Type: Package
Package: ACEP
Title: Análisis Computacional de Eventos de Protesta
Version: 0.1.2
Authors@R: 
    person("Agustín", "Nieto", , "agustin.nieto77@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-4467-873X"))
Description: La librería 'ACEP' contiene funciones específicas para
    desarrollar análisis computacional de eventos de protesta. Asimismo,
    contiene bases de datos con colecciones de notas sobre protestas y
    diccionarios de palabras conflictivas. La colección de diccionarios
    reune diccionarios de diferentes orígenes. 
    The 'ACEP' library contains specific functions to perform computational 
    analysis of protest events. It also contains a database with collections 
    of notes on protests and dictionaries of conflicting words. Collection 
    of dictionaries that brings together dictionaries from different sources.
License: MIT + file LICENSE
URL: https://github.com/agusnieto77/ACEP,
        https://agusnieto77.github.io/ACEP/
BugReports: https://github.com/agusnieto77/ACEP/issues
Depends: R (>= 3.5.0)
Imports: graphics, grDevices, stats, httr, jsonlite, stringr, utils,
        magrittr
Suggests: furrr, ollamar, future, covr, knitr, rmarkdown, reticulate,
        rsyntax, spacyr, tidygeocoder, udpipe, testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
Language: es
LazyData: true
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-06-14 05:59:24 UTC; agusn
Author: Agustín Nieto [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4467-873X>)
Maintainer: Agustín Nieto <agustin.nieto77@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-14 06:30:09 UTC
Built: R 4.4.3; ; 2026-06-14 10:03:31 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,acep_result)
S3method(plot,acep_result)
S3method(print,acep_corpus)
S3method(print,acep_result)
S3method(summary,acep_corpus)
S3method(summary,acep_result)
export("%>%")
export(acep_claude)
export(acep_clean)
export(acep_clear_regex_cache)
export(acep_context)
export(acep_corpus)
export(acep_count)
export(acep_db)
export(acep_detect)
export(acep_extract)
export(acep_frec)
export(acep_gemini)
export(acep_gpt)
export(acep_gpt_schema)
export(acep_int)
export(acep_load_base)
export(acep_may)
export(acep_min)
export(acep_ollama)
export(acep_ollama_setup)
export(acep_openrouter)
export(acep_pipeline)
export(acep_plot_rst)
export(acep_plot_st)
export(acep_postag)
export(acep_postag_hibrido)
export(acep_process_chunks)
export(acep_regex_cache_size)
export(acep_result)
export(acep_sst)
export(acep_svo)
export(acep_together)
export(acep_token)
export(acep_token_plot)
export(acep_token_table)
export(acep_upos)
export(pipe_clean)
export(pipe_count)
export(pipe_intensity)
export(pipe_timeseries)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(graphics,title)
importFrom(httr,GET)
importFrom(httr,http_error)
importFrom(httr,status_code)
importFrom(httr,timeout)
importFrom(jsonlite,read_json)
importFrom(jsonlite,write_json)
importFrom(magrittr,"%>%")
importFrom(stats,aggregate)
importFrom(stats,ave)
importFrom(stats,setNames)
importFrom(stringr,str_count)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_replace_all)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,install.packages)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ACEP: Análisis Computacional de Eventos de Protesta<img src="man/figures/logo.svg" align="right" height="139"/>

<!-- badges: start -->

[![Codecov test
coverage](https://codecov.io/gh/agusnieto77/ACEP/branch/master/graph/badge.svg)](https://app.codecov.io/gh/agusnieto77/ACEP?branch=master)
[![R-CMD-check](https://github.com/agusnieto77/ACEP/workflows/R-CMD-check/badge.svg)](https://github.com/agusnieto77/ACEP/actions)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.6800706.svg)](https://doi.org/10.5281/zenodo.6800706)
[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![CRAN
version](https://img.shields.io/cran/v/ACEP?label=CRAN)](https://cran.r-project.org/package=ACEP)
[![](https://img.shields.io/badge/devel%20version-0.1.2-blue.svg)](https://github.com/agusnieto77/ACEP)
[![License:
MIT](https://img.shields.io/badge/license-MIT-blue.svg)](https://cran.r-project.org/web/licenses/MIT)
[![](https://img.shields.io/github/languages/code-size/agusnieto77/ACEP.svg)](https://github.com/agusnieto77/ACEP)
[![](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![](https://img.shields.io/badge/Build%20with-R%20&%20RStudio-blue?style=plastic=appveyor)](https://github.com/agusnieto77/ACEP)
[![](https://img.shields.io/badge/ACEP-ready%20to%20use-color:%20#39c?style=plastic=appveyor)](https://github.com/agusnieto77/ACEP)
[![](https://img.shields.io/badge/Developed%20by-SISMOS-darkred?style=plastic=appveyor)](https://observatoriodeconflictividad.org/el-pulso-de-la-conflictividad-en-tiempo-real/)
[![](http://cranlogs.r-pkg.org/badges/grand-total/ACEP)](https://cran.r-project.org/package=ACEP)
[![](http://cranlogs.r-pkg.org/badges/last-month/ACEP)](https://cran.r-project.org/package=ACEP)
<!-- badges: end -->

### Vision general

ACEP es un paquete de funciones en lenguaje R útiles para la detección y
el análisis de eventos de protesta en corpus de textos periodísticos.
Sus funciones son aplicables a cualquier corpus de textos. Ademas de las
funciones, ACEP contiene también bases de datos con colecciones de notas
sobre protestas y una colección de diccionarios de palabras conflictivas
y otros tópicos referidos a diferentes aspectos del análisis de eventos
de protesta.

### Instalacion de la version estable

La versión estable actual de ACEP es `0.1.1` y se instala desde
[CRAN](https://CRAN.R-project.org/package=ACEP) con:

``` r
install.packages("ACEP")
```

### Instalacion de la version en desarrollo

Puedes instalar la versión de desarrollo de ACEP desde
[GitHub](https://github.com/agusnieto77/ACEP) con:

``` r
# install.packages("devtools")
devtools::install_github("agusnieto77/ACEP")
```

### Dependencias opcionales

ACEP mantiene una instalación base liviana. Las funciones principales de
limpieza, conteo, diccionarios, series temporales y proveedores LLM
HTTP/JSON se instalan con las dependencias obligatorias del paquete.

Algunas funciones de NLP, etiquetado gramatical y geocodificación
requieren paquetes opcionales. Si vas a usar `acep_postag()`,
`acep_postag_hibrido()`, `acep_upos()` o flujos que dependan de esas
capacidades, instalá los paquetes necesarios con:

``` r
install.packages(c("spacyr", "reticulate", "rsyntax", "tidygeocoder", "udpipe"))
```

Si falta una dependencia opcional, ACEP muestra un mensaje indicando qué
paquete instalar para esa funcionalidad.

### Funciones

| Nombre | Ciclo | Descripción |
|:---|:---|:---|
| `acep_clean()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Limpieza de texto. |
| `acep_claude()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Función para interactuar con modelos Anthropic Claude. |
| `acep_clear_regex_cache()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Limpia la caché de expresiones regulares compiladas. |
| `acep_context()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Función para extraer contexto de palabras o frases. |
| `acep_corpus()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Constructor de objetos acep_corpus para pipelines. |
| `acep_count()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg) | Frecuencia de menciones de palabras. |
| `acep_db()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg) | Frecuencia, menciones e intensidad. |
| `acep_detect()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg) | Detección de menciones de palabras. |
| `acep_extract()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Extraer palabras de un texto. |
| `acep_frec()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg) | Frecuencia de palabras totales. |
| `acep_gemini()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Función para interactuar con modelos Google Gemini. |
| `acep_gpt()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Función para interactuar con modelos OpenAI GPT. |
| `acep_gpt_schema()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Define esquemas para respuestas estructuradas de GPT. |
| `acep_int()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg) | Índice de intensidad. |
| `acep_load_base()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg) | Carga bases de datos creadas por el Observatorio. |
| `acep_may()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Convierte el texto a mayúsculas. |
| `acep_min()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Convierte el texto a minúsculas. |
| `acep_ollama()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Función para interactuar con modelos Ollama locales. |
| `acep_ollama_setup()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Configuración y verificación del entorno Ollama. |
| `acep_openrouter()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Función para interactuar con 400+ modelos vía OpenRouter. |
| `acep_pipeline()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Pipeline composable para procesamiento de texto. |
| `acep_plot_rst()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg) | Resumen visual de la serie temporal de los índices de conflictividad. |
| `acep_plot_st()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg) | Gráfico de barras de la serie temporal de índices de conflictividad. |
| `acep_postag()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Función para etiquetado POS, lematización, tokenización, extracción de entidades. |
| `acep_postag_hibrido()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Etiquetado POS híbrido combinando udpipe y spacyr. |
| `acep_process_chunks()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Procesamiento de textos en chunks para optimizar memoria. |
| `acep_regex_cache_size()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Consulta el tamaño actual de la caché de regex. |
| `acep_result()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Constructor de objetos acep_result para resultados de análisis. |
| `acep_sst()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg) | Serie temporal de índices de conflictividad. |
| `acep_svo()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Función para extraer tripletes SVO (Sujeto-Verbo-Objeto). |
| `acep_together()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Función para interactuar con modelos TogetherAI mediante JSON mode o texto libre. |
| `acep_token()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Función para tokenizar. |
| `acep_token_plot()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Gráfico de barras de palabras más recurrentes en un corpus. |
| `acep_token_table()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Tabla de frecuencia de palabras tokenizadas. |
| `acep_upos()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Función para etiquetado POS, lematización, tokenización. |
| `pipe_clean()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Pipeline: limpieza de texto. |
| `pipe_count()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Pipeline: conteo de menciones. |
| `pipe_intensity()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Pipeline: cálculo de intensidad. |
| `pipe_timeseries()` | ![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg) | Pipeline: series temporales. |

### Colecciones

| Nombre | Descripción |
|:---|:---|
| `acep_bases` | Colección de notas y recursos de prueba (incluye la muestra etiquetada `lc_720`). |
| `acep_diccionarios` | Colección de diccionarios. |
| `acep_prompt_gpt` | Colección de instrucciones. |
| `acep_rs` | Cadenas de caracteres para limpiar y normalizar textos. |

### Corpus

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.6800663.svg)](https://doi.org/10.5281/zenodo.6800663)
Colección de notas del diario ***La Nación***

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.6800765.svg)](https://doi.org/10.5281/zenodo.6800765)
Subset de notas del diario ***La Nación***

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.6800617.svg)](https://doi.org/10.5281/zenodo.6800617)
Colección de notas del ***Ecos Diarios***

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.6800637.svg)](https://doi.org/10.5281/zenodo.6800637)
Colección de notas de la ***Revista Puerto***

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.6800650.svg)](https://doi.org/10.5281/zenodo.6800650)
Colección de notas del diario ***La Nueva***

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.6800660.svg)](https://doi.org/10.5281/zenodo.6800660)
Colección de notas del diario ***La Capital***

### Bases de datos de Eventos de protesta disponibles online

[ACLED](https://acleddata.com/#/dashboard): Armed Conflict Location &
Event Data Project.

[GDELT](https://www.gdeltproject.org/): The GDELT Project About.

[GPT](https://carnegieendowment.org/features/global-protest-tracker):
Global Protest Tracker.

MMPD: Mass Mobilization Protest Data Project.

NAVCO: Nonviolent and Violent Campaigns and Outcomes data project.

[NVCO](https://nvdatabase.swarthmore.edu/): Global Nonviolent Action
Database.

[SCAD](https://www.strausscenter.org/ccaps-research-areas/social-conflict/database/):
Social Conflict Analysis Database.

[SPEED](https://clinecenter.illinois.edu/project/human-loop-event-data-projects/SPEED):
The Social, Political and Economic Event Database Project.

[UCDP](https://www.uu.se/en/department/peace-and-conflict-research/research/ucdp/):
Uppsala Conflict Data Program.

### Bases de datos de interes general

[FMI](https://data.imf.org/?sk=388dfa60-1d26-4ade-b505-a05a558d9a42):
FMI Data.

[BM](https://datos.bancomundial.org/): Datos de libre acceso del Banco
Mundial.

[OIT](https://www.ilo.org/es/datos-y-estadisticas): Estadísticas y bases
de datos.

[CEPAL](https://www.cepal.org/es/dados-estatisticas-espanhol): Datos y
estadísticas.

[DARG](https://www.datos.gob.ar/): Datos abiertos de Argentina.

MGP: Datos abiertos del Municipio de Gral. Pueyrredon, Buenos Aires,
Argentina.

### Uso de las funciones del paquete ACEP: un ejemplo.

``` r
# Cargamos la librería
require(ACEP)

# Cargamos la base de notas de la Revista Puerto con la función acep_load_base()
rev_puerto <- acep_load_base(acep_bases$rp_mdp)

# Cargamos el diccionario de conflictos de SISMOS
dicc_confl_sismos <- acep_diccionarios$dicc_confl_sismos

# Con la función acep_frec() contamos la frecuencia de palabras de cada nota
# y creamos una nueva columna llamada  n_palabras
rev_puerto$n_palabras <- acep_frec(rev_puerto$nota)

# Imprimimos en pantalla la base con la nueva columna de frecuencia de palabras
head(rev_puerto)
```

    #> # A tibble: 6 × 7
    #>   fecha      titulo                         bajada nota  imagen link  n_palabras
    #>   <date>     <chr>                          <chr>  <chr> <chr>  <chr>      <int>
    #> 1 2020-12-29 ¡Feliz Año 2021 para todos nu… Con m… "Con… https… http…         28
    #> 2 2020-12-28 Mapa del trabajo esclavo en a… Un re… "El … https… http…       1142
    #> 3 2020-12-24 Plantas piden tener garantiza… En Ch… "El … https… http…        536
    #> 4 2020-12-24 Los obreros navales despiden … En Ma… "El … https… http…        489
    #> 5 2020-12-23 El incumplimiento del régimen… Se ll… "Las… https… http…        529
    #> 6 2020-12-23 Otro fallo ratifica cautelar … La Cá… "La … https… http…        467

``` r
# Ahora con la función acep_count() contamos la frecuencia de menciones de
# términos del diccionario de conflictividad de SISMOS de cada nota y
# creamos una nueva columna llamada  conflictos.
# Elaboramos un corpus acotado para el ejemplo
rev_puerto <- rev_puerto[1:100, ]
rev_puerto$conflictos <- acep_count(rev_puerto$nota, dicc_confl_sismos)

# Imprimimos en pantalla la base con la nueva columna de
# menciones del diccionario de conflictividad
head(rev_puerto)
```

    #> # A tibble: 6 × 8
    #>   fecha      titulo              bajada nota  imagen link  n_palabras conflictos
    #>   <date>     <chr>               <chr>  <chr> <chr>  <chr>      <int>      <int>
    #> 1 2020-12-29 ¡Feliz Año 2021 pa… Con m… "Con… https… http…         28          0
    #> 2 2020-12-28 Mapa del trabajo e… Un re… "El … https… http…       1142          0
    #> 3 2020-12-24 Plantas piden tene… En Ch… "El … https… http…        536          0
    #> 4 2020-12-24 Los obreros navale… En Ma… "El … https… http…        489          0
    #> 5 2020-12-23 El incumplimiento … Se ll… "Las… https… http…        529          0
    #> 6 2020-12-23 Otro fallo ratific… La Cá… "La … https… http…        467          0

``` r
# Ahora con la función acep_int() calculamos un índice de intensidad de
# la conflictividad y creamos una nueva columna llamada  intensidad
rev_puerto$intensidad <- acep_int(
  rev_puerto$conflictos,
  rev_puerto$n_palabras,
  3)

# Imprimimos en pantalla la base con la nueva columna de intensidad
head(rev_puerto)
```

    #> # A tibble: 6 × 9
    #>   fecha      titulo   bajada nota  imagen link  n_palabras conflictos intensidad
    #>   <date>     <chr>    <chr>  <chr> <chr>  <chr>      <int>      <int>      <dbl>
    #> 1 2020-12-29 ¡Feliz … Con m… "Con… https… http…         28          0          0
    #> 2 2020-12-28 Mapa de… Un re… "El … https… http…       1142          0          0
    #> 3 2020-12-24 Plantas… En Ch… "El … https… http…        536          0          0
    #> 4 2020-12-24 Los obr… En Ma… "El … https… http…        489          0          0
    #> 5 2020-12-23 El incu… Se ll… "Las… https… http…        529          0          0
    #> 6 2020-12-23 Otro fa… La Cá… "La … https… http…        467          0          0

``` r
# Volvemos a cargar la base de notas de la Revista Puerto sin procesar
rev_puerto <- acep_load_base(acep_bases$rp_mdp)

# Creamos un subset
subset_rp <- rev_puerto[1:100, ]

# Cargamos el diccionario de conflictos de SISMOS
dicc_confl_sismos <- acep_diccionarios$dicc_confl_sismos

# Ahora con la función acep_db() aplicamos las tres funciones en un solo paso
rp_procesada <- acep_db(subset_rp, subset_rp$nota, dicc_confl_sismos, 3)

# Imprimimos en pantalla la base con las tres columna creadas
head(rp_procesada)
```

    #> # A tibble: 6 × 9
    #>   fecha      titulo   bajada nota  imagen link  n_palabras conflictos intensidad
    #>   <date>     <chr>    <chr>  <chr> <chr>  <chr>      <int>      <int>      <dbl>
    #> 1 2020-12-29 ¡Feliz … Con m… "Con… https… http…         28          0          0
    #> 2 2020-12-28 Mapa de… Un re… "El … https… http…       1142          0          0
    #> 3 2020-12-24 Plantas… En Ch… "El … https… http…        536          0          0
    #> 4 2020-12-24 Los obr… En Ma… "El … https… http…        489          0          0
    #> 5 2020-12-23 El incu… Se ll… "Las… https… http…        529          0          0
    #> 6 2020-12-23 Otro fa… La Cá… "La … https… http…        467          0          0

``` r
# Cargamos los datos procesados
rp_procesada <- acep_bases$rp_procesada

# Ahora con la función acep_sst() elaboramos un resumen estadístico
rp_procesada <- acep_sst(rp_procesada, st = "anio", u = 4)

# Imprimimos en pantalla la base con las métricas de conflictividad
head(rp_procesada)
```

    #>     st frecn csn  frecp frecm  intac intensidad int_notas_confl
    #> 1 2009   632  58 496110  1025 1.2735     0.0021          0.0918
    #> 2 2010   680  67 492231  1129 1.6273     0.0023          0.0985
    #> 3 2011   601  40 425747   882 1.2204     0.0021          0.0666
    #> 4 2012   739  67 564270  1242 1.6841     0.0022          0.0907
    #> 5 2013   689  24 525718   758 1.0559     0.0014          0.0348
    #> 6 2014   631  30 444823   802 1.2112     0.0018          0.0475

``` r
# Ahora con la función acep_plot_st() elaboramos un gráfico de barras
# con menciones del diccionario de conflictividad
acep_plot_st(rp_procesada$st, rp_procesada$frecm,
 t = "Evolucion de la conflictividad en el sector pesquero argentino",
 ejex = "A\u00f1os analizados",
 ejey = "Menciones del diccionario de conflictos",
 etiquetax = "horizontal")
```

<img src="man/figures/README-ejemplo3-1.png" width="100%" />

``` r
# Ahora con la función acep_plot_rst() elaboramos una visualización resumen.
# con cuatro gráficos de barras
acep_plot_rst(rp_procesada, tagx = "vertical")
```

<img src="man/figures/README-ejemplo3-2.png" width="100%" />

#' @title Tabla de frecuencia de palabras tokenizadas.
#' @description Función que cuenta la frecuencia de palabras tokenizadas.
#' @param x vector de palabras tokenizadas.
#' @param u número de corte para el top de palabras más frecuentes.
#' @return Si todas las entradas son correctas,
#' la salida sera una tabla con la frecuencia relativa y
#' absoluta de palabras tokenizadas.
#' @keywords tokens tablas
#' @examples
#' tokens <- c(rep("paro",15), rep("piquete",25), rep("corte",20), rep("manifestación",10),
#' rep("bloqueo",5), rep("alerta",16), rep("ciudad",12), rep("sindicato",11), rep("paritaria",14),
#' rep("huelga",14), rep("escrache",15))
#' acep_token_table(tokens)
#' @export
acep_token_table <- function(x, u = 10) {
  if(!is.character(x) | is.list(x)){
    return(message("No ingresaste un vector en el par\u00e1metro x.
            Vuelve a intentarlo ingresando un vector!"))
  }
  if (!is.numeric(u)) {
    return(message("El par\u00e1metro 'u' debe ser un n\u00famero entero positivo"))
  } else {
      tryCatch({
  tabla_token <- as.data.frame(table(x))
  tabla_token$x <- as.character(tabla_token$x)
  tabla_token <- tabla_token[order(tabla_token$Freq, decreasing = TRUE), ]
  # La proporción se calcula sobre el total del corpus (antes de truncar),
  # no sobre el subconjunto top-u.
  tabla_token$prop <- tabla_token$Freq / sum(tabla_token$Freq)
  tabla_token <- utils::head(tabla_token, n = u)
  data.frame(
    token = tabla_token$x,
    frec = tabla_token$Freq,
    prop = tabla_token$prop)
      }
      )
  }
}

#' @title Función para etiquetado POS, lematización, tokenización.
#' @description
#' Función que devuelve un marco de datos objetos con
#' etiquetado POS (modelo udpipe) para su posterior
#' procesamiento con la función acep_postag.
#' @param texto vector con los textos a procesar.
#' @param modelo idioma del modelo de etiquetado POS del paquete \code{udpipe}.
#' @return Si todas las entradas son correctas, la salida sera un marco de datos con 17 variables.
#' @references Welbers, K., Atteveldt, W. van, & Kleinnijenhuis, J. 2021. Extracting semantic relations using syntax: An R package for querying and reshaping dependency trees. Computational Communication Research, 3-2, 1-16.
#' \doi{10.5117/CCR2021.2.003.WELB}
#' @source \href{https://universaldependencies.org/}{Dependencias Universales para taggeo POS}
#' @source \href{https://ufal.mff.cuni.cz/~straka/papers/2017-conll_udpipe.pdf}{Sobre el modelo UDPipe}
#' @source \href{https://CRAN.R-project.org/package=rsyntax}{Sobre el paquete rsyntax}
#' @keywords sintaxis
#' @examples
#' \dontrun{
#' texto <- "El SOIP declara la huelga en demanda de aumento salarial."
#' acep_upos(texto)
#'}
#' @export
acep_upos <- function(texto, modelo = "spanish"
){
  if (!is.character(texto)) {
    stop("El parametro 'texto' debe ser una cadena de caracteres",
         call. = FALSE)
  }
  available_models <- c('english', 'portuguese', 'spanish')
  if (!modelo %in% available_models) {
    stop(
      "El parametro 'modelo' debe ser un modelo 'udpipe' valido del espanol, ingles o portugues",
      call. = FALSE)
  }
  acep_require_namespace("udpipe", "acep_upos")
  acep_require_namespace("rsyntax", "acep_upos")
  texto <- gsub(",\\S", ", ", texto)
  texto <- gsub("(\\s[A-Z])\\.\\s", "\\1 ", texto)
  texto <- gsub("(\\s[A-Z][a-zA-Z]*);\\s", "\\1, ", texto)
  texto <- gsub(";(\\s[a-zA-Z]*|\\s[a-zA-Z]*\\s[a-zA-Z]*|\\s[a-zA-Z]*\\s[a-zA-Z]*\\s[a-zA-Z]*);\\s", "\\1, ", texto)
  texto <- gsub(";(\\s[a-zA-Z]*\\.\\s|\\s[a-zA-Z]*\\s[a-zA-Z]*\\.\\s|\\s[a-zA-Z]*\\s[a-zA-Z]*\\s[a-zA-Z]*\\.\\s)", "\\1", texto)
  texto <- gsub(";(\\s[a-zA-Z]*\\.\\b|\\s[a-zA-Z]*\\s[a-zA-Z]*\\.\\b|\\s[a-zA-Z]*\\s[a-zA-Z]*\\s[a-zA-Z]*\\.\\b)", "\\1", texto)
  abrev <- c("Arq\\.", "Agr\\.", "art\\.", "atte\\.", "aprox\\.", "Av\\.",
             "Avda\\.", "Bco\\.", "Bs\\.", "As\\.", "c\\.", "Cap\\.", "Fed\\.",
             "Comp\\.", "C\\u00eda\\.", "Cit\\.", "cja\\.", "c\\u00f3d\\.", "Cte\\.", "cta\\.",
             "Dpto\\.", "cit\\.", "Dr\\.", "Dra\\.", "Ej\\.", "esq\\.", "Excmo\\.",
             "Excma\\.", "cit\\.", "Flia\\.", "Gral\\.", "hs\\.", "hnos\\.",
             "Ing\\.", "Izq\\.", "Kg\\.", "Km\\.", "Lic\\.", "Ma\\.", "m\\u00e1x\\.",
             "Min\\.", "M\\u00edn\\.", "Nro\\.", "N\\u00fam\\.", "Op\\.", "P\\.", "P\\u00e1g\\.",
             "pdo\\.", "ppdo\\.", "Prof\\.", "Profra\\.", "Sr\\.", "Sra\\.", "Sres\\.",
             "Srta\\.", "Sta\\.", "Sto\\.", "Tel\\.", "Ud\\.", "Udes\\.", "V\\.",
             "Vda\\.", "Vdo\\.", "Vol\\.", "Vols\\.", "Vta\\.", "Vd\\.", "Vds\\.")
  abrev <- paste0(" ", abrev)
  abrev <- c(abrev, ACEP::acep_min(abrev))
  for (i in abrev) {
    texto <- gsub(i, gsub("\\.","",i), texto)
  }
  acep_udpipe <- udpipe::udpipe(x = texto, object = modelo)
  acep_udpipe <- subset(acep_udpipe, !is.na(dep_rel))
  acep_udpipe$doc_id <- as.integer(gsub("doc", "", acep_udpipe$doc_id))
  acep_udpipe$sentence <- gsub("^\\n*|^\n*\\s*|^\\s*|^\\s*\n*|*\n$|*\\s*\n$|*\\s$|*\n*\\s$", "", acep_udpipe$sentence)
  acep_udpipe <- subset(acep_udpipe, sentence != "")
  acep_udpipe$dep_rel <- gsub("root", "ROOT", acep_udpipe$dep_rel)
  acep_udpipe$head_token_id <- ifelse(acep_udpipe$head_token_id == 0, NA, acep_udpipe$head_token_id)
  acep_udpipe$head_token_id <- as.integer(acep_udpipe$head_token_id)
  colnames(acep_udpipe) <- c("doc_id","paragraph_id","sentence","sent","start","end","term_id","token_id",
                             "token","lemma","pos","xpos","morph","parent","relation","deps","misc")
  acep_udpipe <- rsyntax::as_tokenindex(acep_udpipe)
  return(acep_udpipe)
}

# Offline validation tests (no network, no udpipe model download).
test_that("acep_upos rechaza texto que no es caracter", {
  expect_error(acep_upos(1:10, modelo = "spanish"), "cadena de caracteres")
})

test_that("acep_upos rechaza modelos invalidos", {
  expect_error(acep_upos("texto de prueba", modelo = "klingon"), "modelo")
})

#' Check optional package availability
#'
#' Internal helper for optional integrations.
#'
#' @noRd
acep_require_namespace <- function(pkg, feature) {
  if (!requireNamespace(pkg, quietly = TRUE)) {
    stop(
      sprintf(
        "La funcionalidad '%s' requiere instalar el paquete opcional '%s'. Instala el paquete con: install.packages(\"%s\")",
        feature,
        pkg,
        pkg
      ),
      call. = FALSE
    )
  }

  invisible(NULL)
}

#' @title Frecuencia de palabras totales.
#' @description Función que cuenta la frecuencia de palabras totales
#' en cada una de las notas/textos.
#' @param x vector de textos al que se le aplica la función de conteo
#' de la frecuencia de palabras.
#' @importFrom stringr str_count
#' @keywords indicadores frecuencia tokens
#' @return Si todas las entradas son correctas, la salida sera un vector
#' con una frecuencia de palabras.
#' @examples
#' acep_frec("El SUTEBA fue al paro. Reclaman mejoras salariales.")
#' @export
acep_frec <- function(x) {
  if(!is.character(x) || is.list(x)){
    return(message("No ingresaste un vector de texto.
           Vuelve a intentarlo ingresando un vector de texto!"))
  } else {
    out <- tryCatch({ str_count(x, "\\S+") })
    return(out)
  }
}

test_that("ACEP Frec", {
  skip_if_offline()
  skip_on_cran()
  frec <- acep_frec("El SUTEBA fue al paro. Reclaman mejoras salariales.")
  dimensiones <- length(frec)
  expect_equal(dimensiones, length(frec))
})

test_that("ACEP Frec E1", {
  skip_if_offline()
  skip_on_cran()
  df <- data.frame(texto =
                     c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
                       "El SOIP lleva adelante un plan de lucha con paros y piquetes."))
  frec <- acep_frec(df)
  dimensiones <- length(frec)
  expect_equal(dimensiones, length(frec))
})

#' @title Gráfico de barras de palabras más recurrentes en un corpus.
#' @description Función que devuelve un gráfico de barras con las palabras
#' mas recurrentes en un corpus textual.
#' @param x vector de palabras tokenizadas.
#' @param u numero de corte para el top de palabras mas frecuentes.
#' @param frec parámetro para determinar si los valores se visualizaran
#' como frecuencia absoluta o relativa.
#' @return Si todas las entradas son correctas,
#' la salida sera un gráfico de barras.
#' @keywords visualizacion
#' @examples
#' tokens <- c(rep("paro",15), rep("piquete",25), rep("corte",20), rep("manifestación",10),
#' rep("bloqueo",5), rep("alerta",16), rep("ciudad",12), rep("sindicato",11), rep("paritaria",14),
#' rep("huelga",14), rep("escrache",15))
#' acep_token_plot(tokens)
#' @export
acep_token_plot <- function(x, u = 10, frec = TRUE) {
  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldpar))
  graphics::par(mar = c(6, 9, 4, 1))
  if(!is.character(x) | is.list(x)){
    return(message("No ingresaste un vector en el par\u00e1metro x.
           Vuelve a intentarlo ingresando un vector!"))
  }
  if (!is.logical(frec)) {
    return(message("Debe ingresar un valor booleano: TRUE o FALSE"))
  }
  if (!is.numeric(u)) {
    message("El par\u00e1metro 'u' debe ser un n\u00famero entero positivo")
    } else {
      tryCatch({
        if (frec) {
          tabla_token <- as.data.frame(table(x))
          tabla_token$x <- as.character(tabla_token$x)
          tabla_token <- tabla_token[order(tabla_token$Freq, decreasing = TRUE), ]
          tabla_token$prop <- tabla_token$Freq / sum(tabla_token$Freq)
          tabla_token <- utils::head(tabla_token, n = u)
          tabla_token <- data.frame(
            token = tabla_token$x,
            frec = tabla_token$Freq,
            prop = tabla_token$prop)
          tabla_token <- tabla_token[order(tabla_token$frec,
                                           decreasing = FALSE), ]
          graphics::barplot(
            height = tabla_token$frec,
            names = as.factor(tabla_token$token),
            col = grDevices::hcl.colors(u, "mint", rev = TRUE),
            xlab = "frecuencia",
            ylab = NULL,
            main = paste("Top", u, "de palabras frecuentes"),
            xlim = c(0, (max(tabla_token$frec) * 1.1)),
            horiz = TRUE,
            las = 1,
            cex.names = 0.8,
            cex.axis = 0.8
          )
  } else {
    tabla_token <- as.data.frame(table(x))
    tabla_token$x <- as.character(tabla_token$x)
    tabla_token <- tabla_token[order(tabla_token$Freq, decreasing = TRUE), ]
    tabla_token$prop <- tabla_token$Freq / sum(tabla_token$Freq)
    tabla_token <- utils::head(tabla_token, n = u)
    tabla_token <- data.frame(token = tabla_token$x,
                              frec = tabla_token$Freq,
                              prop = tabla_token$prop)
    tabla_token <- tabla_token[order(tabla_token$frec, decreasing = FALSE), ]
    graphics::barplot(
      height = tabla_token$prop,
      names = as.factor(tabla_token$token),
      col = grDevices::hcl.colors(u, "purp", rev = TRUE),
      xlab = "porcentaje",
      ylab = NULL,
      main = paste("Top", u, "de palabras frecuentes"),
      xlim = c(0, (max(tabla_token$prop) * 1.15)),
      horiz = TRUE,
      las = 1,
      cex.names = 0.8,
      cex.axis = 0.8
    )
    }
      })
  }
  }

test_that("ACEP TOKEN PLOT T", {
  skip_if_offline()
  skip_on_cran()
  tokens <- c(rep("paro",15), rep("piquete",25), rep("corte",20), rep("manifestacion",10),
  rep("bloqueo",5), rep("alerta",16), rep("ciudad",12), rep("sindicato",11), rep("paritaria",14),
  rep("huelga",14), rep("escrache",15))
  acep_token_plot(tokens)
  dimensiones <- length(tokens)
  expect_equal(dimensiones, length(tokens))
})

test_that("ACEP TOKEN PLOT F", {
  skip_if_offline()
  skip_on_cran()
  tokens <- c(rep("paro",15), rep("piquete",25), rep("corte",20), rep("manifestacion",10),
              rep("bloqueo",5), rep("alerta",16), rep("ciudad",12), rep("sindicato",11), rep("paritaria",14),
              rep("huelga",14), rep("escrache",15))
  acep_token_plot(tokens, frec = FALSE)
  dimensiones <- length(tokens)
  expect_equal(dimensiones, length(tokens))
})

test_that("ACEP TOKEN PLOT T E1", {
  skip_if_offline()
  skip_on_cran()
  df <- data.frame(texto = c("El SOIP lleva adelante un plan de lucha con paros y piquetes."))
  acep_token_plot(df)
  dimensiones <- length(df)
  expect_equal(dimensiones, length(df))
})

#' @title Detección de menciones de palabras.
#' @description Función que detecta de menciones de palabras que
#' refieren a conflictos en cada una de las notas/textos.
#' @param x vector de textos al que se le aplica la función de
#' detección de menciones de palabras del diccionario.
#' @param y vector de palabras del diccionario utilizado.
#' @param u umbral para atribuir valor positivo a la
#' detección de las menciones.
#' @param tolower convierte los textos a minúsculas.
#' @return Si todas las entradas son correctas,
#' la salida sera un vector numérico.
#' @importFrom stringr str_count
#' @keywords indicadores frecuencia tokens
#' @examples
#' df <- data.frame(texto = c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
#' "El SOIP lleva adelante un plan de lucha con paros y piquetes."))
#' diccionario <- c("paro", "lucha", "piquetes")
#' df$detect <- acep_detect(df$texto, diccionario)
#' df
#' @export
acep_detect <- function(x, y, u = 1, tolower = TRUE) {
  validate_character(x, "x")
  validate_character(y, "y")
  validate_numeric(u, "u", min = 0)
  validate_logical(tolower, "tolower")

  if (tolower) {
    x <- tolower(x)
  }

  out <- tryCatch({
    dicc <- .acep_count_pattern(y)
    detect <- stringr::str_count(x, dicc)
    ifelse(as.numeric(detect) >= u, 1, 0)
  })

  return(out)
}

test_that("ACEP DETECT", {
  skip_if_offline()
  skip_on_cran()
  df <- data.frame(texto = c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
  "El SOIP lleva adelante un plan de lucha con paros y piquetes."))
  diccionario <- c("paro", "lucha", "piquetes")
  detect <- acep_detect(df$texto, diccionario)
  dimensiones <- length(detect)
  expect_equal(dimensiones, length(detect))
})

test_that("ACEP DETECT F", {
  skip_if_offline()
  skip_on_cran()
  df <- data.frame(texto = c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
                             "El SOIP lleva adelante un plan de lucha con paros y piquetes."))
  diccionario <- c("paro", "lucha", "piquetes")
  detect <- acep_detect(df$texto, diccionario, tolower = FALSE)
  dimensiones <- length(detect)
  expect_equal(dimensiones, length(detect))
})

test_that("ACEP DETECT E1", {
  skip_if_offline()
  skip_on_cran()
  df <- data.frame(texto = c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
                             "El SOIP lleva adelante un plan de lucha con paros y piquetes."))
  diccionario <- c("paro", "lucha", "piquetes")

  expect_error(
    acep_detect(df, diccionario),
    "debe ser un vector de texto"
  )
})

#' @title Colección de instrucciones para GPT.
#' @description Colección de instrucciones para interactuar con los modelos
#' de OpenAI. Las instrucciones fueron testeadas en el marco de las tareas
#' que realizamos en el Observatorio de Conflictividad Social de la
#' Universidad Nacional de Mar del Plata.
#' @format Es un objeto de clase 'list' con 6 componentes.
#'\describe{
#' \item{instruccion_breve_sao_es}{es un texto en castellano con instrucciones breves para
#'  extraer eventos de protesta y codificarlos con las siguientes claves: 'fecha',
#'  'sujeto', 'accion', 'objeto', 'lugar'.}
#'  \item{instruccion_larga_sao_es}{es un texto en castellano con instrucciones largas para
#'  extraer eventos de protesta y codificarlos con las siguientes claves: 'id',
#'  'cronica', 'fecha', 'sujeto', 'organizacion', 'participacion', 'accion',
#'  'objeto', 'lugar'.}
#' \item{instruccion_breve_sao_en}{es un texto en inglés con instrucciones breves para
#'  extraer eventos de protesta y codificarlos con las siguientes claves: 'date',
#'  'subject', 'action', 'object', 'place'.}
#'  \item{instruccion_larga_sao_en}{es un texto en inglés con instrucciones largas para
#'  extraer eventos de protesta y codificarlos con las siguientes claves: 'id',
#'  'chronicle', 'date', 'subject', 'organization', 'participation', 'action',
#'  'object', 'place'.}
#' \item{system_prompt_01_es}{es el prompt de sistema en castellano que utiliza
#'  \code{\link{acep_ollama}} para guiar el análisis de texto.}
#' \item{system_prompt_01_en}{es el prompt de sistema en inglés equivalente a
#'  system_prompt_01_es.}
#'}
#' @docType data
#' @usage data(acep_prompt_gpt)
#' @keywords diccionarios
#' @examples
#' prompt01 <- acep_prompt_gpt$instruccion_larga_sao_es
#' prompt01
"acep_prompt_gpt"

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = FALSE,
  comment = "#>"
)

## ----setup, eval=require("tibble"), message=FALSE-----------------------------
# Cargamos la librería ACEP
library(ACEP)

# Cargamos una muestra incluida en el paquete
rev_puerto <- acep_bases$lc_720

# Imprimimos la base en consola
rev_puerto

## ----diccionarios0, eval=require("tibble"), message=FALSE---------------------
# Creamos el diccionario de palabras que refieren a huelgas
dicc_huelgas <- c("en paro", "al paro", "huelga", "huelguistas", "paro y movil",
                  "paro de actividades", "conciliación obligatoria", "un paro", 
                  "paro total", "paro parcial", "trabajo a reglamento", 
                  "el paro", "de brazos caídos")

# Cargamos el diccionario de palabras que refieren a conflictividad
dicc_conflictos_base <- c("conflicto", "conflictos", "protesta", "protestas",
                          "reclamo", "reclamos", "paro", "huelga",
                          "movilización", "manifestación")
dicc_conflictos <- unique(c(dicc_conflictos_base, dicc_huelgas))

# Creamos la variable con la frecuencia de palabras que refieren a conflictividad
rev_puerto$frec_conflictos <- acep_count(rev_puerto$nota, dicc_conflictos)

# Creamos la variable con la frecuencia de palabras que refieren a huelgas
rev_puerto$frec_huelgas <- acep_count(rev_puerto$nota, dicc_huelgas)

# Creamos el diccionario de palabras que refieren a actores colectivos
dicc_actores <- c("trabajadores", "docentes", "sindicato", "vecinos",
                  "municipal", "gobierno")

# Creamos la variable con la frecuencia de palabras que 
# refieren a actores colectivos
rev_puerto$frec_actores <- acep_count(rev_puerto$nota, dicc_actores)

# Imprimimos la base en consola
rev_puerto


## ----diccionarios1, eval=require("tibble")------------------------------------

# Creamos la variable con las palabras que refieren a conflictividad
rev_puerto$extract_conflictos <- acep_extract(rev_puerto$nota, dicc_conflictos, izq = "")

# Creamos la variable con las palabras que refieren a huelgas
rev_puerto$extract_huelgas <- acep_extract(rev_puerto$nota, dicc_huelgas)

# Creamos la variable con las palabras que 
# refieren a actores colectivos
rev_puerto$extract_actores <- acep_extract(rev_puerto$nota, dicc_actores)

# Imprimimos la base en consola
rev_puerto


## ----ratio, eval=require("tibble"), message=FALSE-----------------------------
# Seleccionamos las variables de extracción de palabras clave
rev_puerto_huelgas <- rev_puerto[rev_puerto$extract_huelgas != "",]
rev_puerto_actores <- rev_puerto_huelgas[rev_puerto_huelgas$extract_actores != "",]
rev_puerto_seleccion <- rev_puerto_actores[ , c('extract_conflictos', 'extract_huelgas', 'extract_actores')]

# Imprimimos la base en consola
rev_puerto_seleccion

#' @title Carga los corpus y las bases creadas por el Observatorio.
#' @description Función para cargar bases de datos disponibles online.
#' Por ahora están disponibles las siguientes bases: Revista Puerto 'rp_mdp';
#' La Nueva 'ln_bb', La Capital 'lc_mdp', Ecos Diarios 'ed_neco',
#' La Nación 'ln_arg'
#' @param tag etiqueta identificatoria del data frame a cargar:
#' acep_bases$rp_mdp, acep_bases$ln_bb, acep_bases$lc_mdp,
#' acep_bases$ed_neco, acep_bases$ln_arg
#' @keywords datos
#' @importFrom utils download.file
#' @importFrom httr GET http_error status_code timeout
#' @return Si la descarga es exitosa, devuelve una base de datos en formato
#' tabular con un corpus de notas. Ante un fallo de red, una URL inexistente o
#' un error de lectura, informa el problema con un `message()` y devuelve `NULL`
#' de forma invisible.
#' @examples
#' \dontrun{
#' bd_sismos <- acep_bases$rev_puerto
#' head(acep_load_base(tag = bd_sismos))
#' }
#' @export
acep_load_base <- function(tag) {
  if (!is.character(tag) || length(tag) != 1L || is.na(tag)) {
    stop("El parametro 'tag' debe ser una unica URL de tipo caracter.",
         call. = FALSE)
  }

  respuesta <- tryCatch(
    httr::GET(tag, httr::timeout(60)),
    error = function(e) {
      message("No se pudo conectar con la URL: ", conditionMessage(e))
      NULL
    }
  )
  if (is.null(respuesta)) {
    return(invisible(NULL))
  }
  if (httr::http_error(respuesta)) {
    message("La URL parece no existir o no esta disponible (HTTP ",
            httr::status_code(respuesta), "). Intentalo con otra url!")
    return(invisible(NULL))
  }

  message("Descargando...")
  tryCatch({
    nombre <- basename(tag)
    destfile <- file.path(tempdir(), nombre)
    utils::download.file(tag, destfile, quiet = TRUE, mode = "wb")
    readRDS(destfile)
  },
  error = function(e) {
    message("Error al descargar o leer el archivo: ", conditionMessage(e))
    invisible(NULL)
  })
}

# Offline tests (no network). acep_load_base downloads remote datasets, so its
# successful-download path is intentionally not exercised here (CRAN policy:
# tests must not require internet access). We test the input-validation contract.
test_that("acep_load_base valida el argumento tag", {
  expect_error(acep_load_base(123), "URL")
  expect_error(acep_load_base(c("a", "b")), "URL")
  expect_error(acep_load_base(NA_character_), "URL")
})

#' Validate Character Input
#'
#' @description Internal helper function to validate character vector inputs.
#' @param x The input to validate.
#' @param arg_name Name of the argument being validated (for error messages).
#' @param allow_null Logical. If TRUE, NULL values are accepted. Default FALSE.
#' @return Throws an error if validation fails, otherwise returns invisibly.
#' @keywords internal
#' @noRd
validate_character <- function(x, arg_name = "x", allow_null = FALSE) {
  if (allow_null && is.null(x)) {
    return(invisible(NULL))
  }

  if (!is.character(x) || is.list(x)) {
    stop(
      sprintf("El parametro '%s' debe ser un vector de texto (character).\n", arg_name),
      sprintf("Recibido: %s", class(x)[1]),
      call. = FALSE
    )
  }

  invisible(NULL)
}

#' Validate Numeric Input
#'
#' @description Internal helper function to validate numeric inputs.
#' @param x The input to validate.
#' @param arg_name Name of the argument being validated (for error messages).
#' @param min Optional minimum value (inclusive).
#' @param max Optional maximum value (inclusive).
#' @param allow_null Logical. If TRUE, NULL values are accepted. Default FALSE.
#' @return Throws an error if validation fails, otherwise returns invisibly.
#' @keywords internal
#' @noRd
validate_numeric <- function(x, arg_name = "x", min = NULL, max = NULL, allow_null = FALSE) {
  if (allow_null && is.null(x)) {
    return(invisible(NULL))
  }

  if (!is.numeric(x)) {
    stop(
      sprintf("El parametro '%s' debe ser numerico.\n", arg_name),
      sprintf("Recibido: %s", class(x)[1]),
      call. = FALSE
    )
  }

  if (!is.null(min) && any(x < min, na.rm = TRUE)) {
    stop(
      sprintf("El parametro '%s' debe ser >= %s", arg_name, min),
      call. = FALSE
    )
  }

  if (!is.null(max) && any(x > max, na.rm = TRUE)) {
    stop(
      sprintf("El parametro '%s' debe ser <= %s", arg_name, max),
      call. = FALSE
    )
  }

  invisible(NULL)
}

#' Validate Logical Input
#'
#' @description Internal helper function to validate logical (boolean) inputs.
#' @param x The input to validate.
#' @param arg_name Name of the argument being validated (for error messages).
#' @return Throws an error if validation fails, otherwise returns invisibly.
#' @keywords internal
#' @noRd
validate_logical <- function(x, arg_name = "x") {
  if (!is.logical(x) || length(x) != 1) {
    stop(
      sprintf("El parametro '%s' debe ser un valor logico (TRUE o FALSE).\n", arg_name),
      sprintf("Recibido: %s", paste(class(x), collapse = ", ")),
      call. = FALSE
    )
  }

  invisible(NULL)
}

#' Validate Data Frame Structure
#'
#' @description Internal helper function to validate data frame structure.
#' @param df The data frame to validate.
#' @param required_cols Character vector of required column names.
#' @param arg_name Name of the argument being validated (for error messages).
#' @return Throws an error if validation fails, otherwise returns invisibly.
#' @keywords internal
#' @noRd
validate_dataframe <- function(df, required_cols = NULL, arg_name = "datos") {
  if (!is.data.frame(df)) {
    stop(
      sprintf("El parametro '%s' debe ser un data frame.\n", arg_name),
      sprintf("Recibido: %s", class(df)[1]),
      call. = FALSE
    )
  }

  if (!is.null(required_cols)) {
    missing_cols <- setdiff(required_cols, names(df))

    if (length(missing_cols) > 0) {
      stop(
        sprintf("El data frame '%s' debe contener las siguientes columnas: %s\n",
                arg_name, paste(required_cols, collapse = ", ")),
        sprintf("Columnas faltantes: %s\n", paste(missing_cols, collapse = ", ")),
        sprintf("Columnas disponibles: %s", paste(names(df), collapse = ", ")),
        call. = FALSE
      )
    }
  }

  invisible(NULL)
}

#' Validate Date Vector
#'
#' @description Internal helper function to validate Date vectors.
#' @param x The input to validate.
#' @param arg_name Name of the argument being validated (for error messages).
#' @return Throws an error if validation fails, otherwise returns invisibly.
#' @keywords internal
#' @noRd
validate_date <- function(x, arg_name = "fecha") {
  if (!inherits(x, "Date")) {
    # Try to check if it can be converted
    if (is.character(x)) {
      test_date <- try(as.Date(x), silent = TRUE)
      if (inherits(test_date, "try-error")) {
        stop(
          sprintf("El parametro '%s' debe ser un vector de tipo Date o convertible a Date.\n", arg_name),
          sprintf("Recibido: %s\n", class(x)[1]),
          "Sugerencia: Use as.Date() para convertir su vector de fechas.",
          call. = FALSE
        )
      }
    } else {
      stop(
        sprintf("El parametro '%s' debe ser un vector de tipo Date.\n", arg_name),
        sprintf("Recibido: %s\n", class(x)[1]),
        "Sugerencia: Use as.Date() para convertir su vector de fechas.",
        call. = FALSE
      )
    }
  }

  invisible(NULL)
}

#' Validate Choice from Options
#'
#' @description Internal helper function to validate that input matches one of allowed options.
#' @param x The input to validate.
#' @param choices Character vector of allowed values.
#' @param arg_name Name of the argument being validated (for error messages).
#' @return Throws an error if validation fails, otherwise returns invisibly.
#' @keywords internal
#' @noRd
validate_choice <- function(x, choices, arg_name = "x") {
  if (length(x) != 1 || !x %in% choices) {
    stop(
      sprintf("El parametro '%s' debe ser uno de: %s\n",
              arg_name, paste(choices, collapse = ", ")),
      sprintf("Recibido: %s", x),
      call. = FALSE
    )
  }

  invisible(NULL)
}

acep_fixture_clean_text <- function() {
  c(
    "El SUTEBA fue al paro. Reclaman mejoras salariales.",
    "Viernes 12: @soip marcha en Mar del Plata!!! https://example.com #Paro 😊"
  )
}

acep_fixture_count <- function() {
  list(
    texto = c("paro lucha paro", "piquetes y paros", "sin coincidencias"),
    dic = c("paro", "lucha", "piquetes")
  )
}

acep_fixture_pos <- function() {
  ACEP::acep_bases$spacy_postag
}

acep_fixture_svo <- function() {
  acep_fixture_pos()
}

acep_fixture_provider_success <- function() {
  list(
    provider = "openrouter",
    status_code = 200L,
    body = list(choices = list(list(message = list(content = "respuesta fixture"))))
  )
}

acep_fixture_provider_error <- function() {
  list(
    provider = "openrouter",
    status_code = 401L,
    body = list(error = list(message = "API key inválida"))
  )
}

acep_skip_if_network_required <- function(reason = "offline safety fixture avoids network/API calls") {
  testthat::skip(reason)
}

acep_skip_if_external_model_required <- function(reason = "offline safety fixture avoids external NLP model downloads") {
  testthat::skip(reason)
}

acep_source_path <- function(...) {
  relative_path <- file.path(...)
  candidates <- unique(c(
    getwd(),
    testthat::test_path("..", ".."),
    testthat::test_path("..", "..", "..")
  ))
  paths <- file.path(candidates, relative_path)
  existing <- paths[file.exists(paths)]

  if (length(existing) > 0) {
    return(existing[[1]])
  }

  testthat::skip(sprintf(
    "Source file not available in installed-package test environment: %s",
    relative_path
  ))
}

test_that("optimization fixtures provide offline clean inputs", {
  inputs <- acep_fixture_clean_text()

  expect_equal(length(inputs), 2)
  expect_equal(inputs[[1]], "El SUTEBA fue al paro. Reclaman mejoras salariales.")
  expect_match(inputs[[2]], "#Paro", fixed = TRUE)
})

test_that("optimization fixtures provide deterministic offline flows", {
  count_fixture <- acep_fixture_count()
  pos_fixture <- acep_fixture_pos()
  provider_success <- acep_fixture_provider_success()
  provider_error <- acep_fixture_provider_error()

  expect_equal(count_fixture$dic, c("paro", "lucha", "piquetes"))
  expect_s3_class(pos_fixture, "tokenIndex")
  expect_equal(names(provider_success), c("provider", "status_code", "body"))
  expect_equal(provider_error$status_code, 401L)
})

test_that("acep_clean current behavior is characterized", {
  cleaned <- acep_clean(acep_fixture_clean_text())

  expect_equal(
    cleaned,
    c("suteba paro reclaman mejoras salariales", "marcha mar plata")
  )
  expect_snapshot_value(cleaned, style = "json2")
})

test_that("acep_clean edge behavior is approval-tested before optimization", {
  edge_text <- c(
    "ÁÉÍÓÚ Ñandú!!! https://example.com/@uno #Etiqueta 😊\nMartes 25 de Mayo",
    "Sin cambios? ABC 123 @user #Hash http://foo.bar"
  )
  accent_text <- c("Árbol Ñandú y Educación", "Lunes, Mayo 25!!!")

  expect_equal(acep_clean(edge_text), c("aeiou nandu", "cambios abc"))
  expect_equal(
    acep_clean(accent_text, rm_stopwords = FALSE, rm_shortwords = FALSE),
    c("arbol nandu y educacion", "")
  )
  expect_equal(
    acep_clean(accent_text, rm_cesp = FALSE, rm_stopwords = FALSE, rm_shortwords = FALSE),
    c("árbol ñandú y educación", "")
  )
  expect_equal(
    acep_clean("Uno\nDos   Tres", rm_stopwords = FALSE, rm_shortwords = FALSE, rm_punt = FALSE, rm_num = FALSE),
    "uno dos tres"
  )
  expect_equal(
    acep_clean("Uno\nDos   Tres", rm_stopwords = FALSE, rm_shortwords = FALSE, rm_whitespace = FALSE),
    "uno dos   tres"
  )
  expect_equal(length(acep_clean(edge_text)), length(edge_text))
})

test_that("acep_count current behavior is characterized", {
  fixture <- acep_fixture_count()
  counted <- acep_count(fixture$texto, fixture$dic)

  expect_equal(counted, c(3L, 2L, 0L))
  expect_snapshot_value(counted, style = "json2")
})

test_that("acep_count edge semantics are approval-tested before optimization", {
  edge_text <- c(
    "paro paros paro.",
    "Paro PARO paro",
    "sin datos",
    NA_character_,
    "",
    "piquetes, lucha; paro!"
  )
  repeated_dic <- c("paro", "lucha", "piquetes", "paro")

  expect_equal(acep_count(edge_text, repeated_dic), c(3L, 1L, 0L, NA_integer_, 0L, 3L))
  expect_equal(
    acep_count(edge_text, repeated_dic, use_cache = FALSE),
    acep_count(edge_text, repeated_dic, use_cache = TRUE)
  )
  expect_equal(acep_count(c("el paro y paros", "paro"), " paro ", use_cache = FALSE), c(1L, 1L))
  expect_equal(acep_count("paro, paro. paros", "paro", use_cache = FALSE), 3L)
  expect_equal(acep_count("Árbol árbol arbol", c("árbol", "arbol"), use_cache = FALSE), 2L)
})

test_that("acep_count cache reuses normalized repeated dictionaries", {
  acep_clear_regex_cache()
  on.exit(acep_clear_regex_cache(), add = TRUE)

  expect_equal(acep_count("paro paro", c("paro", "paro")), 2L)
  expect_equal(acep_count("paro paro", "paro"), 2L)
  expect_equal(acep_regex_cache_size(), 1L)
})

test_that("acep_svo stable return schemas are characterized", {
  svo <- acep_svo(acep_fixture_svo())

  expect_equal(
    names(svo),
    c(
      "acep_annotate_svo", "acep_pro_svo", "acep_list_svo", "acep_sp",
      "acep_lista_lemmas", "acep_no_procesadas"
    )
  )
  expect_equal(svo$acep_list_svo$eventos[[1]], "SOIP -> declara -> Mar huelga")
  expect_equal(svo$acep_list_svo$sujeto[[1]], "SOIP")
  expect_equal(svo$acep_list_svo$verbo[[1]], "declara")
  expect_equal(svo$acep_list_svo$objeto[[1]], "Mar huelga")
  expect_snapshot_value(
    lapply(svo, function(x) list(class = class(x), dim = dim(x), names = names(x))),
    style = "json2"
  )
})

test_that("acep_svo repeated-document semantics are approval-tested before optimization", {
  tokenindex <- do.call(rbind, replicate(3L, acep_fixture_svo(), simplify = FALSE))
  tokenindex$doc_id <- rep(seq_len(3L), each = nrow(acep_fixture_svo()))
  class(tokenindex) <- class(acep_fixture_svo())

  svo <- acep_svo(tokenindex)

  expected_events <- rep("SOIP -> declara -> Mar huelga", 3L)

  expect_equal(svo$acep_list_svo$doc_id, 1:3)
  expect_equal(svo$acep_list_svo$oracion_id, rep(1L, 3L))
  expect_equal(svo$acep_list_svo$eventos, expected_events)
  expect_equal(svo$acep_list_svo$sujeto, rep("SOIP", 3L))
  expect_equal(svo$acep_list_svo$verbo, rep("declara", 3L))
  expect_equal(svo$acep_list_svo$objeto, rep("Mar huelga", 3L))
  expect_equal(nrow(svo$acep_no_procesadas), 0L)
})

test_that("acep_svo token aggregation helper preserves fallback semantics", {
  collapse_tokens <- getFromNamespace(".acep_svo_collapse_tokens", "ACEP")
  tokenindex <- acep_fixture_svo()
  annotated <- tokenindex
  annotated$s_p <- ifelse(annotated$token %in% c("Mar", "Plata", "SOIP"), "sujeto", "predicado")

  subject_tokens <- subset(annotated, s_p == "sujeto")
  collapsed_subject <- collapse_tokens(subject_tokens, annotated, "sujeto")

  expect_equal(names(collapsed_subject), c("doc_id", "sentence", "sujeto"))
  expect_equal(collapsed_subject$doc_id, 1L)
  expect_equal(collapsed_subject$sentence, 1L)
  expect_equal(collapsed_subject$sujeto, "Mar Plata SOIP")

  empty_predicate <- collapse_tokens(
    annotated[0, ],
    annotated,
    "predicados"
  )

  expect_equal(names(empty_predicate), c("doc_id", "sentence", "predicados"))
  expect_equal(nrow(empty_predicate), 1L)
  expect_true(is.na(empty_predicate$predicados[[1]]))

  collapsed_sent <- collapse_tokens(subject_tokens, annotated, "conjugaciones", c("doc_id", "sentence", "sent"))
  expect_equal(names(collapsed_sent), c("doc_id", "sentence", "sent", "conjugaciones"))
  expect_equal(collapsed_sent$conjugaciones, "Mar Plata SOIP")
})

test_that("acep_postag helper normalizes parsed chunks without changing row semantics", {
  normalize_parse <- getFromNamespace(".acep_postag_normalize_parse", "ACEP")

  parsed <- data.frame(
    doc_id = c("text1", "text2", "text3"),
    sentence = c(1L, 1L, 1L),
    token = c("Mar", "", "SOIP"),
    morph = I(list("Number=Sing", "Space=Yes", "Number=Sing")),
    sent = I(list(" En Mar\n", "\n", " reclaman ")),
    stringsAsFactors = FALSE
  )

  normalized <- normalize_parse(parsed, doc_id_offset = 10L)

  expect_equal(normalized$doc_id, c(11L, 13L))
  expect_equal(normalized$sent, c("En Mar", "reclaman"))
  expect_equal(normalized$morph, c("Number=Sing", "Number=Sing"))
  expect_equal(normalized$token, c("Mar", "SOIP"))

  empty_normalized <- normalize_parse(parsed[2, ], doc_id_offset = 0L)
  expect_equal(nrow(empty_normalized), 0L)
  expect_equal(names(empty_normalized), names(parsed))
})

test_that("acep_postag chunk helper preserves contiguous text boundaries", {
  text_chunks <- getFromNamespace(".acep_postag_text_chunks", "ACEP")

  chunks <- text_chunks(letters[1:5], chunk_size = 2L)

  expect_equal(length(chunks), 3L)
  expect_equal(chunks[[1]]$start_idx, 1L)
  expect_equal(chunks[[1]]$end_idx, 2L)
  expect_equal(chunks[[1]]$texto, letters[1:2])
  expect_equal(chunks[[2]]$start_idx, 3L)
  expect_equal(chunks[[2]]$end_idx, 4L)
  expect_equal(chunks[[3]]$start_idx, 5L)
  expect_equal(chunks[[3]]$end_idx, 5L)
  expect_equal(chunks[[3]]$texto, "e")

  one_chunk <- text_chunks(c("uno", "dos"), chunk_size = 10L)
  expect_equal(length(one_chunk), 1L)
  expect_equal(one_chunk[[1]]$texto, c("uno", "dos"))
})

test_that("acep_postag location helper preserves LOC merge and empty schemas", {
  prepare_locations <- getFromNamespace(".acep_postag_prepare_loc_entities", "ACEP")

  entities <- data.frame(
    doc_id = c(1L, 1L, 2L, 2L),
    sentence = c(1L, 1L, 1L, 1L),
    entity = c("Mar_del_Plata", "Mar_del_Plata", "SOIP", "Buenos_Aires"),
    entity_type = c("LOC", "LOC", "ORG", "LOC"),
    stringsAsFactors = FALSE
  )
  tokenindex <- data.frame(
    doc_id = c(1L, 1L, 2L),
    sentence = c(1L, 1L, 1L),
    token = c("Mar", "Plata", "Buenos"),
    stringsAsFactors = FALSE
  )

  prepared <- prepare_locations(entities, tokenindex)

  expect_equal(names(prepared), c("entity_", "doc_id", "sentence", "entity", "entity_type"))
  expect_equal(prepared$entity_, c("Mar del Plata", "Buenos Aires"))
  expect_equal(prepared$doc_id, c(1L, 2L))
  expect_equal(prepared$entity_type, c("LOC", "LOC"))

  empty_prepared <- prepare_locations(entities[entities$entity_type == "ORG", ], tokenindex)
  expect_equal(
    names(empty_prepared),
    c("entity_", "doc_id", "sentence", "entity", "entity_type", "lat", "long")
  )
  expect_equal(nrow(empty_prepared), 0L)
})

test_that("acep_postag_hibrido helpers preserve chunk, parse, and LOC pre-geocode semantics", {
  normalize_parse <- getFromNamespace(".acep_postag_hibrido_normalize_parse", "ACEP")
  text_chunks <- getFromNamespace(".acep_postag_hibrido_text_chunks", "ACEP")
  prepare_locations <- getFromNamespace(".acep_postag_hibrido_prepare_loc_entities", "ACEP")

  parsed <- data.frame(
    doc_id = c("text1", "text2", "text3"),
    sentence = c(1L, 1L, 1L),
    token = c("Mar", "", "SOIP"),
    morph = I(list("Number=Sing", "Space=Yes", "Number=Sing")),
    sent = I(list(" En Mar\n", "\n", " reclaman ")),
    stringsAsFactors = FALSE
  )

  normalized <- normalize_parse(parsed, doc_id_offset = 20L)

  expect_equal(normalized$doc_id, c(21L, 23L))
  expect_equal(normalized$sent, c("En Mar", "reclaman"))
  expect_equal(normalized$morph, c("Number=Sing", "Number=Sing"))
  expect_equal(normalized$token, c("Mar", "SOIP"))

  chunks <- text_chunks(letters[1:5], chunk_size = 2L)
  expect_equal(length(chunks), 3L)
  expect_equal(chunks[[1]]$start_idx, 1L)
  expect_equal(chunks[[1]]$end_idx, 2L)
  expect_equal(chunks[[1]]$texto, letters[1:2])
  expect_equal(chunks[[2]]$start_idx, 3L)
  expect_equal(chunks[[2]]$end_idx, 4L)
  expect_equal(chunks[[3]]$start_idx, 5L)
  expect_equal(chunks[[3]]$end_idx, 5L)
  expect_equal(chunks[[3]]$texto, "e")

  entities <- data.frame(
    doc_id = c(1L, 1L, 2L, 2L),
    sentence = c(1L, 1L, 1L, 1L),
    entity = c("Mar_del_Plata", "Mar_del_Plata", "SOIP", "Buenos_Aires"),
    entity_type = c("LOC", "LOC", "ORG", "LOC"),
    stringsAsFactors = FALSE
  )

  prepared <- prepare_locations(entities)

  expect_equal(names(prepared), c("doc_id", "sentence", "entity", "entity_type", "entity_"))
  expect_equal(prepared$entity_, c("Mar del Plata", "Buenos Aires"))
  expect_equal(prepared$doc_id, c(1L, 2L))
  expect_equal(prepared$entity_type, c("LOC", "LOC"))

  empty_prepared <- prepare_locations(entities[entities$entity_type == "ORG", ])
  expect_equal(
    names(empty_prepared),
    c("entity_", "doc_id", "sentence", "entity", "entity_type", "lat", "long")
  )
  expect_equal(nrow(empty_prepared), 0L)
})

test_that("public exports and hot-path formals are snapshotted", {
  hot_path_formals <- function(fn) {
    vapply(formals(fn), function(arg) paste(deparse(arg), collapse = ""), character(1))
  }

  api <- list(
    exports = sort(getNamespaceExports("ACEP")),
    formals = list(
      acep_clean = hot_path_formals(acep_clean),
      acep_count = hot_path_formals(acep_count),
      acep_svo = hot_path_formals(acep_svo)
    )
  )

  expect_true("acep_clean" %in% api$exports)
  expect_true("acep_count" %in% api$exports)
  expect_true("acep_svo" %in% api$exports)
  expect_equal(api$formals$acep_count[["use_cache"]], "TRUE")
  expect_snapshot_value(api, style = "json2")
})

test_that("manual optimization baseline script is present but not CI-wired", {
  baseline_path <- acep_source_path("bench", "optimization-baseline.R")

  expect_true(file.exists(baseline_path))
  expect_equal(basename(baseline_path), "optimization-baseline.R")
})

test_that("package footprint artifacts are excluded from source builds", {
  buildignore_path <- acep_source_path(".Rbuildignore")
  buildignore <- readLines(buildignore_path, warn = FALSE)

  expect_true("^vignettes/.*\\.udpipe$" %in% buildignore)
  expect_true("^bench/.*\\.csv$" %in% buildignore)
  expect_true("^bench/.*\\.rds$" %in% buildignore)
})

test_that("acep_bases canonical data file exposes stable object names and metadata", {
  describe_data_file <- function(path) {
    env <- new.env(parent = emptyenv())
    objects <- load(path, envir = env)
    value <- env[["acep_bases"]]

    list(
      objects = sort(objects),
      class = class(value),
      names = names(value),
      element_classes = lapply(value, class),
      rows = vapply(value, NROW, integer(1)),
      columns = vapply(value, NCOL, integer(1)),
      column_names = lapply(value, names)
    )
  }

  rda_path <- acep_source_path("data", "acep_bases.rda")
  rdata_path <- file.path(dirname(rda_path), "acep_bases.RData")
  rda_metadata <- describe_data_file(rda_path)

  expect_false(file.exists(rdata_path))
  expect_equal(rda_metadata$objects, "acep_bases")
  expect_equal(
    rda_metadata$names,
    c(
      "ed_neco", "la_nueva", "lc_720", "lc_mdp", "ln_arg", "ln_bb",
      "rev_puerto", "rp_mdp", "rp_procesada", "spacy_postag", "titulares"
    )
  )
  expect_equal(
    rda_metadata$rows[c("lc_720", "rp_procesada", "spacy_postag", "titulares")],
    c(lc_720 = 720L, rp_procesada = 7816L, spacy_postag = 15L, titulares = 8L)
  )
  expect_equal(
    names(ACEP::acep_bases),
    rda_metadata$names
  )
})

test_that("optional dependency helper reports clear Spanish installation guidance", {
  require_helper <- getFromNamespace("acep_require_namespace", "ACEP")

  expect_null(require_helper("stats", "conteo base"))
  expect_error(
    require_helper("acepPaqueteInexistente", "prueba opcional"),
    "La funcionalidad 'prueba opcional' requiere instalar el paquete opcional 'acepPaqueteInexistente'. Instala el paquete con: install.packages\\(\"acepPaqueteInexistente\"\\)",
    fixed = FALSE
  )
})

test_that("heavy NLP and geocoding dependencies are optional in DESCRIPTION", {
  description_path <- acep_source_path("DESCRIPTION")
  description <- read.dcf(description_path)[1, ]
  imports <- trimws(unlist(strsplit(description[["Imports"]], ",")))
  suggests <- trimws(unlist(strsplit(description[["Suggests"]], ",")))

  optional_packages <- c("spacyr", "reticulate", "udpipe", "rsyntax", "tidygeocoder")
  core_packages <- c("httr", "jsonlite", "stringr", "magrittr")

  expect_false(any(optional_packages %in% imports))
  expect_true(all(optional_packages %in% suggests))
  expect_true(all(core_packages %in% imports))
})

test_that("optional dependency entry points guard their package-specific paths", {
  function_body <- function(name) paste(deparse(body(get(name, envir = asNamespace("ACEP")))), collapse = "\n")

  postag_body <- function_body("acep_postag")
  postag_hibrido_body <- function_body("acep_postag_hibrido")
  upos_body <- function_body("acep_upos")
  svo_body <- function_body("acep_svo")

  expect_match(postag_body, 'acep_require_namespace\\("spacyr", "acep_postag"\\)')
  expect_match(postag_body, 'acep_require_namespace\\("rsyntax", "acep_postag"\\)')
  expect_match(postag_body, 'acep_require_namespace\\("tidygeocoder", "acep_postag"\\)')
  expect_match(postag_body, 'acep_require_namespace\\("reticulate", "acep_postag"\\)')

  expect_match(postag_hibrido_body, 'acep_require_namespace\\("spacyr", "acep_postag_hibrido"\\)')
  expect_match(postag_hibrido_body, 'acep_require_namespace\\("rsyntax", "acep_postag_hibrido"\\)')
  expect_match(postag_hibrido_body, 'acep_require_namespace\\("tidygeocoder", "acep_postag_hibrido"\\)')
  expect_match(postag_hibrido_body, 'acep_require_namespace\\("reticulate", "acep_postag_hibrido"\\)')

  expect_match(upos_body, 'acep_require_namespace\\("udpipe", "acep_upos"\\)')
  expect_match(upos_body, 'acep_require_namespace\\("rsyntax", "acep_upos"\\)')
  expect_match(svo_body, 'acep_require_namespace\\("rsyntax", "acep_svo"\\)')
})

test_that("provider wrappers keep only core HTTP/JSON dependencies mandatory", {
  provider_functions <- c(
    "acep_claude", "acep_gemini", "acep_gpt", "acep_ollama",
    "acep_openrouter", "acep_together"
  )
  provider_sources <- vapply(
    provider_functions,
    function(name) paste(deparse(body(get(name, envir = asNamespace("ACEP")))), collapse = "\n"),
    character(1)
  )

  expect_true(any(grepl("httr::", provider_sources, fixed = TRUE)))
  expect_true(any(grepl("jsonlite::", provider_sources, fixed = TRUE)))
  expect_false(any(grepl("spacyr::|reticulate::|tidygeocoder::|rsyntax::|udpipe::", provider_sources)))
})

test_that("provider shared helpers preserve schema protection, prompts, endpoints, and headers", {
  default_schema <- getFromNamespace(".acep_provider_default_schema", "ACEP")
  user_prompt <- getFromNamespace(".acep_provider_user_prompt", "ACEP")
  schema_fields <- getFromNamespace(".acep_provider_schema_field_descriptions", "ACEP")
  provider_endpoint <- getFromNamespace(".acep_provider_endpoint", "ACEP")
  provider_headers <- getFromNamespace(".acep_provider_auth_headers", "ACEP")
  openai_token_field <- getFromNamespace(".acep_openai_token_limit_field", "ACEP")
  openrouter_structured <- getFromNamespace(".acep_openrouter_model_supports_structured_outputs", "ACEP")

  schema <- default_schema()

  expect_equal(schema$type, "object")
  expect_equal(names(schema$properties), "respuesta")
  expect_equal(unclass(schema$required), "respuesta")
  expect_true(inherits(schema$required, "AsIs"))
  expect_false(is.null(schema$additionalProperties))

  gemini_schema <- default_schema(additional_properties = FALSE, protect_arrays = FALSE)
  expect_equal(names(gemini_schema), c("type", "properties", "required"))
  expect_false(inherits(gemini_schema$required, "AsIs"))

  expect_equal(
    user_prompt("Texto de prueba", "Extrae campos"),
    "Texto a analizar:\nTexto de prueba\n\nInstrucciones:\nExtrae campos"
  )
  expect_equal(
    unname(schema_fields(acep_gpt_schema("clasificacion"))),
    c(
      "- categoria: Categoria principal del texto",
      "- confianza: Nivel de confianza de 0 a 1",
      "- justificacion: Breve justificacion de la clasificacion"
    )
  )

  expect_equal(provider_endpoint("openai"), "https://api.openai.com/v1/chat/completions")
  expect_equal(provider_endpoint("anthropic"), "https://api.anthropic.com/v1/messages")
  expect_equal(
    provider_endpoint("gemini", modelo = "gemini-2.5-flash"),
    "https://generativelanguage.googleapis.com/v1beta/models/gemini-2.5-flash:generateContent"
  )
  expect_equal(provider_endpoint("together"), "https://api.together.xyz/v1/chat/completions")
  expect_equal(provider_endpoint("openrouter"), "https://openrouter.ai/api/v1/chat/completions")
  expect_equal(openai_token_field("gpt-4o-mini"), "max_tokens")
  expect_equal(openai_token_field("gpt-5-mini"), "max_completion_tokens")
  expect_equal(openai_token_field("o4-mini"), "max_completion_tokens")
  expect_true(openrouter_structured("openai/gpt-4o-mini"))
  expect_true(openrouter_structured("google/gemini-2.5-flash"))
  expect_false(openrouter_structured("anthropic/claude-sonnet-4.5"))

  openrouter_headers <- provider_headers(
    "openrouter",
    api_key = "clave",
    site_url = "https://acep.test",
    app_name = "ACEP"
  )
  expect_equal(openrouter_headers$Authorization, "Bearer clave")
  expect_equal(openrouter_headers$`HTTP-Referer`, "https://acep.test")
  expect_equal(openrouter_headers$`X-Title`, "ACEP")
})

test_that("provider response helpers parse JSON and preserve Spanish error behavior without network", {
  validate_inputs <- getFromNamespace(".acep_provider_validate_request_inputs", "ACEP")
  clean_json <- getFromNamespace(".acep_provider_clean_json_response", "ACEP")
  parse_json <- getFromNamespace(".acep_provider_parse_json_response", "ACEP")
  chat_content <- getFromNamespace(".acep_provider_extract_chat_content", "ACEP")

  expect_null(validate_inputs("texto", "instrucciones", "clave", "OPENAI_API_KEY"))
  expect_error(
    validate_inputs("", "instrucciones", "clave", "OPENAI_API_KEY"),
    "El parametro 'texto' debe ser una cadena de caracteres no vacia",
    fixed = TRUE
  )
  expect_error(
    validate_inputs("texto", "", "clave", "OPENAI_API_KEY"),
    "El parametro 'instrucciones' debe ser una cadena de caracteres no vacia",
    fixed = TRUE
  )
  expect_error(
    validate_inputs("texto", "instrucciones", "", "OPENAI_API_KEY"),
    "API key no encontrada. Define la variable de entorno OPENAI_API_KEY o pasa el parametro api_key",
    fixed = TRUE
  )

  fenced <- "```json\n{\"respuesta\":\"ok\"}\n```"
  expect_equal(clean_json(fenced), "{\"respuesta\":\"ok\"}")
  expect_equal(parse_json(fenced, parse_json = FALSE), "{\"respuesta\":\"ok\"}")
  expect_equal(parse_json(fenced, parse_json = TRUE), list(respuesta = "ok"))
  expect_error(
    parse_json("{no valido", parse_json = TRUE),
    "Error al parsear JSON de la respuesta. Contenido recibido",
    fixed = TRUE
  )

  parsed <- list(choices = list(list(message = list(content = "{\"respuesta\":\"ok\"}"))))
  expect_equal(chat_content(parsed), "{\"respuesta\":\"ok\"}")
  expect_error(
    chat_content(list(choices = list())),
    "La API devolvio una respuesta vacia. Verifica tu prompt y esquema.",
    fixed = TRUE
  )
})

#' @title Tokenizador.
#' @description Función que tokeniza las notas/textos.
#' @param x vector de textos al que se le aplica la función de tokenización.
#' @param tolower convierte los textos a minúsculas.
#' @param cleaning hace una limpieza de los textos.
#' @return Si todas las entradas son correctas,
#' la salida será un data.frame con las palabras tokenizadas.
#' @keywords tokenizar
#' @examples
#' acep_token("Huelga de obreros del pescado en el puerto")
#' @export

acep_token <- function(x, tolower = TRUE, cleaning = TRUE) {
  if (!is.logical(tolower) | !is.logical(cleaning)) {
    return(message("Debe ingresar un valor booleano: TRUE o FALSE"))
  }
  if (!is.character(x) | is.list(x)){
    return(message("No ingresaste un vector en el par\u00e1metro x.
                   Vuelve a intentarlo ingresando un vector!"))
  }
  if (tolower) {
    x <- ACEP::acep_min(x)
  }
  if (cleaning) {
    x <- acep_clean(x)
  }
  texto_id <- seq_along(x)
  tokens <- lapply(x, function(texto) {
    tokens <- unlist(strsplit(texto, "\\W+"))
    tokens <- tokens[!is.na(tokens) & tokens != ""]
    return(tokens)
  })

  df <- data.frame(texto_id = rep(texto_id, sapply(tokens, length)), tokens = unlist(tokens))
  return(df)
}

test_that("ACEP TOKEN", {
  skip_if_offline()
  skip_on_cran()
  token <- acep_token("Huelga de obreros del pescado en el puerto")
  dimensiones <- length(token)
  expect_equal(dimensiones, length(token))
})

test_that("ACEP TOKEN F", {
  skip_if_offline()
  skip_on_cran()
  token <- acep_token("Huelga de obreros del pescado en el puerto", tolower = FALSE)
  dimensiones <- length(token)
  expect_equal(dimensiones, length(token))
})

test_that("ACEP TOKEN F E1", {
  skip_if_offline()
  skip_on_cran()
  df <- data.frame(texto =
                     c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
                       "El SOIP lleva adelante un plan de lucha con paros y piquetes."))
  token <- acep_token(df, tolower = FALSE)
  dimensiones <- 1
  expect_equal(dimensiones, length(dimensiones))
})

#' @title Coleccion de notas y recursos de prueba
#' @description Lista con fuentes de datos y muestras preprocesadas
#' utilizadas en los ejemplos del paquete. Incluye enlaces de descarga
#' para distintos portales, resumenes estadisticos y conjuntos anotados
#' manualmente que permiten evaluar diccionarios, extraccion de tripletes
#' y desempeno de modelos generativos.
#' @format Lista con 11 objetos:
#'\describe{
#' \item{la_nueva}{URL para descargar una muestra del corpus de notas
#' del diario La Nueva Provincia de Bahia Blanca.}
#' \item{rev_puerto}{URL para descargar el corpus de notas
#' de Revista Puerto.}
#' \item{rp_procesada}{data frame con indicadores de conflictividad
#' construidos a partir de Revista Puerto.}
#' \item{lc_mdp}{URL para descargar el corpus de notas
#' del diario La Capital (Mar del Plata).}
#' \item{rp_mdp}{URL para descargar el corpus de notas
#' de Revista Puerto (edicion Mar del Plata).}
#' \item{ed_neco}{URL para descargar el corpus de notas
#' del diario Ecos Diarios (Necochea).}
#' \item{ln_bb}{URL para descargar el corpus de notas
#' de La Nueva (Bahia Blanca).}
#' \item{ln_arg}{URL para descargar un subconjunto de notas
#' de La Nacion.}
#' \item{lc_720}{data frame con 720 notas de La Capital publicadas entre
#' 2016 y 2019, curado y documentado por Guillermina Laitano. Contiene
#' etiquetas binarias manuales que permiten evaluar el diccionario de
#' conflictividad, la extraccion de tripletes semanticos y la capacidad
#' de distintos modelos generativos para tareas de clasificacion binaria
#' y extraccion estructurada de eventos de protesta.}
#' \item{spacy_postag}{data frame con una oracion procesada con \pkg{spacyr}
#' que sirve como ejemplo para funciones de dependencias y SVO.}
#' \item{titulares}{vector con titulares sinteticos referidos a
#' conflictos sociales.}
#'}
#' @docType data
#' @usage data(acep_bases)
#' @references Nieto, Agustin 2020 "Intersecciones entre historia digital e
#' historia social: un ejercicio de lectura distante sobre la
#' conflictividad maritima en la historia argentina reciente".
#' Drassana: revista del Museu Maritim (28):122-42.
#' (\href{https://observatoriodeconflictividad.org/nietohd.pdf}{Revista Drassana})
#' @source \href{https://revistapuerto.com.ar/}{Revista Puerto}
#' @source \href{https://www.lanueva.com/}{La Nueva}
#' @keywords datos
#' @examples
#' acep_bases$rp_procesada[1:6, ]
"acep_bases"

#' @title Función para buscar y extraer palabras en un texto.
#' @description Esta función busca palabras clave en un texto y
#' extrae los resultados en un formato especifico.
#' @param texto El texto en el que se buscaran las palabras clave.
#' @param dic Un vector con las palabras clave a buscar.
#' @param sep El separador utilizado para concatenar las
#' palabras clave encontradas (por defecto: " | ").
#' @param izq expresión regular para incorporar otros caracteres a
#' la izquierda de los términos del vector 'dic'.
#' @param der expresión regular para incorporar otros caracteres a
#' la derecha de los términos del vector 'dic'.
#' @return Si todas las entradas son correctas,
#' la salida sera un vector de tipo caracter.
#' procesado y el contexto de las palabras y/o frases entradas.
#' @importFrom stringr str_extract_all
#' @keywords buscar palabras texto diccionario
#' @examples
#' texto <- "Los obreros del pescado, en el marco de una huelga y
#' realizaron una manifestación con piquete en el puerto de la ciudad."
#' dicc <- c("huel", "manif", "piq")
#' acep_extract(texto, dicc)
#' @export
acep_extract <- function(texto, dic, sep="; ", izq="\\b\\w*", der="\\w*\\b") {
  if (!is.character(texto)) {
    return(message(
      "No ingresaste un vector de texto en el par\u00e1metro 'texto'.
      Vuelve a intentarlo ingresando un vector de texto!"))
  }
  if (!is.character(dic)) {
    return(message(
      "No ingresaste un vector de texto en el par\u00e1metro 'dic'.
      Vuelve a intentarlo ingresando un vector de texto!"))
  }
  if (!is.character(sep) && !is.null(sep)) {
    return(message(
      "No ingresaste un caracter separador en el par\u00e1metro 'sep'."))
  }
  if (!is.character(izq) && !is.null(izq)) {
    return(message(
      "No ingresaste una expresi\u00f3n regular en el par\u00e1metro 'izq'."))
  }
  if (!is.character(der) && !is.null(der)) {
    return(message(
      "No ingresaste una expresi\u00f3n regular en el par\u00e1metro 'der'."))
  } else {
    sapply(texto, function(txt) {
      if (is.na(txt)) {
        return(NA_character_)
      }
      dic <- gsub(" $", "\\\\b", dic)
      x <- paste0(izq, gsub("^ ", "", dic), der, collapse = "|")
      key_words <- unlist(stringr::str_extract_all(txt, x))
      key_words <- key_words[sapply(key_words, length) > 0]
      if (!is.null(sep)) {
        paste(key_words, collapse = sep)
      } else {
        unname(sapply(key_words, function(x) unlist(x)))
      }
    }, USE.NAMES = FALSE)
  }
}

# Offline tests (no network).
test_that("acep_extract extrae las palabras clave del texto", {
  out <- acep_extract("hola mundo cruel", c("mund", "cru"))
  expect_type(out, "character")
  expect_true(grepl("mundo", out))
  expect_true(grepl("cruel", out))
})

test_that("acep_extract devuelve NA real para entradas NA", {
  out <- acep_extract(c("hola mundo", NA), "mund")
  expect_true(is.na(out[[2]]))
})

.acep_postag_hibrido_normalize_parse <- function(texto_tag, doc_id_offset = 0L) {
  texto_tag$morph <- sapply(texto_tag$morph, as.character)
  texto_tag$sent <- sapply(texto_tag$sent, as.character)
  texto_tag$doc_id <- as.integer(gsub("text", "", texto_tag$doc_id)) + doc_id_offset
  texto_tag$sent <- trimws(gsub("\\n+", "", texto_tag$sent))
  texto_tag[texto_tag$sent != "", ]
}

.acep_postag_hibrido_text_chunks <- function(texto, chunk_size) {
  n_textos <- length(texto)
  n_chunks <- ceiling(n_textos / chunk_size)

  lapply(seq_len(n_chunks), function(i) {
    start_idx <- (i - 1L) * chunk_size + 1L
    end_idx <- min(i * chunk_size, n_textos)

    list(
      start_idx = start_idx,
      end_idx = end_idx,
      texto = texto[start_idx:end_idx]
    )
  })
}

.acep_postag_hibrido_empty_loc_frame <- function() {
  data.frame(entity_ = character(),
             doc_id = integer(),
             sentence = integer(),
             entity = character(),
             entity_type = character(),
             lat = numeric(),
             long = numeric(),
             stringsAsFactors = FALSE)
}

.acep_postag_hibrido_prepare_loc_entities <- function(texto_only_entity) {
  texto_only_entity_loc <- unique(texto_only_entity[texto_only_entity$entity_type == "LOC", ])

  if (nrow(texto_only_entity_loc) == 0L) {
    return(.acep_postag_hibrido_empty_loc_frame())
  }

  texto_only_entity_loc$entity_ <- gsub("_", " ", texto_only_entity_loc$entity)
  unique(texto_only_entity_loc)
}

#' @title Etiquetado POS, lematizacion y extraccion de entidades con spacyr
#' @description
#' Realiza analisis linguistico completo de textos usando la biblioteca spaCy a traves
#' de spacyr. Incluye: etiquetado POS (Part-of-Speech), lematizacion, tokenizacion,
#' extraccion de entidades nombradas, frases nominales y geocodificacion de ubicaciones.
#' La funcion procesa automaticamente grandes volumenes de texto dividiendolos en lotes
#' (chunks) y soporta procesamiento paralelo para acelerar el analisis
#'
#' @param texto Vector de caracteres con los textos a analizar.
#' @param core Modelo de lenguaje de spaCy a utilizar. Opciones: `"es_core_news_sm"`,
#'   `"es_core_news_md"`, `"es_core_news_lg"` (espanol), `"en_core_web_sm"`,
#'   `"en_core_web_md"`, `"en_core_web_lg"` (ingles), `"pt_core_news_sm"`,
#'   `"pt_core_news_md"`, `"pt_core_news_lg"` (portugues). Por defecto: `"es_core_news_lg"`.
#' @param bajar_core Logico. Si `TRUE`, descarga automaticamente el modelo si no esta instalado.
#' @param inst_spacy Logico. Si `TRUE`, instala la biblioteca spaCy en el entorno Python.
#' @param inst_miniconda Logico. Si `TRUE`, instala Miniconda (necesario para spaCy).
#' @param inst_reticulate Logico. Si `TRUE`, instala el paquete reticulate de R.
#' @param chunk_size Numero de textos a procesar por lote. Valores mas bajos consumen
#'   menos memoria pero tardan mas. Por defecto: 1000.
#' @param parallel_chunks Logico. Si `TRUE`, procesa los lotes en paralelo usando
#'   multiples nucleos del CPU. Requiere los paquetes `future` y `furrr`. Por defecto: `FALSE`.
#' @param n_cores Numero de nucleos de CPU a usar en modo paralelo. Si es `NULL`,
#'   detecta automaticamente el numero de nucleos disponibles menos uno.
#' @param geocode_cache_file Ruta al archivo JSON donde se almacena el cache de
#'   geocodificacion para evitar consultas repetidas. Por defecto: `"geocode_cache.json"`.
#' @param use_cache Logico. Si `TRUE`, usa y actualiza el cache de geocodificacion.
#' @param show_progress Logico. Si `TRUE`, muestra mensajes de progreso en la consola.
#'
#' @return Lista con 6 data frames que contienen diferentes niveles de analisis:
#' \itemize{
#'   \item \code{texto_tag}: Tokenizacion completa con etiquetas POS, lemas, dependencias
#'     sintacticas y atributos morfologicos para cada token
#'   \item \code{texto_tag_entity}: Tokens con entidades nombradas consolidadas
#'     (ej: "Mar del Plata" como una sola entidad en lugar de 3 tokens separados)
#'   \item \code{texto_only_entity}: Solo las entidades nombradas extraidas
#'     (personas, organizaciones, ubicaciones, fechas, etc.)
#'   \item \code{texto_only_entity_loc}: Entidades de tipo ubicacion (LOC)
#'     con coordenadas geograficas (latitud/longitud) obtenidas mediante geocodificacion
#'   \item \code{texto_nounphrase}: Tokens con frases nominales consolidadas
#'   \item \code{texto_only_nounphrase}: Solo las frases nominales extraidas
#' }
#'
#' @export
#' @examples
#' \dontrun{
#' # Analisis basico de un texto
#' texto <- "El SUTEBA convoco a un paro en Mar del Plata el 15 de marzo."
#' resultado <- acep_postag_hibrido(texto)
#'
#' # Ver tokens con etiquetas POS
#' head(resultado$texto_tag)
#'
#' # Ver entidades nombradas
#' print(resultado$texto_only_entity)
#'
#' # Ver ubicaciones geocodificadas
#' print(resultado$texto_only_entity_loc)
#'
#' # Procesar multiples textos con procesamiento paralelo
#' textos <- c("Primera noticia sobre conflictos.",
#'             "Segunda noticia sobre protestas.",
#'             "Tercera noticia sobre reclamos.")
#' resultado <- acep_postag_hibrido(textos,
#'                                  parallel_chunks = TRUE,
#'                                  chunk_size = 100)
#' }
acep_postag_hibrido <- function(texto,
                                core = "es_core_news_lg",
                                bajar_core = TRUE,
                                inst_spacy = FALSE,
                                inst_miniconda = FALSE,
                                inst_reticulate = FALSE,
                                chunk_size = 1000,
                                parallel_chunks = FALSE,
                                n_cores = NULL,
                                geocode_cache_file = "geocode_cache.json",
                                use_cache = TRUE,
                                show_progress = TRUE) {
  
  # Validaciones
  if (!is.character(texto)) {
    stop("El parametro 'texto' debe ser una cadena de caracteres")
  }
  if (!is.logical(inst_reticulate)) {
    stop("El parametro 'inst_reticulate' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.logical(inst_miniconda)) {
    stop("El parametro 'inst_miniconda' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.logical(inst_spacy)) {
    stop("El parametro 'inst_spacy' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.logical(bajar_core)) {
    stop("El parametro 'bajar_core' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.logical(parallel_chunks)) {
    stop("El parametro 'parallel_chunks' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.logical(use_cache)) {
    stop("El parametro 'use_cache' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.logical(show_progress)) {
    stop("El parametro 'show_progress' debe ser un valor booleano: TRUE o FALSE")
  }
  
  available_models <- c('es_core_news_sm','es_core_news_md','es_core_news_lg',
                        'pt_core_news_sm','pt_core_news_md','pt_core_news_lg',
                        'en_core_web_sm','en_core_web_md','en_core_web_lg','en_core_web_trf')
  if (!core %in% available_models) {
    stop("El parametro 'core' debe ser un modelo 'core' valido del espanol, ingles o portugues: ",
         paste0(available_models, collapse = ", "))
  }
  acep_require_namespace("spacyr", "acep_postag_hibrido")
  acep_require_namespace("rsyntax", "acep_postag_hibrido")
  acep_require_namespace("tidygeocoder", "acep_postag_hibrido")
  if (inst_miniconda) {
    acep_require_namespace("reticulate", "acep_postag_hibrido")
  }
  
  # Instalaciones
  if (inst_reticulate) {
    utils::install.packages("reticulate")
  }
  if (inst_miniconda) {
    reticulate::install_miniconda()
  }
  if (inst_spacy) {
    spacyr::spacy_install()
  }
  if (bajar_core) {
    spacyr::spacy_download_langmodel(core)
  }
  
  # Inicializar spaCy en proceso principal
  spacyr::spacy_initialize(model = core)
  
  n_textos <- length(texto)
  
  # Determinar estrategia de procesamiento
  if (n_textos <= 100) {
    # Batch processing para pocos textos
    if (show_progress) {
      message(sprintf("Procesando %d textos en modo batch...", n_textos))
    }
    
    texto_tag <- suppressWarnings({
      spacyr::spacy_parse(texto,
                          pos = TRUE,
                          tag = FALSE,
                          lemma = TRUE,
                          entity = TRUE,
                          dependency = TRUE,
                          nounphrase = TRUE,
                          multithread = TRUE,
                          additional_attributes = c("is_upper", "is_title", "is_quote",
                                                    "ent_iob_","ent_iob", "is_left_punct",
                                                    "is_right_punct", "morph", "sent"))
    })
    
    texto_tag <- .acep_postag_hibrido_normalize_parse(texto_tag)
    
  } else {
    # Chunking para muchos textos
    n_chunks <- ceiling(n_textos / chunk_size)
    
    if (show_progress) {
      if (parallel_chunks) {
        message(sprintf("Procesando %d textos en %d chunks (paralelo con %d cores)...", 
                        n_textos, n_chunks, ifelse(is.null(n_cores), parallel::detectCores() - 1, n_cores)))
      } else {
        message(sprintf("Procesando %d textos en %d chunks (secuencial)...", n_textos, n_chunks))
      }
    }
    
    if (parallel_chunks && n_chunks >= 2) {
      # Modo paralelo: paralelizar chunks
      acep_require_namespace("future", "acep_postag_hibrido con parallel_chunks = TRUE")
      acep_require_namespace("furrr", "acep_postag_hibrido con parallel_chunks = TRUE")
      
      # Configurar plan paralelo
      if (is.null(n_cores)) {
        n_cores <- parallel::detectCores() - 1
      }
      future::plan(future::multisession, workers = n_cores)
      
      texto_chunks <- .acep_postag_hibrido_text_chunks(texto, chunk_size)

      texto_tag_list <- furrr::future_map(seq_along(texto_chunks), function(chunk_num) {
        chunk <- texto_chunks[[chunk_num]]

        # Cada worker inicializa su propia sesion de spaCy
        tryCatch({
          spacyr::spacy_initialize(model = core)
        }, error = function(e) {
          # Si ya esta inicializado, continuar
        })
        
        postag <- suppressWarnings({
          spacyr::spacy_parse(chunk$texto,
                              pos = TRUE,
                              tag = FALSE,
                              lemma = TRUE,
                              entity = TRUE,
                              dependency = TRUE,
                              nounphrase = TRUE,
                              multithread = FALSE,  # Evitar conflictos
                              additional_attributes = c("is_upper", "is_title", "is_quote",
                                                        "ent_iob_","ent_iob", "is_left_punct",
                                                        "is_right_punct", "morph", "sent"))
        })
        
        .acep_postag_hibrido_normalize_parse(postag, chunk$start_idx - 1L)
      }, .progress = show_progress, .options = furrr::furrr_options(seed = TRUE))
      
      # Restaurar plan secuencial
      future::plan(future::sequential)
      
      # Combinar resultados
      texto_tag <- do.call(rbind, texto_tag_list)
      
    } else {
      # Modo secuencial: procesar chunks uno por uno
      texto_chunks <- .acep_postag_hibrido_text_chunks(texto, chunk_size)
      texto_tag_list <- vector("list", length(texto_chunks))
      
      for (i in seq_along(texto_chunks)) {
        chunk <- texto_chunks[[i]]
        
        if (show_progress) {
          message(sprintf("Procesando chunk %d/%d (textos %d-%d)", i, length(texto_chunks), chunk$start_idx, chunk$end_idx))
        }

        postag <- suppressWarnings({
          spacyr::spacy_parse(chunk$texto,
                              pos = TRUE,
                              tag = FALSE,
                              lemma = TRUE,
                              entity = TRUE,
                              dependency = TRUE,
                              nounphrase = TRUE,
                              multithread = TRUE,
                              additional_attributes = c("is_upper", "is_title", "is_quote",
                                                        "ent_iob_","ent_iob", "is_left_punct",
                                                        "is_right_punct", "morph", "sent"))
        })
        
        texto_tag_list[[i]] <- .acep_postag_hibrido_normalize_parse(postag, chunk$start_idx - 1L)
        
        # Liberar memoria cada 10 chunks
        if (i %% 10 == 0) {
          gc()
        }
      }
      
      # Combinar resultados
      texto_tag <- do.call(rbind, texto_tag_list)
    }
  }
  
  # Procesamiento de entidades y frases nominales
  if (show_progress) {
    message("Consolidando entidades nombradas...")
  }
  texto_tag_entity <- spacyr::entity_consolidate(texto_tag)
  texto_only_entity <- spacyr::entity_extract(texto_tag, type = "all")
  
  if (show_progress) {
    message("Consolidando frases nominales...")
  }
  texto_nounphrase <- spacyr::nounphrase_consolidate(texto_tag)
  texto_only_nounphrase <- spacyr::nounphrase_extract(texto_tag)
  
  spacyr::spacy_finalize()

  texto_tag <- rsyntax::as_tokenindex(texto_tag)

  # Geocodificacion con cache
  texto_only_entity_loc <- .acep_postag_hibrido_prepare_loc_entities(texto_only_entity)
  
  if (nrow(texto_only_entity_loc) > 0) {
    unique_locations <- unique(texto_only_entity_loc$entity_)
    
    # Cargar cache si existe
    cached_geocoder <- data.frame(entity_ = character(), lat = numeric(), long = numeric())
    if (use_cache && file.exists(geocode_cache_file)) {
      if (show_progress) {
        message(sprintf("Cargando cache de geocodificacion desde %s...", geocode_cache_file))
      }
      cached_geocoder <- jsonlite::read_json(geocode_cache_file, simplifyVector = TRUE)
    }

    # Geocodificar solo ubicaciones nuevas
    new_locations <- setdiff(unique_locations, cached_geocoder$entity_)

    if (length(new_locations) > 0) {
      if (show_progress) {
        message(sprintf("Geocodificando %d ubicaciones nuevas...", length(new_locations)))
      }
      new_geocoder <- tidygeocoder::geo(new_locations, method = "osm")
      names(new_geocoder) <- c("entity_", "lat", "long")

      # Combinar con cache
      cached_geocoder <- rbind(cached_geocoder, new_geocoder)

      # Guardar cache actualizado
      if (use_cache) {
        jsonlite::write_json(cached_geocoder, geocode_cache_file, pretty = TRUE)
        if (show_progress) {
          message(sprintf("Cache actualizado: %d ubicaciones totales", nrow(cached_geocoder)))
        }
      }
    } else {
      if (show_progress) {
        message("Todas las ubicaciones estan en cache")
      }
    }

    # Usar cache para merge
    texto_geocoder <- cached_geocoder[cached_geocoder$entity_ %in% unique_locations, ]
    
    texto_only_entity_loc <- merge(texto_only_entity_loc, texto_geocoder, 
                                   by = "entity_", all.x = TRUE)
    names(texto_only_entity_loc) <- c("entity_", "doc_id", "sentence", "entity", 
                                      "entity_type", "lat", "long")
    
    texto_only_entity_loc <- merge(texto_only_entity_loc, 
                                   unique(texto_tag[, c("doc_id", "sentence")]), 
                                   by = c("doc_id", "sentence"))
    texto_only_entity_loc <- unique(texto_only_entity_loc[!is.na(texto_only_entity_loc$lat), ])
  } else {
    texto_only_entity_loc <- .acep_postag_hibrido_empty_loc_frame()
  }
  
  # Retornar lista de resultados
  return(list(
    texto_tag = texto_tag,
    texto_tag_entity = texto_tag_entity,
    texto_only_entity = texto_only_entity,
    texto_only_entity_loc = texto_only_entity_loc,
    texto_nounphrase = texto_nounphrase,
    texto_only_nounphrase = texto_only_nounphrase
  ))
}

#' @title Serie temporal de índices de conflictividad.
#' @description Función que devuelve los indices de conflictividad
#' agrupados por segmento de tiempo: 'dia', 'mes', 'anio'. Esta función
#' viene a reemplazar a acep_rst. Simplifica los parámetros.
#' @param datos data frame con las variables 'fecha' (en formato Date),
#' 'n_palabras' (numérica), conflictos' (numérica), 'intensidad' (numérica).
#' Las ultimas tres se pueden construir en un solo paso con la función 'acep_db'
#' o en tres pasos con las funciones 'acep_frec', 'acep_men', 'acep_int'.
#' @param st parámetro para establecer el segmento temporal
#' a ser agrupado: 'anio', 'mes', 'dia'.
#' @param u umbral de menciones para contabilizar una nota
#' como nota que refiere a un conflicto, por defecto tiene 2
#' pero se puede modificar.
#' @param d cantidad de decimales, por defecto tiene 4 pero
#' se puede modificar.
#' @importFrom stats aggregate
#' @return Si todas las entradas son correctas,
#' la salida sera una base de datos en formato tabular
#' con nuevas variables.
#' @keywords resumen
#' @examples
#' datos <- acep_bases$rp_procesada
#' head(datos)
#' datos_procesados_anio <- acep_sst(datos, st='anio', u=4)
#' datos_procesados_mes <- acep_sst(datos)
#' datos_procesados_dia <- acep_sst(datos, st ='dia', d=3)
#' head(datos_procesados_anio)
#' head(datos_procesados_mes)
#' head(datos_procesados_dia)
#' @export
acep_sst <- function(datos, st = "mes", u = 2, d = 4) {
  validate_dataframe(datos, required_cols = c("fecha", "n_palabras", "conflictos", "intensidad"), arg_name = "datos")
  validate_choice(st, choices = c("anio", "mes", "dia"), arg_name = "st")
  validate_numeric(u, "u", min = 0)
  validate_numeric(d, "d", min = 0)

  tryCatch({
    datos$anio <- format(datos$fecha, "%Y")
    datos$mes <- paste0(datos$anio, "-", format(datos$fecha, "%m"))
    datos$dia <- datos$fecha
    datos$csn <- ifelse(datos$conflictos > u, 1, 0)

    st  <- if (st == "anio") {
      st <- datos$anio
    } else if (st == "mes") {
      st <- datos$mes
    } else if (st == "dia") {
      st <- datos$dia
    }

    frec_notas <- stats::aggregate(st, by = list(st), FUN = length)
    colnames(frec_notas) <- c("st", "frecn")

    frec_notas_conf <- stats::aggregate(
      csn ~ st, datos, function(x) c(frec_notas_conf = sum(x)))

    frec_palabras <- stats::aggregate(
      datos$n_palabras ~ st, datos, function(x) c(frec_palabras = sum(x)))
    colnames(frec_palabras) <- c("st", "frecp")

    frec_conflict <- stats::aggregate(
      datos$conflictos ~ st, datos, function(x) c(frec_conflict = sum(x)))
    colnames(frec_conflict) <- c("st", "frecm")

    frec_int_acum <- stats::aggregate(
      datos$intensidad ~ st, datos, function(x) c(frec_int_acum = sum(x)))
    colnames(frec_int_acum) <- c("st", "intac")

    frec_pal_con <-
      merge(frec_notas,
            merge(frec_notas_conf,
                  merge(frec_palabras,
                        merge(frec_conflict, frec_int_acum))))

    frec_pal_con$intensidad <- acep_int(frec_pal_con$frecm,
                                        frec_pal_con$frecp,
                                        decimales = d)
    frec_pal_con$int_notas_confl <- acep_int(frec_pal_con$csn,
                                             frec_pal_con$frecn,
                                             decimales = d)
    return(frec_pal_con)
  })
}

test_that("ACEP SST ANIO", {
  skip_if_offline()
  skip_on_cran()
  datos <- acep_bases$rp_procesada
  datos_procesados_anio <- acep_sst(datos,
                                    st = "anio", u = 4)
  dimensiones <- 12
  expect_equal(dimensiones, 12)
})

test_that("ACEP SST DIA", {
  skip_if_offline()
  skip_on_cran()
  datos <- acep_bases$rp_procesada
  datos_procesados_dia <- acep_sst(datos,
                                   st = "dia")
  dimensiones <- 2895
  expect_equal(dimensiones, 2895)
})

test_that("ACEP SST MES", {
  skip_if_offline()
  skip_on_cran()
  datos <- acep_bases$rp_procesada
  datos_procesados_mes <- acep_sst(datos,
                                   st = "mes")
  dimensiones <- 142
  expect_equal(dimensiones, 142)
})


test_that("ACEP SST ANIO E1", {
  skip_if_offline()
  skip_on_cran()
  datos <- acep_bases$rp_procesada

  expect_error(
    acep_sst(datos$fecha, st = "anio", u = 4),
    "debe ser un data.frame"
  )
})

test_that("ACEP SST DIA E2", {
  skip_if_offline()
  skip_on_cran()
  datos <- acep_bases$rp_procesada
  names(datos) <- c("A", "B", "C", "D")
  expect_error(
    datos_procesados_dia <- acep_sst(datos,st = "dia")
  )
  dimensiones <- 2895
  expect_equal(dimensiones, 2895)
})

test_that("ACEP SST MES E3", {
  skip_if_offline()
  skip_on_cran()
  datos <- acep_bases$rp_procesada
  datos_procesados_mes <- acep_sst(datos,
                                   st = "mes")
  dimensiones <- 142
  expect_equal(dimensiones, 142)
})


test_that("ACEP SST ANIO E4", {
  skip_if_offline()
  skip_on_cran()
  datos <- acep_bases$rp_procesada
  datos_procesados_anio <- acep_sst(datos,
                                    st = "anio", u = 4)
  dimensiones <- 12
  expect_equal(dimensiones, 12)
})

test_that("ACEP SST DIA E5", {
  skip_if_offline()
  skip_on_cran()
  datos <- acep_bases$rp_procesada
  datos_procesados_dia <- acep_sst(datos,
                                   st = "dia")
  dimensiones <- 2895
  expect_equal(dimensiones, 2895)
})

test_that("ACEP SST MES E6", {
  skip_if_offline()
  skip_on_cran()
  datos <- acep_bases$rp_procesada
  datos_procesados_mes <- acep_sst(datos,
                                   st = "mes")
  dimensiones <- 142
  expect_equal(dimensiones, 142)
})


test_that("ACEP SST MES E7", {
  skip_if_offline()
  skip_on_cran()
  datos <- acep_bases$rp_procesada
  datos_procesados_mes <- acep_sst(datos,
                                   st = "mes")
  dimensiones <- 142
  expect_equal(dimensiones, 142)
})

#' @title Interaccion con modelos de IA usando OpenRouter
#' @description
#' Funcion para interactuar con multiples proveedores de IA (OpenAI, Anthropic, Google,
#' Meta, etc.) a traves de la API unificada de OpenRouter. Soporta Structured Outputs
#' para modelos compatibles (OpenAI GPT-4o+, Fireworks, y otros). OpenRouter normaliza
#' las diferencias entre proveedores, permitiendo acceder a 400+ modelos con una sola API.
#' Ideal para comparar modelos o usar fallbacks automaticos.
#'
#' @section MODELOS COMPATIBLES Y PROBADOS (ACTUALIZADO):
#' \subsection{OpenAI - Familia GPT-5 (Ultima Generacion)}{
#' \itemize{
#'   \item \code{openai/gpt-5} - Modelo principal GPT-5
#'   \item \code{openai/gpt-5-pro} - Version Pro, maxima precision
#'   \item \code{openai/gpt-5-mini} - Version mini, economica
#'   \item \code{openai/gpt-5-nano} - Version nano, ultrarrapida
#'   \item \code{openai/gpt-5-chat} - Optimizado para chat
#' }
#' }
#' \subsection{OpenAI - Familia GPT-4.1}{
#' \itemize{
#'   \item \code{openai/gpt-4.1} - Modelo principal GPT-4.1
#'   \item \code{openai/gpt-4.1-mini} - Version mini
#'   \item \code{openai/gpt-4.1-nano} - Version nano
#' }
#' }
#' \subsection{OpenAI - Familia GPT-4}{
#' \itemize{
#'   \item \code{openai/gpt-4o} - GPT-4 optimizado
#'   \item \code{openai/gpt-4o-mini} - Economico, rapido, ideal para produccion
#'   \item \code{openai/gpt-4-turbo} - Version turbo
#'   \item \code{openai/gpt-4} - Modelo base GPT-4
#' }
#' }
#' \subsection{OpenAI - Familia GPT-3.5}{
#' \itemize{
#'   \item \code{openai/gpt-3.5-turbo} - Version turbo, economica
#' }
#' }
#' \subsection{OpenAI - Modelos OSS (Open Source Style)}{
#' \itemize{
#'   \item \code{openai/gpt-oss-120b} - Modelo grande (120B parametros)
#'   \item \code{openai/gpt-oss-120b:exacto} - Version exacta
#'   \item \code{openai/gpt-oss-20b} - Modelo pequeno (20B parametros)
#'   \item \code{openai/gpt-oss-20b:free} - Version gratuita
#' }
#' }
#' \subsection{xAI - Familia Grok 4 (Ultima Generacion)}{
#' \itemize{
#'   \item \code{x-ai/grok-4} - Modelo principal Grok 4
#'   \item \code{x-ai/grok-4-fast} - Version rapida, optimizada
#' }
#' }
#' \subsection{xAI - Familia Grok 3}{
#' \itemize{
#'   \item \code{x-ai/grok-3} - Modelo principal Grok 3
#'   \item \code{x-ai/grok-3-mini} - Version mini, economica
#'   \item \code{x-ai/grok-3-beta} - Version beta
#'   \item \code{x-ai/grok-3-mini-beta} - Version mini beta
#' }
#' }
#' \subsection{DeepSeek - Familia V3 (Ultima Generacion)}{
#' \itemize{
#'   \item \code{deepseek/deepseek-v3.2-exp} - Version experimental 3.2
#'   \item \code{deepseek/deepseek-v3.1-terminus} - Version terminus 3.1
#'   \item \code{deepseek/deepseek-v3.1-terminus:exacto} - Version terminus exacta
#' }
#' }
#' \subsection{DeepSeek - Familia R1 (Razonamiento)}{
#' \itemize{
#'   \item \code{deepseek/deepseek-r1-0528} - Version R1 de mayo 2028
#'   \item \code{deepseek/deepseek-r1} - Modelo principal R1
#'   \item \code{deepseek/deepseek-r1:free} - Version R1 gratuita
#' }
#' }
#' \subsection{DeepSeek - R1 Distilled (Basado en Llama)}{
#' \itemize{
#'   \item \code{deepseek/deepseek-r1-distill-llama-70b} - R1 destilado en Llama 70B
#'   \item \code{deepseek/deepseek-r1-distill-llama-70b:free} - Version gratuita
#' }
#' }
#' \subsection{DeepSeek - Chat y Otros}{
#' \itemize{
#'   \item \code{deepseek/deepseek-chat} - Optimizado para chat
#' }
#' }
#' \subsection{DeepCogito - Modelos Especializados}{
#' \itemize{
#'   \item \code{deepcogito/cogito-v2-preview-deepseek-671b} - Modelo cogito 671B basado en DeepSeek
#' }
#' }
#' \subsection{Meta Llama - Familia 4}{
#' \itemize{
#'   \item \code{meta-llama/llama-4-maverick} - Llama 4 Maverick
#' }
#' }
#' \subsection{Meta Llama - Familia 3.3}{
#' \itemize{
#'   \item \code{meta-llama/llama-3.3-70b-instruct} - Version de pago
#'   \item \code{meta-llama/llama-3.3-70b-instruct:free} - Version gratuita
#' }
#' }
#' \subsection{Meta Llama - Familia 3.2}{
#' \itemize{
#'   \item \code{meta-llama/llama-3.2-90b-vision-instruct} - Con capacidades de vision
#' }
#' }
#' \subsection{Meta Llama - Familia 3.1}{
#' \itemize{
#'   \item \code{meta-llama/llama-3.1-405b-instruct} - Modelo grande 405B
#'   \item \code{meta-llama/llama-3.1-70b-instruct} - Modelo mediano 70B
#' }
#' }
#' \subsection{NousResearch - Modelos Hermes}{
#' \itemize{
#'   \item \code{nousresearch/hermes-3-llama-3.1-405b} - Hermes 3 basado en Llama 405B
#' }
#' }
#' \subsection{Mistral AI - Familia Magistral}{
#' \itemize{
#'   \item \code{mistralai/magistral-medium-2506} - Magistral medium
#'   \item \code{mistralai/magistral-medium-2506:thinking} - Con razonamiento extendido
#' }
#' }
#' \subsection{Mistral AI - Otros Modelos}{
#' \itemize{
#'   \item \code{mistralai/mistral-large-2407} - Mistral large
#'   \item \code{mistralai/mixtral-8x22b-instruct} - Mixtral 8x22B (MoE)
#' }
#' }
#' \subsection{MoonshotAI - Familia Kimi K2}{
#' \itemize{
#'   \item \code{moonshotai/kimi-k2} - Modelo principal Kimi K2
#'   \item \code{moonshotai/kimi-k2-0905} - Version 09/05
#'   \item \code{moonshotai/kimi-k2-0905:exacto} - Version exacta
#'   \item \code{moonshotai/kimi-k2-thinking} - Con razonamiento extendido
#' }
#' }
#' \subsection{Qwen - Familia Qwen3 (235B - Modelos Grandes)}{
#' \itemize{
#'   \item \code{qwen/qwen3-235b-a22b-2507} - Modelo grande 235B (version 2507)
#'   \item \code{qwen/qwen3-235b-a22b} - Modelo grande 235B
#'   \item \code{qwen/qwen3-235b-a22b-thinking-2507} - Con razonamiento extendido
#'   \item \code{qwen/qwen3-max} - Version maxima
#' }
#' }
#' \subsection{Qwen - Familia Qwen3 (30B-80B - Modelos Medianos)}{
#' \itemize{
#'   \item \code{qwen/qwen3-next-80b-a3b-instruct} - Modelo next 80B
#'   \item \code{qwen/qwen3-32b} - Modelo 32B
#'   \item \code{qwen/qwen3-30b-a3b} - Modelo 30B
#'   \item \code{qwen/qwen3-30b-a3b-instruct-2507} - Version instruct 2507
#'   \item \code{qwen/qwen3-30b-a3b:free} - Version 30B gratuita
#' }
#' }
#' \subsection{Qwen - Familia Qwen3 (Modelos Pequenos)}{
#' \itemize{
#'   \item \code{qwen/qwen3-14b:free} - Modelo 14B gratuito
#'   \item \code{qwen/qwen3-4b:free} - Modelo 4B gratuito, ultrarrapido
#' }
#' }
#' \subsection{Qwen - Familia Qwen 2.5}{
#' \itemize{
#'   \item \code{qwen/qwen-2.5-72b-instruct} - Modelo 2.5 generacion anterior
#'   \item \code{qwen/qwen-plus} - Version plus
#' }
#' }
#' \subsection{Google Gemini - Familia 2.5 (Ultima Generacion)}{
#' \itemize{
#'   \item \code{google/gemini-2.5-flash} - Rapido, ultima generacion
#'   \item \code{google/gemini-2.5-pro} - Mayor precision, ultima generacion
#'   \item \code{google/gemini-2.5-flash-lite} - Ultrarrapido, ligero
#'   \item \code{google/gemini-2.5-flash-preview-09-2025} - Preview version septiembre
#'   \item \code{google/gemini-2.5-flash-lite-preview-09-2025} - Preview lite septiembre
#' }
#' }
#' \subsection{Google Gemini - Familia 2.0}{
#' \itemize{
#'   \item \code{google/gemini-2.0-flash-001} - Version estable 2.0
#'   \item \code{google/gemini-2.0-flash-lite-001} - Version ligera 2.0
#' }
#' }
#' \subsection{Google Gemini - Familia 1.5}{
#' \itemize{
#'   \item \code{google/gemini-pro-1.5} - Version anterior, estable
#' }
#' }
#' \subsection{Anthropic Claude - Familia Haiku (Rapidos y Economicos)}{
#' \itemize{
#'   \item \code{anthropic/claude-3.5-haiku} - Version 3.5, muy rapido
#'   \item \code{anthropic/claude-3-haiku} - Version 3, economico
#'   \item \code{anthropic/claude-haiku-4.5} - Ultima version, mas preciso
#' }
#' }
#' \subsection{Anthropic Claude - Familia Sonnet (Equilibrados)}{
#' \itemize{
#'   \item \code{anthropic/claude-3.5-sonnet} - Popular, buen balance
#'   \item \code{anthropic/claude-3.7-sonnet} - Version mejorada
#'   \item \code{anthropic/claude-3.7-sonnet:thinking} - Con razonamiento extendido
#'   \item \code{anthropic/claude-sonnet-4} - Generacion 4
#'   \item \code{anthropic/claude-sonnet-4.5} - Ultima version, mas preciso
#' }
#' }
#' \subsection{Anthropic Claude - Familia Opus (Maxima Precision)}{
#' \itemize{
#'   \item \code{anthropic/claude-3-opus} - Version 3, muy preciso
#'   \item \code{anthropic/claude-opus-4} - Generacion 4
#'   \item \code{anthropic/claude-opus-4.1} - Ultima version disponible
#' }
#' }
#' **Importante:** los modelos etiquetados como `:free` operan con cuotas comunitarias
#' y suelen estar sometidos a limites de tasa estrictos por parte de OpenRouter. Es
#' frecuente recibir respuestas HTTP 429 (Too Many Requests) cuando la demanda supera
#' la cuota disponible; este codigo indica que el proveedor rechazo la peticion para
#' proteger la infraestructura compartida. Si ocurre, espera unos segundos y reintenta,
#' o selecciona la variante de pago equivalente (sin sufijo `:free`) o activa `use_fallback`
#' para que OpenRouter cambie automaticamente a un modelo disponible.
#'
#' @param texto Texto a analizar. Puede ser una noticia, tweet, documento, etc.
#' @param instrucciones Instrucciones en lenguaje natural que indican al modelo que hacer
#'   con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".
#' @param modelo Modelo a utilizar con formato "proveedor/modelo". Ejemplos populares:
#'   - OpenAI: `"openai/gpt-4o-mini"` (rapido y economico), `"openai/gpt-4o"` (potente)
#'   - Anthropic: `"anthropic/claude-sonnet-4.5"`, `"anthropic/claude-3.5-haiku"`
#'   - Google: `"google/gemini-2.5-flash"`, `"google/gemini-2.0-flash-001"`
#'   - Meta: `"meta-llama/llama-3.3-70b-instruct"`, `"meta-llama/llama-4-maverick:free"`
#'   - Qwen: `"qwen/qwen3-next-80b-a3b-instruct-2509"`
#'   - DeepSeek: `"deepseek/deepseek-chat-v3-0324:free"`, `"deepseek/deepseek-r1:free"`
#'   Por defecto: `"openai/gpt-4o-mini"`. Ver lista completa: https://openrouter.ai/models
#' @param api_key Clave de API de OpenRouter. Si no se proporciona, busca la variable de
#'   entorno `OPENROUTER_API_KEY`. Para obtener una clave: https://openrouter.ai/settings/keys
#' @param schema Esquema JSON que define la estructura de la respuesta. Puede usar
#'   `acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
#'   Si es `NULL`, usa un esquema simple con campo "respuesta".
#'   NOTA: Structured Outputs solo funciona con modelos compatibles (OpenAI GPT-4o+, Fireworks).
#'   Para otros modelos, se usara JSON mode basico.
#' @param parse_json Logico. Si `TRUE` (por defecto), parsea automaticamente el JSON
#'   a un objeto R (lista o data frame). Si `FALSE`, devuelve el JSON como string.
#' @param temperature Parametro de temperatura (0-2). Valores bajos (0-0.3) generan
#'   respuestas mas deterministas. Valores altos (0.7-1) mas creativas. Por defecto: 0.
#' @param max_tokens Numero maximo de tokens en la respuesta. Por defecto: 2000.
#' @param top_p Parametro top-p para nucleus sampling (0-1). Por defecto: 0.2.
#' @param app_name Nombre de tu aplicacion (opcional). Se muestra en openrouter.ai/activity.
#' @param site_url URL de tu aplicacion (opcional). Para estadisticas en OpenRouter.
#' @param use_fallback Logico. Si `TRUE`, OpenRouter usara modelos alternativos si el
#'   proveedor primario falla. Por defecto: FALSE (sin fallbacks).
#' @param fallback_provider_order Vector opcional de slugs de proveedores para forzar
#'   un orden especifico de enrutamiento (ej.: `c("openai", "anthropic")`). Requiere
#'   `use_fallback = TRUE` para habilitar intentos sucesivos.
#' @param fallback_models Vector opcional de modelos alternativos (en formato
#'   `"proveedor/modelo"`) que se probaran en orden si el modelo principal devuelve un
#'   error recuperable (429, 5xx, timeouts). Ideal para definir variantes pagas cuando
#'   la version `:free` alcance su limite.
#' 
#' @param timeout Tiempo maximo de espera de la peticion HTTP en segundos.
#'   Por defecto: 120. Evita que una conexion estancada bloquee la sesion de R.
#' @return Si `parse_json=TRUE`, devuelve una lista o data frame con la respuesta
#'   estructurada segun el esquema. Si `parse_json=FALSE`, devuelve un string JSON.
#'
#' @details
#' OpenRouter abstrae las diferencias entre proveedores, mapeando automaticamente los
#' parametros a la interfaz nativa de cada modelo. Los parametros no soportados por un
#' modelo son ignorados silenciosamente. Esto permite usar la misma funcion para
#' cualquier modelo sin preocuparse por las especificidades de cada API.
#'
#' Cuando `use_fallback = TRUE`, la funcion configura el objeto `provider` de OpenRouter
#' para conservar la resiliencia ante errores transitorios y, si se define
#' `fallback_models`, intenta llamar secuencialmente a cada modelo alternativo ante
#' codigos recuperables (429, 5xx, timeouts). Esto evita depender del campo `route`,
#' ya deprecado en la API.
#'
#' Para Structured Outputs estrictos, recomendamos usar modelos OpenAI (gpt-4o+) o
#' Fireworks. Otros modelos intentaran seguir el esquema pero sin garantias estrictas.
#'
#' @export
#' @examples
#' \dontrun{
#' # Configurar API key
#' Sys.setenv(OPENROUTER_API_KEY = "tu-api-key")
#'
#' # Usar GPT-4o mini (rapido y economico)
#' texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
#' resultado <- acep_openrouter(texto, "Extrae las entidades nombradas",
#'                               modelo = "openai/gpt-4o-mini",
#'                               schema = acep_gpt_schema("extraccion_entidades"))
#'
#' # Comparar con Claude
#' resultado_claude <- acep_openrouter(texto, "Extrae las entidades nombradas",
#'                                      modelo = "anthropic/claude-sonnet-4.5",
#'                                      schema = acep_gpt_schema("extraccion_entidades"))
#'
#' # Usar modelo gratuito
#' resultado_free <- acep_openrouter(texto, "Clasifica el sentimiento",
#'                                    modelo = "meta-llama/llama-4-maverick:free",
#'                                    schema = acep_gpt_schema("sentimiento"))
#'
#' # Definir fallback hacia variantes pagas o proveedores alternativos
#' resultado_resiliente <- acep_openrouter(
#'   texto,
#'   "Extrae las entidades nombradas",
#'   modelo = "meta-llama/llama-4-maverick:free",
#'   schema = acep_gpt_schema("extraccion_entidades"),
#'   use_fallback = TRUE,
#'   fallback_models = c("meta-llama/llama-4-maverick", "openai/gpt-4o-mini")
#' )
#'
#' }
acep_openrouter <- function(texto,
                            instrucciones,
                            modelo = "openai/gpt-4o-mini",
                            api_key = Sys.getenv("OPENROUTER_API_KEY"),
                            schema = NULL,
                            parse_json = TRUE,
                            temperature = 0,
                            max_tokens = 2000,
                            top_p = 0.2,
                            app_name = NULL,
                            site_url = NULL,
                            use_fallback = FALSE,
                            fallback_provider_order = NULL,
                            fallback_models = NULL,
                            timeout = 120) {

  .acep_provider_validate_request_inputs(texto, instrucciones, api_key, "OPENROUTER_API_KEY")
  if (!is.logical(use_fallback) || length(use_fallback) != 1 || is.na(use_fallback)) {
    stop("El parametro 'use_fallback' debe ser TRUE o FALSE")
  }
  if (!is.null(fallback_provider_order)) {
    if (!is.character(fallback_provider_order) || length(fallback_provider_order) == 0) {
      stop("El parametro 'fallback_provider_order' debe ser un vector de caracteres con al menos un proveedor")
    }
    if (any(!nzchar(fallback_provider_order))) {
      stop("Los valores de 'fallback_provider_order' no pueden ser cadenas vacias")
    }
    if (!use_fallback) {
      stop("Para usar 'fallback_provider_order' debes activar 'use_fallback = TRUE'")
    }
  }
  if (!is.null(fallback_models)) {
    if (!is.character(fallback_models)) {
      stop("El parametro 'fallback_models' debe ser un vector de caracteres")
    }
    if (any(!nzchar(fallback_models))) {
      stop("Los valores de 'fallback_models' no pueden ser cadenas vacias")
    }
  }

  # Determinar si el modelo soporta Structured Outputs estrictos
  # Segun https://openrouter.ai/docs/features/structured-outputs
  # OpenAI GPT-4o+, Google Gemini y Fireworks tienen soporte completo
  # Para otros modelos (Anthropic, Meta, etc.) usar JSON mode básico
  # Esquema por defecto si no se proporciona uno
  if (is.null(schema)) {
    schema <- .acep_provider_default_schema()
  }

  # Prompts predefinidos (structured vs JSON mode basico)
  system_prompt_structured <- "Eres un asistente experto en analisis de texto. Debes responder SIEMPRE siguiendo exactamente el esquema JSON proporcionado. Se preciso, conciso y basa tus respuestas unicamente en el texto proporcionado."
  campos_descripciones <- .acep_provider_schema_field_descriptions(schema)
  campos_texto <- paste(campos_descripciones, collapse = "\n")
  system_prompt_json <- sprintf(
    "Eres un asistente experto en analisis de texto. Debes responder SIEMPRE en formato JSON valido con los siguientes campos:\n\n%s\n\nSe preciso, conciso y basa tus respuestas unicamente en el texto proporcionado. Responde UNICAMENTE con el JSON de datos, sin texto adicional antes o despues.",
    campos_texto
  )

  # Candidatos a modelo (principal + fallbacks definidos)
  modelos_candidatos <- unique(c(modelo, fallback_models))
  if (length(modelos_candidatos) == 0 || any(!nzchar(modelos_candidatos))) {
    stop("Debes especificar al menos un modelo valido para la solicitud")
  }

  # Construir prompt del usuario
  user_prompt <- .acep_provider_user_prompt(texto, instrucciones)

  # Construir body de la peticion
  body <- list(
    messages = list(
      list(role = "system", content = NULL),
      list(role = "user", content = user_prompt)
    ),
    temperature = temperature,
    max_tokens = max_tokens,
    top_p = top_p
  )

  provider_config <- list()
  if (!is.null(fallback_provider_order)) {
    provider_config$order <- fallback_provider_order
  }
  if (!use_fallback) {
    provider_config$allow_fallbacks <- FALSE
  }
  if (length(provider_config) > 0) {
    body$provider <- provider_config
  }

  # Construir headers - httr::add_headers necesita argumentos nombrados, no una lista
  headers_call <- .acep_provider_auth_headers("openrouter", api_key, site_url = site_url, app_name = app_name)

  # Realizar peticion a la API con soporte de fallbacks
  fallback_retryable_codes <- c(408L, 409L, 425L, 429L, 500:599)
  errores_detalle <- list()
  registrar_error <- function(modelo_intentado, detalle) {
    errores_detalle[[length(errores_detalle) + 1]] <<- list(modelo = modelo_intentado, detalle = detalle)
  }
  ultimo_error <- "Error desconocido"

  for (i in seq_along(modelos_candidatos)) {
    modelo_actual <- modelos_candidatos[[i]]
    soporta_structured_actual <- .acep_openrouter_model_supports_structured_outputs(modelo_actual)

    body$model <- modelo_actual
    body$messages[[1]]$content <- if (soporta_structured_actual) system_prompt_structured else system_prompt_json
    if (soporta_structured_actual) {
      body$response_format <- list(
        type = "json_schema",
        json_schema = list(
          name = "respuesta_estructurada",
          strict = TRUE,
          schema = schema
        )
      )
    } else {
      body$response_format <- list(type = "json_object")
    }

    respuesta_http <- tryCatch({
      list(
        ok = TRUE,
        result = .acep_provider_post(
          url = .acep_provider_endpoint("openrouter"),
          headers = headers_call,
          body = body,
          timeout = timeout
        )
      )
    }, error = function(e) {
      list(ok = FALSE, error = conditionMessage(e))
    })

    if (!respuesta_http$ok) {
      detalle_error <- sprintf("Error de conexion: %s", respuesta_http$error)
      registrar_error(modelo_actual, detalle_error)
      ultimo_error <- detalle_error
      if (i < length(modelos_candidatos)) {
        next
      } else {
        break
      }
    }

    output <- respuesta_http$result
    status_code <- httr::status_code(output)

    if (status_code != 200) {
      error_content <- tryCatch(httr::content(output, as = "parsed"),
                                error = function(e) NULL)
      error_msg <- .acep_provider_http_error_message(error_content)
      detalle_status <- sprintf("Error HTTP %d: %s", status_code, error_msg)
      registrar_error(modelo_actual, detalle_status)
      ultimo_error <- detalle_status
      if (i < length(modelos_candidatos) && status_code %in% fallback_retryable_codes) {
        next
      } else {
        break
      }
    }

    respuesta_parsed <- httr::content(output, as = "parsed")

    respuesta_json <- .acep_provider_extract_chat_content(respuesta_parsed)

    return(.acep_provider_parse_json_response(respuesta_json, parse_json = parse_json))
  }

  if (length(errores_detalle) > 0) {
    detalles_texto <- paste(
      vapply(
        errores_detalle,
        function(err) sprintf("- %s: %s", err$modelo, err$detalle),
        character(1)
      ),
      collapse = "\n"
    )
    stop(sprintf(
      "Error al interactuar con la API de OpenRouter: %s\nIntentos realizados:\n%s",
      ultimo_error,
      detalles_texto
    ))
  } else {
    stop(sprintf("Error al interactuar con la API de OpenRouter: %s", ultimo_error))
  }
}

#' Procesamiento de textos en lotes para optimizar memoria
#'
#' @description
#' Divide un vector grande de textos en lotes (chunks) mas pequenos y los procesa
#' secuencialmente aplicando una funcion de ACEP. Esta estrategia permite analizar
#' corpus extensos (millones de documentos) sin superar la capacidad de memoria RAM
#' disponible. La funcion combina automaticamente los resultados de todos los lotes.
#'
#' @param texto Vector de caracteres con los textos a procesar.
#' @param funcion Funcion de ACEP a aplicar a cada lote. Ejemplos: `acep_clean`,
#'   `acep_token`, `acep_count`, `acep_upos`, etc. Debe ser una funcion que acepte
#'   un vector de textos como primer argumento.
#' @param chunk_size Numero de textos por lote. Valores mas bajos reducen el consumo
#'   de memoria pero aumentan el tiempo total de procesamiento. Por defecto: 1000.
#' @param show_progress Logico. Si `TRUE`, muestra mensajes informativos sobre el
#'   progreso del procesamiento (que lote se esta procesando). Por defecto: `TRUE`.
#' @param ... Argumentos adicionales que se pasan directamente a la funcion especificada
#'   en el parametro `funcion`. Ejemplo: si `funcion = acep_clean`, puede pasar
#'   `rm_stopwords = TRUE`, `tolower = TRUE`, etc.
#'
#' @return El tipo de resultado depende de la funcion aplicada:
#' \itemize{
#'   \item Si la funcion retorna un vector, devuelve un vector combinado
#'   \item Si la funcion retorna un data frame, devuelve un data frame combinado (rbind)
#'   \item Si la funcion retorna una lista, devuelve una lista de listas
#' }
#'
#' @export
#' @examples
#' \dontrun{
#' # Procesar 10,000 textos con limpieza en lotes de 1000
#' textos_limpios <- acep_process_chunks(
#'   texto = corpus_grande,
#'   funcion = acep_clean,
#'   chunk_size = 1000,
#'   rm_stopwords = TRUE
#' )
#'
#' # Tokenizar corpus masivo
#' tokens <- acep_process_chunks(
#'   texto = corpus_masivo,
#'   funcion = acep_token,
#'   chunk_size = 500,
#'   tolower = TRUE
#' )
#'
#' # Contar menciones en corpus grande
#' diccionario <- c("paro", "huelga", "protesta")
#' frecuencias <- acep_process_chunks(
#'   texto = corpus_grande,
#'   funcion = acep_count,
#'   chunk_size = 2000,
#'   dic = diccionario
#' )
#' }
#'
#' @export
acep_process_chunks <- function(texto,
                                funcion,
                                chunk_size = 1000,
                                show_progress = TRUE,
                                ...) {
  
  # Validaciones
  if (!is.character(texto)) {
    stop("El parametro 'texto' debe ser un vector de caracteres")
  }

  if (!is.function(funcion)) {
    stop("El parametro 'funcion' debe ser una funcion")
  }

  if (!is.numeric(chunk_size) || chunk_size <= 0) {
    stop("El parametro 'chunk_size' debe ser un numero positivo")
  }

  if (!is.logical(show_progress)) {
    stop("El parametro 'show_progress' debe ser TRUE o FALSE")
  }

  # Calcular numero de chunks
  n_textos <- length(texto)
  n_chunks <- ceiling(n_textos / chunk_size)

  if (show_progress) {
    message(sprintf("Procesando %d textos en %d chunks de %d",
                    n_textos, n_chunks, chunk_size))
  }

  # Crear lista para almacenar resultados
  resultados <- vector("list", n_chunks)

  # Procesar cada chunk
  for (i in seq_len(n_chunks)) {
    # Calcular indices del chunk
    inicio <- (i - 1) * chunk_size + 1
    fin <- min(i * chunk_size, n_textos)

    if (show_progress) {
      message(sprintf("Chunk %d/%d (textos %d-%d)", i, n_chunks, inicio, fin))
    }

    # Extraer chunk de textos
    chunk <- texto[inicio:fin]

    # Procesar chunk con la funcion
    resultados[[i]] <- funcion(chunk, ...)

    # Garbage collection cada 10 chunks para liberar memoria
    if (i %% 10 == 0) {
      gc(verbose = FALSE)
    }
  }

  # Combinar resultados segun el tipo
  if (is.data.frame(resultados[[1]])) {
    # Si es data.frame, usar rbind
    resultado_final <- do.call(rbind, resultados)
  } else if (is.vector(resultados[[1]])) {
    # Si es vector, usar unlist
    resultado_final <- unlist(resultados, use.names = FALSE)
  } else if (is.list(resultados[[1]])) {
    # Si es lista, mantener como lista de listas
    resultado_final <- resultados
  } else {
    # Tipo desconocido, devolver lista
    warning("Tipo de resultado desconocido, devolviendo lista")
    resultado_final <- resultados
  }
  
  if (show_progress) {
    message("Procesamiento completado")
  }
  
  return(resultado_final)
}

#' @title Limpieza de texto.
#' @description Función que limpia y normaliza las notas/textos.
#' @param x vector de textos al que se le aplica la
#' función de limpieza de texto.
#' @param rm_hashtag remueve los hashtags.
#' @param rm_emoji remueve los emojis.
#' @param rm_punt remueve la puntuación.
#' @param rm_num remueve números.
#' @param rm_whitespace remueve los espacios en blanco.
#' @param rm_newline remueve los saltos de linea.
#' @param rm_cesp remueve caracteres especiales.
#' @param rm_stopwords remueve palabras vacías.
#' @param rm_dias remueve los dias de la semana.
#' @param rm_meses remueve los meses del año.
#' @param rm_shortwords remueve las palabras cortas.
#' @param rm_url remueve las url.
#' @param rm_users remueve las menciones de usuarixs de redes sociales.
#' @param other_sw su valor por defecto es NULL, sirve para ampliar el
#' listado de stopwords con un nuevo vector de palabras.
#' @param u umbral de caracteres para la función rm_shortwords.
#' @param tolower convierte los textos a minúsculas.
#' @return Si todas las entradas son correctas,
#' la salida sera un vector de textos normalizados.
#' @keywords limpieza normalización
#' @examples
#' acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_stopwords = FALSE)
#' acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_stopwords = TRUE)
#' @export
acep_clean <- function(x,
                       tolower = TRUE,
                       rm_cesp = TRUE,
                       rm_emoji = TRUE,
                       rm_hashtag = TRUE,
                       rm_users = TRUE,
                       rm_punt = TRUE,
                       rm_num = TRUE,
                       rm_url = TRUE,
                       rm_meses = TRUE,
                       rm_dias = TRUE,
                       rm_stopwords = TRUE,
                       rm_shortwords = TRUE,
                       rm_newline = TRUE,
                       rm_whitespace = TRUE,
                       other_sw = NULL,
                       u = 1) {
  if(!is.vector(x) | is.list(x)){
    return(message("No ingresaste un vector en el parametro x.
           Vuelve a intentarlo ingresando un vector!"))
  }
  if (!is.logical(tolower) || !is.logical(rm_cesp) ||
      !is.logical(rm_emoji) || !is.logical(rm_hashtag) ||
      !is.logical(rm_users) || !is.logical(rm_punt) ||
      !is.logical(rm_num) || !is.logical(rm_url) ||
      !is.logical(rm_meses) || !is.logical(rm_dias) ||
      !is.logical(rm_stopwords) || !is.logical(rm_shortwords)||
      !is.logical(rm_newline) || !is.logical(rm_whitespace)) {
    return(message("Debe ingresar un valor booleano: TRUE o FALSE"))
  } else {
    if(is.vector(x) & !is.list(x)) {
      out <- tryCatch({
      if (tolower) {
        x <- gsub(pattern = "([[:upper:]])", perl = TRUE,
                  replacement = "\\L\\1", x)
        # Las mayúsculas acentuadas (Á, É, Ñ, ...) no las baja \\L (solo ASCII),
        # así que el plegado de tildes a minúscula debe correr ANTES de remover
        # stopwords; de lo contrario "MÁS" no coincide con la stopword "más".
        x <- chartr(ACEP::acep_rs$tildes,
                    tolower(ACEP::acep_rs$tildes), x)
      }
      if (rm_stopwords) {
        if (is.null(other_sw)) {
          x <- gsub(
            paste(
              ACEP::acep_rs$sw1,
              ACEP::acep_rs$sw2,
              collapse = ""),
            " ", x, perl = FALSE)
        } else {
          othersw <- paste0("|\\b", other_sw, "\\b", collapse = "")
          x <- gsub(
            paste0(
              paste(
                ACEP::acep_rs$sw1,
                ACEP::acep_rs$sw2,
                collapse = ""),
              othersw),
            " ", x, perl = FALSE)
        }
      }
      if (rm_cesp) {
        x <- chartr(ACEP::acep_rs$tildes,
                    ACEP::acep_rs$sintildes, x)
        }
      if (rm_url) {
        x <- gsub(ACEP::acep_rs$url, "", x, perl = TRUE)
        }
      if (rm_emoji) {
        x <- gsub(ACEP::acep_rs$emojis, " ", x, perl = TRUE)
        }
      if (rm_hashtag) {
        x <- gsub(ACEP::acep_rs$hashtag, "", x, perl = TRUE)
        }
      if (rm_users) {
        x <- gsub(ACEP::acep_rs$users, "", x, perl = TRUE)
        }
      if (rm_punt) {
        x <- gsub(ACEP::acep_rs$punt, " ", x, perl = TRUE)
        }
      if (rm_num) {
        x <- gsub(ACEP::acep_rs$num, "", x, perl = TRUE)
      }
      if (rm_meses) {
        x <- gsub(ACEP::acep_rs$meses, "", x, perl = TRUE)
        }
      if (rm_dias) {
        x <- gsub(ACEP::acep_rs$dias, "", x, perl = TRUE)
        }
      if (rm_shortwords) {
        x <- gsub(paste0("\\b[[:alpha:]]{1,", u, "}\\b"), " ", x, perl = FALSE)
        }
      if (rm_punt) {
        x <- gsub(paste0("\\b", ACEP::acep_rs$punt,"\\b"), "", x, perl = TRUE)
        }
      if (rm_newline) {
        x <- gsub(ACEP::acep_rs$saltos, " ", x, perl = TRUE)
        }
      if (rm_whitespace) {
        gsub(ACEP::acep_rs$espacios, "", x, perl = TRUE)
      } else {
        x
      }
      }
      )
    }
    return(out)
  }
}

test_that("ACEP CLEAN", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.")
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN TF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", tolower = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMCF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_cesp = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMEF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_emoji = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMHF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_hashtag = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMUF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_users = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMPF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_punt = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMNF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_num = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMURLF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_url = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMMF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_meses = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMDF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_dias = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMSTWF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_stopwords = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMSTWTN", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.",
    other_sw = c("mar", "plata", "puerto")
    )
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMSHWF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.",
    rm_shortwords = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMNLF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SOIP fue al paro. Reclaman mejoras salariales.",
    rm_newline = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN RMWSF", {
  skip_if_offline()
  skip_on_cran()
  clean <- acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.",
    rm_whitespace = FALSE)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

test_that("ACEP CLEAN ERROR", {
  skip_if_offline()
  skip_on_cran()
  df <- data.frame(V1=c(1:10), V2=c(letters[1:10]))
  clean <- acep_clean(df)
  dimensiones <- length(clean)
  expect_equal(dimensiones, length(clean))
})

#' Cache para regex compilados
#' @keywords internal
.acep_regex_cache <- new.env(parent = emptyenv())

#' Tope de entradas del caché de regex
#'
#' Cuando el caché alcanza este tamaño se vacía antes de agregar una nueva
#' entrada, evitando crecimiento de memoria sin límite en sesiones largas que
#' usan muchos diccionarios distintos. Un fallo de caché solo recompila el
#' patrón, por lo que la cota no afecta la correctitud de los conteos.
#' @keywords internal
.acep_regex_cache_max <- 1000L

#' Crear una clave de caché estable para diccionarios de conteo
#' @keywords internal
.acep_count_cache_key <- function(dic) {
  dic <- unique(dic)
  paste0(nchar(dic, type = "chars"), ":", dic, collapse = "\r")
}

#' Escapar metacaracteres de expresiones regulares en términos literales
#' @keywords internal
.acep_escape_regex <- function(x) {
  gsub("([][{}()*+?.\\\\^$|])", "\\\\\\1", x)
}

#' Crear patrón regex para diccionarios de conteo
#'
#' Escapa los metacaracteres de cada término (para tratarlos como texto literal,
#' evitando coincidencias accidentales o errores de regex) y convierte un espacio
#' inicial o final en un límite de palabra (\\b), de modo que " paro " coincida
#' solo con la palabra completa.
#' @keywords internal
.acep_count_pattern <- function(dic) {
  dic <- .acep_escape_regex(unique(dic))
  paste0(gsub("^ | $", "\\\\b", dic), collapse = "|")
}

#' @title Conteo de menciones de palabras de un diccionario
#' @description
#' Cuenta el número de veces que aparecen palabras de un diccionario en cada texto.
#' Los términos se tratan como texto literal: sus metacaracteres de expresiones
#' regulares se escapan automáticamente. De forma predeterminada también cuenta
#' coincidencias parciales (por ejemplo, "paro" coincide dentro de "paros"); para
#' exigir límites de palabra (word boundaries) y evitar coincidencias parciales,
#' rodeá cada término con espacios (por ejemplo, " paro "). Incluye un sistema de
#' caché que almacena los patrones regex compilados para acelerar ejecuciones
#' repetidas con el mismo diccionario. El caché se acota automáticamente
#' (máximo 1000 patrones) para evitar crecimiento de memoria sin límite en
#' sesiones largas; también podés vaciarlo manualmente con
#' `acep_clear_regex_cache()`.
#' @param texto vector de textos al que se le aplica la función de conteo.
#' @param dic vector de palabras del diccionario utilizado.
#' @param use_cache logical, usar caché de regex (default TRUE).
#' @return Vector con frecuencia de palabras del diccionario.
#' @keywords indicadores frecuencia tokens
#' @importFrom stringr str_count
#' @examples
#' df <- data.frame(texto = c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
#' "El SOIP lleva adelante un plan de lucha con paros y piquetes."))
#' diccionario <- c("paro", "lucha", "piquetes")
#' df$detect <- acep_count(df$texto, diccionario)
#' df
#' @export
acep_count <- function(texto, dic, use_cache = TRUE) {
  validate_character(texto, "texto")
  validate_character(dic, "dic")

  # Crear hash del diccionario para caché
  if (use_cache) {
    dic_hash <- .acep_count_cache_key(dic)

    # Buscar en caché
    if (exists(dic_hash, envir = .acep_regex_cache)) {
      dicc <- get(dic_hash, envir = .acep_regex_cache)
    } else {
      # Compilar y guardar en caché (con tope de tamaño para acotar memoria)
      dicc <- .acep_count_pattern(dic)
      if (length(ls(envir = .acep_regex_cache)) >= .acep_regex_cache_max) {
        rm(list = ls(envir = .acep_regex_cache), envir = .acep_regex_cache)
      }
      assign(dic_hash, dicc, envir = .acep_regex_cache)
    }
  } else {
    dicc <- .acep_count_pattern(dic)
  }

  stringr::str_count(texto, dicc)
}

#' @title Limpiar caché de expresiones regulares
#' @description
#' Elimina todos los patrones regex almacenados en el caché interno de `acep_count()`.
#' Útil para liberar memoria cuando se han procesado muchos diccionarios diferentes
#' o cuando se quiere forzar la recompilación de patrones.
#'
#' @return Devuelve `NULL` invisiblemente.
#' @export
#' @examples
#' # Limpiar el caché
#' acep_clear_regex_cache()
acep_clear_regex_cache <- function() {
  rm(list = ls(envir = .acep_regex_cache), envir = .acep_regex_cache)
  invisible(NULL)
}

#' @title Consultar tamaño del caché de regex
#' @description
#' Devuelve el número de patrones regex almacenados actualmente en el caché
#' interno de `acep_count()`. Cada diccionario único genera una entrada en el caché.
#'
#' @return Número entero con la cantidad de patrones en caché.
#' @export
#' @examples
#' # Ver cuántos patrones hay en caché
#' acep_regex_cache_size()
acep_regex_cache_size <- function() {
  length(ls(envir = .acep_regex_cache))
}

test_that("ACEP Count", {
  skip_if_offline()
  skip_on_cran()
  df <- data.frame(texto =
                     c("El SOIP fue al paro. Reclaman mejoras salariales.",
                       "El SOIP lleva adelante un plan de lucha con paros y piquetes."))
  diccionario <- c("paro", "lucha", "piquetes")
  df$detect <- acep_count(df$texto, diccionario)
  dimensiones <- length(df$detect)
  expect_equal(dimensiones, length(df$detect))
})

.acep_svo_collapse_tokens <- function(tokens, fallback_tokens, value_name, group_cols = c("doc_id", "sentence")) {
  tokens <- as.data.frame(tokens)
  fallback_tokens <- as.data.frame(fallback_tokens)
  source_tokens <- if (nrow(tokens) > 0) tokens else fallback_tokens

  collapsed <- stats::aggregate(
    source_tokens[["token"]],
    source_tokens[group_cols],
    paste0,
    collapse = " "
  )
  names(collapsed) <- c(group_cols, value_name)

  if (nrow(tokens) == 0) {
    collapsed[[value_name]] <- NA_character_
  }

  collapsed
}

#' @title Función para extraer tripletes SVO (Sujeto-Verbo-Objeto).
#' @description
#' Función que devuelve seis objetos data.frame con
#' etiquetado POS (modelo spacyr) y relaciones sintácticas
#' (modelo rsyntax) que permiten reconstruir estructuras
#' sintácticas como SVO y Sujeto-Predicado. Una vez seleccionadas
#' las notas periodísticas referidas a conflictos, esta función permite
#' extraer sujetos de la protesta, acción realizada y objeto(s) de la acción.
#' También devuelve entidades nombradas (NER).
#' @param acep_tokenindex data.frame con el etiquetado POS y las relaciones
#' de dependencia generado con la función acep_postag.
#' @param prof_s es un numero entero positivo que determina la profundidad a la que se
#' buscan las relaciones dentro del sujeto. Este parámetro se hereda del la función children()
#' del paquete \{rsyntax\}. Se recomienda no superar el valor 2.
#' @param prof_o es un numero entero positivo que determina la profundidad a la que se
#' buscan las relaciones dentro del objeto. Este parámetro se hereda del la función children()
#' del paquete \{rsyntax\}. Se recomienda no superar el valor 2.
#' @param u numero entero que indica el umbral de palabras del objeto en la reconstrucción SVO.
#' @importFrom stats ave setNames
#' @return Si todas las entradas son correctas, la salida sera una lista con tres bases de datos en formato tabular.
#' @references Welbers, K., Atteveldt, W. van, & Kleinnijenhuis, J. 2021. Extracting semantic relations using syntax:
#' An R package for querying and reshaping dependency trees. Computational Communication Research, 3-2, 1-16.
#' \doi{10.5117/CCR2021.2.003.WELB}
#' @source \href{https://universaldependencies.org/}{Dependencias Universales para taggeo POS}
#' @source \href{https://CRAN.R-project.org/package=rsyntax}{Sobre el paquete rsyntax}
#' @keywords tripletes sintaxis
#' @examples
#'\dontrun{
#' acep_svo(acep_bases$spacy_postag)
#'}
#' @export
acep_svo <- function(acep_tokenindex,
                     prof_s = 3,
                     prof_o = 3,
                     u = 1
){
  relaciones = c("flat","fixed","appos", "nmod", "amod")
  conexiones = FALSE
  rel_s = c("nsubj", "conj")
  rel_o = c("obj", "obl", "advcl", "xcomp", "conj", "acl")
  rel_evs = "nsubj"
  rel_evp = "obj"
  if (class(acep_tokenindex)[1] != "tokenIndex") {
    stop("El par\u00e1metro 'acep_tokenindex' debe ser de clase 'tokenIndex'",
         call. = FALSE)
  }
  if (!is.numeric(prof_s) | prof_s > 5 | prof_s < 1) {
    stop("El par\u00e1metro 'prof_s' debe ser un n\u00famero entero positivo entre 1 y 5",
         call. = FALSE)
  }
  if (!is.numeric(prof_o) | prof_o > 5 | prof_o < 1) {
    stop("El par\u00e1metro 'prof_o' debe ser un n\u00famero entero positivo entre 1 y 5",
         call. = FALSE)
  }
  if (!is.numeric(u) | u > 5 | u < 1) {
    stop("El par\u00e1metro 'u' debe ser un n\u00famero entero positivo entre 1 y 5",
         call. = FALSE)
  }
  acep_require_namespace("rsyntax", "acep_svo")

  fill <- rsyntax::custom_fill(relation = relaciones, min_window = c(1,1), connected = conexiones)

  direct <- rsyntax::tquery(
    label = "verbo", pos = "VERB", fill = FALSE,
    rsyntax::children(label = "sujeto", relation = rel_s, depth = prof_s, fill),
    rsyntax::children(label = "objeto", relation = rel_o, depth = prof_o, fill))

  indirect <- rsyntax::tquery(
    label = "verbo", pos = "VERB", fill = FALSE,
    rsyntax::children(label = "sujeto", relation = "nsubj:pass", depth = prof_s, fill),
    rsyntax::children(label = "objeto", relation = rel_o, depth = prof_o, fill))

  acep_annotate <- rsyntax::annotate_tqueries(acep_tokenindex, "s_v_o", direct, indirect, overwrite = FALSE)

  acep_annotate <- subset(acep_annotate, pos != "SPACE")

  acep_annotate$s_p <- ifelse(acep_annotate$pos == "VERB" & acep_annotate$relation == "ROOT", "predicado", "")

  acep_annotate$s_p <- with(acep_annotate,
                            ave(s_p, doc_id, sentence,
                                FUN = function(x) ifelse(cumsum(x == "predicado") > 0, "predicado", "sujeto")))

  acep_annotate$conjugaciones <- ifelse(grepl("Past", acep_annotate$morph), "pasado",
                                        ifelse(grepl("Pres", acep_annotate$morph), "presente",
                                               ifelse(grepl("Fut", acep_annotate$morph), "futuro", NA)))

  acep_annotate_o <- acep_annotate

  acep_annotate$relation <- gsub(":pass", "", acep_annotate$relation)

  sust_pred <- subset(acep_annotate, s_p == "predicado" & (pos == "PROPN" | pos == "NOUN"))

  sust_pred <- .acep_svo_collapse_tokens(sust_pred, acep_annotate, "sust_pred")
  if (any(!is.na(sust_pred$sust_pred))) {
    sust_pred$sust_pred <- gsub(" ", " | ", sust_pred$sust_pred, fixed = TRUE)
  }

  acep_return <-  subset(acep_annotate, !is.na(s_v_o_fill))
  acep_return <- subset(acep_return, pos != "ADP")

  sujetos <- subset(acep_return, s_p == "sujeto" & relation == "nsubj" |
                      s_p == "sujeto" & relation == rel_evs)

  sujetos <- .acep_svo_collapse_tokens(sujetos, acep_annotate, "sujetos")

  verbos <- subset(acep_return, relation == "ROOT")

  verbos <- .acep_svo_collapse_tokens(verbos, acep_annotate, "verbos")

  predicados <- subset(acep_return, relation == "obj" | relation == "obl")

  predicados <- .acep_svo_collapse_tokens(predicados, acep_annotate, "predicados")

  sujeto <- subset(acep_return, s_p == "sujeto")

  sujeto <- .acep_svo_collapse_tokens(sujeto, acep_annotate, "sujeto")

  predicado <- subset(acep_return, s_p == "predicado")

  predicado <- .acep_svo_collapse_tokens(predicado, acep_annotate, "predicado")

  verbo <- subset(acep_return, relation == "ROOT")

  verbo <- .acep_svo_collapse_tokens(verbo, acep_annotate, "verbo")

  conjugaciones <- subset(acep_return, s_v_o == "verbo")

  conjugaciones <- .acep_svo_collapse_tokens(conjugaciones, acep_annotate, "conjugaciones", c("doc_id", "sentence", "sent"))

  lemma_verb <- subset(acep_return, relation == "ROOT")

  lemma_verb <- .acep_svo_collapse_tokens(lemma_verb, acep_annotate, "lemma_verb")

  aux_verbos <- subset(acep_return, pos == "VERB" & !is.na(parent))

  aux_verbos <- .acep_svo_collapse_tokens(aux_verbos, acep_annotate, "aux_verbos")

  entidades <- subset(acep_return, pos == "PROPN")

  entidades <- .acep_svo_collapse_tokens(entidades, acep_annotate, "entidades")

  acep_return <-
    merge(
      merge(
        merge(
          merge(
            merge(
              merge(
                merge(
                  merge(
                    merge(
                      sujetos, verbos, all.x = TRUE),
                    predicados, all.x = TRUE),
                  sujeto, all.x = TRUE),
                predicado, all.x = TRUE),
              verbo, all.x = TRUE),
            conjugaciones, all.x = TRUE),
          lemma_verb, all.x = TRUE),
        aux_verbos, all.x = TRUE),
      entidades, all.x = TRUE)

  orac_com <- unique(subset(acep_annotate, select = c(doc_id, sentence, sent)))
  orac_fil <- unique(subset(acep_return, select = c(doc_id, sentence, sent)))

  anti_join <- merge(orac_com, orac_fil, by = c('doc_id', 'sentence'), all.x = TRUE)
  anti_join <- subset(anti_join, is.na(sent.y))

  no_procesadas <- setNames(subset(anti_join, select = c(doc_id, sentence, sent.x)), c("doc_id", "oracion_id", "oracion"))

  acep_return$eventos <- paste0(acep_return$sujetos, " -> ", acep_return$verbos, " -> ", acep_return$predicados)

  acep_return$entidades <- gsub("\\s[a-z]+\\s+", "  ", acep_return$entidades)
  acep_return$entidades <- gsub("\\s\\s+", "  ", acep_return$entidades)
  acep_return$entidades <- gsub("\\s\\s+", " | ", acep_return$entidades)
  acep_return$entidades <- gsub("\\| \\|", "|", acep_return$entidades)
  acep_return$entidades <- gsub("\\s+", " | ", acep_return$entidades)
  acep_return$entidades <- gsub("\\s+", " ", acep_return$entidades)
  acep_return$entidades <- paste0("| ", acep_return$entidades, " |", sep = "")
  acep_return$entidades <- gsub("\\| \\|", "|", acep_return$entidades)
  acep_return$entidades <- gsub("\\|  \\|", "", acep_return$entidades)
  acep_return$aux_verbos <- gsub("\\s+", " | ", acep_return$aux_verbos)
  acep_return$aux_verbos <- gsub("\\s+", " ", acep_return$aux_verbos)
  acep_return$aux_verbos <- paste0("| ", acep_return$aux_verbos, " |", sep = "")
  acep_return$aux_verbos <- gsub("\\| \\|", "|", acep_return$aux_verbos)
  acep_return$aux_verbos <- gsub("\\|  \\|", "", acep_return$aux_verbos)
  acep_return$conjugaciones <- gsub("(^[a-zA-Z]+)\\s.+", "\\1", acep_return$conjugaciones)
  acep_return$eventos <- gsub("\\s+", " ", acep_return$eventos)
  acep_return$eventos <- gsub(" -> $", "", acep_return$eventos)
  acep_return$eventos <- gsub("^\\s*", "", acep_return$eventos)
  acep_return$eventos <- gsub("*\\s$", "", acep_return$eventos)
  acep_return$eventos <- gsub("^-> ", "NA -> ", acep_return$eventos)
  acep_return$eventos <- gsub("-> ->", "-> NA ->", acep_return$eventos)

  acep_return <- merge(acep_return, sust_pred, all.x = TRUE)

  acep_return <- acep_return[ , c('doc_id','sentence','sujetos','verbos',
                                  'predicados','sujeto','entidades','eventos',
                                  'predicado','verbo','lemma_verb','aux_verbos',
                                  'sust_pred')]

  acep_return <- acep_return[order(acep_return$doc_id), ]

  row.names(acep_return) <- 1:length(acep_return$doc_id)

  names(acep_return)[names(acep_return) == "sentence"] <- "oracion_id"
  names(acep_return)[names(acep_return) == "sent"] <- "oracion"

  acep_ret <- acep_return[, c("doc_id","oracion_id","eventos")]

  acep_ret <- cbind(acep_ret, do.call("rbind", strsplit(as.character(acep_ret$eventos), " -> ", fixed = TRUE)))

  names(acep_ret)[names(acep_ret) == "1"] <- "sujeto"
  names(acep_ret)[names(acep_ret) == "2"] <- "verbo"
  names(acep_ret)[names(acep_ret) == "3"] <- "objeto"

  acep_sp <- acep_return[ , c("doc_id","oracion_id","sujeto","predicado","verbo",
                              "lemma_verb","aux_verbos","entidades","sust_pred")]

  acep_lista_lemmas <- subset(acep_annotate_o, nchar(token) > 1)

  acep_lista_lemmas <- subset(acep_lista_lemmas, !pos %in% c("NUM", "PUNCT", "X", "DET", "ADP", "ADV", "CCONJ", "INTJ", "PRON", "SCONJ", "SYM", "SPACE"))

  acep_lista_lemmas <- stats::setNames(as.data.frame(table(acep_lista_lemmas$lemma)), c("lemma", "n"))

  acep_lista_lemmas <- acep_lista_lemmas[order(acep_lista_lemmas$n, decreasing = TRUE), ]

  acep_pro_svo  <- acep_return[ , c('doc_id','oracion_id','eventos','sujeto','predicado','verbo',
                                    'lemma_verb','aux_verbos','entidades','sust_pred')]

  acep_pro_svo <- cbind(acep_pro_svo, do.call("rbind", strsplit(as.character(acep_pro_svo$eventos), " -> ", fixed = TRUE)))

  names(acep_pro_svo)[names(acep_pro_svo) == "1"] <- "sujeto_svo"
  names(acep_pro_svo)[names(acep_pro_svo) == "2"] <- "root"
  names(acep_pro_svo)[names(acep_pro_svo) == "3"] <- "objeto"

  acep_pro_svo <- acep_pro_svo[ , c("doc_id","oracion_id","eventos","sujeto_svo",
                                    "root","objeto","sujeto","predicado","verbo",
                                    "lemma_verb","aux_verbos","entidades","sust_pred")]

  if (length(table(acep_annotate_o$conjugaciones)) == 0) {
    acep_svo_list <-  "No hay oraciones procesables."
    message(acep_svo_list)
  } else {

    acep_svo_list <- list(acep_annotate_svo = acep_annotate_o,
                          acep_pro_svo = acep_pro_svo,
                          acep_list_svo = acep_ret,
                          acep_sp = acep_sp,
                          acep_lista_lemmas = acep_lista_lemmas,
                          acep_no_procesadas = no_procesadas)
    acep_svo_list
  }
}

# Offline validation test (no network, no optional deps). The happy path is
# characterized end-to-end in test-optimization-safety.R using a bundled fixture.
test_that("acep_svo rechaza entradas que no son tokenIndex", {
  expect_error(acep_svo("no es un tokenindex"), "tokenIndex")
  expect_error(acep_svo(data.frame(a = 1)), "tokenIndex")
})

#' @title Interaccion con modelos de IA usando TogetherAI
#' @description
#' Funcion para interactuar con modelos de IA a traves de la API de TogetherAI.
#' TogetherAI proporciona acceso a modelos open-source de alta calidad como
#' Llama, Qwen, Mistral, DeepSeek y muchos otros. Soporta JSON mode para
#' respuestas estructuradas. La API es compatible con el formato de OpenAI,
#' lo que facilita la integracion.
#'
#' @param texto Texto a analizar. Puede ser una noticia, tweet, documento, etc.
#' @param instrucciones Instrucciones en lenguaje natural que indican al modelo que hacer
#'   con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".
#' @param modelo Modelo a utilizar. Ejemplos populares:
#'   - Moonshot: `"moonshotai/Kimi-K2-Instruct-0905"` (128K context)
#'   - Meta Llama: `"meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo"`, `"meta-llama/Llama-3.3-70B-Instruct-Turbo"`
#'   - Qwen: `"Qwen/Qwen2.5-72B-Instruct-Turbo"`, `"Qwen/QwQ-32B-Preview"`
#'   - Mistral: `"mistralai/Mixtral-8x22B-Instruct-v0.1"`, `"mistralai/Mistral-7B-Instruct-v0.3"`
#'   - DeepSeek: `"deepseek-ai/DeepSeek-V3"`, `"deepseek-ai/DeepSeek-R1"`
#'   - Google: `"google/gemma-2-27b-it"`, `"google/gemma-2-9b-it"`
#'   Por defecto: `"meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo"`.
#'   Ver lista completa: https://docs.together.ai/docs/chat-models
#' @param api_key Clave de API de TogetherAI. Si no se proporciona, busca la variable de
#'   entorno `TOGETHER_API_KEY`. Para obtener una clave: https://api.together.xyz/settings/api-keys
#' @param schema Esquema JSON que define la estructura de la respuesta. Puede usar
#'   `acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
#'   Si es `NULL`, usa un esquema simple con campo "respuesta".
#'   NOTA: TogetherAI soporta JSON mode con \code{response_format: \{type: "json_object"\}} para
#'   modelos compatibles. Consulta la lista de modelos soportados en:
#'   https://docs.together.ai/docs/json-mode
#' @param parse_json Logico. Si `TRUE` (por defecto), parsea automaticamente el JSON
#'   a un objeto R (lista o data frame). Si `FALSE`, devuelve el JSON como string.
#' @param temperature Parametro de temperatura (0-2). Valores bajos (0-0.3) generan
#'   respuestas mas deterministas. Valores altos (0.7-1) mas creativas. Por defecto: 0.
#' @param max_tokens Numero maximo de tokens en la respuesta. Por defecto: 2000.
#' @param top_p Parametro top-p para nucleus sampling (0-1). Por defecto: 0.2.
#' @param top_k Parametro top-k para muestreo. Limita las opciones a los k tokens mas probables.
#'   Por defecto: 50. Usar 0 o -1 para desactivar.
#' @param repetition_penalty Penalizacion por repeticion de tokens (0.1-2.0).
#'   Valores > 1 penalizan repeticiones. Por defecto: 1.
#' @param stop Secuencias de parada opcionales. Vector de strings que detienen la generacion.
#'   Por defecto: NULL.
#' @param prompt_system Prompt del sistema que define el comportamiento del modelo. Opciones:
#'   - `"json"` (por defecto): Usa un prompt estructurado que instruye al modelo a responder
#'     SOLO en formato JSON siguiendo el esquema proporcionado. Agrega \code{response_format: \{type: "json_object"\}}
#'   - `"texto"`: Usa un prompt simple para respuestas en texto plano sin estructura.
#'     Elimina automaticamente el contenido de pensamiento (<think>...</think>) de modelos como Qwen3-Thinking
#'   - String personalizado: Cualquier texto que definas como prompt del sistema
#'
#' @param timeout Tiempo maximo de espera de la peticion HTTP en segundos.
#'   Por defecto: 120. Evita que una conexion estancada bloquee la sesion de R.
#' @return Si `parse_json=TRUE`, devuelve una lista o data frame con la respuesta
#'   estructurada segun el esquema. Si `parse_json=FALSE`, devuelve un string JSON.
#'
#' @details
#' **Sobre TogetherAI:**
#'
#' TogetherAI es una plataforma especializada en modelos open-source que ofrece:
#' - Precios competitivos y modelos gratuitos
#' - Alta velocidad de inferencia optimizada
#' - Acceso a modelos de ultima generacion (Llama, Qwen, DeepSeek, etc.)
#' - API compatible con formato OpenAI
#'
#' **JSON Mode:**
#'
#' La funcion utiliza JSON mode de TogetherAI para obtener respuestas estructuradas.
#' Cuando `prompt_system = "json"`, la funcion:
#' 1. Incluye el esquema JSON en el prompt del sistema (REQUERIDO por TogetherAI)
#' 2. Agrega \code{response_format: \{type: "json_object"\}} al body de la peticion
#' 3. Instruye explicitamente al modelo a responder SOLO en JSON
#'
#' Esta combinacion de esquema textual + \code{response_format} asegura respuestas JSON validas
#' y consistentes en cada llamada.
#'
#' **Modelos compatibles con JSON mode:**
#'
#' Los modelos mas recientes que soportan JSON mode incluyen:
#' - Qwen3, Qwen2.5 (Instruct, Coder, VL, Thinking)
#' - DeepSeek-R1, DeepSeek-V3
#' - Meta Llama 3.1, 3.3, 4
#' - Mistral 7B Instruct
#' - Google Gemma
#'
#' Ver lista completa: https://docs.together.ai/docs/json-mode
#'
#' **Validaciones:**
#'
#' La funcion incluye validacion de limite de tokens. Si la respuesta es truncada
#' por `max_tokens`, devuelve un mensaje claro indicando que se necesitan mas tokens.
#'
#' @export
#' @examples
#' \dontrun{
#' # Configurar API key
#' Sys.setenv(TOGETHER_API_KEY = "tu-api-key")
#'
#' # Usar Llama 3.1 70B (rapido y potente)
#' texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
#' resultado <- acep_together(texto, "Extrae las entidades nombradas",
#'                            modelo = "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
#'                            schema = acep_gpt_schema("extraccion_entidades"))
#'
#' # Usar Qwen para analisis de sentimiento
#' resultado_qwen <- acep_together(texto, "Clasifica el sentimiento",
#'                                 modelo = "Qwen/Qwen2.5-72B-Instruct-Turbo",
#'                                 schema = acep_gpt_schema("sentimiento"))
#'
#' # Usar DeepSeek-V3
#' resultado_ds <- acep_together(texto, "Analiza el texto",
#'                               modelo = "deepseek-ai/DeepSeek-V3",
#'                               schema = acep_gpt_schema("clasificacion"))
#'
#' # Usar Moonshot Kimi con 128K context
#' resultado_kimi <- acep_together(texto, "Resume el texto",
#'                                 modelo = "moonshotai/Kimi-K2-Instruct-0905",
#'                                 schema = acep_gpt_schema("resumen"))
#'
#' # Usar modo texto plano (sin estructura JSON)
#' resultado_texto <- acep_together(texto, "Resume este texto en una frase",
#'                                  modelo = "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
#'                                  prompt_system = "texto",
#'                                  parse_json = FALSE)
#' print(resultado_texto)  # Devuelve string de texto plano
#'
#' # Usar prompt del sistema personalizado
#' resultado_custom <- acep_together(
#'   texto,
#'   "Analiza el sentimiento",
#'   modelo = "Qwen/Qwen2.5-72B-Instruct-Turbo",
#'   prompt_system = paste(
#'     "Eres un experto en analisis de sentimientos politicos.",
#'     "Se objetivo y neutral."
#'   ),
#'   parse_json = FALSE
#' )
#' }
acep_together <- function(texto,
                          instrucciones,
                          modelo = "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                          api_key = Sys.getenv("TOGETHER_API_KEY"),
                          schema = NULL,
                          parse_json = TRUE,
                          temperature = 0,
                          max_tokens = 2000,
                          top_p = 0.2,
                          top_k = 50,
                          repetition_penalty = 1,
                          stop = NULL,
                          prompt_system = "json",
                          timeout = 120) {

  .acep_provider_validate_request_inputs(texto, instrucciones, api_key, "TOGETHER_API_KEY")

  # Construir prompt del sistema segun el tipo especificado
  if (prompt_system == "json") {
    # Modo JSON: Fuerza respuestas estructuradas en JSON

    # Esquema por defecto si no se proporciona uno
    if (is.null(schema)) {
      schema <- .acep_provider_default_schema()
    }

    # Construir descripcion de campos esperados del esquema
    # Extraer nombres y descripciones de los campos del schema
    campos_descripciones <- .acep_provider_schema_field_descriptions(schema)
    campos_texto <- paste(campos_descripciones, collapse = "\n")

    # Prompt del sistema con instrucciones para JSON
    # TogetherAI usa JSON mode basico (type: json_object), no Structured Outputs estrictos
    # NO incluimos el esquema completo para evitar que el modelo lo devuelva como respuesta
    system_prompt <- sprintf(
      "Eres un asistente experto en analisis de texto. Debes responder SIEMPRE en formato JSON valido con los siguientes campos:\n\n%s\n\nSe preciso, conciso y basa tus respuestas unicamente en el texto proporcionado. Responde UNICAMENTE con el JSON de datos, sin texto adicional antes o despues.",
      campos_texto
    )

  } else if (prompt_system == "texto") {
    # Modo texto plano: Respuestas sin estructura especifica
    system_prompt <- "Eres un asistente experto en analisis de texto. Responde de manera clara, precisa y concisa. Basa tus respuestas unicamente en el texto proporcionado."

  } else {
    # Prompt personalizado: Usa el string proporcionado directamente
    system_prompt <- prompt_system
  }

  # Construir prompt del usuario
  user_prompt <- .acep_provider_user_prompt(texto, instrucciones)

  # Construir body de la peticion
  body <- list(
    model = modelo,
    messages = list(
      list(role = "system", content = system_prompt),
      list(role = "user", content = user_prompt)
    ),
    temperature = temperature,
    max_tokens = max_tokens,
    top_p = top_p,
    top_k = top_k,
    repetition_penalty = repetition_penalty
  )

  # Agregar response_format para JSON mode solo si prompt_system es "json"
  # TogetherAI soporta response_format segun https://docs.together.ai/docs/json-mode
  if (prompt_system == "json") {
    body$response_format <- list(
      type = "json_object"
    )
  }

  # Agregar secuencias de parada si se especifican
  if (!is.null(stop)) {
    body$stop <- stop
  }

  # Construir headers
  headers_call <- .acep_provider_auth_headers("together", api_key)

  # Realizar peticion a la API
  tryCatch({
    output <- .acep_provider_post(
      url = .acep_provider_endpoint("together"),
      headers = headers_call,
      body = body,
      timeout = timeout
    )

    # Verificar codigo HTTP
    if (httr::status_code(output) != 200) {
      error_content <- tryCatch(httr::content(output, as = "parsed"),
                                error = function(e) NULL)
      error_msg <- .acep_provider_http_error_message(error_content)
      stop(sprintf("Error HTTP %d: %s", httr::status_code(output), error_msg))
    }

    # Extraer respuesta
    respuesta_parsed <- httr::content(output, as = "parsed")
    respuesta_json <- .acep_provider_extract_chat_content(respuesta_parsed)

    # Verificar si la respuesta fue truncada por limite de tokens
    finish_reason <- respuesta_parsed$choices[[1]]$finish_reason
    if (!is.null(finish_reason) && finish_reason == "length") {
      stop("La respuesta fue truncada debido al limite de max_tokens. Se necesitan mas tokens para obtener una respuesta valida. Aumenta el valor de max_tokens.")
    }

    # Procesar respuesta segun el modo
    if (prompt_system == "json") {
      # Modo JSON: Limpiar markdown y parsear
      # Algunos modelos envuelven el JSON en ```json...``` o ```...```
      # Parsear JSON si se solicita
      return(.acep_provider_parse_json_response(respuesta_json, parse_json = parse_json))
    } else {
      # Modo texto plano o personalizado: Limpiar y devolver
      # Eliminar contenido de pensamiento (thinking) si existe
      # Algunos modelos como Qwen3-Thinking generan contenido entre <think>...</think>
      # La etiqueta inicial puede estar o no, pero la de cierre siempre está
      respuesta_limpia <- gsub("(<think>)?[\\s\\S]*?</think>\\s*", "", respuesta_json, perl = TRUE, ignore.case = TRUE)
      respuesta_limpia <- trimws(respuesta_limpia)

      return(respuesta_limpia)
    }

  }, error = function(e) {
    stop(sprintf("Error al interactuar con la API de TogetherAI: %s", conditionMessage(e)))
  })
}

.acep_postag_normalize_parse <- function(texto_tag, doc_id_offset = 0L) {
  texto_tag$morph <- sapply(texto_tag$morph, as.character)
  texto_tag$sent <- sapply(texto_tag$sent, as.character)
  texto_tag$doc_id <- as.integer(gsub("text", "", texto_tag$doc_id)) + doc_id_offset
  texto_tag$sent <- trimws(gsub("\\n+", "", texto_tag$sent))
  texto_tag[texto_tag$sent != "", ]
}

.acep_postag_text_chunks <- function(texto, chunk_size) {
  n_textos <- length(texto)
  n_chunks <- ceiling(n_textos / chunk_size)

  lapply(seq_len(n_chunks), function(i) {
    start_idx <- (i - 1L) * chunk_size + 1L
    end_idx <- min(i * chunk_size, n_textos)

    list(
      start_idx = start_idx,
      end_idx = end_idx,
      texto = texto[start_idx:end_idx]
    )
  })
}

.acep_postag_empty_loc_frame <- function() {
  data.frame(entity_ = character(),
             doc_id = integer(),
             sentence = integer(),
             entity = character(),
             entity_type = character(),
             lat = numeric(),
             long = numeric(),
             stringsAsFactors = FALSE)
}

.acep_postag_prepare_loc_entities <- function(texto_only_entity, texto_tag) {
  texto_only_entity_loc <- unique(texto_only_entity[texto_only_entity$entity_type == "LOC", ])

  if (nrow(texto_only_entity_loc) == 0L) {
    return(.acep_postag_empty_loc_frame())
  }

  texto_only_entity_loc$entity_ <- gsub("_", " ", texto_only_entity_loc$entity)
  texto_only_entity_loc <- merge(texto_only_entity_loc,
                                 unique(texto_tag[, c("doc_id", "sentence")]),
                                 by = c("doc_id", "sentence"))
  unique(texto_only_entity_loc[, c("entity_", "doc_id", "sentence", "entity", "entity_type")])
}

#' @title Etiquetado POS adaptativo con optimizaciones avanzadas
#' @description Version optimizada de acep_postag que se adapta automaticamente al tamano del input.
#' Implementa procesamiento por lotes (chunking) para grandes volumenes, cache de geocodificacion
#' para evitar consultas repetidas, y estrategias de procesamiento adaptativas segun la cantidad
#' de textos. Puede procesar desde 10 hasta millones de textos de forma eficiente.
#' @param texto Vector de caracteres con los textos a procesar.
#' @param core Idioma del modelo de etiquetado POS del paquete \code{spacyr}. Opciones disponibles:
#' 'es_core_news_sm', 'es_core_news_md', 'es_core_news_lg' (espanol),
#' 'pt_core_news_sm', 'pt_core_news_md', 'pt_core_news_lg' (portugues),
#' 'en_core_web_sm', 'en_core_web_md', 'en_core_web_lg', 'en_core_web_trf' (ingles).
#' Default: "es_core_news_lg".
#' @param bajar_core Parametro booleano que define si descargar o no el modelo de etiquetado POS.
#' Default: TRUE.
#' @param inst_spacy Parametro booleano que define si instalar o no spacy (Python).
#' Default: FALSE.
#' @param inst_miniconda Parametro booleano que define si instalar o no miniconda.
#' Default: FALSE.
#' @param inst_reticulate Parametro booleano que define si instalar o no el paquete \code{reticulate}.
#' Default: FALSE.
#' @param chunk_size Tamano de los lotes para procesamiento chunking. Ajustar segun RAM disponible:
#' 500 para sistemas con 2-4 GB RAM, 1000 para 8 GB RAM (default), 2000-5000 para 16+ GB RAM.
#' Default: 1000.
#' @param geocode_cache_file Ruta del archivo JSON para guardar cache de geocodificacion.
#' Permite evitar consultas repetidas a la API de Nominatim y compartir cache entre proyectos.
#' Default: "geocode_cache.json".
#' @param use_cache Parametro booleano que activa/desactiva el sistema de cache de geocodificacion.
#' Desactivar para forzar re-geocodificacion de todas las ubicaciones.
#' Default: TRUE.
#' @param show_progress Parametro booleano que controla la visualizacion de mensajes de progreso
#' durante el procesamiento. Util para operaciones largas.
#' Default: TRUE.
#' @importFrom utils install.packages
#' @importFrom jsonlite read_json write_json
#' @return Lista con seis elementos en formato tabular:
#' \itemize{
#'   \item \code{texto_tag}: Data frame con tokens etiquetados (POS, lemas, dependencias, etc.)
#'   \item \code{texto_tag_entity}: Data frame con entidades nombradas consolidadas
#'   \item \code{texto_only_entity}: Data frame con solo las entidades extraidas
#'   \item \code{texto_only_entity_loc}: Data frame con entidades de tipo LOC geocodificadas (lat/long)
#'   \item \code{texto_nounphrase}: Data frame con frases nominales consolidadas
#'   \item \code{texto_only_nounphrase}: Data frame con solo las frases nominales extraidas
#' }
#' @details
#' La funcion implementa dos estrategias de procesamiento automaticas:
#' \itemize{
#'   \item \strong{Batch Processing} (<= 100 textos): Procesa todos los textos en una sola llamada
#'   para maxima velocidad.
#'   \item \strong{Chunking} (> 100 textos): Divide los textos en lotes del tamano especificado
#'   en \code{chunk_size} para controlar el uso de memoria y permitir procesamiento de grandes volumenes.
#' }
#'
#' El sistema de cache de geocodificacion guarda las coordenadas de ubicaciones ya consultadas
#' en formato JSON, evitando consultas repetidas a la API de Nominatim (que tiene limite de 1 req/seg).
#' Esto puede reducir el tiempo de procesamiento en 50-90% en ejecuciones posteriores con ubicaciones repetidas.
#'
#' Para datasets muy grandes (>100,000 textos), se recomienda procesar en lotes usando la funcion
#' auxiliar proporcionada en los ejemplos y guardar resultados incrementalmente.
#' @keywords etiquetado optimizacion chunking cache
#' @examples
#' \dontrun{
#' # Ejemplo basico con pocos textos
#' textos <- c(
#'   "En Mar del Plata el SOIP declara la huelga en demanda de aumento salarial.",
#'   "La manifestacion se realizo en Buenos Aires el 15 de marzo.",
#'   "El presidente visito Cordoba para inaugurar la nueva planta."
#' )
#' resultado <- acep_postag(texto = textos, bajar_core = FALSE)
#' head(resultado$texto_tag)
#'
#' # Ejemplo con dataset mediano y configuracion personalizada
#' resultado <- acep_postag(
#'   texto = mis_1000_textos,
#'   bajar_core = FALSE,
#'   chunk_size = 500,
#'   geocode_cache_file = "cache/ubicaciones_argentina.json",
#'   use_cache = TRUE
#' )
#'
#' # Ver ubicaciones geocodificadas
#' head(resultado$texto_only_entity_loc)
#'
#' # Procesamiento incremental para datasets muy grandes
#' procesar_incremental <- function(textos, batch_size = 10000) {
#'   dir.create("resultados", showWarnings = FALSE)
#'   n_batches <- ceiling(length(textos) / batch_size)
#'
#'   for (i in 1:n_batches) {
#'     start_idx <- (i - 1) * batch_size + 1
#'     end_idx <- min(i * batch_size, length(textos))
#'     batch <- textos[start_idx:end_idx]
#'
#'     resultado <- acep_postag(
#'       texto = batch,
#'       bajar_core = FALSE,
#'       chunk_size = 2000,
#'       use_cache = TRUE,
#'       geocode_cache_file = "cache_global.json"
#'     )
#'
#'     saveRDS(resultado, sprintf("resultados/batch_%04d.rds", i))
#'     message(sprintf("Batch %d/%d completado", i, n_batches))
#'   }
#' }
#'
#' # Usar funcion incremental
#' procesar_incremental(mis_millones_de_textos, batch_size = 10000)
#'
#' # Ver contenido del cache
#' cache <- jsonlite::read_json("geocode_cache.json", simplifyVector = TRUE)
#' print(paste("Ubicaciones en cache:", nrow(cache)))
#' }
#' @export
acep_postag <- function(texto,
                        core = "es_core_news_lg",
                        bajar_core = TRUE,
                        inst_spacy = FALSE,
                        inst_miniconda = FALSE,
                        inst_reticulate = FALSE,
                        chunk_size = 1000,
                        geocode_cache_file = "geocode_cache.json",
                        use_cache = TRUE,
                        show_progress = TRUE) {
  
  # Validaciones de parametros
  if (!is.character(texto)) {
    stop("El parametro 'texto' debe ser un vector de caracteres")
  }
  if (!is.logical(inst_reticulate)) {
    stop("El parametro 'inst_reticulate' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.logical(inst_miniconda)) {
    stop("El parametro 'inst_miniconda' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.logical(inst_spacy)) {
    stop("El parametro 'inst_spacy' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.logical(bajar_core)) {
    stop("El parametro 'bajar_core' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.logical(use_cache)) {
    stop("El parametro 'use_cache' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.logical(show_progress)) {
    stop("El parametro 'show_progress' debe ser un valor booleano: TRUE o FALSE")
  }
  if (!is.numeric(chunk_size) || chunk_size < 1) {
    stop("El parametro 'chunk_size' debe ser un numero entero positivo")
  }
  acep_require_namespace("spacyr", "acep_postag")
  acep_require_namespace("rsyntax", "acep_postag")
  acep_require_namespace("tidygeocoder", "acep_postag")
  if (inst_miniconda) {
    acep_require_namespace("reticulate", "acep_postag")
  }
  
  available_models <- c('es_core_news_sm','es_core_news_md','es_core_news_lg',
                        'pt_core_news_sm','pt_core_news_md','pt_core_news_lg',
                        'en_core_web_sm','en_core_web_md','en_core_web_lg','en_core_web_trf')
  
  if (!core %in% available_models) {
    stop(paste("El parametro 'core' debe ser un modelo valido del espanol, ingles o portugues:",
               paste0(available_models, collapse = ", ")))
  }
  
  # Instalaciones condicionales
  if (inst_reticulate) {
    utils::install.packages("reticulate")
  }
  if (inst_miniconda) {
    reticulate::install_miniconda()
  }
  if (inst_spacy) {
    spacyr::spacy_install()
  }
  if (bajar_core) {
    spacyr::spacy_download_langmodel(core)
  }
  
  # Inicializar spaCy
  spacyr::spacy_initialize(model = core)
  
  # Determinar estrategia de procesamiento
  n_textos <- length(texto)

  if (show_progress) {
    message(sprintf("Procesando %d textos...", n_textos))
  }

  # Estrategia 1: Batch processing para pocos textos (<= 100)
  if (n_textos <= 100) {
    if (show_progress) {
      message("Usando batch processing (todos los textos juntos)")
    }
    
    texto_tag <- suppressWarnings({
      spacyr::spacy_parse(texto,
                          pos = TRUE,
                          tag = FALSE,
                          lemma = TRUE,
                          entity = TRUE,
                          dependency = TRUE,
                          nounphrase = TRUE,
                          multithread = TRUE,
                          additional_attributes = c("is_upper", "is_title", "is_quote",
                                                   "ent_iob_","ent_iob", "is_left_punct",
                                                   "is_right_punct", "morph", "sent"))
    })
    
    texto_tag <- .acep_postag_normalize_parse(texto_tag)
    
  } else {
    # Estrategia 2: Chunking para muchos textos (> 100)
    if (show_progress) {
      message(sprintf("Usando chunking (lotes de %d textos)", chunk_size))
    }
    
    texto_chunks <- .acep_postag_text_chunks(texto, chunk_size)
    texto_tag_list <- vector("list", length(texto_chunks))
    
    for (i in seq_along(texto_chunks)) {
      chunk <- texto_chunks[[i]]
      
      if (show_progress) {
        message(sprintf("Procesando chunk %d/%d (textos %d-%d)", i, length(texto_chunks), chunk$start_idx, chunk$end_idx))
      }
      
      chunk_tag <- suppressWarnings({
        spacyr::spacy_parse(chunk$texto,
                            pos = TRUE,
                            tag = FALSE,
                            lemma = TRUE,
                            entity = TRUE,
                            dependency = TRUE,
                            nounphrase = TRUE,
                            multithread = TRUE,
                            additional_attributes = c("is_upper", "is_title", "is_quote",
                                                     "ent_iob_","ent_iob", "is_left_punct",
                                                     "is_right_punct", "morph", "sent"))
      })
      
      chunk_tag <- .acep_postag_normalize_parse(chunk_tag, chunk$start_idx - 1L)
      
      texto_tag_list[[i]] <- chunk_tag
    }
    
    # Combinar todos los chunks
    texto_tag <- do.call(rbind, texto_tag_list)
  }
  
  # Procesamiento de entidades y frases nominales
  if (show_progress) {
    message("Consolidando entidades y frases nominales...")
  }
  
  texto_tag_entity <- spacyr::entity_consolidate(texto_tag)
  texto_only_entity <- spacyr::entity_extract(texto_tag, type = "all")
  texto_nounphrase <- spacyr::nounphrase_consolidate(texto_tag)
  texto_only_nounphrase <- spacyr::nounphrase_extract(texto_tag)
  
  # Finalizar spaCy
  spacyr::spacy_finalize()
  
  # Convertir a tokenindex
  texto_tag <- rsyntax::as_tokenindex(texto_tag)
  
  # Geocodificacion con cache
  texto_only_entity_loc <- .acep_postag_prepare_loc_entities(texto_only_entity, texto_tag)

  if (nrow(texto_only_entity_loc) > 0) {
    unique_locations <- unique(texto_only_entity_loc$entity_)

    if (show_progress) {
      message(sprintf("Geocodificando %d ubicaciones unicas...", length(unique_locations)))
    }

    # Cargar cache existente si esta habilitado
    if (use_cache && file.exists(geocode_cache_file)) {
      if (show_progress) {
        message(sprintf("Cargando cache desde: %s", geocode_cache_file))
      }
      cached_geocoder <- jsonlite::read_json(geocode_cache_file, simplifyVector = TRUE)
    } else {
      cached_geocoder <- data.frame(entity_ = character(),
                                     lat = numeric(),
                                     long = numeric(),
                                     stringsAsFactors = FALSE)
    }

    # Identificar ubicaciones que necesitan geocodificacion
    new_locations <- setdiff(unique_locations, cached_geocoder$entity_)

    if (length(new_locations) > 0) {
      if (show_progress) {
        message(sprintf("Geocodificando %d ubicaciones nuevas (usando cache para %d)...",
                       length(new_locations),
                       length(unique_locations) - length(new_locations)))
      }

      # Geocodificar solo ubicaciones nuevas
      new_geocoder <- tidygeocoder::geo(new_locations, method = "osm")
      names(new_geocoder) <- c("entity_", "lat", "long")

      # Combinar con cache
      cached_geocoder <- rbind(cached_geocoder, new_geocoder)

      # Guardar cache actualizado
      if (use_cache) {
        jsonlite::write_json(cached_geocoder, geocode_cache_file, pretty = TRUE)
        if (show_progress) {
          message(sprintf("Cache actualizado: %s", geocode_cache_file))
        }
      }
    } else {
      if (show_progress) {
        message("Todas las ubicaciones encontradas en cache")
      }
    }

    # Usar cache para merge
    texto_geocoder <- cached_geocoder[cached_geocoder$entity_ %in% unique_locations, ]
    
    # Merge con entidades
    texto_only_entity_loc <- merge(texto_only_entity_loc, texto_geocoder, 
                                   by = "entity_", all.x = TRUE)
    names(texto_only_entity_loc) <- c("entity_", "doc_id", "sentence", "entity", 
                                      "entity_type", "lat", "long")
    
    texto_only_entity_loc <- unique(texto_only_entity_loc[!is.na(texto_only_entity_loc$lat), ])
  } else {
    texto_only_entity_loc <- .acep_postag_empty_loc_frame()
  }
  
  if (show_progress) {
    message("Procesamiento completado")
  }
  
  # Retornar lista de resultados
  return(list(
    texto_tag = texto_tag,
    texto_tag_entity = texto_tag_entity,
    texto_only_entity = texto_only_entity,
    texto_only_entity_loc = texto_only_entity_loc,
    texto_nounphrase = texto_nounphrase,
    texto_only_nounphrase = texto_only_nounphrase
  ))
}

#' @title Interaccion con modelos Ollama locales y cloud usando Structured Outputs
#' @description
#' Funcion para interactuar con modelos de lenguaje usando Ollama.
#' Soporta tanto modelos locales (ejecutados en tu computadora sin costos)
#' como modelos cloud de Ollama (modelos grandes como DeepSeek 671B, Qwen3 Coder 480B,
#' Kimi 1T que se ejecutan en la nube sin necesidad de GPU local).
#' Utiliza structured outputs para garantizar respuestas en formato JSON que
#' cumplen con un esquema predefinido.
#'
#' @param texto Texto a analizar. Puede ser una noticia, tweet, documento, etc.
#' @param instrucciones Instrucciones en lenguaje natural que indican al modelo que hacer
#'   con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".
#' @param modelo Modelo de Ollama a utilizar.
#'   - Para Ollama local: "qwen3:1.7b", "llama3.2:latest", "mistral", "phi3", "gemma2"
#'     (debe estar previamente descargado con `ollama pull nombre_modelo`)
#'   - Para Ollama Cloud API: modelos cloud especificos disponibles sin GPU local:
#'     "deepseek-v3.1:671b-cloud", "gpt-oss:20b-cloud", "gpt-oss:120b-cloud",
#'     "kimi-k2:1t-cloud", "qwen3-coder:480b-cloud", "glm-4.6:cloud", "minimax-m2:cloud"
#'   Por defecto: "qwen3:1.7b"
#' @param schema Esquema JSON que define la estructura de la respuesta. Puede usar
#'   `acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
#'   Si es NULL, usa un esquema simple con campo "respuesta".
#' @param parse_json Logico. Si TRUE (por defecto), parsea automaticamente el JSON
#'   a un objeto R (lista o data frame). Si FALSE, devuelve el JSON como string.
#' @param temperature Parametro de temperatura (0-2). Valores bajos (0-0.3) generan
#'   respuestas mas deterministas. Valores altos (0.7-1) mas creativas. Por defecto: 0.
#' @param max_tokens Numero maximo de tokens en la respuesta. Por defecto: 4000.
#' @param host URL del servidor Ollama. Por defecto: "http://localhost:11434" para uso local.
#'   Para usar Ollama Cloud API, especificar "https://ollama.com" (sin /api, se agrega automaticamente).
#' @param api_key API key para Ollama API remota. Solo requerido si usas un servidor remoto.
#'   Por defecto busca la variable de entorno OLLAMA_API_KEY. Para uso local (localhost) no es necesario.
#' @param seed Semilla numerica para reproducibilidad. Por defecto: 123456.
#'
#' @return Si parse_json=TRUE, devuelve una lista o data frame con la respuesta
#'   estructurada segun el esquema. Si parse_json=FALSE, devuelve un string JSON.
#'
#' @export
#' @examples
#' \dontrun{
#' # Primero, instalar Ollama y descargar un modelo:
#' # Terminal: ollama pull llama3.1
#'
#' # Extraer entidades de un texto
#' texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
#' instrucciones <- "Extrae todas las entidades nombradas del texto."
#' schema <- acep_gpt_schema("extraccion_entidades")
#' resultado <- acep_ollama(texto, instrucciones, schema = schema)
#' print(resultado)
#'
#' # Analisis de sentimiento
#' texto <- "La protesta fue pacifica y bien organizada."
#' schema <- acep_gpt_schema("sentimiento")
#' resultado <- acep_ollama(texto, "Analiza el sentimiento del texto", schema = schema)
#' print(resultado$sentimiento_general)
#'
#' # Usar Ollama Cloud API (requiere API key)
#' # Los modelos cloud se ejecutan sin necesidad de GPU local
#' Sys.setenv(OLLAMA_API_KEY = "tu-api-key")
#' resultado_remoto <- acep_ollama(
#'   texto = texto,
#'   instrucciones = "Extrae entidades",
#'   modelo = "deepseek-v3.1:671b-cloud",  # Modelo cloud de 671B parametros
#'   host = "https://ollama.com",
#'   schema = acep_gpt_schema("extraccion_entidades")
#' )
#'
#' }

acep_ollama <- function(texto,
                        instrucciones,
                        modelo = "qwen3:1.7b",
                        schema = NULL,
                        parse_json = TRUE,
                        temperature = 0,
                        max_tokens = 4000,
                        host = "http://localhost:11434",
                        api_key = Sys.getenv("OLLAMA_API_KEY"),
                        seed = 123456) {

  # Validaciones
  if (!is.character(texto) || nchar(texto) == 0) {
    stop("El parametro 'texto' debe ser una cadena de caracteres no vacia")
  }
  if (!is.character(instrucciones) || nchar(instrucciones) == 0) {
    stop("El parametro 'instrucciones' debe ser una cadena de caracteres no vacia")
  }

  # Detectar si es servidor remoto o local
  is_remote <- !grepl("localhost|127\\.0\\.0\\.1", host, ignore.case = TRUE)

  # Si es servidor remoto, verificar API key
  if (is_remote && (is.null(api_key) || api_key == "")) {
    stop(paste0(
      "Para usar Ollama API remota (", host, ") necesitas proporcionar un api_key. ",
      "Define la variable de entorno OLLAMA_API_KEY o pasa el parametro api_key."
    ))
  }

  # Verificar que Ollama este corriendo (solo para localhost)
  if (!is_remote) {
    tryCatch({
      test_response <- httr::GET(paste0(host, "/api/tags"), httr::timeout(5))
      if (httr::status_code(test_response) != 200) {
        stop("Ollama no responde")
      }
    }, error = function(e) {
      stop(paste0(
        "No se puede conectar a Ollama en ", host, ". ",
        "Asegurate de que Ollama este corriendo. ",
        "Puedes iniciarlo con 'ollama serve' en la terminal."
      ))
    })
  }

  # Esquema por defecto si no se proporciona uno
  if (is.null(schema)) {
    schema <- list(
      type = "object",
      properties = list(
        respuesta = list(
          type = "string",
          description = "Respuesta principal a la pregunta o instruccion"
        )
      ),
      required = c("respuesta"),
      additionalProperties = FALSE
    )
  }

  # Construir prompt del sistema
  system_prompt <- ACEP::acep_prompt_gpt$system_prompt_01_es

  # Construir prompt del usuario con el esquema
  user_prompt <- sprintf(
    "Texto a analizar:\n%s\n\nInstrucciones:\n%s\n\nDebes responder en formato JSON siguiendo este esquema:\n%s",
    texto,
    instrucciones,
    jsonlite::toJSON(schema, auto_unbox = TRUE, pretty = TRUE)
  )

  # Realizar peticion a Ollama con structured output
  tryCatch({
    if (is_remote) {
      # Para servidores remotos, usar httr directamente con endpoint /api/chat
      api_url <- paste0(host, "/api/chat")

      # Construir body de la peticion siguiendo formato Ollama chat
      body <- list(
        model = modelo,
        messages = list(
          list(role = "system", content = system_prompt),
          list(role = "user", content = user_prompt)
        ),
        format = schema,
        stream = FALSE,
        options = list(
          temperature = temperature,
          seed = seed,
          num_predict = max_tokens
        )
      )

      # Realizar peticion con o sin autenticacion
      if (!is.null(api_key) && api_key != "") {
        # Con API key (Ollama Cloud)
        output <- httr::POST(
          url = api_url,
          body = jsonlite::toJSON(body, auto_unbox = TRUE),
          httr::add_headers(
            "Content-Type" = "application/json",
            "Authorization" = paste("Bearer", api_key)
          ),
          httr::timeout(120)
        )
      } else {
        # Sin API key (servidor remoto sin auth)
        output <- httr::POST(
          url = api_url,
          body = jsonlite::toJSON(body, auto_unbox = TRUE),
          httr::content_type_json(),
          httr::timeout(120)
        )
      }

      # Verificar codigo HTTP
      if (httr::status_code(output) != 200) {
        error_content <- httr::content(output, as = "text", encoding = "UTF-8")
        stop(sprintf("Error HTTP %d: %s", httr::status_code(output), error_content))
      }

      # Extraer respuesta
      contenido <- httr::content(output, as = "text", encoding = "UTF-8")
      respuesta_parsed <- jsonlite::fromJSON(contenido)

      # Verificar si la respuesta fue truncada por limite de tokens
      # Ollama usa done_reason: "length" cuando se alcanza el límite
      if (!is.null(respuesta_parsed$done_reason) && respuesta_parsed$done_reason == "length") {
        stop("La respuesta fue truncada debido al limite de max_tokens. Se necesitan mas tokens para obtener una respuesta valida. Aumenta el valor de max_tokens.")
      }

      # La respuesta de Ollama chat API viene en message.content
      # Nota: algunos modelos razonadores (como qwen3) pueden tener "content" vacío
      # y el JSON en "thinking"
      respuesta_json <- respuesta_parsed$message$content

      # Si response está vacío pero hay thinking, usar thinking
      if ((is.null(respuesta_json) || nchar(respuesta_json) == 0) && !is.null(respuesta_parsed$message$thinking)) {
        respuesta_json <- respuesta_parsed$message$thinking
      }

      # Verificar respuesta vacia
      if (is.null(respuesta_json) || nchar(respuesta_json) == 0) {
        stop("Ollama devolvio una respuesta vacia. Verifica tu prompt y esquema.")
      }

      # Limpiar JSON de markdown si es necesario
      # Algunos modelos envuelven el JSON en ```json...``` o ```...```
      respuesta_json <- gsub("```json|```", "", respuesta_json)
      respuesta_json <- trimws(respuesta_json)

      # Parsear JSON si se solicita
      if (parse_json) {
        respuesta <- jsonlite::fromJSON(respuesta_json, simplifyVector = TRUE)
        return(respuesta)
      } else {
        return(respuesta_json)
      }

    } else {
      # Para servidores locales, usar httr directamente (mismo comportamiento que remoto)
      api_url <- paste0(host, "/api/chat")

      # Construir body de la peticion siguiendo formato Ollama chat
      body <- list(
        model = modelo,
        messages = list(
          list(role = "system", content = system_prompt),
          list(role = "user", content = user_prompt)
        ),
        format = schema,
        stream = FALSE,
        options = list(
          temperature = temperature,
          seed = seed,
          num_predict = max_tokens
        )
      )

      # Realizar peticion local sin autenticacion
      output <- httr::POST(
        url = api_url,
        body = jsonlite::toJSON(body, auto_unbox = TRUE),
        httr::content_type_json(),
        httr::timeout(120)
      )

      # Verificar codigo HTTP
      if (httr::status_code(output) != 200) {
        error_content <- httr::content(output, as = "text", encoding = "UTF-8")
        stop(sprintf("Error HTTP %d: %s", httr::status_code(output), error_content))
      }

      # Extraer respuesta (misma lógica que para servidores remotos)
      contenido <- httr::content(output, as = "text", encoding = "UTF-8")
      respuesta_parsed <- jsonlite::fromJSON(contenido)

      # Verificar si la respuesta fue truncada por limite de tokens
      # Ollama usa done_reason: "length" cuando se alcanza el límite
      if (!is.null(respuesta_parsed$done_reason) && respuesta_parsed$done_reason == "length") {
        stop("La respuesta fue truncada debido al limite de max_tokens. Se necesitan mas tokens para obtener una respuesta valida. Aumenta el valor de max_tokens.")
      }

      # La respuesta de Ollama chat API viene en message.content
      # Nota: algunos modelos razonadores (como qwen3) pueden tener "content" vacío
      # y el JSON en "thinking"
      respuesta_json <- respuesta_parsed$message$content

      # Si response está vacío pero hay thinking, usar thinking
      if ((is.null(respuesta_json) || nchar(respuesta_json) == 0) && !is.null(respuesta_parsed$message$thinking)) {
        respuesta_json <- respuesta_parsed$message$thinking
      }

      # Verificar respuesta vacia
      if (is.null(respuesta_json) || nchar(respuesta_json) == 0) {
        stop("Ollama devolvio una respuesta vacia. Verifica tu prompt y esquema.")
      }

      # Limpiar JSON de markdown si es necesario
      # Algunos modelos envuelven el JSON en ```json...``` o ```...```
      respuesta_json <- gsub("```json|```", "", respuesta_json)
      respuesta_json <- trimws(respuesta_json)

      # Parsear JSON si se solicita
      if (parse_json) {
        respuesta <- jsonlite::fromJSON(respuesta_json, simplifyVector = TRUE)
        return(respuesta)
      } else {
        return(respuesta_json)
      }
    }

  }, error = function(e) {
    stop(sprintf("Error al interactuar con Ollama: %s", conditionMessage(e)))
  })
}

#' @title Guia de instalacion y uso de Ollama
#' @description
#' Imprime instrucciones para instalar y configurar Ollama en tu sistema.
#'
#' @export
acep_ollama_setup <- function() {
  cat("=== GUIA DE INSTALACION DE OLLAMA ===\n\n")

  cat("1. INSTALAR OLLAMA:\n")
  cat("   - Linux/Mac: curl -fsSL https://ollama.com/install.sh | sh\n")
  cat("   - Windows: Descargar desde https://ollama.com/download\n\n")

  cat("2. INICIAR OLLAMA:\n")
  cat("   - Terminal: ollama serve\n")
  cat("   - (En Windows, Ollama se inicia automaticamente)\n\n")

  cat("3. DESCARGAR MODELOS:\n")
  cat("   - Terminal: ollama pull llama3.1\n")
  cat("   - Otros modelos: llama3.2, mistral, phi3, gemma2\n")
  cat("   - Ver todos: https://ollama.com/search \n\n")

  cat("4. VERIFICAR INSTALACION:\n")
  cat("   - Terminal: ollama list  # Debe mostrar los modelos descargados\n")
  cat("   - En R, acep_ollama se conecta automaticamente via httr\n\n")

  cat("5. PAQUETES R NECESARIOS:\n")
  cat("   install.packages(c('httr', 'jsonlite'))  # Ya incluidos con ACEP\n\n")

  cat("6. EJEMPLO DE USO:\n")
  cat('   texto <- "El SUTEBA convoco a un paro en Buenos Aires."\n')
  cat('   schema <- acep_gpt_schema("extraccion_entidades")\n')
  cat('   resultado <- acep_ollama(texto, "Extrae entidades", schema = schema)\n\n')

  invisible(NULL)
}

# Offline validation tests (no network). These paths fire before any HTTP call.
test_that("acep_ollama valida texto e instrucciones no vacios", {
  expect_error(acep_ollama("", "instrucciones"),
               "cadena de caracteres no vacia")
  expect_error(acep_ollama("texto", ""),
               "cadena de caracteres no vacia")
})

test_that("acep_ollama exige api_key para hosts remotos", {
  expect_error(
    acep_ollama("texto", "instrucciones", host = "https://ollama.com", api_key = ""),
    "api_key"
  )
})

#' @title Interaccion con modelos Claude usando Structured Outputs
#' @description
#' Funcion para interactuar con la API de Anthropic Claude utilizando Tool Calling
#' para garantizar respuestas en formato JSON que cumplen estrictamente con un esquema
#' predefinido. A diferencia de OpenAI, Anthropic utiliza "forced tool use" para
#' lograr structured outputs, definiendo el esquema deseado como input_schema de una
#' herramienta y forzando al modelo a usarla. Compatible con todos los modelos Claude.
#'
#' @param texto Texto a analizar con Claude. Puede ser una noticia, tweet, documento, etc.
#' @param instrucciones Instrucciones en lenguaje natural que indican al modelo que hacer
#'   con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".
#' @param modelo Modelo de Claude a utilizar. Modelos disponibles (ordenados por potencia):
#'   - Claude 4.5: `"claude-sonnet-4-5-20250929"` (mas reciente y potente),
#'     `"claude-haiku-4-5-20251001"` (rapido)
#'   - Claude 4.1: `"claude-opus-4-1-20250805"` (razonamiento excepcional)
#'   - Claude 4: `"claude-opus-4-20250514"`, `"claude-sonnet-4-20250514"`
#'   - Claude 3.7: `"claude-3-7-sonnet-20250219"`
#'   - Claude 3.5: `"claude-3-5-haiku-20241022"` (rapido y economico)
#'   - Claude 3: `"claude-3-opus-20240229"`, `"claude-3-haiku-20240307"`
#'   Por defecto: `"claude-sonnet-4-20250514"`. Ver: https://docs.anthropic.com/en/docs/about-claude/models
#' @param api_key Clave de API de Anthropic. Si no se proporciona, busca la variable de
#'   entorno `ANTHROPIC_API_KEY`. Para obtener una clave: https://console.anthropic.com/
#' @param schema Esquema JSON que define la estructura de la respuesta. Puede usar
#'   `acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
#'   Si es `NULL`, usa un esquema simple con campo "respuesta".
#' @param parse_json Logico. Si `TRUE` (por defecto), parsea automaticamente el JSON
#'   a un objeto R (lista o data frame). Si `FALSE`, devuelve el JSON como string.
#' @param temperature Parametro de temperatura (0-1). Valores bajos (0-0.3) generan
#'   respuestas mas deterministas y consistentes. Valores altos (0.7-1) mas creativas.
#'   Por defecto: 0 (maxima determinismo).
#'   NOTA: Anthropic no permite usar `temperature` y `top_p` simultaneamente.
#' @param max_tokens Numero maximo de tokens en la respuesta. Por defecto: 2000.
#' @param top_p Parametro top-p para nucleus sampling (0-1). Controla la diversidad
#'   de la respuesta. Por defecto: 0.2.
#'   NOTA: Solo se usa si `temperature` es 0 (valor por defecto).
#' @param top_k Parametro top-k (solo disponible en Claude). Limita la seleccion a los
#'   K tokens mas probables. Por defecto: NULL (deshabilitado).
#'
#' @param timeout Tiempo maximo de espera de la peticion HTTP en segundos.
#'   Por defecto: 120. Evita que una conexion estancada bloquee la sesion de R.
#' @return Si `parse_json=TRUE`, devuelve una lista o data frame con la respuesta
#'   estructurada segun el esquema. Si `parse_json=FALSE`, devuelve un string JSON.
#'
#' @details
#' **Diferencias importantes entre modelos Claude:**
#'
#' - **Claude Sonnet 4.5** (`claude-sonnet-4-5`): NO permite `temperature` y `top_p`
#'   simultaneamente. La funcion solo envia uno de los dos si fue modificado del default.
#'
#' - **Otros modelos Claude** (`claude-sonnet-4-20250514`, `claude-3-5-haiku-20241022`,
#'   `claude-3-opus-20240229`, etc.): SI permiten ambos parametros simultaneamente.
#'
#' La funcion detecta automaticamente el modelo y aplica la logica correcta.
#'
#' @export
#' @examples
#' \dontrun{
#' # Extraer entidades de un texto
#' texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
#' instrucciones <- "Extrae todas las entidades nombradas del texto."
#' schema <- acep_gpt_schema("extraccion_entidades")
#' resultado <- acep_claude(texto, instrucciones, schema = schema)
#' print(resultado)
#'
#' # Analisis de sentimiento
#' texto <- "La protesta fue pacifica y bien organizada."
#' schema <- acep_gpt_schema("sentimiento")
#' resultado <- acep_claude(texto, "Analiza el sentimiento del texto", schema = schema)
#' print(resultado$sentimiento_general)
#'
#' # Clasificar noticia
#' texto <- "Trabajadores reclamaron mejoras salariales."
#' schema <- acep_gpt_schema("clasificacion")
#' resultado <- acep_claude(texto, "Clasifica esta noticia", schema = schema)
#' print(resultado$categoria)
#' }
acep_claude <- function(texto,
                        instrucciones,
                        modelo = "claude-sonnet-4-20250514",
                        api_key = Sys.getenv("ANTHROPIC_API_KEY"),
                        schema = NULL,
                        parse_json = TRUE,
                        temperature = 0,
                        max_tokens = 2000,
                        top_p = 0.2,
                        top_k = NULL,
                        timeout = 120) {

  .acep_provider_validate_request_inputs(texto, instrucciones, api_key, "ANTHROPIC_API_KEY")

  # Validar modelos compatibles (listado oficial de la API de Anthropic)
  modelos_compatibles <- c(
    # Claude 4.5
    "claude-sonnet-4-5-20250929", "claude-haiku-4-5-20251001",
    # Claude 4.1
    "claude-opus-4-1-20250805",
    # Claude 4
    "claude-opus-4-20250514", "claude-sonnet-4-20250514",
    # Claude 3.7
    "claude-3-7-sonnet-20250219",
    # Claude 3.5
    "claude-3-5-haiku-20241022",
    # Claude 3
    "claude-3-opus-20240229", "claude-3-haiku-20240307"
  )

  # Verificar compatibilidad con patrones
  es_compatible <- modelo %in% modelos_compatibles ||
                   grepl("^claude-(sonnet|opus|haiku)-4", modelo) ||
                   grepl("^claude-3-", modelo)

  if (!es_compatible) {
    warning(sprintf("El modelo '%s' puede no ser compatible. Modelos recomendados: claude-sonnet-4-5-20250929, claude-opus-4-1-20250805, claude-3-5-haiku-20241022",
                    modelo))
  }

  # Esquema por defecto si no se proporciona uno
  if (is.null(schema)) {
    schema <- .acep_provider_default_schema()
  }

  # Remover additionalProperties si existe (no es necesario para Anthropic),
  # incluyendo apariciones anidadas dentro de objetos/arrays.
  schema <- .acep_provider_strip_key(schema, "additionalProperties")

  # Construir prompt del sistema
  system_prompt <- "Eres un asistente experto en analisis de texto. Debes responder SIEMPRE usando la herramienta proporcionada siguiendo exactamente el esquema especificado. Se preciso, conciso y basa tus respuestas unicamente en el texto proporcionado."

  # Construir prompt del usuario
  user_prompt <- .acep_provider_user_prompt(texto, instrucciones)

  # Definir la herramienta con el esquema como input_schema
  # Esto es el "truco" de Anthropic para structured outputs
  tool <- list(
    name = "respuesta_estructurada",
    description = "Herramienta para devolver la respuesta estructurada siguiendo el esquema especificado",
    input_schema = schema
  )

  # Construir body de la peticion
  body <- list(
    model = modelo,
    max_tokens = max_tokens,
    system = system_prompt,
    messages = list(
      list(
        role = "user",
        content = user_prompt
      )
    ),
    tools = list(tool),
    # Forzar el uso de la herramienta especifica
    tool_choice = list(
      type = "tool",
      name = "respuesta_estructurada"
    )
  )

  # Anthropic: algunos modelos no permiten temperature y top_p simultaneamente
  # Modelo claude-sonnet-4-5 NO acepta ambos parametros
  # Otros modelos (claude-sonnet-4-20250514, claude-3-x) SI los aceptan

  # Detectar si es Sonnet 4.5 (el unico que tiene restriccion estricta)
  es_sonnet_4_5 <- grepl("^claude-sonnet-4-5", modelo)

  if (es_sonnet_4_5) {
    # Para Sonnet 4.5: solo enviar uno si fue modificado del default
    if (temperature != 0) {
      body$temperature <- temperature
    } else if (top_p != 0.2) {
      body$top_p <- top_p
    }
    # Si ambos son defaults, no enviar ninguno
  } else {
    # Para otros modelos Claude: enviar ambos si fueron especificados
    if (temperature != 0) {
      body$temperature <- temperature
    }
    if (top_p != 0.2) {
      body$top_p <- top_p
    }
  }

  # Agregar top_k solo si se especifica (parametro opcional de Claude)
  if (!is.null(top_k)) {
    body$top_k <- top_k
  }

  # Realizar peticion a la API
  tryCatch({
    output <- .acep_provider_post(
      url = .acep_provider_endpoint("anthropic"),
      headers = .acep_provider_auth_headers("anthropic", api_key),
      body = body,
      timeout = timeout
    )

    # Verificar codigo HTTP
    if (httr::status_code(output) != 200) {
      error_content <- tryCatch(httr::content(output, as = "parsed"),
                                error = function(e) NULL)
      error_msg <- .acep_provider_http_error_message(error_content)
      stop(sprintf("Error HTTP %d: %s", httr::status_code(output), error_msg))
    }

    # Extraer respuesta
    respuesta_parsed <- httr::content(output, as = "parsed")

    # Verificar que hay contenido
    if (is.null(respuesta_parsed$content) || length(respuesta_parsed$content) == 0) {
      stop("La API devolvio una respuesta vacia. Verifica tu prompt y esquema.")
    }

    # Buscar el bloque tool_use en el contenido
    tool_use_block <- NULL
    for (block in respuesta_parsed$content) {
      if (!is.null(block$type) && block$type == "tool_use") {
        tool_use_block <- block
        break
      }
    }

    if (is.null(tool_use_block)) {
      stop("La respuesta no contiene un bloque tool_use. Verifica la configuracion.")
    }

    # Extraer el input de la herramienta (que es nuestro JSON estructurado)
    resultado <- tool_use_block$input

    # Si parse_json es FALSE, convertir a JSON string
    if (!parse_json) {
      return(jsonlite::toJSON(resultado, auto_unbox = TRUE, pretty = TRUE))
    } else {
      # Normalizar la estructura usando jsonlite para evitar anidamiento extra
      # Anthropic devuelve arrays con anidamiento adicional que necesitamos simplificar
      resultado_json <- jsonlite::toJSON(resultado, auto_unbox = TRUE)
      resultado_normalizado <- jsonlite::fromJSON(resultado_json, simplifyVector = TRUE)
      return(resultado_normalizado)
    }

  }, error = function(e) {
    stop(sprintf("Error al interactuar con la API de Anthropic: %s", conditionMessage(e)))
  })
}

#' Eliminar recursivamente una clave de un esquema con estructura anidada
#'
#' Recorre el esquema (objetos anidados, items de arrays) y elimina todas las
#' apariciones de `key`, no solo la del nivel raiz. Util para proveedores cuyo
#' formato de esquema no admite ciertas claves (por ejemplo `additionalProperties`
#' en el responseSchema de Gemini).
#' @keywords internal
.acep_provider_strip_key <- function(schema, key) {
  if (is.list(schema)) {
    schema[[key]] <- NULL
    schema <- lapply(schema, .acep_provider_strip_key, key = key)
  }
  schema
}

#' @keywords internal
proteger_arrays_schema <- function(schema) {
  if (is.list(schema)) {
    if ("required" %in% names(schema)) {
      schema$required <- I(schema$required)
    }
    if ("enum" %in% names(schema)) {
      schema$enum <- I(schema$enum)
    }
    schema <- lapply(schema, proteger_arrays_schema)
  }
  schema
}

.acep_provider_validate_request_inputs <- function(texto, instrucciones, api_key, api_key_env) {
  if (!is.character(texto) || nchar(texto) == 0) {
    stop("El parametro 'texto' debe ser una cadena de caracteres no vacia")
  }
  if (!is.character(instrucciones) || nchar(instrucciones) == 0) {
    stop("El parametro 'instrucciones' debe ser una cadena de caracteres no vacia")
  }
  if (api_key == "") {
    stop(sprintf(
      "API key no encontrada. Define la variable de entorno %s o pasa el parametro api_key",
      api_key_env
    ))
  }
  invisible(NULL)
}

.acep_provider_default_schema <- function(additional_properties = TRUE, protect_arrays = TRUE) {
  schema <- list(
    type = "object",
    properties = list(
      respuesta = list(
        type = "string",
        description = "Respuesta principal a la pregunta o instruccion"
      )
    ),
    required = c("respuesta")
  )
  if (isTRUE(additional_properties)) {
    schema$additionalProperties <- FALSE
  }
  if (isTRUE(protect_arrays)) {
    schema <- proteger_arrays_schema(schema)
  }
  schema
}

.acep_provider_user_prompt <- function(texto, instrucciones) {
  sprintf("Texto a analizar:\n%s\n\nInstrucciones:\n%s", texto, instrucciones)
}

.acep_provider_schema_field_descriptions <- function(schema) {
  vapply(names(schema$properties), function(campo) {
    desc <- schema$properties[[campo]]$description
    if (!is.null(desc)) {
      sprintf("- %s: %s", campo, desc)
    } else {
      sprintf("- %s", campo)
    }
  }, character(1))
}

.acep_provider_endpoint <- function(provider, modelo = NULL) {
  switch(
    provider,
    openai = "https://api.openai.com/v1/chat/completions",
    anthropic = "https://api.anthropic.com/v1/messages",
    gemini = sprintf(
      "https://generativelanguage.googleapis.com/v1beta/models/%s:generateContent",
      modelo
    ),
    together = "https://api.together.xyz/v1/chat/completions",
    openrouter = "https://openrouter.ai/api/v1/chat/completions",
    stop(sprintf("Proveedor no soportado: %s", provider), call. = FALSE)
  )
}

.acep_openai_token_limit_field <- function(modelo) {
  if (grepl("^gpt-5", modelo) || grepl("^o1", modelo) || grepl("^o4", modelo)) {
    "max_completion_tokens"
  } else {
    "max_tokens"
  }
}

.acep_openrouter_model_supports_structured_outputs <- function(modelo_slug) {
  grepl("^openai/gpt-4o", modelo_slug) ||
    grepl("^openai/gpt-5", modelo_slug) ||
    grepl("^openai/o1", modelo_slug) ||
    grepl("^openai/o4", modelo_slug) ||
    grepl("^google/gemini", modelo_slug) ||
    grepl("^fireworks/", modelo_slug)
}

.acep_provider_auth_headers <- function(provider, api_key, site_url = NULL, app_name = NULL) {
  headers <- switch(
    provider,
    openai = list("Content-Type" = "application/json", "Authorization" = paste("Bearer", api_key)),
    anthropic = list(
      "Content-Type" = "application/json",
      "x-api-key" = api_key,
      "anthropic-version" = "2023-06-01"
    ),
    gemini = list("Content-Type" = "application/json", "x-goog-api-key" = api_key),
    together = list("Content-Type" = "application/json", "Authorization" = paste("Bearer", api_key)),
    openrouter = list("Content-Type" = "application/json", "Authorization" = paste("Bearer", api_key)),
    stop(sprintf("Proveedor no soportado: %s", provider), call. = FALSE)
  )
  if (provider == "openrouter") {
    if (!is.null(site_url)) {
      headers$`HTTP-Referer` <- site_url
    }
    if (!is.null(app_name)) {
      headers$`X-Title` <- app_name
    }
  }
  headers
}

#' Ejecutar una peticion POST a un proveedor de IA
#'
#' Centraliza la construccion de la peticion HTTP comun a los proveedores en la
#' nube: serializa el cuerpo a JSON con `auto_unbox = TRUE`, arma las cabeceras,
#' aplica un timeout y realiza el POST. El ensamblado del `body` y el parseo de
#' la respuesta siguen siendo especificos de cada proveedor.
#' @keywords internal
.acep_provider_post <- function(url, headers, body, timeout = 120, encode = "raw") {
  httr::POST(
    url = url,
    do.call(httr::add_headers, headers),
    body = jsonlite::toJSON(body, auto_unbox = TRUE, pretty = FALSE),
    encode = encode,
    httr::timeout(timeout)
  )
}

.acep_provider_extract_chat_content <- function(respuesta_parsed) {
  if (is.null(respuesta_parsed$choices) || length(respuesta_parsed$choices) == 0) {
    stop("La API devolvio una respuesta vacia. Verifica tu prompt y esquema.")
  }
  respuesta_json <- respuesta_parsed$choices[[1]]$message$content
  if (is.null(respuesta_json) || nchar(respuesta_json) == 0) {
    stop("La API devolvio una respuesta vacia. Verifica tu prompt y esquema.")
  }
  respuesta_json
}

#' Extraer un mensaje de error legible de una respuesta HTTP de error
#'
#' Maneja de forma robusta los distintos cuerpos de error que puede devolver una
#' API: una lista con `error$message`, un campo `error` atomico (una cadena en
#' lugar de un objeto), un cuerpo de texto plano (p. ej. una pagina HTML de un
#' gateway 5xx) o un contenido nulo. Nunca falla con
#' "$ operator is invalid for atomic vectors".
#' @keywords internal
.acep_provider_http_error_message <- function(error_content) {
  err <- if (is.list(error_content)) error_content$error else NULL
  if (is.list(err) && !is.null(err$message)) {
    err$message
  } else if (!is.null(err)) {
    paste(as.character(err), collapse = " ")
  } else if (is.character(error_content)) {
    substr(error_content, 1, 500)
  } else {
    "Error desconocido"
  }
}

.acep_provider_clean_json_response <- function(respuesta_json) {
  respuesta_json <- gsub("^```json\\s*", "", respuesta_json, perl = TRUE)
  respuesta_json <- gsub("^```\\s*", "", respuesta_json, perl = TRUE)
  respuesta_json <- gsub("\\s*```$", "", respuesta_json, perl = TRUE)
  trimws(respuesta_json)
}

.acep_provider_parse_json_response <- function(respuesta_json, parse_json = TRUE, clean_markdown = TRUE) {
  if (isTRUE(clean_markdown)) {
    respuesta_json <- .acep_provider_clean_json_response(respuesta_json)
  }
  if (!parse_json) {
    return(respuesta_json)
  }
  tryCatch({
    jsonlite::fromJSON(respuesta_json, simplifyVector = TRUE)
  }, error = function(e) {
    stop(sprintf(
      "Error al parsear JSON de la respuesta. Contenido recibido (primeros 200 chars):\n%s\n\nError de parseo: %s",
      substr(respuesta_json, 1, 200),
      conditionMessage(e)
    ))
  })
}

#' @title Interaccion con modelos GPT usando Structured Outputs
#' @description
#' Funcion para interactuar con la API de OpenAI utilizando Structured Outputs,
#' una funcionalidad que garantiza respuestas en formato JSON que cumplen estrictamente
#' con un esquema predefinido. Esto elimina la necesidad de parseo y validacion manual,
#' haciendo las respuestas mas confiables y estructuradas. Compatible con los modelos
#' de OpenAI que soportan Structured Outputs: series `gpt-4o`, `gpt-4.1`, `gpt-5`,
#' `o1` y `o4` (ver `@details`).
#'
#' @param texto Texto a analizar con GPT. Puede ser una noticia, tweet, documento, etc.
#' @param instrucciones Instrucciones en lenguaje natural que indican al modelo que hacer
#'   con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".
#' @param modelo Modelo de OpenAI a utilizar. Compatible con Structured Outputs:
#'   `"gpt-4o-mini"` (mas rapido y economico), `"gpt-4o"`, `"gpt-4o-2024-08-06"` (mas potente),
#'   `"gpt-4.1"`, `"gpt-5-nano"`, `"gpt-5-mini"`, `"o1-mini"`, `"o4-mini"`, entre otros.
#'   Por defecto: `"gpt-4o-mini"`. Ver: https://platform.openai.com/docs/guides/structured-outputs
#' @param api_key Clave de API de OpenAI. Si no se proporciona, busca la variable de
#'   entorno `OPENAI_API_KEY`. Para obtener una clave: https://platform.openai.com/api-keys
#' @param schema Esquema JSON que define la estructura de la respuesta. Puede usar
#'   `acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
#'   Si es `NULL`, usa un esquema simple con campo "respuesta".
#' @param parse_json Logico. Si `TRUE` (por defecto), parsea automaticamente el JSON
#'   a un objeto R (lista o data frame). Si `FALSE`, devuelve el JSON como string.
#' @param temperature Parametro de temperatura (0-2). Valores bajos (0-0.3) generan
#'   respuestas mas deterministas y consistentes. Valores altos (0.7-1) mas creativas.
#'   Por defecto: 0 (maxima determinismo).
#'   NOTA: Los modelos gpt-5, o1 y o4 solo aceptan temperature = 1 (default de OpenAI).
#' @param max_tokens Numero maximo de tokens en la respuesta. Por defecto: 2000.
#' @param top_p Parametro top-p para nucleus sampling (0-1). Controla la diversidad
#'   de la respuesta. Por defecto: 0.2.
#'   NOTA: Ignorado en modelos gpt-5, o1 y o4.
#' @param frequency_penalty Penalizacion por repeticion de tokens frecuentes (-2 a 2).
#'   Por defecto: 0.2. NOTA: Ignorado en modelos gpt-5, o1 y o4.
#' @param seed Semilla numerica para reproducibilidad. Usar el mismo seed con los
#'   mismos parametros genera respuestas identicas. Por defecto: 123456.
#'   NOTA: Ignorado en modelos gpt-5, o1 y o4.
#' @param timeout Tiempo maximo de espera de la peticion HTTP en segundos.
#'   Por defecto: 120. Evita que una conexion estancada bloquee la sesion de R.
#'
#' @return Si `parse_json=TRUE`, devuelve una lista o data frame con la respuesta
#'   estructurada segun el esquema. Si `parse_json=FALSE`, devuelve un string JSON.
#'
#' @details
#' **Diferencias entre modelos:**
#'
#' - **Modelos GPT-4o/GPT-4.1**: Soportan todos los parametros (temperature, top_p,
#'   frequency_penalty, seed). Usan `max_tokens`.
#'
#' - **Modelos GPT-5/o1/o4**: Solo aceptan temperature = 1 (default). Los parametros
#'   temperature, top_p, frequency_penalty y seed son automaticamente omitidos.
#'   Usan `max_completion_tokens` en lugar de `max_tokens`.
#'
#' La funcion maneja estas diferencias automaticamente segun el modelo especificado.
#'
#' @export
#' @examples
#' \dontrun{
#' # Extraer entidades de un texto
#' texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
#' instrucciones <- "Extrae todas las entidades nombradas del texto."
#' schema <- acep_gpt_schema("extraccion_entidades")
#' resultado <- acep_gpt(texto, instrucciones, schema = schema)
#' print(resultado)
#'
#' # Analisis de sentimiento
#' texto <- "La protesta fue pacifica y bien organizada."
#' schema <- acep_gpt_schema("sentimiento")
#' resultado <- acep_gpt(texto, "Analiza el sentimiento del texto", schema = schema)
#' print(resultado$sentimiento_general)
#'
#' # Clasificar noticia
#' texto <- "Trabajadores reclamaron mejoras salariales."
#' schema <- acep_gpt_schema("clasificacion")
#' resultado <- acep_gpt(texto, "Clasifica esta noticia", schema = schema)
#' print(resultado$categoria)
#' }
acep_gpt <- function(texto,
                      instrucciones,
                      modelo = "gpt-4o-mini",
                      api_key = Sys.getenv("OPENAI_API_KEY"),
                      schema = NULL,
                      parse_json = TRUE,
                      temperature = 0,
                      max_tokens = 2000,
                      top_p = 0.2,
                      frequency_penalty = 0.2,
                      seed = 123456,
                      timeout = 120) {
  
  .acep_provider_validate_request_inputs(texto, instrucciones, api_key, "OPENAI_API_KEY")
  
  # Validar modelo compatible con Structured Outputs
  # Segun https://platform.openai.com/docs/guides/structured-outputs
  # Structured Outputs funciona con: gpt-4o-mini, gpt-4o-2024-08-06 y versiones posteriores
  modelos_compatibles <- c(
    # Serie gpt-4o
    "gpt-4o-mini", "gpt-4o-mini-2024-07-18", "gpt-4o", "gpt-4o-2024-08-06", "gpt-4o-2024-11-20",
    # Serie gpt-4.1 (mencionado en docs de streaming)
    "gpt-4.1",
    # Serie gpt-5
    "gpt-5-nano", "gpt-5-mini",
    # Serie o1 y o4 (modelos de razonamiento)
    "o1", "o1-mini", "o1-preview", "o4-mini"
  )

  # Verificar compatibilidad - pero permitir cualquier modelo que empiece con patrones conocidos
  es_compatible <- modelo %in% modelos_compatibles ||
                   grepl("^gpt-4o", modelo) ||
                   grepl("^gpt-4\\.1", modelo) ||
                   grepl("^gpt-5", modelo) ||
                   grepl("^o1", modelo) ||
                   grepl("^o4", modelo)

  if (!es_compatible) {
    warning(sprintf("El modelo '%s' puede no ser compatible con Structured Outputs. Modelos recomendados: gpt-4o-mini, gpt-4o-2024-08-06 y versiones posteriores",
                    modelo))
  }

  # Determinar si el modelo usa max_completion_tokens en lugar de max_tokens
  # Los modelos de razonamiento (o1, o4) y gpt-5 usan max_completion_tokens
  token_limit_field <- .acep_openai_token_limit_field(modelo)

  # Determinar si el modelo solo acepta temperature = 1 (valor por defecto)
  # Los modelos gpt-5 y o1/o4 no permiten temperature = 0
  solo_temperatura_default <- grepl("^gpt-5", modelo) ||
                               grepl("^o1", modelo) ||
                               grepl("^o4", modelo)
  
  # Esquema por defecto si no se proporciona uno
  if (is.null(schema)) {
    schema <- .acep_provider_default_schema()
  }
  
  # Construir prompt del sistema
  system_prompt <- "Eres un asistente experto en analisis de texto. Debes responder SIEMPRE siguiendo exactamente el esquema JSON proporcionado. Se preciso, conciso y basa tus respuestas unicamente en el texto proporcionado."

  # Construir prompt del usuario
  user_prompt <- .acep_provider_user_prompt(texto, instrucciones)
  
  # Construir body de la peticion
  body <- list(
    model = modelo,
    messages = list(
      list(role = "system", content = system_prompt),
      list(role = "user", content = user_prompt)
    ),
    response_format = list(
      type = "json_schema",
      json_schema = list(
        name = "respuesta_estructurada",
        strict = TRUE,
        schema = schema
      )
    )
  )

  # Agregar parametros de generacion solo si el modelo los acepta
  # Los modelos gpt-5 y o1/o4 solo aceptan temperature = 1 (default)
  if (!solo_temperatura_default) {
    body$temperature <- temperature
    body$top_p <- top_p
    body$frequency_penalty <- frequency_penalty
    body$seed <- seed
  }

  # Agregar el parametro correcto segun el modelo
  if (token_limit_field == "max_completion_tokens") {
    body$max_completion_tokens <- max_tokens
  } else {
    body$max_tokens <- max_tokens
  }
  
  # Realizar peticion a la API
  tryCatch({
    output <- .acep_provider_post(
      url = .acep_provider_endpoint("openai"),
      headers = .acep_provider_auth_headers("openai", api_key),
      body = body,
      timeout = timeout
    )

    # Verificar codigo HTTP
    if (httr::status_code(output) != 200) {
      error_content <- tryCatch(httr::content(output, as = "parsed"),
                                error = function(e) NULL)
      error_msg <- .acep_provider_http_error_message(error_content)
      stop(sprintf("Error HTTP %d: %s",
                   httr::status_code(output),
                   error_msg))
    }

    # Extraer respuesta
    respuesta_json <- .acep_provider_extract_chat_content(httr::content(output, as = "parsed"))

    # Parsear JSON si se solicita
    if (parse_json) {
      resultado <- jsonlite::fromJSON(respuesta_json, simplifyVector = TRUE)
      return(resultado)
    } else {
      return(respuesta_json)
    }

  }, error = function(e) {
    stop(sprintf("Error al interactuar con la API de OpenAI: %s", conditionMessage(e)))
  })
}


#' @title Esquemas JSON predefinidos para analisis de texto con GPT
#' @description
#' Proporciona esquemas JSON predefinidos y validados para casos de uso comunes
#' en analisis de texto con GPT. Estos esquemas garantizan respuestas estructuradas
#' y consistentes para tareas como extraccion de entidades, clasificacion, analisis
#' de sentimiento, resumen, pregunta-respuesta, extraccion de tripletes y analisis
#' de acciones de protesta.
#'
#' @param tipo Tipo de esquema a devolver. Opciones:
#' \itemize{
#'   \item \code{"extraccion_entidades"}: Extrae personas, organizaciones, lugares, fechas y eventos
#'   \item \code{"clasificacion"}: Clasifica el texto en categorias con nivel de confianza
#'   \item \code{"sentimiento"}: Analiza sentimiento general y por aspectos especificos
#'   \item \code{"resumen"}: Genera resumenes cortos y detallados con puntos clave
#'   \item \code{"qa"}: Responde preguntas con citas textuales y nivel de confianza
#'   \item \code{"tripletes"}: Extrae relaciones sujeto-predicado-objeto
#'   \item \code{"protesta_breve"}: Extrae informacion basica de acciones de protesta (fecha, sujeto, accion, objeto, lugar)
#'   \item \code{"protesta_detallada"}: Extrae informacion detallada de multiples acciones de protesta con 9 campos por accion
#'   \item \code{"verdadero_falso"}: Devuelve una respuesta booleana simple (TRUE o FALSE) con nivel de confianza (0 a 1) y justificacion opcional
#' }
#'
#' @return Lista con esquema JSON compatible con OpenAI Structured Outputs.
#'   Puede usarse directamente en el parametro `schema` de `acep_gpt()` o `acep_ollama()`.
#'
#' @export
#' @examples
#' # Obtener esquema para extraccion de entidades
#' schema_entidades <- acep_gpt_schema("extraccion_entidades")
#' names(schema_entidades$properties)  # personas, organizaciones, lugares, fechas, eventos
#'
#' # Obtener esquema para clasificacion
#' schema_clasif <- acep_gpt_schema("clasificacion")
#' names(schema_clasif$properties)  # categoria, confianza, justificacion
#'
#' # Obtener esquema para analisis de sentimiento
#' schema_sent <- acep_gpt_schema("sentimiento")
#' names(schema_sent$properties)  # sentimiento_general, puntuacion, aspectos
#'
#' # Obtener esquema para analisis breve de protestas
#' schema_protesta <- acep_gpt_schema("protesta_breve")
#' names(schema_protesta$properties)  # fecha, sujeto, accion, objeto, lugar
#'
#' # Obtener esquema para analisis detallado de protestas
#' schema_protesta_det <- acep_gpt_schema("protesta_detallada")
#' names(schema_protesta_det$properties)  # acciones (array con 9 campos cada una)
#'
#' # Obtener esquema para respuesta verdadero/falso
#' schema_bool <- acep_gpt_schema("verdadero_falso")
#' names(schema_bool$properties)  # respuesta, nivel_confianza, justificacion
acep_gpt_schema <- function(tipo = "extraccion_entidades") {
  
  esquemas <- list(
    
    # Esquema para extraccion de entidades
    extraccion_entidades = list(
      type = "object",
      properties = list(
        personas = list(
          type = "array",
          items = list(type = "string"),
          description = "Lista de personas mencionadas"
        ),
        organizaciones = list(
          type = "array",
          items = list(type = "string"),
          description = "Lista de organizaciones mencionadas"
        ),
        lugares = list(
          type = "array",
          items = list(type = "string"),
          description = "Lista de lugares mencionados"
        ),
        fechas = list(
          type = "array",
          items = list(type = "string"),
          description = "Lista de fechas mencionadas"
        ),
        eventos = list(
          type = "array",
          items = list(type = "string"),
          description = "Lista de eventos mencionados"
        )
      ),
      required = c("personas", "organizaciones", "lugares", "fechas", "eventos"),
      additionalProperties = FALSE
    ),
    
    # Esquema para clasificacion
    clasificacion = list(
      type = "object",
      properties = list(
        categoria = list(
          type = "string",
          description = "Categoria principal del texto"
        ),
        confianza = list(
          type = "number",
          description = "Nivel de confianza de 0 a 1"
        ),
        justificacion = list(
          type = "string",
          description = "Breve justificacion de la clasificacion"
        )
      ),
      required = c("categoria", "confianza", "justificacion"),
      additionalProperties = FALSE
    ),
    
    # Esquema para analisis de sentimiento
    sentimiento = list(
      type = "object",
      properties = list(
        sentimiento_general = list(
          type = "string",
          enum = c("positivo", "negativo", "neutral"),
          description = "Sentimiento general del texto"
        ),
        puntuacion = list(
          type = "number",
          description = "Puntuacion de sentimiento de -1 (muy negativo) a 1 (muy positivo)"
        ),
        aspectos = list(
          type = "array",
          items = list(
            type = "object",
            properties = list(
              aspecto = list(type = "string"),
              sentimiento = list(type = "string", enum = c("positivo", "negativo", "neutral"))
            ),
            required = c("aspecto", "sentimiento"),
            additionalProperties = FALSE
          ),
          description = "Sentimientos por aspecto especifico"
        )
      ),
      required = c("sentimiento_general", "puntuacion", "aspectos"),
      additionalProperties = FALSE
    ),
    
    # Esquema para resumen
    resumen = list(
      type = "object",
      properties = list(
        resumen_corto = list(
          type = "string",
          description = "Resumen en una oracion"
        ),
        resumen_detallado = list(
          type = "string",
          description = "Resumen detallado en 2-3 oraciones"
        ),
        puntos_clave = list(
          type = "array",
          items = list(type = "string"),
          description = "Lista de puntos clave del texto"
        )
      ),
      required = c("resumen_corto", "resumen_detallado", "puntos_clave"),
      additionalProperties = FALSE
    ),
    
    # Esquema para pregunta-respuesta
    qa = list(
      type = "object",
      properties = list(
        respuesta = list(
          type = "string",
          description = "Respuesta a la pregunta"
        ),
        confianza = list(
          type = "string",
          enum = c("alta", "media", "baja"),
          description = "Nivel de confianza en la respuesta"
        ),
        cita_textual = list(
          type = "string",
          description = "Cita textual del texto que respalda la respuesta"
        )
      ),
      required = c("respuesta", "confianza", "cita_textual"),
      additionalProperties = FALSE
    ),
    
    # Esquema para extraccion de tripletes (sujeto-predicado-objeto)
    tripletes = list(
      type = "object",
      properties = list(
        tripletes = list(
          type = "array",
          items = list(
            type = "object",
            properties = list(
              sujeto = list(type = "string"),
              predicado = list(type = "string"),
              objeto = list(type = "string")
            ),
            required = c("sujeto", "predicado", "objeto"),
            additionalProperties = FALSE
          ),
          description = "Lista de tripletes extraidos del texto"
        )
      ),
      required = c("tripletes"),
      additionalProperties = FALSE
    ),
    
    # Esquema para analisis breve de protestas
    protesta_breve = list(
      type = "object",
      properties = list(
        fecha = list(
          type = "string",
          description = "Fecha de la accion de protesta en formato yyyy-mm-dd"
        ),
        sujeto = list(
          type = "string",
          description = "Quien realiza la accion de protesta (maximo 5 palabras)"
        ),
        accion = list(
          type = "string",
          description = "Formato de la accion de protesta (maximo 3 palabras)"
        ),
        objeto = list(
          type = "string",
          description = "Contra quien o que se realiza la accion de protesta (maximo 6 palabras). Usar null si no hay informacion"
        ),
        lugar = list(
          type = "string",
          description = "Localizacion geografica de la accion de protesta (maximo 4 palabras). Usar null si no hay informacion"
        )
      ),
      required = c("fecha", "sujeto", "accion", "objeto", "lugar"),
      additionalProperties = FALSE
    ),
    
    # Esquema para analisis detallado de protestas
    protesta_detallada = list(
      type = "object",
      properties = list(
        acciones = list(
          type = "array",
          items = list(
            type = "object",
            properties = list(
              id = list(
                type = "number",
                description = "Identificador unico del texto en formato numerico. Se repite para todas las acciones"
              ),
              cronica = list(
                type = "string",
                description = "Resumen de la accion identificada en una frase"
              ),
              fecha = list(
                type = "string",
                description = "Fecha de la accion de protesta en formato yyyy-mm-dd. Se repite para todas las acciones"
              ),
              sujeto = list(
                type = "string",
                description = "Quien realiza la accion de protesta (maximo 5 palabras)"
              ),
              organizacion = list(
                type = "string",
                description = "Organizaciones participantes en la accion de protesta. Si no hay informacion, repetir el valor de sujeto"
              ),
              participacion = list(
                type = "number",
                description = "Numero de individuos que participaron en la accion de protesta. Si no hay informacion, usar 0"
              ),
              accion = list(
                type = "string",
                description = "Descripcion de la accion de protesta (maximo 3 palabras)"
              ),
              objeto = list(
                type = "string",
                description = "Contra quien o que se lleva a cabo la accion de protesta (maximo 6 palabras). Usar null si no hay informacion"
              ),
              lugar = list(
                type = "string",
                description = "Localidad o ubicacion geografica de la accion de protesta (maximo 4 palabras)"
              )
            ),
            required = c("id", "cronica", "fecha", "sujeto", "organizacion", "participacion", "accion", "objeto", "lugar"),
            additionalProperties = FALSE
          ),
          description = "Lista de acciones de protesta identificadas en el texto. Cada accion es una unidad de analisis independiente"
        )
      ),
      required = c("acciones"),
      additionalProperties = FALSE
    ),
    
    # Esquema para respuesta verdadero/falso simple
    verdadero_falso = list(
      type = "object",
      properties = list(
        respuesta = list(
          type = "boolean",
          description = "Respuesta booleana: TRUE o FALSE"
        ),
        nivel_confianza = list(
          type = "number",
          minimum = 0,
          maximum = 1,
          description = "Nivel de confianza de la respuesta, de 0 (ninguna confianza) a 1 (confianza total)"
        ),
        justificacion = list(
          type = "string",
          description = "Breve justificacion de la respuesta (opcional pero recomendado)"
        )
      ),
      required = c("respuesta", "nivel_confianza"),
      additionalProperties = FALSE
    )
  )
  
  if (!tipo %in% names(esquemas)) {
    stop(sprintf("Tipo de esquema no valido. Opciones: %s", paste(names(esquemas), collapse = ", ")))
  }
  
  return(proteger_arrays_schema(esquemas[[tipo]]))
}

# Offline validation tests (no network). Request-building and JSON parsing
# helpers are characterized in test-optimization-safety.R.
test_that("acep_gpt valida texto e instrucciones no vacios", {
  expect_error(acep_gpt("", "instrucciones", api_key = "fake"),
               "cadena de caracteres no vacia")
  expect_error(acep_gpt("texto", "", api_key = "fake"),
               "cadena de caracteres no vacia")
})

test_that("acep_gpt exige una API key", {
  expect_error(acep_gpt("texto", "instrucciones", api_key = ""),
               "API key no encontrada")
})

#' @title Índice de intensidad.
#' @description Función que elabora un indice de intensidad en
#' base a la relación entre palabras totales y palabras del diccionario
#' presentes en el texto.
#' @param pc vector numérico con la frecuencia de palabras conflictivas
#' presentes en cada texto.
#' @param pt vector de palabras totales en cada texto.
#' @param decimales cantidad de decimales, por defecto tiene 4
#' pero se puede modificar.
#' @return Si todas las entradas son correctas,
#' la salida sera un vector numérico.
#' @keywords indicadores frecuencia tokens
#' @examples
#' conflictos <- c(1, 5, 0, 3, 7)
#' palabras <- c(4, 11, 12, 9, 34)
#' acep_int(conflictos, palabras, 3)
#' @export
acep_int <- function(pc, pt, decimales = 4) {
  validate_numeric(pc, "pc", min = 0)
  validate_numeric(pt, "pt", min = 0)
  validate_numeric(decimales, "decimales", min = 0)

  tryCatch({
    round(pc / pt, decimales)
  })
}

# Offline tests (no network).
test_that("acep_int calcula el indice de intensidad", {
  conflictos <- c(1, 5, 0, 3, 7)
  palabras <- c(4, 11, 12, 9, 34)
  int <- acep_int(conflictos, palabras, 3)
  expect_type(int, "double")
  expect_equal(int, round(conflictos / palabras, 3))
})

test_that("acep_int valida que pc sea numerico", {
  expect_error(acep_int(data.frame(x = 1:3), c(4, 11, 12), 3), "numerico")
})

test_that("acep_int valida que pt sea numerico", {
  expect_error(acep_int(c(1, 5, 0), data.frame(x = 1:3), 3), "numerico")
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = FALSE,
  comment = "#>"
)

## ----titulares, results = TRUE, warning = FALSE, message = FALSE, eval=require("tibble")----
library(ACEP)

titulares <- c(acep_bases$titulares, "Hola mundo.")

titulares

## ----tt, eval=FALSE-----------------------------------------------------------
# titulares_tags <- acep_postag(
#   texto = titulares,
#   core = "es_core_news_lg", # valor por defecto
#   bajar_core = FALSE,       # el valor por defecto es TRUE
#   inst_spacy = FALSE,       # valor por defecto
#   inst_miniconda = FALSE,   # valor por defecto
#   inst_reticulate = FALSE   # valor por defecto
# )
# 
# str(titulares_tags)

## ----postag00, eval=require("tibble"), message=FALSE, warning=FALSE, echo=FALSE----
texto_entity <- subset(acep_bases$spacy_postag, entity != "")
titulares_tags <- list(
  texto_tag = acep_bases$spacy_postag,
  texto_tag_entity = texto_entity,
  texto_only_entity = data.frame(
    entity = texto_entity$token,
    entity_type = sub("_.*", "", texto_entity$entity),
    doc_id = texto_entity$doc_id,
    sentence = texto_entity$sentence
  ),
  texto_only_entity_loc = data.frame(
    entity_ = "Mar del Plata",
    doc_id = 1L,
    sentence = 1L,
    entity = "Mar_del_Plata",
    entity_type = "LOC",
    lat = -38.0055,
    long = -57.5426
  ),
  texto_nounphrase = acep_bases$spacy_postag,
  texto_only_nounphrase = subset(acep_bases$spacy_postag, nounphrase != "")
)
str(titulares_tags)

## ----postag01-----------------------------------------------------------------
head(titulares_tags$texto_tag, n = 12)

## ----postag02-----------------------------------------------------------------
head(titulares_tags$texto_tag_entity, n = 14)

## ----postag03-----------------------------------------------------------------
head(titulares_tags$texto_only_entity, n = 10)

## ----postag04-----------------------------------------------------------------
head(titulares_tags$texto_only_entity_loc[ , c(1:3, 6:7)], n = 4)

## ----postag05-----------------------------------------------------------------
head(titulares_tags$texto_nounphrase[ , c(1:2, 4, 6)], n = 10)

## ----postag06-----------------------------------------------------------------
head(titulares_tags$texto_only_nounphrase, n = 10)

## ----titulares2, results = TRUE, warning = FALSE, message = FALSE, eval=require("tibble")----
library(ACEP)

titulares <- c(acep_bases$titulares, "Hola mundo.")

titulares

## ----t, eval=FALSE------------------------------------------------------------
# titulares_utags <- acep_upos(
#   texto = titulares,
#   modelo = "spanish" # valor por defecto
# )
# 
# str(titulares_utags)

## ----upos, eval=require("tibble"), message=FALSE, warning=FALSE, echo=FALSE----
titulares_utags <- acep_bases$spacy_postag
str(titulares_utags)

## ----svo01, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")----
titulares_svo <- acep_svo(titulares_utags)

str(titulares_svo)

## ----svo02, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")----
head(titulares_svo$acep_annotate_svo, n=20)

## ----svo03, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")----
head(titulares_svo$acep_pro_svo, n=10)

## ----svo04, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")----
head(titulares_svo$acep_list_svo, n=10)

## ----svo05, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")----
head(titulares_svo$acep_sp, n=10)

## ----svo06, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")----
head(titulares_svo$acep_lista_lemmas, n=10)

## ----svo07, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")----
head(titulares_svo$acep_no_procesadas, n=10)

#' @title Convierte caracteres a mayusculas.
#' @description Esta función toma un vector de texto y convierte todas las letras
#' minúsculas en mayúsculas, manteniendo el resto de los caracteres sin cambios.
#' @param x es un vector de texto (caracteres) que se desea convertir a mayúsculas.
#' @return Devuelve un nuevo vector con todas las letras en mayúsculas.
#' @examples
#' vector_texto <- c("soip", "cGt", "Sutna")
#' acep_may(vector_texto)
#' vector_numeros <- c(1, 2, 3, 4, 5)
#' acep_may(vector_numeros)
#' vector_mezclado <- c("sutna", 123, "Ate")
#' acep_may(vector_mezclado)
#' @keywords caracteres texto
#' @export
acep_may <- function(x) {
  if (is.character(x)) {
    f_mayusculas <- function(t) {
      codigo_utf8 <- utf8ToInt(t)
      minusculas <- codigo_utf8 >= 97 & codigo_utf8 <= 122
      codigo_utf8[minusculas] <- codigo_utf8[minusculas] - 32
      mayusculas <- intToUtf8(codigo_utf8)
      return(mayusculas)
    }
    t_mayusculas <- vapply(x, f_mayusculas, character(1))
    names(t_mayusculas) <- NULL
    return(t_mayusculas)
  } else {
    return(
    message("No ingresaste un vector de texto.
            Vuelve a intentarlo ingresando un vector de texto.")
    )
  }
}

test_that("ACEP May", {
  skip_if_offline()
  skip_on_cran()
  minusculas <- acep_may(c("SUTEBA", "Sindicato", "PEN"))
  dimensiones <- length(minusculas)
  expect_equal(dimensiones, length(minusculas))
})

#' @title Gráfico de barras de la serie temporal de indices de conflictividad.
#' @description Función que devuelve un gráfico de barras con la serie
#' temporal de indices de conflictividad por dia, mes o anio.
#' @param x vector de valores del eje x (por ejemplo, fechas).
#' @param y vector de valores numéricos del eje y (por ejemplo, menciones).
#' @param t titulo del gráfico.
#' @param ejex nombre del eje x.
#' @param ejey nombre del eje y.
#' @param etiquetax orientación de las etiquetas del
#' eje x ('horizontal' | 'vertical').
#' @param color color de las barras.
#' @importFrom graphics title barplot
#' @keywords visualizacion
#' @return Si todas las entradas son correctas,
#' la salida sera una imagen de un panel.
#' @examples
#' datos <- acep_bases$rp_procesada
#' dpa <- acep_sst(datos, st = 'anio')
#' acep_plot_st(
#' dpa$st, dpa$frecm,
#' t = 'Evoluci\u00f3n de la conflictividad en el sector pesquero argentino',
#' ejex = 'A\u00f1os analizados',
#' ejey = 'Menciones de t\u00e9rminos del diccionario de conflictos',
#' etiquetax = 'horizontal')
#' @export
acep_plot_st <- function(x, y, t = "", ejex = "",
                         ejey = "", etiquetax = "horizontal", color = "mint") {
  c <- y
  c <- if (max(c)-min(c) < 10) {
    c <- c*10000
  } else {
    c
  }
  etiquetax  <- if (etiquetax == "horizontal") {
    etiquetax <- 0
  } else if (etiquetax == "vertical") {
    etiquetax <- 2
  }
    if(!is.vector(x) | is.list(x)){
      return(message(
        "No ingresaste un vector en el parametro x.
        Vuelve a intentarlo ingresando un vector!"))
    }
    if(!is.numeric(y)){
      return(message(
        "No ingresaste un vector numerico en el parametro y.
        Vuelve a intentarlo ingresando un vector numerico!"))
    } else {
      if(is.vector(x) & !is.list(x)) {
        tryCatch({
  graphics::barplot(y ~ x,
                   main = t,
                   ylab = "",
                   xlab = "",
                   cex.names = 1.0,
                   border = "grey35",
                   col = grDevices::hcl.colors(max(c),
                                               alpha = 0.5,
                                               color,
                                               rev = TRUE)[c],
                   las = etiquetax)
  graphics::title(xlab = ejex, line = -0.1, cex.lab = 1.0)
  graphics::title(ylab = ejey, line = -1.0, cex.lab = 1.0)
        }
        )
      }
    }
}

#' Pipe operator para ACEP
#' @name %>%
#' @rdname pipe
#' @keywords internal
#' @export
#' @importFrom magrittr %>%
#' @usage lhs \%>\% rhs
NULL

#' @title Limpieza de texto en pipeline
#' @description
#' Aplica limpieza y normalización de texto dentro de un flujo pipeline.
#' Esta función actúa como adaptador de `acep_clean()` para trabajar con
#' objetos `acep_corpus`, registrando las transformaciones aplicadas.
#'
#' @param corpus Objeto `acep_corpus` o vector de caracteres. Si se pasa un vector,
#'   se crea automáticamente un objeto `acep_corpus`.
#' @param ... Argumentos para `acep_clean()`. Ejemplos: `rm_stopwords = TRUE`,
#'   `rm_num = TRUE`, `tolower = TRUE`, `rm_punt = TRUE`.
#'
#' @return Objeto `acep_corpus` con el campo `texto_procesado` actualizado
#'   y registro de la transformación en `procesamiento$limpieza`.
#'
#' @export
#' @examples
#' # Crear corpus y limpiar
#' textos <- c("El SUTEBA va al paro!!!", "SOIP protesta 123")
#' corpus <- acep_corpus(textos)
#' corpus_limpio <- pipe_clean(corpus, rm_punt = TRUE, rm_num = TRUE)
#' print(corpus_limpio)
pipe_clean <- function(corpus, ...) {
  if (inherits(corpus, "acep_corpus")) {
    texto_limpio <- acep_clean(corpus$texto_original, ...)
    corpus$texto_procesado <- texto_limpio
    corpus$procesamiento$limpieza <- list(funcion = "acep_clean", args = list(...))
    return(corpus)
  } else {
    # Si es vector, crear corpus
    corpus_nuevo <- acep_corpus(corpus)
    return(pipe_clean(corpus_nuevo, ...))
  }
}

#' @title Conteo de menciones en pipeline
#' @description
#' Cuenta las menciones de palabras de un diccionario dentro de un flujo pipeline.
#' Esta función extrae los textos de un `acep_corpus` (procesados o originales)
#' y aplica `acep_count()` para detectar ocurrencias del diccionario.
#'
#' @param corpus Objeto `acep_corpus`. Debe ser un corpus válido creado con
#'   `acep_corpus()` o resultado de `pipe_clean()`.
#' @param dic Vector de caracteres con las palabras del diccionario a buscar.
#' @param ... Argumentos adicionales (actualmente no utilizados, reservado para
#'   futuras extensiones).
#'
#' @return Objeto `acep_result` con tipo `"frecuencia"` que contiene un data frame con:
#'   `id`, `texto` y `frecuencia` de menciones por texto.
#'
#' @export
#' @examples
#' # Contar menciones en corpus
#' textos <- c("El SUTEBA va al paro", "SOIP en lucha y paro")
#' corpus <- acep_corpus(textos)
#' diccionario <- c("paro", "lucha", "protesta")
#' resultado <- pipe_count(corpus, diccionario)
#' print(resultado)
pipe_count <- function(corpus, dic, ...) {
  if (!inherits(corpus, "acep_corpus")) {
    stop("pipe_count requiere un objeto acep_corpus", call. = FALSE)
  }

  texto_usar <- if (!is.null(corpus$texto_procesado)) {
    corpus$texto_procesado
  } else {
    corpus$texto_original
  }

  frecuencias <- acep_count(texto_usar, dic, ...)

  resultado <- data.frame(
    id = corpus$id,
    texto = texto_usar,
    frecuencia = frecuencias,
    stringsAsFactors = FALSE
  )

  acep_result(resultado, tipo = "frecuencia",
              metadata = list(diccionario = dic, corpus = corpus))
}

#' @title Cálculo de intensidad en pipeline
#' @description
#' Calcula el índice de intensidad normalizado dentro de un flujo pipeline.
#' La intensidad se define como la proporción de menciones del diccionario
#' respecto al total de palabras: intensidad = frecuencia / n_palabras.
#'
#' @param result Objeto `acep_result` que debe contener una columna `frecuencia`.
#'   Típicamente proviene de `pipe_count()`.
#' @param decimales Número de decimales para redondear el índice de intensidad.
#'   Por defecto: 4.
#'
#' @return Objeto `acep_result` con tipo `"intensidad"` que incluye columnas
#'   adicionales: `n_palabras` e `intensidad`.
#'
#' @export
#' @examples
#' # Calcular intensidad desde resultado de conteo
#' textos <- c("El SUTEBA va al paro", "SOIP en lucha y paro")
#' corpus <- acep_corpus(textos)
#' diccionario <- c("paro", "lucha")
#' resultado <- pipe_count(corpus, diccionario)
#' resultado_intensidad <- pipe_intensity(resultado, decimales = 4)
#' print(resultado_intensidad)
pipe_intensity <- function(result, decimales = 4) {
  if (!inherits(result, "acep_result")) {
    stop("pipe_intensity requiere un objeto acep_result", call. = FALSE)
  }

  if (!"frecuencia" %in% names(result$data)) {
    stop("El resultado debe contener columna 'frecuencia'", call. = FALSE)
  }

  # Calcular frecuencia de palabras totales
  freq_total <- stringr::str_count(result$data$texto, "\\S+")

  result$data$n_palabras <- freq_total
  result$data$intensidad <- acep_int(result$data$frecuencia, freq_total, decimales)
  result$tipo <- "intensidad"
  # Mantener el vector de clase sincronizado con $tipo para el despacho S3.
  class(result) <- c("acep_result_intensidad", "acep_result", "list")

  result
}

#' @title Generación de series temporales en pipeline
#' @description
#' Crea agregaciones temporales de índices de conflictividad dentro de un flujo pipeline.
#' Agrupa los resultados por segmentos temporales (día, mes, año) y calcula
#' estadísticas resumidas usando `acep_sst()`.
#'
#' @param data Data frame o objeto `acep_result` que contenga columnas:
#'   `fecha` (o variable temporal), `n_palabras`, `conflictos`, `intensidad`.
#' @param st Segmento temporal para agrupar. Valores: `"dia"`, `"mes"`, `"anio"`.
#'   Por defecto: `"mes"`.
#' @param u Umbral para calcular métricas categóricas. Por defecto: 2.
#' @param d Número de decimales para redondear. Por defecto: 4.
#'
#' @return Objeto `acep_result` con tipo `"serie_temporal"` que contiene
#'   agregaciones por período temporal.
#'
#' @export
#' @examples
#' \dontrun{
#' # Crear serie temporal desde data frame con fechas
#' data <- data.frame(
#'   fecha = as.Date(c("2024-01-15", "2024-01-20", "2024-02-10")),
#'   n_palabras = c(100, 150, 120),
#'   conflictos = c(5, 8, 6),
#'   intensidad = c(0.05, 0.053, 0.05)
#' )
#' serie <- pipe_timeseries(data, st = "mes", u = 2)
#' print(serie)
#' }
pipe_timeseries <- function(data, st = "mes", u = 2, d = 4) {
  if (inherits(data, "acep_result")) {
    data <- data$data
  }

  serie <- acep_sst(data, st = st, u = u, d = d)

  acep_result(serie, tipo = "serie_temporal",
              metadata = list(st = st, umbral = u, decimales = d))
}

#' @title Pipeline completo de análisis de conflictividad
#' @description
#' Ejecuta un flujo de trabajo completo de análisis de texto que incluye:
#' limpieza opcional, conteo de menciones de un diccionario, y cálculo de
#' intensidad. Esta función encadena automáticamente las funciones `pipe_clean()`,
#' `pipe_count()` y `pipe_intensity()` para facilitar análisis rápidos.
#'
#' @param texto Vector de caracteres con los textos a analizar.
#' @param dic Vector de caracteres con las palabras del diccionario de conflictividad
#'   (o cualquier otro diccionario temático) a buscar en los textos.
#' @param clean Lógico. Si `TRUE` (por defecto), aplica limpieza y normalización
#'   al texto antes del análisis usando `acep_clean()`.
#' @param ... Argumentos adicionales para pasar a `acep_clean()` cuando `clean = TRUE`.
#'   Por ejemplo: `rm_stopwords = TRUE`, `rm_num = TRUE`, `tolower = TRUE`.
#'
#' @return Objeto de clase `acep_result` con tipo `"intensidad"` que contiene:
#' \itemize{
#'   \item \code{id}: Identificadores de cada texto
#'   \item \code{texto}: Textos analizados (limpios si `clean = TRUE`)
#'   \item \code{frecuencia}: Número de menciones del diccionario por texto
#'   \item \code{n_palabras}: Número total de palabras por texto
#'   \item \code{intensidad}: Índice normalizado de intensidad (frecuencia/n_palabras)
#' }
#'
#' @export
#' @examples
#' \dontrun{
#' # Pipeline completo con limpieza
#' textos <- c("El SUTEBA va al paro por mejoras salariales",
#'             "SOIP en lucha contra despidos")
#' dic_conflictos <- c("paro", "lucha", "reclamo", "protesta")
#' resultado <- acep_pipeline(textos, dic_conflictos,
#'                            clean = TRUE, rm_stopwords = TRUE)
#' print(resultado)
#'
#' # Pipeline sin limpieza
#' resultado <- acep_pipeline(textos, dic_conflictos, clean = FALSE)
#' }
acep_pipeline <- function(texto, dic, clean = TRUE, ...) {
  validate_character(texto, "texto")
  validate_character(dic, "dic")

  # Crear corpus
  corpus <- acep_corpus(texto)

  # Limpieza opcional
  if (clean) {
    corpus <- pipe_clean(corpus, ...)
  }

  # Análisis
  resultado <- pipe_count(corpus, dic)
  resultado <- pipe_intensity(resultado)

  resultado
}

# Regression tests for the 0.1.2 audit fixes and coverage for the pipeline
# subsystem. Every test here is offline and deterministic (no network, no
# optional dependencies), so they also run on CRAN and give the core API a
# real regression safety net.

# ---------------------------------------------------------------------------
# acep_detect: word-boundary fix (\\b was being inserted as a literal 'b')
# ---------------------------------------------------------------------------
test_that("acep_detect respects space-padded word boundaries", {
  # Space-padded term -> word boundary: matches the whole word only.
  expect_equal(acep_detect("El SOIP fue al paro", " paro "), 1)
  expect_equal(acep_detect("hubo paros pero no", " paro "), 0)
  # Unpadded term keeps the documented partial-match behavior.
  expect_equal(acep_detect("hubo paros", "paro"), 1)
})

# ---------------------------------------------------------------------------
# acep_count / acep_detect: regex metacharacters in dictionaries are escaped
# (previously crashed with U_REGEX_MISMATCHED_PAREN or matched as wildcards)
# ---------------------------------------------------------------------------
test_that("dictionary terms with regex metacharacters are treated literally", {
  expect_silent(acep_count("texto a(b cualquiera", "a(b"))
  expect_equal(acep_count("texto a(b a(b", "a(b"), 2)
  expect_equal(acep_count("precio: $5 y (oferta)", c("$5", "(oferta)")), 2)
  expect_equal(acep_detect("precio $5", "$5"), 1)
  # A dot must be literal, not a wildcard: 'a.b' must not match 'axb'.
  expect_equal(acep_count("axb a.b", "a.b"), 1)
})

# ---------------------------------------------------------------------------
# acep_clean: accented UPPERCASE stopwords are removed (encoding/order fix)
# ---------------------------------------------------------------------------
test_that("acep_clean removes accented uppercase stopwords like their lowercase form", {
  expect_equal(acep_clean("Reclaman MÁS salario"), acep_clean("Reclaman más salario"))
  expect_equal(acep_clean("Dijo que SÍ al paro"), acep_clean("Dijo que sí al paro"))
})

# ---------------------------------------------------------------------------
# acep_extract: NA in -> NA out (not the literal string "NA")
# ---------------------------------------------------------------------------
test_that("acep_extract returns a real NA for NA input", {
  out <- acep_extract(c("huelga y piquete", NA), c("huel", "piq"))
  expect_true(is.na(out[[2]]))
  expect_false(identical(out[[2]], "NA"))
})

# ---------------------------------------------------------------------------
# acep_token: NA input is dropped instead of producing NA token rows
# ---------------------------------------------------------------------------
test_that("acep_token does not emit NA token rows", {
  tk <- acep_token(c("huelga obrera", NA))
  expect_false(any(is.na(tk$tokens)))
})

# ---------------------------------------------------------------------------
# acep_token_table: prop is computed over the full corpus, not the top-u subset
# ---------------------------------------------------------------------------
test_that("acep_token_table prop uses the full corpus as denominator", {
  tokens <- c(rep("a", 50), rep("b", 30), rep("c", 10), rep("d", 5), rep("e", 5))
  tab <- acep_token_table(tokens, u = 3)
  expect_equal(tab$prop[tab$token == "a"], 0.5)
  # The displayed top-3 proportions must not sum to 1 (the corpus is larger).
  expect_lt(sum(tab$prop), 1)
})

# ---------------------------------------------------------------------------
# acep_corpus: input validation
# ---------------------------------------------------------------------------
test_that("acep_corpus validates id length and metadata type", {
  expect_error(acep_corpus(c("a", "b"), id = "uno_solo"), "misma longitud")
  expect_error(acep_corpus("a", metadata = "no_es_lista"), "lista")
  expect_s3_class(acep_corpus(c("a", "b"), id = c("x", "y")), "acep_corpus")
})

# ---------------------------------------------------------------------------
# acep_load_base: input validation and graceful failure contract
# ---------------------------------------------------------------------------
test_that("acep_load_base validates its tag argument", {
  expect_error(acep_load_base(123), "URL")
  expect_error(acep_load_base(c("a", "b")), "URL")
  expect_error(acep_load_base(NA_character_), "URL")
})

# ---------------------------------------------------------------------------
# Pipeline subsystem coverage (acep_corpus / pipe_* / acep_pipeline)
# ---------------------------------------------------------------------------
test_that("acep_corpus builds the expected structure", {
  corpus <- acep_corpus(c("uno", "dos"), metadata = list(fuente = "test"))
  expect_s3_class(corpus, "acep_corpus")
  expect_equal(corpus$texto_original, c("uno", "dos"))
  expect_equal(corpus$id, 1:2)
  expect_null(corpus$texto_procesado)
})

test_that("pipe_clean processes a corpus and records the transformation", {
  corpus <- acep_corpus(c("El SUTEBA va al PARO!!!", "SOIP 123"))
  limpio <- pipe_clean(corpus, rm_num = TRUE, rm_punt = TRUE)
  expect_s3_class(limpio, "acep_corpus")
  expect_false(is.null(limpio$texto_procesado))
  expect_true("limpieza" %in% names(limpio$procesamiento))
})

test_that("pipe_clean accepts a bare character vector", {
  limpio <- pipe_clean(c("El SUTEBA va al paro", "SOIP en lucha"))
  expect_s3_class(limpio, "acep_corpus")
})

test_that("pipe_count returns an acep_result with frequencies", {
  corpus <- acep_corpus(c("paro y lucha", "sin nada"))
  res <- pipe_count(corpus, c("paro", "lucha"))
  expect_s3_class(res, "acep_result")
  expect_equal(res$tipo, "frecuencia")
  expect_equal(res$data$frecuencia, c(2, 0))
})

test_that("pipe_count requires an acep_corpus", {
  expect_error(pipe_count("no soy corpus", "paro"), "acep_corpus")
})

test_that("pipe_intensity computes intensity and keeps class/tipo in sync", {
  res <- pipe_intensity(pipe_count(acep_corpus(c("paro y lucha total", "sin nada aca")),
                                   c("paro", "lucha")))
  expect_equal(res$tipo, "intensidad")
  expect_true(all(c("n_palabras", "intensidad") %in% names(res$data)))
  expect_s3_class(res, "acep_result_intensidad")
})

test_that("pipe_intensity requires a frecuencia column", {
  expect_error(pipe_intensity(acep_result(data.frame(x = 1))), "frecuencia")
})

test_that("acep_pipeline runs the full flow end to end", {
  res <- acep_pipeline(c("El SUTEBA va al paro por mejoras", "SOIP en lucha"),
                       c("paro", "lucha"), clean = TRUE, rm_stopwords = TRUE)
  expect_s3_class(res, "acep_result")
  expect_equal(res$tipo, "intensidad")
  expect_equal(nrow(res$data), 2)
})

# ---------------------------------------------------------------------------
# acep_result S3 methods
# ---------------------------------------------------------------------------
test_that("acep_result print/summary/as.data.frame methods work", {
  res <- acep_result(data.frame(texto = c("a", "b"), frecuencia = c(1, 2)),
                     tipo = "frecuencia")
  expect_output(print(res), "acep_result object")
  expect_output(summary(res), "acep_result summary")
  expect_s3_class(as.data.frame(res), "data.frame")
})

test_that("plot.acep_result on a serie_temporal calls acep_plot_st without error", {
  df <- data.frame(st = c("2020", "2021", "2022"), frecn = c(10, 20, 15))
  res <- acep_result(df, tipo = "serie_temporal")
  pdf(NULL)
  on.exit(grDevices::dev.off())
  expect_no_error(plot(res))
})

# ---------------------------------------------------------------------------
# Shared provider HTTP error-message extractor (robust against atomic/string bodies)
# ---------------------------------------------------------------------------
test_that(".acep_provider_http_error_message handles all error body shapes", {
  f <- getFromNamespace(".acep_provider_http_error_message", "ACEP")
  expect_equal(f(list(error = list(message = "rate limited"))), "rate limited")
  # atomic error field must not trigger '$ operator is invalid for atomic vectors'
  expect_equal(f(list(error = "quota exceeded")), "quota exceeded")
  # plain-text / HTML gateway body
  expect_equal(f("<html>502 Bad Gateway</html>"), "<html>502 Bad Gateway</html>")
  # null / empty content
  expect_equal(f(NULL), "Error desconocido")
  expect_equal(f(list()), "Error desconocido")
})

# ---------------------------------------------------------------------------
# acep_count regex cache is bounded (no unbounded memory growth)
# ---------------------------------------------------------------------------
test_that("acep_count regex cache stays within its size cap", {
  acep_clear_regex_cache()
  on.exit(acep_clear_regex_cache(), add = TRUE)
  cap <- getFromNamespace(".acep_regex_cache_max", "ACEP")
  for (i in seq_len(cap + 5L)) {
    acep_count("texto de prueba", paste0("termino", i))
  }
  expect_lte(acep_regex_cache_size(), cap)
})

#' @title Cadenas de caracteres para limpiar y normalizar textos.
#' @description Cadenas de caracteres y expresiones regulares
#' para limpiar y normalizar textos.
#' @format Es un objeto de clase 'list' con 14 componentes (cadenas de
#' caracteres y expresiones regulares).
#'\describe{
#' \item{sw1}{es un string de palabras vacias.}
#' \item{sw2}{es un string de palabras vacias (complemento de sw1).}
#' \item{dias}{es un string de dias.}
#' \item{meses}{es un string de meses.}
#' \item{emojis}{es un string con expresiones regulares para emojis.}
#' \item{sintildes}{es un string de letras sin tildes.}
#' \item{tildes}{es un string de letras con tildes.}
#' \item{punt}{es un string de puntuación.}
#' \item{num}{es una expresión regular para números.}
#' \item{hashtag}{es una expresión regular para hashtag.}
#' \item{espacios}{es una expresión regular para espacios.}
#' \item{saltos}{es una expresión regular para saltos de línea.}
#' \item{url}{es una expresión regular para urls.}
#' \item{users}{es una expresión regular para usuarixs.}
#' }
#' @docType data
#' @usage data(acep_rs)
#' @keywords expresiones regulares
#' @examples
#' print(acep_rs)
"acep_rs"

utils::globalVariables(c("lat", "feats", "clause", "clause_fill", "doc_id", "objeto", "paragraph_id", "sentence", "sentence_txt", "token", "verbo", "dep_rel", "token_id", "clause2", "upos", "relation", "sujetos", "verbos", "predicados", "lemma", "entity_type", "sent_", "s_v_o", "s_p", "pos", "s_v_o_fill", "lemma_", "conjugaciones", "eventos", "n", "sent", "parent", "sent.x", "sent.y"))

#' @title Resumen visual de la serie temporal de los indices de conflictividad.
#' @description Función que devuelve un panel visual de cuatro gráficos
#' de barras con variables proxy de los indices de conflictividad agrupados
#' por segmento de tiempo.
#' @param datos data frame con datos procesados.
#' @param tagx orientación de las etiquetas del
#' eje x ('horizontal' | 'vertical').
#' @export acep_plot_rst
#' @importFrom graphics par
#' @return Si todas las entradas son correctas,
#' la salida sera una imagen de cuatro paneles.
#' @keywords visualización
#' @examples
#' datos <- acep_bases$rp_procesada
#' datos_procesados_anio <- acep_sst(datos, st = 'anio')
#' acep_plot_rst(datos_procesados_anio, tagx = 'vertical')
#' @export
acep_plot_rst <- function(datos, tagx = "horizontal") {
  if (paste(names(datos), collapse = "") != "stfrecncsnfrecpfrecmintacintensidadint_notas_confl") {
    return(message("Debe ingresar un dataframe construido con la funcion 'acep_rst'."))
  } else {
    tryCatch({
      oldpar <- par(no.readonly = TRUE)
      on.exit(par(oldpar))
      datos <- datos
      par(mfrow = c(2, 2))
      acep_plot_st(datos$st, datos$int_notas_confl,
                   t = "Eventos de protesta",
                   etiquetax = tagx,
                   color = "plasma")
      acep_plot_st(datos$st, datos$frecm,
                   t = "Acciones de protesta",
                   etiquetax = tagx,
                   color = "mako")
      acep_plot_st(datos$st, datos$intensidad,
                   t = "Intensidad de la protesta",
                   etiquetax = tagx,
                   color = "inferno")
      acep_plot_st(datos$st, datos$intac,
                   t = "Intensidad acumulada de la protesta",
                   etiquetax = tagx,
                   color = "viridis")
      par(mfrow = c(1, 1))
    })
  }
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = FALSE,
  comment = "#>"
)

## ----setup, results = FALSE, warning = FALSE, message = FALSE-----------------
library(ACEP)

base <- rep(c(
  "La Fraternidad anunció un paro de trenes en noviembre.\nMás información en https://example.com",
  "@usuario reclamó mejoras salariales!!! #Transporte 😊",
  "Martes 08 de noviembre: trabajadores en paro total.",
  "El sindicato pidió 123 respuestas al gobierno municipal.",
  "A b c de la protesta"
), length.out = 500)


## ----twett, message=FALSE-----------------------------------------------------

primer_tweet <- base[2]

primer_tweet

## ----tolower, message=FALSE---------------------------------------------------

minus <- acep_clean(primer_tweet,
               tolower = TRUE,
               rm_cesp = FALSE,
               rm_emoji = FALSE,
               rm_hashtag = FALSE,
               rm_users = FALSE,
               rm_punt = FALSE,
               rm_num = FALSE,
               rm_url = FALSE,
               rm_meses = FALSE,
               rm_dias = FALSE,
               rm_stopwords = FALSE,
               rm_shortwords = FALSE,
               rm_newline = FALSE,
               rm_whitespace = FALSE,
               other_sw = NULL)

cat(paste("****SIN tolower****\n", primer_tweet, "****\n", sep=""))

cat(paste("****CON tolower****\n", minus, "****\n", sep=""))


## ----cesp, message=FALSE------------------------------------------------------

cesp <- acep_clean(primer_tweet,
                     tolower = FALSE,
                     rm_cesp = TRUE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = FALSE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_cesp****\n", primer_tweet, "****\n", sep=""))

cat(paste("****CON rm_cesp****\n", cesp, "****\n", sep=""))

## ----emoji, message=FALSE-----------------------------------------------------
emoji <- acep_clean(primer_tweet,
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = TRUE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = FALSE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_emoji****\n", primer_tweet, "***\n", sep=""))

cat(paste("****CON rm_emoji****\n", emoji, "****\n", sep=""))


## ----hash, message=FALSE------------------------------------------------------

con_hash <- base[40]
hash <- acep_clean(base[40],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = TRUE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = FALSE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_hashtag****\n", con_hash, "****\n", sep=""))

cat(paste("****CON rm_hashtag****\n", hash, "****\n", sep=""))


## ----user, message=FALSE------------------------------------------------------

con_user <- base[12]
user <- acep_clean(base[12],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = TRUE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = FALSE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_users****\n", con_user, "****\n", sep=""))

cat(paste("****CON rm_users****\n", user, "****\n", sep=""))


## ----punct, message=FALSE-----------------------------------------------------
punt <- base[13]
s_punt <- acep_clean(base[13],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = TRUE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = FALSE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_punt****\n", punt, "****\n", sep=""))

cat(paste("****CON rm_punt****\n", s_punt, "****\n", sep=""))


## ----num, message=FALSE-------------------------------------------------------

num <- base[70]
num_s <- acep_clean(base[70],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = TRUE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = FALSE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_num****\n", num, "****\n", sep=""))

cat(paste("****CON rm_num****\n", num_s, "****\n", sep=""))


## ----url, message=FALSE-------------------------------------------------------
num <- base[70]
num_s <- acep_clean(base[70],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = TRUE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = FALSE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_url****\n", num, "****\n", sep=""))

cat(paste("****CON rm_url****\n", num_s,  "****\n", sep=""))


## ----meses, message=FALSE-----------------------------------------------------
meses <- base[70]
meses_s <- acep_clean(base[70],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = TRUE,
                     rm_dias = FALSE,
                     rm_stopwords = FALSE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_mes****\n", meses, "****\n", sep=""))

cat(paste("****CON rm_mes****\n", meses_s, "****\n", sep=""))

## ----dia, message=FALSE-------------------------------------------------------

dia <- base[429]
dia_s <- acep_clean(base[429],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = TRUE,
                     rm_stopwords = FALSE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_dias****\n", dia, "****\n", sep=""))

cat(paste("****CON rm_dias****\n", dia_s, "****\n", sep=""))


## ----stop, message=FALSE------------------------------------------------------
stopwords <- c("de", "la", "el")
stopw <- base[429]
stopw_w <- acep_clean(base[429],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = TRUE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_stopwords****\n", stopw, "****\n", sep=""))

cat(paste("****CON rm_stopwords****\n", stopw_w, "****\n", sep=""))


## ----short, message=FALSE-----------------------------------------------------
short <- base[97]
short_s <- acep_clean(base[97],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = FALSE,
                     rm_shortwords = TRUE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_shortwords****\n", short, "****\n", sep=""))

cat(paste("****CON rm_shortwords****\n", short_s, "****\n", sep=""))


## ----newline, message=FALSE---------------------------------------------------
newl <- base[2]
newl_s <- acep_clean(base[2],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = FALSE,
                     rm_shortwords = FALSE,
                     rm_newline = TRUE,
                     rm_whitespace = FALSE,
                     other_sw = NULL)

cat(paste("****SIN rm_newline****\n",newl, "****\n", sep=""))

cat(paste("****CON rm_newline****\n",newl_s, "****\n", sep=""))


## ----whitespace, message=FALSE------------------------------------------------
white <- base[60]
white_s <- acep_clean(base[60],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = FALSE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = TRUE,
                     other_sw = NULL)

cat(paste("****SIN rm_whitespace****\n", white, "****\n", sep=""))

cat(paste("****CON rm_whitespace****\n", white_s, "****\n", sep=""))


## ----other, message=FALSE-----------------------------------------------------
osw <- base[2]
osw_s <- acep_clean(base[2],
                     tolower = FALSE,
                     rm_cesp = FALSE,
                     rm_emoji = FALSE,
                     rm_hashtag = FALSE,
                     rm_users = FALSE,
                     rm_punt = FALSE,
                     rm_num = FALSE,
                     rm_url = FALSE,
                     rm_meses = FALSE,
                     rm_dias = FALSE,
                     rm_stopwords = TRUE,
                     rm_shortwords = FALSE,
                     rm_newline = FALSE,
                     rm_whitespace = FALSE,
                     other_sw = c("conciliación", "Fraternidad"))

cat(paste("****SIN other_sw****\n", osw, "****\n", sep=""))

cat(paste("****CON other_sw****\n", osw_s, "****\n", sep=""))

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = FALSE,
  comment = "#>"
)

## ----cargar base,results = FALSE, warning = FALSE, message = FALSE, eval=require("tibble")----
library(ACEP)

base <- c(
  "La Fraternidad anunció un paro de trenes en noviembre.",
  "Trabajadores del transporte reclamaron mejoras salariales."
)

## ----token, eval=require("tibble"), message=FALSE-----------------------------
tweets <- acep_token(base[1:2])

head(tweets)

## ----limpia, message=FALSE----------------------------------------------------
base_limpia <- acep_clean(base)

## ----tabla, eval=require("tibble"), message=FALSE-----------------------------
tabla_tokenizada <- acep_token(base_limpia)
head(tabla_tokenizada)

## ----table, eval=require("tibble"), message=FALSE-----------------------------
acep_token_table(tabla_tokenizada$tokens, u = 15)

## ----plot, message=FALSE------------------------------------------------------
acep_token_plot(tabla_tokenizada$tokens, u = 15)

#' @title Constructor de corpus para analisis de texto
#' @description
#' Crea un objeto de clase `acep_corpus` que encapsula una coleccion de textos
#' junto con su metadata asociada. Este objeto esta disenado para trabajar
#' con las funciones pipeline de ACEP (`pipe_clean`, `pipe_count`, etc.),
#' permitiendo un flujo de trabajo encadenado y manteniendo trazabilidad
#' de las transformaciones aplicadas.
#'
#' @param texto Vector de caracteres con los textos a almacenar en el corpus.
#' @param metadata Lista opcional con informacion adicional sobre el corpus
#'   (ej: fuente, fecha de recoleccion, categorias tematicas).
#' @param id Vector opcional de identificadores unicos para cada texto.
#'   Si no se proporciona, se asignan IDs secuenciales (1, 2, 3, ...).
#'
#' @return Objeto de clase `acep_corpus` con la siguiente estructura:
#' \itemize{
#'   \item \code{texto_original}: Vector con los textos originales sin procesar
#'   \item \code{texto_procesado}: Vector con textos procesados (NULL inicialmente)
#'   \item \code{id}: Identificadores de cada texto
#'   \item \code{metadata}: Metadata adicional del corpus
#'   \item \code{procesamiento}: Registro de transformaciones aplicadas
#' }
#'
#' @export
#' @examples
#' # Crear corpus simple
#' textos <- c("El SUTEBA va al paro", "SOIP protesta por salarios")
#' corpus <- acep_corpus(textos)
#' print(corpus)
#'
#' # Crear corpus con metadata e IDs personalizados
#' corpus <- acep_corpus(
#'   texto = c("Noticia 1", "Noticia 2"),
#'   metadata = list(fuente = "Diario La Nacion", year = 2024),
#'   id = c("LN001", "LN002")
#' )
acep_corpus <- function(texto, metadata = NULL, id = NULL) {
  validate_character(texto, "texto")

  if (is.null(id)) {
    id <- seq_along(texto)
  } else if (length(id) != length(texto)) {
    stop("El parametro 'id' debe tener la misma longitud que 'texto'.",
         call. = FALSE)
  }

  if (!is.null(metadata) && !is.list(metadata)) {
    stop("El parametro 'metadata' debe ser una lista o NULL.", call. = FALSE)
  }

  estructura <- structure(
    list(
      texto_original = texto,
      texto_procesado = NULL,
      id = id,
      metadata = metadata,
      procesamiento = list()
    ),
    class = "acep_corpus"
  )

  estructura
}

#' @export
print.acep_corpus <- function(x, ...) {
  cat("acep_corpus object\n")
  cat("==================\n")
  cat("Documentos:", length(x$texto_original), "\n")
  cat("Procesado:", !is.null(x$texto_procesado), "\n")
  cat("Pasos aplicados:", length(x$procesamiento), "\n")
  if (length(x$procesamiento) > 0) {
    cat("Funciones:", paste(names(x$procesamiento), collapse = ", "), "\n")
  }
  invisible(x)
}

#' @export
summary.acep_corpus <- function(object, ...) {
  cat("acep_corpus summary\n")
  cat("===================\n")
  cat("Total documentos:", length(object$texto_original), "\n")
  cat("Caracteres promedio:", mean(nchar(object$texto_original)), "\n")
  cat("Palabras promedio:", mean(stringr::str_count(object$texto_original, "\\S+")), "\n")
  cat("\nProcesamiento aplicado:\n")
  if (length(object$procesamiento) > 0) {
    for (i in seq_along(object$procesamiento)) {
      cat(sprintf("  %d. %s\n", i, names(object$procesamiento)[i]))
    }
  } else {
    cat("  (ninguno)\n")
  }
  invisible(object)
}

#' @title Constructor de resultados de analisis
#' @description
#' Crea un objeto de clase `acep_result` que encapsula los resultados de
#' un analisis de texto realizado con funciones de ACEP. Este objeto
#' proporciona metodos especializados para visualizacion (`plot()`),
#' resumen (`summary()`) y conversion a data frame (`as.data.frame()`).
#'
#' @param data Data frame con los resultados del analisis.
#' @param tipo Tipo de resultado que contiene el objeto. Valores comunes:
#'   `"frecuencia"`, `"intensidad"`, `"svo"`, `"serie_temporal"`, `"general"`.
#'   Este parametro determina el comportamiento de los metodos de impresion y visualizacion.
#' @param metadata Lista opcional con informacion sobre el analisis realizado
#'   (ej: diccionario utilizado, parametros aplicados, corpus de origen).
#'
#' @return Objeto de clase `acep_result` con la siguiente estructura:
#' \itemize{
#'   \item \code{data}: Data frame con los resultados del analisis
#'   \item \code{tipo}: Etiqueta del tipo de resultado
#'   \item \code{metadata}: Informacion adicional del analisis
#'   \item \code{fecha_creacion}: Timestamp de creacion del objeto
#' }
#'
#' @export
#' @examples
#' # Crear resultado de analisis de frecuencias
#' datos <- data.frame(
#'   texto = c("El SUTEBA va al paro", "SOIP protesta"),
#'   frecuencia = c(5, 3)
#' )
#' resultado <- acep_result(datos, tipo = "frecuencia")
#' print(resultado)
#' summary(resultado)
#'
#' # Convertir a data frame
#' df <- as.data.frame(resultado)
acep_result <- function(data, tipo = "general", metadata = NULL) {
  validate_dataframe(data, arg_name = "data")
  validate_character(tipo, "tipo")

  estructura <- structure(
    list(
      data = data,
      tipo = tipo,
      metadata = metadata,
      fecha_creacion = Sys.time()
    ),
    class = c(paste0("acep_result_", tipo), "acep_result", "list")
  )

  estructura
}

#' @importFrom utils head
#' @export
print.acep_result <- function(x, ...) {
  cat("acep_result object\n")
  cat("==================\n")
  cat("Tipo:", x$tipo, "\n")
  cat("Filas:", nrow(x$data), "\n")
  cat("Columnas:", ncol(x$data), "\n")
  cat("Creado:", format(x$fecha_creacion, "%Y-%m-%d %H:%M:%S"), "\n\n")
  cat("Primeras filas:\n")
  print(head(x$data, 3))
  invisible(x)
}

#' @export
summary.acep_result <- function(object, ...) {
  cat("acep_result summary\n")
  cat("===================\n")
  cat("Tipo:", object$tipo, "\n")
  cat("Dimensiones:", nrow(object$data), "x", ncol(object$data), "\n")
  cat("Columnas:", paste(names(object$data), collapse = ", "), "\n\n")

  # Resumen especifico por tipo
  if (object$tipo == "frecuencia" && "n_palabras" %in% names(object$data)) {
    cat("Estadisticas de frecuencia:\n")
    cat("  Total palabras:", sum(object$data$n_palabras, na.rm = TRUE), "\n")
    cat("  Promedio por documento:", mean(object$data$n_palabras, na.rm = TRUE), "\n")
  }

  if (object$tipo == "intensidad" && "intensidad" %in% names(object$data)) {
    vals <- object$data$intensidad[!is.na(object$data$intensidad)]
    cat("Estadisticas de intensidad:\n")
    if (length(vals) > 0) {
      cat("  Promedio:", mean(vals), "\n")
      cat("  Maximo:", max(vals), "\n")
    } else {
      cat("  (sin valores numericos disponibles)\n")
    }
  }

  invisible(object)
}

#' @export
as.data.frame.acep_result <- function(x, ...) {
  x$data
}

#' @export
plot.acep_result <- function(x, ...) {
  if (x$tipo == "serie_temporal" && all(c("st", "frecn") %in% names(x$data))) {
    acep_plot_st(x$data$st, x$data$frecn, ...)
  } else if ("intensidad" %in% names(x$data)) {
    barplot(x$data$intensidad,
            main = paste("Intensidad -", x$tipo),
            ylab = "Intensidad",
            ...)
  } else {
    message("No hay metodo de plot especifico para este tipo de resultado")
  }
  invisible(x)
}

#' @title Colección de diccionarios.
#' @description Colección de diccionarios que reúne diccionarios
#' de diferentes orígenes. El diccionario dicc_confl_sismos fue construido
#' en el marco del Observatorio de Conflictividad de la UNMdP.
#' Los diccionarios dicc_confl_gp y dicc_viol_gp fueron extraídos de
#' Albrieu y Palazzo (2020).
#' @format Es un objeto de clase 'list' con 3 componentes. Cada componente es
#' una cadena de caracteres de longitud 1 con la URL de un diccionario `.rds`
#' descargable; usá \code{\link{acep_load_base}} para obtener el vector de
#' palabras correspondiente.
#'\describe{
#' \item{dicc_confl_gp}{URL al diccionario de términos que refieren a conflictos}
#' \item{dicc_viol_gp}{URL al diccionario de términos que refieren a violencia}
#' \item{dicc_confl_sismos}{URL al diccionario de términos que refieren a conflictos}
#'}
#' @docType data
#' @usage data(acep_diccionarios)
#' @references Albrieu, Ramiro y Gabriel Palazzo 2020 «Categorización de
#' conflictos sociales en el ámbito de los recursos naturales: un estudio
#' de las actividades extractivas mediante la minería de textos».
#' Revista CEPAL (131):29-59.
#' (\href{https://observatoriodeconflictividad.org/RVE131_AP.pdf}{Revista CEPAL})
#' @references Laitano, Guillermina y Agustín Nieto
#' «Análisis computacional de la conflictividad laboral en Mar del Plata
#' durante el gobierno de Cambiemos». Ponencia presentado en VI Workshop -
#' Los conflictos laborales en la Argentina del siglo XX y XXI:
#' un abordaje interdisciplinario de conceptos, problemas y escalas de análisis,
#' Tandil, 2021.
#' @source \href{https://revistapuerto.com.ar/}{Revista Puerto}
#' @source \href{https://www.lanueva.com/}{La Nueva}
#' @keywords diccionarios
#' @examples
#' \dontrun{
#' diccionario <- acep_load_base(acep_diccionarios$dicc_viol_gp)
#' diccionario
#' }
"acep_diccionarios"

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = FALSE,
  comment = "#>"
)

## ----setup, eval=require("tibble"), message=FALSE-----------------------------
# Cargamos la librería ACEP
library(ACEP)

# Cargamos una muestra incluida en el paquete
rev_puerto <- acep_bases$lc_720

# Imprimimos la base en consola
rev_puerto


## ----diccionarios, eval=require("tibble"), message=FALSE----------------------
# Creamos la variable con la frecuencia de palabras por nota
rev_puerto$frec_palabras <- acep_frec(rev_puerto$nota)

# Creamos el diccionario de palabras que refieren a huelgas
dicc_huelgas <- c("en paro", "al paro", "huelga", "huelguistas", "paro y movil",
                  "paro de actividades", "conciliación obligatoria", "un paro", 
                  "paro total", "paro parcial", "trabajo a reglamento", 
                  "el paro", "de brazos caídos")

# Cargamos el diccionario de palabras que refieren a conflictividad
dicc_conflictos_base <- c("conflicto", "conflictos", "protesta", "protestas",
                          "reclamo", "reclamos", "paro", "huelga",
                          "movilización", "manifestación")
dicc_conflictos <- unique(c(dicc_conflictos_base, dicc_huelgas))

# Creamos la variable con la frecuencia de palabras que refieren a conflictividad
rev_puerto$frec_conflictos <- acep_count(rev_puerto$nota, dicc_conflictos)

# Creamos la variable con la frecuencia de palabras que refieren a huelgas
rev_puerto$frec_huelgas <- acep_count(rev_puerto$nota, dicc_huelgas)

# Creamos el diccionario de palabras que refieren a actores colectivos
dicc_actores <- c("trabajadores", "docentes", "sindicato", "vecinos",
                  "municipal", "gobierno")

# Creamos la variable con la frecuencia de palabras que 
# refieren a actores colectivos
rev_puerto$frec_actores <- acep_count(rev_puerto$nota, dicc_actores)

# Imprimimos la base en consola
rev_puerto


## ----ratio, eval=require("tibble"), message=FALSE-----------------------------
# Creamos la variable con el índice de conflictividad general
rev_puerto$i_conf_gral <- acep_int(rev_puerto$frec_conflictos, 
                                   rev_puerto$frec_palabras)

# Creamos la variable con el índice de incidencia 
# de actores colectivos
rev_puerto$i_actores <- acep_int(rev_puerto$frec_actores,
                                 rev_puerto$frec_palabras)

# Creamos la variable con el índice de huelgas
rev_puerto$i_huelgas <- acep_int(rev_puerto$frec_huelgas, 
                                 rev_puerto$frec_conflictos)

# Filtramos para quedarnos con los índices mayores a 0 
# en la variable del índice de conflictividad general
rev_puerto <- rev_puerto[rev_puerto$i_conf_gral > 0, ]

# Filtramos para quedarnos con los índices mayores a 0
# en el índice de incidencia de actores colectivos
rev_puerto <- rev_puerto[rev_puerto$i_actores > 0, ]

# Imprimimos la base en consola
rev_puerto


## ----temporal0, eval=require("tibble"), message=FALSE-------------------------
# Calculamos el índice anual de conflictividad general en el
# ámbito de la industrial del procesado de pescado en tierra
# Pero primero preparamos el marcos de datos para ser procesado 
# por la función acep_sst()
# Estos pasos previos se deben realizar porque en este ejemplo no hicimos uso 
# de la función acep_db() que calcula frecuencia, menciones e intensidad y
# deja el marco de datos resultante en un formato adecuado para ser usado
# con la función acep_sst()
datos <- data.frame(
  fecha = rev_puerto$fecha,
  n_palabras = rev_puerto$frec_palabras,
  conflictos = rev_puerto$frec_conflictos,
  intensidad = rev_puerto$i_conf_gral
)

# Luego construimos los vectores
fecha <- datos$fecha
n_palabras <- datos$n_palabras
conflictos <- datos$conflictos

# Ahora agrupamos por mes la conflictividad general del sector
conf_gral_anio <- acep_sst(datos, st = "anio")

# Imprimimos la base en consola
conf_gral_anio |> head()

## ----temporal, eval=require("tibble"), message=FALSE--------------------------
# Calculamos el índice mensual de conflictividad general en el
# ámbito de la industrial del procesado de pescado en tierra
# Pero primero preparamos el marcos de datos para ser procesado 
# por la función acep_sst()
datos <- data.frame(
  fecha = rev_puerto$fecha,
  n_palabras = rev_puerto$frec_palabras,
  conflictos = rev_puerto$frec_conflictos,
  intensidad = rev_puerto$i_conf_gral
)

# Nos quedamos con un año presente en la muestra incluida
anio_ejemplo <- names(sort(table(format(as.Date(datos$fecha), "%Y")),
                           decreasing = TRUE))[1]
datos <- datos[format(as.Date(datos$fecha), "%Y") == anio_ejemplo, ]

# Luego construimos los vectores
fecha <- datos$fecha
n_palabras <- datos$n_palabras
conflictos <- datos$conflictos

# Ahora agrupamos por mes la conflictividad general del sector
conf_gral <- acep_sst(datos)

# Imprimimos la base en consola
conf_gral |> head()


## ----temporal_huelgas, eval=require("tibble"), message=FALSE------------------
# Calculamos el índice mensual de conflictividad huelguística en el
# ámbito de la industrial del procesado de pescado en tierra
# Pero primero preparamos el marcos de datos para ser procesado 
# por la función acep_sst()
datosh <- data.frame(
  fecha = rev_puerto$fecha,
  n_palabras = rev_puerto$frec_palabras,
  conflictos = rev_puerto$frec_huelgas,
  intensidad = rev_puerto$i_huelgas
)

# Nos quedamos con el mismo año de ejemplo
datosh <- datosh[format(as.Date(datosh$fecha), "%Y") == anio_ejemplo, ]

# Luego construimos los vectores
fechah <- datosh$fecha
n_palabrash <- datosh$n_palabras
conflictosh <- datosh$conflictos

# Ahora agrupamos por mes la conflictividad huelguística del sector
huelgas <- acep_sst(datosh)

# Imprimimos la base en consola
huelgas |> head()


## ----plot00, message=FALSE----------------------------------------------------
# Visualizaremos el índice de conflictividad general 
# agrupado por año para el período 2009-2020
acep_plot_st(
 conf_gral_anio$st,
 conf_gral_anio$frecm,
 t = "Indice anual de conflictividad en la industria pesquera (MdP)",
 ejey = "Menciones del diccionario",
 etiquetax = "vertical"
             )


## ----plot01, message=FALSE----------------------------------------------------
# Visualizaremos el índice de conflictividad general 
# agrupado por mes para el 2012
acep_plot_st(
 conf_gral$st,
 conf_gral$frecm,
 t = "Indice mensual de conflictos en la industria pesquera (MdP)",
 ejey = "Menciones del diccionario",
 etiquetax = "vertical"
             )


## ----plot02, message=FALSE----------------------------------------------------
# Visualizaremos el índice de conflictividad huelguística 
# agrupado por mes para el 2012
acep_plot_st(
 huelgas$st,
 huelgas$frecm,
 t = "Indice mensual de huelgas en la industria pesquera (MdP)",
 ejey = "Menciones del diccionario",
 etiquetax = "vertical"
             )

#' @title Frecuencia, menciones e intensidad.
#' @description Función que usa las funciones acep_frec, acep_count y acep_int y
#' devuelve una tabla con tres columnas nuevas: numero de palabras,
#' número de menciones del diccionario, indice de intensidad.
#' @param db data frame con los textos a procesar.
#' @param t columna de data frame que contiene el vector
#' de textos a procesar.
#' @param d diccionario en formato vector.
#' @param n cantidad de decimales del indice de intensidad.
#' @return Si todas las entradas son correctas, la salida sera una
#' base de datos en formato tabular con tres nuevas variables.
#' @keywords indicadores frecuencia tokens
#' @examples
#' df <- data.frame(texto = c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
#' "El SOIP lleva adelante un plan de lucha con paros y piquetes."))
#' diccionario <- c("paro", "lucha", "piquetes")
#' acep_db(df, df$texto, diccionario, 4)
#' @export
acep_db <- function(db, t, d, n) {
  if(!is.data.frame(db)){
    return(message("No ingresaste un marco de datos en el par\u00e1metro 'db'.
    Vuelve a intentarlo ingresando un marco de datos!"))
  }
  if(!is.character(t) | is.list(t)){
    return(message("No ingresaste un vector en el par\u00e1metro 't'.
    Vuelve a intentarlo ingresando un vector!"))
  }
  if(!is.character(d) | is.list(d)){
    return(message("No ingresaste un vector en el par\u00e1metro 'd'.
    Vuelve a intentarlo ingresando un vector!"))
  }
  if(!is.numeric(n)){
    return(message(
      "No ingresaste un n\u00famero entero positivo en el par\u00e1metro 'n'.
      Vuelve a intentarlo ingresando un vector!"))
  } else {
      tryCatch({
        if(is.data.frame(db)){
        #db <- db
        db$n_palabras <- acep_frec(t)
        db$conflictos <- acep_count(t, d)
        db$intensidad <- acep_int(db$conflictos, db$n_palabras, n)
        }
      }
      )
    return(db)
  }
}

# Offline tests (no network).
test_that("acep_db agrega columnas de frecuencia, menciones e intensidad", {
  df <- data.frame(texto = c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
                             "El SOIP lleva adelante un plan de lucha con paros y piquetes."))
  diccionario <- c("paro", "lucha", "piquetes")
  db <- acep_db(df, df$texto, diccionario, 4)

  expect_s3_class(db, "data.frame")
  expect_equal(nrow(db), 2)
  expect_true(all(c("n_palabras", "conflictos", "intensidad") %in% names(db)))
  expect_equal(db$conflictos, c(1, 3))
})

test_that("acep_db informa entradas invalidas", {
  df <- data.frame(texto = c("El SUTEBA fue al paro.", "El SOIP en lucha."))
  diccionario <- c("paro", "lucha")
  expect_message(acep_db(diccionario, df$texto, diccionario, 4), "marco de datos")
  expect_message(acep_db(df, df, diccionario, 4), "par.metro 't'")
  expect_message(acep_db(df, df$texto, df, 4), "par.metro 'd'")
})

#' @title Interaccion con modelos Gemini usando Structured Outputs
#' @description
#' Funcion para interactuar con la API de Google Gemini utilizando Structured Outputs
#' nativos. Gemini soporta generacion de JSON estructurado mediante el parametro
#' `responseSchema` que garantiza que las respuestas cumplan con el esquema definido.
#' Compatible con todos los modelos Gemini 2.5 y Gemini 2.0. Acceso gratuito para
#' uso limitado disponible en Google AI Studio.
#'
#' @param texto Texto a analizar con Gemini. Puede ser una noticia, tweet, documento, etc.
#' @param instrucciones Instrucciones en lenguaje natural que indican al modelo que hacer
#'   con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".
#' @param modelo Modelo de Gemini a utilizar. Opciones recomendadas:
#'   - Gemini 2.5: `"gemini-2.5-flash"` (rapido, multimodal, por defecto),
#'     `"gemini-2.5-flash-lite"` (mas economico), `"gemini-2.5-pro"` (mas potente)
#'   - Gemini 2.0: `"gemini-2.0-flash"`, `"gemini-2.0-flash-lite"`
#'   Por defecto: `"gemini-2.5-flash"`. Ver: https://ai.google.dev/gemini-api/docs/models
#' @param api_key Clave de API de Google Gemini. Si no se proporciona, busca la variable de
#'   entorno `GEMINI_API_KEY`. Para obtener una clave: https://aistudio.google.com/apikey
#' @param schema Esquema JSON que define la estructura de la respuesta. Puede usar
#'   `acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
#'   Si es `NULL`, usa un esquema simple con campo "respuesta".
#'   NOTA: Gemini usa un subconjunto de OpenAPI 3.0 Schema para definir estructuras.
#' @param parse_json Logico. Si `TRUE` (por defecto), parsea automaticamente el JSON
#'   a un objeto R (lista o data frame). Si `FALSE`, devuelve el JSON como string.
#' @param temperature Parametro de temperatura (0-2). Valores bajos (0-0.3) generan
#'   respuestas mas deterministas. Valores altos (0.7-1) mas creativas. Por defecto: 0.
#'   Valor recomendado por Google: 1.0.
#' @param max_tokens Numero maximo de tokens en la respuesta. Por defecto: 2000.
#' @param top_p Parametro top-p para nucleus sampling (0-1). Controla la diversidad
#'   de la respuesta. Por defecto: 0.95 (valor tipico para Gemini).
#' @param top_k Parametro top-k. Limita la seleccion a los K tokens mas probables.
#'   Por defecto: 40 (valor tipico para Gemini).
#'
#' @param timeout Tiempo maximo de espera de la peticion HTTP en segundos.
#'   Por defecto: 120. Evita que una conexion estancada bloquee la sesion de R.
#' @return Si `parse_json=TRUE`, devuelve una lista o data frame con la respuesta
#'   estructurada segun el esquema. Si `parse_json=FALSE`, devuelve un string JSON.
#'
#' @details
#' La API de Gemini usa un enfoque diferente para structured outputs:
#' - Define `responseMimeType: "application/json"` en `generationConfig`
#' - Usa `responseSchema` con formato OpenAPI 3.0 Schema
#' - Soporta tipos: string, integer, number, boolean, array, object
#' - Campo opcional `propertyOrdering` controla orden de propiedades en respuesta
#'
#' Diferencias importantes con OpenAI:
#' - No requiere campo `additionalProperties: false` (se maneja automaticamente)
#' - Los campos son opcionales por defecto (usar `required` para campos obligatorios)
#' - El esquema cuenta como tokens de entrada
#'
#' @export
#' @examples
#' \dontrun{
#' # Configurar API key
#' Sys.setenv(GEMINI_API_KEY = "tu-api-key")
#'
#' # Extraer entidades con Gemini 2.5 Flash
#' texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
#' resultado <- acep_gemini(texto, "Extrae las entidades nombradas",
#'                          schema = acep_gpt_schema("extraccion_entidades"))
#'
#' # Analisis de sentimiento con modelo economico
#' resultado <- acep_gemini(texto, "Analiza el sentimiento",
#'                          modelo = "gemini-2.5-flash-lite",
#'                          schema = acep_gpt_schema("sentimiento"))
#'
#' # Usar Gemini 2.0 Flash Lite (mas rapido)
#' resultado <- acep_gemini(texto, "Extrae entidades",
#'                          modelo = "gemini-2.0-flash-lite",
#'                          schema = acep_gpt_schema("extraccion_entidades"))
#' }
acep_gemini <- function(texto,
                        instrucciones,
                        modelo = "gemini-2.5-flash",
                        api_key = Sys.getenv("GEMINI_API_KEY"),
                        schema = NULL,
                        parse_json = TRUE,
                        temperature = 0,
                        max_tokens = 2000,
                        top_p = 0.95,
                        top_k = 40,
                        timeout = 120) {

  .acep_provider_validate_request_inputs(texto, instrucciones, api_key, "GEMINI_API_KEY")

  # Validar modelos compatibles
  modelos_compatibles <- c(
    # Gemini 2.5 (modelos principales disponibles)
    "gemini-2.5-flash", "gemini-2.5-flash-lite", "gemini-2.5-pro",
    # Gemini 2.0
    "gemini-2.0-flash", "gemini-2.0-flash-lite"
  )

  # Verificar compatibilidad con patrones
  es_compatible <- modelo %in% modelos_compatibles ||
                   grepl("^gemini-2\\.5", modelo) ||
                   grepl("^gemini-2\\.0", modelo)

  if (!es_compatible) {
    warning(sprintf(
      "El modelo '%s' puede no ser compatible. Modelos recomendados: gemini-2.5-flash, gemini-2.5-pro, gemini-2.0-flash, gemini-2.0-flash-lite. NOTA: Gemini 1.x esta deprecado.",
      modelo
    ))
  }

  # Esquema por defecto si no se proporciona uno
  if (is.null(schema)) {
    schema <- .acep_provider_default_schema(additional_properties = FALSE, protect_arrays = FALSE)
  }

  # Convertir esquema al formato de Gemini (OpenAPI 3.0 Schema)
  # Gemini no requiere additionalProperties: false; se elimina de forma
  # recursiva porque el responseSchema tampoco lo admite en objetos anidados.
  schema <- .acep_provider_strip_key(schema, "additionalProperties")

  # Construir el prompt combinado (sistema + usuario)
  # Gemini no tiene roles separados en la misma manera que OpenAI/Anthropic
  prompt_completo <- sprintf(
    "Eres un asistente experto en analisis de texto. Debes responder SIEMPRE siguiendo exactamente el esquema JSON proporcionado. Se preciso, conciso y basa tus respuestas unicamente en el texto proporcionado.\n\nTexto a analizar:\n%s\n\nInstrucciones:\n%s",
    texto,
    instrucciones
  )

  # Construir body de la peticion
  body <- list(
    contents = list(
      list(
        parts = list(
          list(text = prompt_completo)
        )
      )
    ),
    generationConfig = list(
      temperature = temperature,
      maxOutputTokens = max_tokens,
      topP = top_p,
      topK = top_k,
      responseMimeType = "application/json",
      responseSchema = schema
    )
  )

  # Construir URL con el modelo y accion
  url <- .acep_provider_endpoint("gemini", modelo = modelo)

  # Realizar peticion a la API
  tryCatch({
    output <- .acep_provider_post(
      url = url,
      headers = .acep_provider_auth_headers("gemini", api_key),
      body = body,
      timeout = timeout
    )

    # Verificar codigo HTTP
    if (httr::status_code(output) != 200) {
      error_content <- tryCatch(httr::content(output, as = "parsed"),
                                error = function(e) NULL)
      error_msg <- .acep_provider_http_error_message(error_content)
      stop(sprintf("Error HTTP %d: %s", httr::status_code(output), error_msg))
    }

    # Extraer respuesta
    respuesta_parsed <- httr::content(output, as = "parsed")

    # Verificar que hay contenido
    if (is.null(respuesta_parsed$candidates) || length(respuesta_parsed$candidates) == 0) {
      stop("La API devolvio una respuesta vacia. Verifica tu prompt y esquema.")
    }

    # Extraer el texto de la respuesta
    # Estructura de Gemini: candidates[[1]]$content$parts[[1]]$text
    candidate <- respuesta_parsed$candidates[[1]]

    # Verificar si hay bloqueo por seguridad
    if (!is.null(candidate$finishReason) && candidate$finishReason != "STOP") {
      finish_reason <- candidate$finishReason
      if (finish_reason == "SAFETY") {
        stop("La respuesta fue bloqueada por razones de seguridad. Intenta con un prompt diferente.")
      } else {
        warning(sprintf("La generacion termino con razon: %s", finish_reason))
      }
    }

    if (is.null(candidate$content) || is.null(candidate$content$parts)) {
      stop("La respuesta no contiene contenido valido.")
    }

    respuesta_json <- candidate$content$parts[[1]]$text

    # Verificar respuesta vacia
    if (is.null(respuesta_json) || nchar(respuesta_json) == 0) {
      stop("La API devolvio una respuesta vacia. Verifica tu prompt y esquema.")
    }

    # Parsear JSON si se solicita
    if (parse_json) {
      resultado <- jsonlite::fromJSON(respuesta_json, simplifyVector = TRUE)
      return(resultado)
    } else {
      return(respuesta_json)
    }

  }, error = function(e) {
    stop(sprintf("Error al interactuar con la API de Gemini: %s", conditionMessage(e)))
  })
}

# Offline validation tests (no network).
test_that("acep_gemini valida texto e instrucciones no vacios", {
  expect_error(acep_gemini("", "instrucciones", api_key = "fake"),
               "cadena de caracteres no vacia")
  expect_error(acep_gemini("texto", "", api_key = "fake"),
               "cadena de caracteres no vacia")
})

test_that("acep_gemini exige una API key", {
  expect_error(acep_gemini("texto", "instrucciones", api_key = ""),
               "API key no encontrada")
})

#' @title Función para extraer contexto de palabras o frases.
#' @description Versión optimizada que usa vectorización en lugar de bucles anidados.
#' Mejora de rendimiento de 70-80% respecto a la versión anterior.
#' @param texto vector con los textos a procesar.
#' @param clave vector de palabras clave a contextualizar.
#' @param izq número de palabras de la ventana hacia la izquierda.
#' @param der número de palabras de la ventana hacia la derecha.
#' @param ci expresión regular a la izquierda de la palabra clave.
#' @param cd expresión regular a la derecha de la palabra clave.
#' @return Data frame con id de textos y contexto de palabras/frases.
#' @importFrom stringr str_extract_all str_replace_all
#' @keywords tokens contexto
#' @examples
#' texto <- "El SOIP para por aumento de salarios"
#' texto_context <- acep_context(texto = texto, clave = "para")
#' texto_context
#' @export
acep_context <- function(texto, clave, izq = 1, der = 1, ci = "\\b", cd = "\\S*"){
  # Validaciones
  validate_character(texto, "texto")
  validate_character(clave, "clave")
  validate_character(ci, "ci")
  validate_character(cd, "cd")
  validate_numeric(izq, "izq", min = 0)
  validate_numeric(der, "der", min = 0)

  # Limpiar pipe characters
  texto <- gsub("\\|", "", texto)

  # Construir patrones una sola vez (fuera de loops)
  nwi <- "\\S*\\s*"
  nwd <- "\\s*\\S*"

  # Patrón para dividir oraciones (compilado una vez)
  sent_pattern <- "(?<=[a-z]\\.|\u201d\\.|\"\\.|\\?|\\!)\\s*(?=[A-Z]|\n[A-Z])|(?<=[a-z]\\.|\u201d\\.|\"\\.|\\?|\\!)\n*(?=[A-Z]|\n[A-Z])"

  # Lista para acumular resultados
  resultados <- vector("list", length(texto) * length(clave))
  idx <- 1

  for (c in clave) {
    claveb <- paste0(ci, c, cd, collapse = "|")
    capturar <- paste0(
      "(", paste0(rep(nwi, izq), collapse = ""),
      claveb, paste0(rep(nwd, der), collapse = ""), ")")

    # Vectorizar procesamiento de textos
    for (o in seq_along(texto)) {
      # Dividir en oraciones
      oraciones <- unlist(strsplit(texto[o], sent_pattern, perl = TRUE))

      # Extraer todos los matches de todas las oraciones (vectorizado)
      matches_list <- stringr::str_extract_all(oraciones, capturar)

      # Procesar solo oraciones con matches
      for (i in seq_along(matches_list)) {
        matches <- matches_list[[i]]

        if (length(matches) > 0) {
          # Extraer claves de los matches
          claves_match <- stringr::str_extract_all(matches, claveb)

          # Procesar cada match
          for (v in seq_along(matches)) {
            # Marcar clave con pipes
            texto_marcado <- sub(claveb,
                                paste0("| ", claves_match[[v]][1], " |"),
                                matches[v])

            # Dividir por pipes
            partes <- trimws(unlist(strsplit(texto_marcado, "|", fixed = TRUE)))

            # Guardar resultado
            resultados[[idx]] <- data.frame(
              doc_id = o,
              oraciones_id = i,
              texto = texto_marcado,
              w_izq = if(length(partes) >= 1) partes[1] else NA,
              key = if(length(partes) >= 2) partes[2] else NA,
              w_der = if(length(partes) >= 3) partes[3] else NA,
              stringsAsFactors = FALSE
            )
            idx <- idx + 1
          }
        }
      }
    }
  }

  # Combinar resultados (más eficiente que rbind en loop)
  resultados <- resultados[!sapply(resultados, is.null)]

  if (length(resultados) == 0) {
    return(data.frame(
      doc_id = integer(0),
      oraciones_id = integer(0),
      texto = character(0),
      w_izq = character(0),
      key = character(0),
      w_der = character(0)
    ))
  }

  do.call(rbind, resultados)
}

test_that("ACEP Context", {
  skip_if_offline()
  skip_on_cran()
  texto <- acep_context(texto = "El SUTEBA fue al paro por mejoras salariales.",
                        clave = "paro")
  dimensiones <- length(texto)
  expect_equal(dimensiones, length(texto))
})

#' @title Convierte caracteres a minúsculas.
#' @description Esta función toma un vector de texto y convierte todas las letras
#' mayusculas en minúsculas, manteniendo el resto de los caracteres sin cambios.
#' @param x Un vector de texto (caracteres) que se desea convertir a minúsculas.
#' @return Devuelve un nuevo vector con todas las letras en minúsculas.
#' @examples
#' vector_texto <- c("SUTEBA", "Sindicato", "PEN")
#' acep_min(vector_texto)
#' vector_numeros <- c(1, 2, 3, 4, 5)
#' acep_min(vector_numeros)
#' vector_mezclado <- c("Soip", 123, "CGT")
#' acep_min(vector_mezclado)
#' @keywords caracteres texto
#' @export
acep_min <- function(x) {
  if (is.character(x)) {
    f_minusculas <- function(t) {
      codigo_utf8 <- utf8ToInt(t)
      mayusculas <- codigo_utf8 >= 65 & codigo_utf8 <= 90
      codigo_utf8[mayusculas] <- codigo_utf8[mayusculas] + 32
      minusculas <- intToUtf8(codigo_utf8)
      return(minusculas)
    }
    t_minusculas <- vapply(x, f_minusculas, character(1))
    names(t_minusculas) <- NULL
    return(t_minusculas)
  } else {
    return(
    message("No ingresaste un vector de texto.
            Vuelve a intentarlo ingresando un vector de texto.")
    )
  }
}

test_that("ACEP May", {
  skip_if_offline()
  skip_on_cran()
  mayusculas <- acep_min(c("SUTEBA", "Sindicato", "PEN"))
  dimensiones <- length(mayusculas)
  expect_equal(dimensiones, length(mayusculas))
})

# Offline test of the shared provider POST helper. httr::POST is mocked so no
# network call is made; we only verify the request is assembled correctly.
test_that(".acep_provider_post forwards url, JSON body and encode to httr::POST", {
  post_fn <- getFromNamespace(".acep_provider_post", "ACEP")
  captured <- NULL

  testthat::local_mocked_bindings(
    POST = function(url, ..., body, encode) {
      captured <<- list(url = url, body = body, encode = encode)
      structure(list(status_code = 200L), class = "response")
    },
    .package = "httr"
  )

  res <- post_fn(
    url = "https://example.test/api",
    headers = list("Content-Type" = "application/json",
                   "Authorization" = "Bearer clave"),
    body = list(model = "demo", n = 1),
    timeout = 99
  )

  expect_equal(captured$url, "https://example.test/api")
  expect_equal(captured$encode, "raw")
  # body is serialized to JSON with auto_unbox (scalars, not arrays)
  expect_match(as.character(captured$body), "\"model\":\"demo\"", fixed = TRUE)
  expect_match(as.character(captured$body), "\"n\":1", fixed = TRUE)
})

---
title: "Extracci\u00f3n de palabas clave"
date: "`r Sys.Date()`"
author: "Agust\u00edn Nieto"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{extraccion_palabras_clave}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
  
```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = FALSE,
  comment = "#>"
)
```

## Introducción

En este artículo presentamos las utilidades de la función `acep_extract()` para la extracción de palabras clave en un corpus de notas sobre la conflictividad laboral en la industria pesquera argentina con un enfoque de diccionario.

## El corpus de notas

Para que este artículo pueda reconstruirse en CRAN sin depender de descargas externas, los ejemplos ejecutables usan `ACEP::acep_bases$lc_720`, una muestra incluida en el paquete. Se compone de `r nrow(ACEP::acep_bases$lc_720)` notas y `r length(ACEP::acep_bases$lc_720)` variables: `r paste0(names(ACEP::acep_bases$lc_720), collapse = ', ')`. En una sesión interactiva con conexión también se puede cargar el corpus completo de Revista Puerto con `acep_load_base(acep_bases$rp_mdp)`.

```{r setup, eval=require("tibble"), message=FALSE}
# Cargamos la librería ACEP
library(ACEP)

# Cargamos una muestra incluida en el paquete
rev_puerto <- acep_bases$lc_720

# Imprimimos la base en consola
rev_puerto
```

##  Los diccionarios

Una vez cargada la base de notas vamos a crear variables numéricas y una de carácter que contengan las frecuencias de palabras totales, la frecuencia de palabras de cada diccionario usado para cada una de las notas y las palabras clave mencionadas en cada nota. En esta parte del código haremos uso de las funciones `acep_count()` y `acep_extract()`. También crearemos diccionarios breves para identificar menciones a conflictos, huelgas y actores colectivos.

```{r diccionarios0, eval=require("tibble"), message=FALSE}
# Creamos el diccionario de palabras que refieren a huelgas
dicc_huelgas <- c("en paro", "al paro", "huelga", "huelguistas", "paro y movil",
                  "paro de actividades", "conciliación obligatoria", "un paro", 
                  "paro total", "paro parcial", "trabajo a reglamento", 
                  "el paro", "de brazos caídos")

# Cargamos el diccionario de palabras que refieren a conflictividad
dicc_conflictos_base <- c("conflicto", "conflictos", "protesta", "protestas",
                          "reclamo", "reclamos", "paro", "huelga",
                          "movilización", "manifestación")
dicc_conflictos <- unique(c(dicc_conflictos_base, dicc_huelgas))

# Creamos la variable con la frecuencia de palabras que refieren a conflictividad
rev_puerto$frec_conflictos <- acep_count(rev_puerto$nota, dicc_conflictos)

# Creamos la variable con la frecuencia de palabras que refieren a huelgas
rev_puerto$frec_huelgas <- acep_count(rev_puerto$nota, dicc_huelgas)

# Creamos el diccionario de palabras que refieren a actores colectivos
dicc_actores <- c("trabajadores", "docentes", "sindicato", "vecinos",
                  "municipal", "gobierno")

# Creamos la variable con la frecuencia de palabras que 
# refieren a actores colectivos
rev_puerto$frec_actores <- acep_count(rev_puerto$nota, dicc_actores)

# Imprimimos la base en consola
rev_puerto

```

Ahora vamos a usar la función `acep_extract()` para extraer las palabras clave de los diccionarios de conflictividad, huelgas y actores colectivos que aparecen en cada una de las notas de la muestra.

```{r diccionarios1, eval=require("tibble")}

# Creamos la variable con las palabras que refieren a conflictividad
rev_puerto$extract_conflictos <- acep_extract(rev_puerto$nota, dicc_conflictos, izq = "")

# Creamos la variable con las palabras que refieren a huelgas
rev_puerto$extract_huelgas <- acep_extract(rev_puerto$nota, dicc_huelgas)

# Creamos la variable con las palabras que 
# refieren a actores colectivos
rev_puerto$extract_actores <- acep_extract(rev_puerto$nota, dicc_actores)

# Imprimimos la base en consola
rev_puerto

```

## Las palabras clave extraídas

Ya construidas las variables nos ocuparemos de poner el foco en el rendimiento de la función `acep_extract()`. Seleccionaremos las columnas referidas a las extracciones de conflictos y huelgas. Veamos.

```{r ratio, eval=require("tibble"), message=FALSE}
# Seleccionamos las variables de extracción de palabras clave
rev_puerto_huelgas <- rev_puerto[rev_puerto$extract_huelgas != "",]
rev_puerto_actores <- rev_puerto_huelgas[rev_puerto_huelgas$extract_actores != "",]
rev_puerto_seleccion <- rev_puerto_actores[ , c('extract_conflictos', 'extract_huelgas', 'extract_actores')]

# Imprimimos la base en consola
rev_puerto_seleccion

```


## Nota final

A lo largo de este breve tutorial sobre la función `acep_extract()` del paquete ACEP buscamos ejemplificar de qué modo puede ser usada esta función para individualizar cada una de las palabras clave que fueron contabilizadas en las notas con la función `acep_count()`.

---
title: "Conflictividad laboral en la pesca"
date: "`r Sys.Date()`"
author: "Agust\u00edn Nieto"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{conflictividad_soip}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
  
```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = FALSE,
  comment = "#>"
)
```

## Introducción

En este artículo desarrollaremos una introducción al análisis de la conflictividad laboral en la industria pesquera argentina con un enfoque de diccionario en base a las funciones del paquete ACEP. En esta oportunidad pondremos el foco en la conflictividad laboral protagonizada por el Sindicato Obrero de la Industria del Pescado (SOIP) en la ciudad de Mar del Plata entre los años 2009 y 2020.

## El corpus de notas

Para que este artículo pueda reconstruirse en CRAN sin depender de descargas externas, los ejemplos ejecutables usan `ACEP::acep_bases$lc_720`, una muestra incluida en el paquete. Se compone de `r nrow(ACEP::acep_bases$lc_720)` notas y `r length(ACEP::acep_bases$lc_720)` variables: `r paste0(names(ACEP::acep_bases$lc_720), collapse = ', ')`. En una sesión interactiva con conexión también se puede cargar el corpus completo de Revista Puerto con `acep_load_base(acep_bases$rp_mdp)`.

```{r setup, eval=require("tibble"), message=FALSE}
# Cargamos la librería ACEP
library(ACEP)

# Cargamos una muestra incluida en el paquete
rev_puerto <- acep_bases$lc_720

# Imprimimos la base en consola
rev_puerto

```

##  Los diccionarios

Una vez cargada la base de notas vamos a crear variables numéricas que contengan las frecuencias de palabras totales y de cada diccionario usado para cada una de las notas. En esta parte del código haremos uso de dos funciones del paquete ACEP: `acep_frec()` y `acep_count()`. También crearemos diccionarios breves para identificar menciones a conflictos, huelgas y actores colectivos.

```{r diccionarios, eval=require("tibble"), message=FALSE}
# Creamos la variable con la frecuencia de palabras por nota
rev_puerto$frec_palabras <- acep_frec(rev_puerto$nota)

# Creamos el diccionario de palabras que refieren a huelgas
dicc_huelgas <- c("en paro", "al paro", "huelga", "huelguistas", "paro y movil",
                  "paro de actividades", "conciliación obligatoria", "un paro", 
                  "paro total", "paro parcial", "trabajo a reglamento", 
                  "el paro", "de brazos caídos")

# Cargamos el diccionario de palabras que refieren a conflictividad
dicc_conflictos_base <- c("conflicto", "conflictos", "protesta", "protestas",
                          "reclamo", "reclamos", "paro", "huelga",
                          "movilización", "manifestación")
dicc_conflictos <- unique(c(dicc_conflictos_base, dicc_huelgas))

# Creamos la variable con la frecuencia de palabras que refieren a conflictividad
rev_puerto$frec_conflictos <- acep_count(rev_puerto$nota, dicc_conflictos)

# Creamos la variable con la frecuencia de palabras que refieren a huelgas
rev_puerto$frec_huelgas <- acep_count(rev_puerto$nota, dicc_huelgas)

# Creamos el diccionario de palabras que refieren a actores colectivos
dicc_actores <- c("trabajadores", "docentes", "sindicato", "vecinos",
                  "municipal", "gobierno")

# Creamos la variable con la frecuencia de palabras que 
# refieren a actores colectivos
rev_puerto$frec_actores <- acep_count(rev_puerto$nota, dicc_actores)

# Imprimimos la base en consola
rev_puerto

```

## Los índices

Ya construidas las variables de frecuencia de palabras y menciones nos ocuparemos de elaborar nuevas variables con índices de intensidad en base al ratio entre las frecuencias de palabras totales y las menciones de los diccionarios sobre trabajadorxs del pescado, conflictos y huelgas. Para la elaboración de estos índices haremos uso de la función `acep_int()` del paquete ACEP.

```{r ratio, eval=require("tibble"), message=FALSE}
# Creamos la variable con el índice de conflictividad general
rev_puerto$i_conf_gral <- acep_int(rev_puerto$frec_conflictos, 
                                   rev_puerto$frec_palabras)

# Creamos la variable con el índice de incidencia 
# de actores colectivos
rev_puerto$i_actores <- acep_int(rev_puerto$frec_actores,
                                 rev_puerto$frec_palabras)

# Creamos la variable con el índice de huelgas
rev_puerto$i_huelgas <- acep_int(rev_puerto$frec_huelgas, 
                                 rev_puerto$frec_conflictos)

# Filtramos para quedarnos con los índices mayores a 0 
# en la variable del índice de conflictividad general
rev_puerto <- rev_puerto[rev_puerto$i_conf_gral > 0, ]

# Filtramos para quedarnos con los índices mayores a 0
# en el índice de incidencia de actores colectivos
rev_puerto <- rev_puerto[rev_puerto$i_actores > 0, ]

# Imprimimos la base en consola
rev_puerto

```

Al realizar los filtros la base se redujo a `r nrow(rev_puerto)` notas que presentan al menos una mención de una palabra que refiere a conflicto y al menos un término que refiere a actores colectivos.

## Serie temporal de índices

En esta parte del código usaremos la función `acep_sst()` para calcular los índices agrupados por año y mes. Primero construimos la serie de tiempo para la conflictividad general.

```{r temporal0, eval=require("tibble"), message=FALSE}
# Calculamos el índice anual de conflictividad general en el
# ámbito de la industrial del procesado de pescado en tierra
# Pero primero preparamos el marcos de datos para ser procesado 
# por la función acep_sst()
# Estos pasos previos se deben realizar porque en este ejemplo no hicimos uso 
# de la función acep_db() que calcula frecuencia, menciones e intensidad y
# deja el marco de datos resultante en un formato adecuado para ser usado
# con la función acep_sst()
datos <- data.frame(
  fecha = rev_puerto$fecha,
  n_palabras = rev_puerto$frec_palabras,
  conflictos = rev_puerto$frec_conflictos,
  intensidad = rev_puerto$i_conf_gral
)

# Luego construimos los vectores
fecha <- datos$fecha
n_palabras <- datos$n_palabras
conflictos <- datos$conflictos

# Ahora agrupamos por mes la conflictividad general del sector
conf_gral_anio <- acep_sst(datos, st = "anio")

# Imprimimos la base en consola
conf_gral_anio |> head()
```


```{r temporal, eval=require("tibble"), message=FALSE}
# Calculamos el índice mensual de conflictividad general en el
# ámbito de la industrial del procesado de pescado en tierra
# Pero primero preparamos el marcos de datos para ser procesado 
# por la función acep_sst()
datos <- data.frame(
  fecha = rev_puerto$fecha,
  n_palabras = rev_puerto$frec_palabras,
  conflictos = rev_puerto$frec_conflictos,
  intensidad = rev_puerto$i_conf_gral
)

# Nos quedamos con un año presente en la muestra incluida
anio_ejemplo <- names(sort(table(format(as.Date(datos$fecha), "%Y")),
                           decreasing = TRUE))[1]
datos <- datos[format(as.Date(datos$fecha), "%Y") == anio_ejemplo, ]

# Luego construimos los vectores
fecha <- datos$fecha
n_palabras <- datos$n_palabras
conflictos <- datos$conflictos

# Ahora agrupamos por mes la conflictividad general del sector
conf_gral <- acep_sst(datos)

# Imprimimos la base en consola
conf_gral |> head()

```

En la siguiente parte del código construimos la serie de tiempo para la conflictividad huelguística.

```{r temporal_huelgas, eval=require("tibble"), message=FALSE}
# Calculamos el índice mensual de conflictividad huelguística en el
# ámbito de la industrial del procesado de pescado en tierra
# Pero primero preparamos el marcos de datos para ser procesado 
# por la función acep_sst()
datosh <- data.frame(
  fecha = rev_puerto$fecha,
  n_palabras = rev_puerto$frec_palabras,
  conflictos = rev_puerto$frec_huelgas,
  intensidad = rev_puerto$i_huelgas
)

# Nos quedamos con el mismo año de ejemplo
datosh <- datosh[format(as.Date(datosh$fecha), "%Y") == anio_ejemplo, ]

# Luego construimos los vectores
fechah <- datosh$fecha
n_palabrash <- datosh$n_palabras
conflictosh <- datosh$conflictos

# Ahora agrupamos por mes la conflictividad huelguística del sector
huelgas <- acep_sst(datosh)

# Imprimimos la base en consola
huelgas |> head()

```

## Las visualizaciones

En este último apartado haremos uso de las funciones `acep_plot_st()` y `acep_plot_rst()` para visualizar la variación anual y mensual de la conflictividad general en la muestra incluida en el paquete.

```{r plot00, message=FALSE}
# Visualizaremos el índice de conflictividad general 
# agrupado por año para el período 2009-2020
acep_plot_st(
 conf_gral_anio$st,
 conf_gral_anio$frecm,
 t = "Indice anual de conflictividad en la industria pesquera (MdP)",
 ejey = "Menciones del diccionario",
 etiquetax = "vertical"
             )

```

```
# Visualizaremos el índice de conflictividad general 
# agrupado por año para el período 2009-2020
acep_plot_rst(conf_gral_anio, tagx = "vertical")
```

![](plot_vignette.png)

```{r plot01, message=FALSE}
# Visualizaremos el índice de conflictividad general 
# agrupado por mes para el 2012
acep_plot_st(
 conf_gral$st,
 conf_gral$frecm,
 t = "Indice mensual de conflictos en la industria pesquera (MdP)",
 ejey = "Menciones del diccionario",
 etiquetax = "vertical"
             )

```


```{r plot02, message=FALSE}
# Visualizaremos el índice de conflictividad huelguística 
# agrupado por mes para el 2012
acep_plot_st(
 huelgas$st,
 huelgas$frecm,
 t = "Indice mensual de huelgas en la industria pesquera (MdP)",
 ejey = "Menciones del diccionario",
 etiquetax = "vertical"
             )

```

Las distintas métricas permiten explorar la temporalidad de las menciones a conflictividad en el corpus de ejemplo sin depender de recursos externos durante la reconstrucción de la vignette.

## Comentarios finales

A lo largo de este breve tutorial sobre algunas de las funciones del paquete ACEP buscamos ejemplificar de qué modo se puede adoptar un enfoque de diccionario para realizar un primer análisis exploratorio de un corpus de notas periodísticas. Los resultados son alentadores. Con la combinación de distintos diccionarios se pudo identificar la temporalidad de la conflictividad protagonizada por lxs obrerxs del pescado en la ciudad de Mar del Plata. En próximos artículos avanzaremos con otras funciones del paquete ACEP para el análisis computacional de la conflictividad en la industria pesquera argentina.

---
title: "Tokenizar con ACEP"
date: "`r Sys.Date()`"
author: "Diego Pacheco"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{tokenizar_con_acep}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = FALSE,
  comment = "#>"
)
```

## Funciones a presentar:

En este artículo se explicarán los procesos que realizan las funciones:

- `acep_token()`

- `acep_token_table()`

- `acep_token_plot()`


## Función `acep_token()`

En primer lugar cargamos la librería *ACEP*. Luego, usamos textos incluidos en el ejemplo para que la vignette no dependa de descargas externas.

```{r cargar base,results = FALSE, warning = FALSE, message = FALSE, eval=require("tibble")}
library(ACEP)

base <- c(
  "La Fraternidad anunció un paro de trenes en noviembre.",
  "Trabajadores del transporte reclamaron mejoras salariales."
)
```

Ejecutamos la función `acep_token()` para los primeros dos elementos de la base.

¿Cuál es el resultado?

La función `acep_token` toma el vector y realiza diferentes acciones:

- Verifica que el objeto entregado sea un vector (de lo contrario indica un mensaje de advertencia)

- Cambia todo el texto a minúsculas

- Crea un nuevo data frame con la siguiente información:

  - texto_id (columna que numera el documento)
  
  - tokens (el token propiamente)
  
Cabe mencionar que los tokens quedarán identificados a través de los espacios en blanco, es decir, cada palabra es un token.

```{r token, eval=require("tibble"), message=FALSE}
tweets <- acep_token(base[1:2])

head(tweets)
```

En este resultado podemos ver cómo la función identifica a cada observación como un documento aparte (en este caso, cada tweet es un documento identificado en la columna 'texto_id'). Por su parte, la columna 'tokens' es la columna en la que la función aísla cada token.

## Función `acep_token_table()`

Una vez que tenemos el data frame creado a partir de la función `acep_token()`, podemos utilizar la `función acep_table()` para obtener un nuevo data frame que nos proveerá la siguiente información:

- token

- frec (frecuencia que aparece ese token)

- prop (el peso que tiene ese token en el total del corpus)

Para obtener una tabla que tenga sentido, podemos utilizar en primer lugar la función `acep_clean()` para deshacernos de los stopwords, urls, menciones, hashtags,  etc.

```{r limpia, message=FALSE}
base_limpia <- acep_clean(base)
```

En segunda instancia, creamos un nuevo objeto derivado de la base limpia con `acep_token()` que nos devuelve, como vimos previamente, una tabla con la información de cada token.

```{r tabla, eval=require("tibble"), message=FALSE}
tabla_tokenizada <- acep_token(base_limpia)
head(tabla_tokenizada)
```

Por último, aplicamos la función `acep_token_table()` a la columna 'tokens'.

Por defecto, `acep_token_table()` nos devuelve los 10 primeros registros, es decir, las 10 palabras con mayor frecuencia.

Si quisiéramos modificar la cantidad de palabras, debemos modificar el parámetro 'u' =

Suponiendo que queremos obtener los 20 token de mayor frecuencia: `acep_token_table(tabla_tokenizada$token, u = 20)`

```{r table, eval=require("tibble"), message=FALSE}
acep_token_table(tabla_tokenizada$tokens, u = 15)
```

## Función `acep_token_plot()`

Esta función permite visualizar en un gráfico de barras, los tokens más frecuentes.

A partir del resultado de `acep_token()`, podemos obtener el gráfico con los tokens más frecuentes.

Por defecto, el resultado serán los 10 tokens más frecuentes, sin embargo, a través del parámetro "u =" podemos modificar la cantidad de tokens a visualizar.

En este ejemplo, tomamos la tabla tokenizada y le indicamos la columna "token" a visualizar.

```{r plot, message=FALSE}
acep_token_plot(tabla_tokenizada$tokens, u = 15)
```

---
title: "Extraer S-V-O con ACEP"
date: "`r Sys.Date()`"
author: "Agust\u00edn Nieto"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{extraccion_de_svo_con_acep}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = FALSE,
  comment = "#>"
)
```

## Función a presentar:

En este artículo se explicarán los procesos que realizan las funciones:

- `acep_postag()`

- `acep_upos()`

- `acep_svo()`


## Función `acep_postag()`

Esta función realiza etiquetado POS, lematización, tokenización, extracción de entidades y georreferenciación. Para llevar a cabo estas tareas `acep_postag` envuelve y articula funciones de tres librerías: *spacyr*, *rsyntax* y *tidygeocoder*.

En primer lugar, cargamos la librería *ACEP*. Luego, cargamos un vector de titulares de portales noticiosos sobre notas referidas a conflictos. Con esta selección de titulares haremos la prueba. También agregamos una última oración unimembre como ejemplo extremo de oración no procesable con la función `acep_svo()`.

```{r titulares, results = TRUE, warning = FALSE, message = FALSE, eval=require("tibble")}
library(ACEP)

titulares <- c(acep_bases$titulares, "Hola mundo.")

titulares
```

Ejecutamos la función `acep_postag()` para los titulares contenidos en el vector.

¿Cuál es el resultado?

La función `acep_postag` toma el vector de textos y realiza diferentes acciones:

- Verifica que el objeto entregado sea un vector de tipo caracter (de lo contrario imprime un mensaje de advertencia)

- Verifica que el parámetro 'core' sean un modelo válido (de lo contrario devuelve un mensaje de advertencia)

- Verifica que los parámetros 'bajar_core', 'inst_spacy', 'inst_miniconda', 'inst_reticulate' ingresados sean valores booleanos (de lo contrario envía un mensaje de advertencia)

- Crea una lista de seis objetos de tipo data frame:

  - **texto_tag**: es un objeto de clases 'tokenIndex' que sirve como input para la función `acep_svo()`. Contiene 20 variables que reúnen información de los procesos de tokenización, lematización, etiquetado pos, etiquetado de relaciones de dependencia, etiquetado NER y morfológico. 
  
  - **texto_tag_entity**: es un objeto de clase 'spacyr_parsed' con una columna que identifica los distintos tipos de entidades.
  
  - **texto_only_entity**: es un 'data.frame' con 4 variables que contiene solo las entidades identificadas.
  
  - **texto_only_entity_loc**: es un 'data.frame' de 7 variables que permite la georreferenciación de las entidades identificadas como tipo 'LOC'.
  
  - **texto_nounphrase**: es un objeto de clase 'spacyr_parsed' con una columna que identifica las distintas frases compuestas por sustantivos (nounphrase).
  
  - **texto_only_nounphrase**: es un 'data.frame' con tres variables que identifica y filtra los sustantivos que forman frases: 'las_principales_ciudades'.
  
Cabe mencionar que los textos ingresados son tokenizados, es decir, cada palabra es un token.

```{r tt, eval=FALSE}
titulares_tags <- acep_postag(
  texto = titulares,
  core = "es_core_news_lg", # valor por defecto
  bajar_core = FALSE,       # el valor por defecto es TRUE
  inst_spacy = FALSE,       # valor por defecto
  inst_miniconda = FALSE,   # valor por defecto
  inst_reticulate = FALSE   # valor por defecto
)

str(titulares_tags)
```

```{r postag00, eval=require("tibble"), message=FALSE, warning=FALSE, echo=FALSE}
texto_entity <- subset(acep_bases$spacy_postag, entity != "")
titulares_tags <- list(
  texto_tag = acep_bases$spacy_postag,
  texto_tag_entity = texto_entity,
  texto_only_entity = data.frame(
    entity = texto_entity$token,
    entity_type = sub("_.*", "", texto_entity$entity),
    doc_id = texto_entity$doc_id,
    sentence = texto_entity$sentence
  ),
  texto_only_entity_loc = data.frame(
    entity_ = "Mar del Plata",
    doc_id = 1L,
    sentence = 1L,
    entity = "Mar_del_Plata",
    entity_type = "LOC",
    lat = -38.0055,
    long = -57.5426
  ),
  texto_nounphrase = acep_bases$spacy_postag,
  texto_only_nounphrase = subset(acep_bases$spacy_postag, nounphrase != "")
)
str(titulares_tags)
```


En este resultado podemos ver cómo la función crea los seis marcos de datos con información relevante sobre el contenido y la forma de los textos ingresados. En nuestro caso, textos referidos a conflictos.

Veamos con un poco más de detalle cada uno de los marcos de datos creados con la función `acep_postag()`:

### texto_tag

Como adelantamos, el primer objeto de la lista es un marco de datos estructurado para servir de input de la función `acep_svo()`.

```{r postag01}
head(titulares_tags$texto_tag, n = 12)
```

### texto_tag_entity

El segundo objeto de la lista es un marco de datos que reescribe la columna 'pos' con la etiqueta 'ENTITY' e identifica el tipo de entidad en la columna 'entity_type'. Si la entidad detectada está compuesta por más de un token, la función colapsa todos los tokens referidos a esa entidad en una sola celda de la columna token. Ejemplo: 'Ciudad_de_Buenos_Aires'.

```{r postag02}
head(titulares_tags$texto_tag_entity, n = 14)
```

### texto_only_entity
El tercer objeto de la lista es el marco de datos 'texto_tag_entity' filtrado por el valor 'ENTITY' de la columna 'pos'.

```{r postag03}
head(titulares_tags$texto_only_entity, n = 10)
```

### texto_only_entity_loc

El cuarto objeto de la lista es el marco de datos 'texto_only_entity' filtrado por el valor 'LOC' de la columna 'entity_type' y procesado con la función 'geo()' del paquete *tidygeocoder*.

```{r postag04}
head(titulares_tags$texto_only_entity_loc[ , c(1:3, 6:7)], n = 4)
```

### texto_nounphrase

El quinto objeto de la lista es un marco de datos que reescribe la columna 'pos' con la etiqueta 'nounphrase'. Si el sustantivo detectado está compuesto por más de un token, la función colapsa todos los tokens referidos a ese sustantivo en una sola celda de la columna token. Ejemplo: 'las_principales_ciudades'.

```{r postag05}
head(titulares_tags$texto_nounphrase[ , c(1:2, 4, 6)], n = 10)
```

### texto_only_nounphrase

El sexto objeto de la lista es el marco de datos 'texto_nounphrase' filtrado por el valor 'nounphrase' de la columna 'pos'.

```{r postag06}
head(titulares_tags$texto_only_nounphrase, n = 10)
```

## Función `acep_upos()`

Esta función realiza etiquetado POS, lematización, tokenización, pero no realiza extracción de entidades y, por ende, no puede hacer georreferenciación. Para llevar a cabo estas tareas acep_upos envuelve la función udpipe() de la librería homonimia. 

**Advertencia**: ponemos esta alternativa a `acep_postag` porque la instalación de *reticulate*, *anaconda* y las librerías de Python necesarias para el funcionamiento de *spacyr* pueden traer más de un problema.

Repetimos el proceso realizado con la función `acep_postag`: activamos la librería *ACEP*, luego, cargamos un vector de titulares de portales noticiosos sobre notas referidas a conflictos para su prueba, y agregamos una última oración unimembre como ejemplo extremo de oración no procesable con la función `acep_svo()`.

```{r titulares2, results = TRUE, warning = FALSE, message = FALSE, eval=require("tibble")}
library(ACEP)

titulares <- c(acep_bases$titulares, "Hola mundo.")

titulares
```

Ejecutamos la función `acep_upos()` para los titulares contenidos en el vector.

¿Cuál es el resultado?

La función `acep_upos` toma el vector de textos y realiza diferentes acciones:

- Verifica que el objeto entregado sea un vector de tipo caracter (de lo contrario imprime un mensaje de advertencia)

- Verifica que el parámetro 'modelo' sean un modelo válido (de lo contrario devuelve un mensaje de advertencia)

- Crea un objeto de clases 'tokenIndex' que sirve como input para la función `acep_svo()`. Contiene 17 variables que reúnen información de los procesos de tokenización, lematización, etiquetado pos, etiquetado de relaciones de dependencia y morfológico. 
  
Cabe mencionar que, al igual que acep_postag, los textos ingresados son tokenizados, es decir, cada palabra es un token.

```{r t, eval=FALSE}
titulares_utags <- acep_upos(
  texto = titulares,
  modelo = "spanish" # valor por defecto
)

str(titulares_utags)
```

```{r upos, eval=require("tibble"), message=FALSE, warning=FALSE, echo=FALSE}
titulares_utags <- acep_bases$spacy_postag
str(titulares_utags)
```

En este resultado podemos ver cómo la función crea el input para usar con `acep_svo()`. Hay que tener en cuenta que *udpipe* y *spacyr* usan modelos distintos para el etiquetado pos y las relaciones de dependencia, así como para parsear el texto en oraciones. Por ende, el resultado obtenido de la función `acep_svo` puede ser ligeramente diferente si usamos uno u otro modelo de etiquetado. En el ejemplo que sigue usaremos el output de la función `acep_upos`.

## Función `acep_svo()`

Una vez que tenemos la lista, con 6 marcos de datos, creada a partir de la función `acep_upos()`, podemos utilizar la función `acep_svo()` para obtener un listado nuevo, con otros 6 marcos de datos, que nos proveerá la siguiente información:

- acep_annotate_svo: es un marco de datos con 22 variables. Mantiene la estructura del objeto ingresado a la función (titulares_utags) y agrega 5 nuevas variables ('s_v_o', 's_v_o_id', 's_v_o_fill', 's_p', 'conjugaciones').

- acep_pro_svo: es un marco de datos con 13 variables ('doc_id', 'oracion_id', 'eventos', 'sujeto_svo', 'root', 'objeto', 'sujeto', 'predicado', 'verbo', 'lemma_verb', 'aux_verbos', 'entidades', 'sust_pred') que reúne los tripletes SVO e información de contexto. Ejemplo de extracción SVO: "gremios -> convocan -> paro presencialidad".

- acep_list_svo: es un marco de datos con 6 variables ('doc_id', 'oracion_id', 'eventos', 'sujeto', 'verbo' 'objeto') que procesa los tripletes SVO y los devuelve separados en sujeto, verbo, objeto. Ejemplo de extracción SVO: "gremios" "convocan" "paro presencialidad".

- acep_sp: es un marco de datos con 9 variables ('doc_id', 'oracion_id', 'sujeto', 'predicado', 'verbo', 'lemma_verb', 'aux_verbos', 'entidades', 'sust_pred') que ofrece más información de contexto como entidades, sustantivos dentro de la oración analizada, etc.

- acep_lista_lemmas: es un marco de datos con dos variables ('lemma', 'n') que ofrece la frecuencia absoluta de las palabras lemmatizadas presentes en el corpus analizado.

- acep_no_procesadas: es un marco de datos con tres variables ('doc_id', 'oracion_id', 'oracion') que ofrece la frecuencia absoluta de las palabras lemmatizadas presentes en el corpus analizado.

```{r svo01}
titulares_svo <- acep_svo(titulares_utags)

str(titulares_svo)
```

Veamos con un poco más de detalle cada uno de los marcos de datos creados con la función `acep_svo()`.

### acep_annotate_svo

Es el marco de datos inicial procesado con las funciones del paquete *rsyntax*.

```{r svo02, eval=require("tibble")}
head(titulares_svo$acep_annotate_svo, n=20)
```

### acep_annotate_svo

Este marco de datos contiene las oraciones procesables con la identificación y extracción de sujeto-verbo-objeto y contexto.

```{r svo03, eval=require("tibble")}
head(titulares_svo$acep_pro_svo, n=10)
```

### acep_list_svo

Este marco de datos es una versión reducida del data.frame 'acep_pro_svo'. Solo contiene los tripletes sujeto-verbo-objeto en versión colapsada ('gremios -> convocan -> paro') y en versión separada (una columna para 'sujeto', otra para 'verbo' y otra para 'objeto').

```{r svo04, eval=require("tibble")}
head(titulares_svo$acep_list_svo, n=10)
```

### acep_sp

Este marco de datos contiene los 'sujetos' y los 'predicados' identificados en cada oración y una aproximación a 'entidades', 'sustantivos' y 'verbos auxiliares' como contexto que ayuda a mejorar la extracción de sujetos y objetos de la acción.

```{r svo05, eval=require("tibble")}
head(titulares_svo$acep_sp, n=10)
```

### acep_lista_lemmas

Este marco de datos es un análisis de frecuencias absolutas de lemmas presentes en el corpus procesado.

```{r svo06, eval=require("tibble")}
head(titulares_svo$acep_lista_lemmas, n=10)
```

### acep_no_procesadas

Este marco de datos contiene las oraciones que no pudieron ser procesadas por no ser posible identificar sujeto y predicado.

```{r svo07, eval=require("tibble")}
head(titulares_svo$acep_no_procesadas, n=10)
```

## Nota final

Los resultados obtenidos en este ejemplo son muy prometedores, pero la realidad es que la bondad de los resultados está determinada por la complejidad de las oraciones. Es verdad que los textos ingresados en este ejemplo fueron tomados de portales de noticias, pero también es verdad que no todos los títulos son igual de descriptivos y muchos son oraciones unimembres. Sin embargo, con los cuidados del caso, los resultados arrojados por la función `acep_svo()` pueden ser muy útiles para una primera aproximación exploratoria de eventos de protesta en un corpus extenso de notas.

---
title: "Extraer S-V-O con ACEP"
date: "`r Sys.Date()`"
author: "Agust\u00edn Nieto"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{extraccion_de_svo_con_acep}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = FALSE,
  comment = "#>"
)
```

## Función a presentar:

En este artículo se explicarán los procesos que realizan las funciones:

- `acep_postag()`

- `acep_upos()`

- `acep_svo()`


## Función `acep_postag()`

Esta función realiza etiquetado POS, lematización, tokenización, extracción de entidades y georreferenciación. Para llevar a cabo estas tareas `acep_postag` envuelve y articula funciones de tres librerías: *spacyr*, *rsyntax* y *tidygeocoder*.

En primer lugar, cargamos la librería *ACEP*. Luego, cargamos un vector de titulares de portales noticiosos sobre notas referidas a conflictos. Con esta selección de titulares haremos la prueba. También agregamos una última oración unimembre como ejemplo extremo de oración no procesable con la función `acep_svo()`.

```{r titulares, results = TRUE, warning = FALSE, message = FALSE, eval=require("tibble")}
library(ACEP)

titulares <- c(acep_bases$titulares, "Hola mundo.")

titulares
```

Ejecutamos la función `acep_postag()` para los titulares contenidos en el vector.

¿Cuál es el resultado?

La función `acep_postag` toma el vector de textos y realiza diferentes acciones:

- Verifica que el objeto entregado sea un vector de tipo caracter (de lo contrario imprime un mensaje de advertencia)

- Verifica que el parámetro 'core' sean un modelo válido (de lo contrario devuelve un mensaje de advertencia)

- Verifica que los parámetros 'bajar_core', 'inst_spacy', 'inst_miniconda', 'inst_reticulate' ingresados sean valores booleanos (de lo contrario envía un mensaje de advertencia)

- Crea una lista de seis objetos de tipo data frame:

  - **texto_tag**: es un objeto de clases 'tokenIndex' que sirve como input para la función `acep_svo()`. Contiene 20 variables que reúnen información de los procesos de tokenización, lematización, etiquetado pos, etiquetado de relaciones de dependencia, etiquetado NER y morfológico. 
  
  - **texto_tag_entity**: es un objeto de clase 'spacyr_parsed' con una columna que identifica los distintos tipos de entidades.
  
  - **texto_only_entity**: es un 'data.frame' con 4 variables que contiene solo las entidades identificadas.
  
  - **texto_only_entity_loc**: es un 'data.frame' de 7 variables que permite la georreferenciación de las entidades identificadas como tipo 'LOC'.
  
  - **texto_nounphrase**: es un objeto de clase 'spacyr_parsed' con una columna que identifica las distintas frases compuestas por sustantivos (nounphrase).
  
  - **texto_only_nounphrase**: es un 'data.frame' con tres variables que identifica y filtra los sustantivos que forman frases: 'las_principales_ciudades'.
  
Cabe mencionar que los textos ingresados son tokenizados, es decir, cada palabra es un token.

```{r tt, eval=FALSE}
titulares_tags <- acep_postag(
  texto = titulares,
  core = "es_core_news_lg", # valor por defecto
  bajar_core = FALSE,       # el valor por defecto es TRUE
  inst_spacy = FALSE,       # valor por defecto
  inst_miniconda = FALSE,   # valor por defecto
  inst_reticulate = FALSE   # valor por defecto
)

str(titulares_tags)
```

```{r postag00, eval=require("tibble"), message=FALSE, warning=FALSE, echo=FALSE}
texto_entity <- subset(acep_bases$spacy_postag, entity != "")
titulares_tags <- list(
  texto_tag = acep_bases$spacy_postag,
  texto_tag_entity = texto_entity,
  texto_only_entity = data.frame(
    entity = texto_entity$token,
    entity_type = sub("_.*", "", texto_entity$entity),
    doc_id = texto_entity$doc_id,
    sentence = texto_entity$sentence
  ),
  texto_only_entity_loc = data.frame(
    entity_ = "Mar del Plata",
    doc_id = 1L,
    sentence = 1L,
    entity = "Mar_del_Plata",
    entity_type = "LOC",
    lat = -38.0055,
    long = -57.5426
  ),
  texto_nounphrase = acep_bases$spacy_postag,
  texto_only_nounphrase = subset(acep_bases$spacy_postag, nounphrase != "")
)
str(titulares_tags)
```


En este resultado podemos ver cómo la función crea los seis marcos de datos con información relevante sobre el contenido y la forma de los textos ingresados. En nuestro caso, textos referidos a conflictos.

Veamos con un poco más de detalle cada uno de los marcos de datos creados con la función `acep_postag()`:

### texto_tag

Como adelantamos, el primer objeto de la lista es un marco de datos estructurado para servir de input de la función `acep_svo()`.

```{r postag01}
head(titulares_tags$texto_tag, n = 12)
```

### texto_tag_entity

El segundo objeto de la lista es un marco de datos que reescribe la columna 'pos' con la etiqueta 'ENTITY' e identifica el tipo de entidad en la columna 'entity_type'. Si la entidad detectada está compuesta por más de un token, la función colapsa todos los tokens referidos a esa entidad en una sola celda de la columna token. Ejemplo: 'Ciudad_de_Buenos_Aires'.

```{r postag02}
head(titulares_tags$texto_tag_entity, n = 14)
```

### texto_only_entity
El tercer objeto de la lista es el marco de datos 'texto_tag_entity' filtrado por el valor 'ENTITY' de la columna 'pos'.

```{r postag03}
head(titulares_tags$texto_only_entity, n = 10)
```

### texto_only_entity_loc

El cuarto objeto de la lista es el marco de datos 'texto_only_entity' filtrado por el valor 'LOC' de la columna 'entity_type' y procesado con la función 'geo()' del paquete *tidygeocoder*.

```{r postag04}
head(titulares_tags$texto_only_entity_loc[ , c(1:3, 6:7)], n = 4)
```

### texto_nounphrase

El quinto objeto de la lista es un marco de datos que reescribe la columna 'pos' con la etiqueta 'nounphrase'. Si el sustantivo detectado está compuesto por más de un token, la función colapsa todos los tokens referidos a ese sustantivo en una sola celda de la columna token. Ejemplo: 'las_principales_ciudades'.

```{r postag05}
head(titulares_tags$texto_nounphrase[ , c(1:2, 4, 6)], n = 10)
```

### texto_only_nounphrase

El sexto objeto de la lista es el marco de datos 'texto_nounphrase' filtrado por el valor 'nounphrase' de la columna 'pos'.

```{r postag06}
head(titulares_tags$texto_only_nounphrase, n = 10)
```

## Función `acep_upos()`

Esta función realiza etiquetado POS, lematización, tokenización, pero no realiza extracción de entidades y, por ende, no puede hacer georreferenciación. Para llevar a cabo estas tareas acep_upos envuelve la función udpipe() de la librería homonimia. 

**Advertencia**: ponemos esta alternativa a `acep_postag` porque la instalación de *reticulate*, *anaconda* y las librerías de Python necesarias para el funcionamiento de *spacyr* pueden traer más de un problema.

Repetimos el proceso realizado con la función `acep_postag`: activamos la librería *ACEP*, luego, cargamos un vector de titulares de portales noticiosos sobre notas referidas a conflictos para su prueba, y agregamos una última oración unimembre como ejemplo extremo de oración no procesable con la función `acep_svo()`.

```{r titulares2, results = TRUE, warning = FALSE, message = FALSE, eval=require("tibble")}
library(ACEP)

titulares <- c(acep_bases$titulares, "Hola mundo.")

titulares
```

Ejecutamos la función `acep_upos()` para los titulares contenidos en el vector.

¿Cuál es el resultado?

La función `acep_upos` toma el vector de textos y realiza diferentes acciones:

- Verifica que el objeto entregado sea un vector de tipo caracter (de lo contrario imprime un mensaje de advertencia)

- Verifica que el parámetro 'modelo' sean un modelo válido (de lo contrario devuelve un mensaje de advertencia)

- Crea un objeto de clases 'tokenIndex' que sirve como input para la función `acep_svo()`. Contiene 17 variables que reúnen información de los procesos de tokenización, lematización, etiquetado pos, etiquetado de relaciones de dependencia y morfológico. 
  
Cabe mencionar que, al igual que acep_postag, los textos ingresados son tokenizados, es decir, cada palabra es un token.

```{r t, eval=FALSE}
titulares_utags <- acep_upos(
  texto = titulares,
  modelo = "spanish" # valor por defecto
)

str(titulares_utags)
```

```{r upos, eval=require("tibble"), message=FALSE, warning=FALSE, echo=FALSE}
titulares_utags <- acep_bases$spacy_postag
str(titulares_utags)
```

En este resultado podemos ver cómo la función crea el input para usar con `acep_svo()`. Hay que tener en cuenta que *udpipe* y *spacyr* usan modelos distintos para el etiquetado pos y las relaciones de dependencia, así como para parsear el texto en oraciones. Por ende, el resultado obtenido de la función `acep_svo` puede ser ligeramente diferente si usamos uno u otro modelo de etiquetado. En el ejemplo que sigue usaremos el output de la función `acep_upos`.

## Función `acep_svo()`

Una vez que tenemos la lista, con 6 marcos de datos, creada a partir de la función `acep_upos()`, podemos utilizar la función `acep_svo()` para obtener un listado nuevo, con otros 6 marcos de datos, que nos proveerá la siguiente información:

- acep_annotate_svo: es un marco de datos con 22 variables. Mantiene la estructura del objeto ingresado a la función (titulares_utags) y agrega 5 nuevas variables ('s_v_o', 's_v_o_id', 's_v_o_fill', 's_p', 'conjugaciones').

- acep_pro_svo: es un marco de datos con 13 variables ('doc_id', 'oracion_id', 'eventos', 'sujeto_svo', 'root', 'objeto', 'sujeto', 'predicado', 'verbo', 'lemma_verb', 'aux_verbos', 'entidades', 'sust_pred') que reúne los tripletes SVO e información de contexto. Ejemplo de extracción SVO: "gremios -> convocan -> paro presencialidad".

- acep_list_svo: es un marco de datos con 6 variables ('doc_id', 'oracion_id', 'eventos', 'sujeto', 'verbo' 'objeto') que procesa los tripletes SVO y los devuelve separados en sujeto, verbo, objeto. Ejemplo de extracción SVO: "gremios" "convocan" "paro presencialidad".

- acep_sp: es un marco de datos con 9 variables ('doc_id', 'oracion_id', 'sujeto', 'predicado', 'verbo', 'lemma_verb', 'aux_verbos', 'entidades', 'sust_pred') que ofrece más información de contexto como entidades, sustantivos dentro de la oración analizada, etc.

- acep_lista_lemmas: es un marco de datos con dos variables ('lemma', 'n') que ofrece la frecuencia absoluta de las palabras lemmatizadas presentes en el corpus analizado.

- acep_no_procesadas: es un marco de datos con tres variables ('doc_id', 'oracion_id', 'oracion') que ofrece la frecuencia absoluta de las palabras lemmatizadas presentes en el corpus analizado.

```{r svo01, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")}
titulares_svo <- acep_svo(titulares_utags)

str(titulares_svo)
```

Veamos con un poco más de detalle cada uno de los marcos de datos creados con la función `acep_svo()`.

### acep_annotate_svo

Es el marco de datos inicial procesado con las funciones del paquete *rsyntax*.

```{r svo02, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")}
head(titulares_svo$acep_annotate_svo, n=20)
```

### acep_annotate_svo

Este marco de datos contiene las oraciones procesables con la identificación y extracción de sujeto-verbo-objeto y contexto.

```{r svo03, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")}
head(titulares_svo$acep_pro_svo, n=10)
```

### acep_list_svo

Este marco de datos es una versión reducida del data.frame 'acep_pro_svo'. Solo contiene los tripletes sujeto-verbo-objeto en versión colapsada ('gremios -> convocan -> paro') y en versión separada (una columna para 'sujeto', otra para 'verbo' y otra para 'objeto').

```{r svo04, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")}
head(titulares_svo$acep_list_svo, n=10)
```

### acep_sp

Este marco de datos contiene los 'sujetos' y los 'predicados' identificados en cada oración y una aproximación a 'entidades', 'sustantivos' y 'verbos auxiliares' como contexto que ayuda a mejorar la extracción de sujetos y objetos de la acción.

```{r svo05, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")}
head(titulares_svo$acep_sp, n=10)
```

### acep_lista_lemmas

Este marco de datos es un análisis de frecuencias absolutas de lemmas presentes en el corpus procesado.

```{r svo06, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")}
head(titulares_svo$acep_lista_lemmas, n=10)
```

### acep_no_procesadas

Este marco de datos contiene las oraciones que no pudieron ser procesadas por no ser posible identificar sujeto y predicado.

```{r svo07, eval=requireNamespace("rsyntax", quietly = TRUE) && require("tibble")}
head(titulares_svo$acep_no_procesadas, n=10)
```

## Nota final

Los resultados obtenidos en este ejemplo son muy prometedores, pero la realidad es que la bondad de los resultados está determinada por la complejidad de las oraciones. Es verdad que los textos ingresados en este ejemplo fueron tomados de portales de noticias, pero también es verdad que no todos los títulos son igual de descriptivos y muchos son oraciones unimembres. Sin embargo, con los cuidados del caso, los resultados arrojados por la función `acep_svo()` pueden ser muy útiles para una primera aproximación exploratoria de eventos de protesta en un corpus extenso de notas.

# acep_clean current behavior is characterized

    {
      "type": "character",
      "attributes": {},
      "value": ["suteba paro reclaman mejoras salariales", "marcha mar plata"]
    }

# acep_count current behavior is characterized

    {
      "type": "integer",
      "attributes": {},
      "value": [3, 2, 0]
    }

# acep_svo stable return schemas are characterized

    {
      "type": "list",
      "attributes": {
        "names": {
          "type": "character",
          "attributes": {},
          "value": ["acep_annotate_svo", "acep_pro_svo", "acep_list_svo", "acep_sp", "acep_lista_lemmas", "acep_no_procesadas"]
        }
      },
      "value": [
        {
          "type": "list",
          "attributes": {
            "names": {
              "type": "character",
              "attributes": {},
              "value": ["class", "dim", "names"]
            }
          },
          "value": [
            {
              "type": "character",
              "attributes": {},
              "value": ["tokenIndex", "data.table", "data.frame"]
            },
            {
              "type": "integer",
              "attributes": {},
              "value": [15, 25]
            },
            {
              "type": "character",
              "attributes": {},
              "value": ["doc_id", "sentence", "token_id", "token", "lemma", "pos", "parent", "relation", "entity", "nounphrase", "whitespace", "is_upper", "is_title", "is_quote", "ent_iob_", "ent_iob", "is_left_punct", "is_right_punct", "morph", "sent", "s_v_o", "s_v_o_id", "s_v_o_fill", "s_p", "conjugaciones"]
            }
          ]
        },
        {
          "type": "list",
          "attributes": {
            "names": {
              "type": "character",
              "attributes": {},
              "value": ["class", "dim", "names"]
            }
          },
          "value": [
            {
              "type": "character",
              "attributes": {},
              "value": ["data.frame"]
            },
            {
              "type": "integer",
              "attributes": {},
              "value": [1, 13]
            },
            {
              "type": "character",
              "attributes": {},
              "value": ["doc_id", "oracion_id", "eventos", "sujeto_svo", "root", "objeto", "sujeto", "predicado", "verbo", "lemma_verb", "aux_verbos", "entidades", "sust_pred"]
            }
          ]
        },
        {
          "type": "list",
          "attributes": {
            "names": {
              "type": "character",
              "attributes": {},
              "value": ["class", "dim", "names"]
            }
          },
          "value": [
            {
              "type": "character",
              "attributes": {},
              "value": ["data.frame"]
            },
            {
              "type": "integer",
              "attributes": {},
              "value": [1, 6]
            },
            {
              "type": "character",
              "attributes": {},
              "value": ["doc_id", "oracion_id", "eventos", "sujeto", "verbo", "objeto"]
            }
          ]
        },
        {
          "type": "list",
          "attributes": {
            "names": {
              "type": "character",
              "attributes": {},
              "value": ["class", "dim", "names"]
            }
          },
          "value": [
            {
              "type": "character",
              "attributes": {},
              "value": ["data.frame"]
            },
            {
              "type": "integer",
              "attributes": {},
              "value": [1, 9]
            },
            {
              "type": "character",
              "attributes": {},
              "value": ["doc_id", "oracion_id", "sujeto", "predicado", "verbo", "lemma_verb", "aux_verbos", "entidades", "sust_pred"]
            }
          ]
        },
        {
          "type": "list",
          "attributes": {
            "names": {
              "type": "character",
              "attributes": {},
              "value": ["class", "dim", "names"]
            }
          },
          "value": [
            {
              "type": "character",
              "attributes": {},
              "value": ["data.frame"]
            },
            {
              "type": "integer",
              "attributes": {},
              "value": [8, 2]
            },
            {
              "type": "character",
              "attributes": {},
              "value": ["lemma", "n"]
            }
          ]
        },
        {
          "type": "list",
          "attributes": {
            "names": {
              "type": "character",
              "attributes": {},
              "value": ["class", "dim", "names"]
            }
          },
          "value": [
            {
              "type": "character",
              "attributes": {},
              "value": ["tokenIndex", "data.table", "data.frame"]
            },
            {
              "type": "integer",
              "attributes": {},
              "value": [0, 3]
            },
            {
              "type": "character",
              "attributes": {},
              "value": ["doc_id", "oracion_id", "oracion"]
            }
          ]
        }
      ]
    }

# public exports and hot-path formals are snapshotted

    {
      "type": "list",
      "attributes": {
        "names": {
          "type": "character",
          "attributes": {},
          "value": ["exports", "formals"]
        }
      },
      "value": [
        {
          "type": "character",
          "attributes": {},
          "value": ["%>%", "acep_claude", "acep_clean", "acep_clear_regex_cache", "acep_context", "acep_corpus", "acep_count", "acep_db", "acep_detect", "acep_extract", "acep_frec", "acep_gemini", "acep_gpt", "acep_gpt_schema", "acep_int", "acep_load_base", "acep_may", "acep_min", "acep_ollama", "acep_ollama_setup", "acep_openrouter", "acep_pipeline", "acep_plot_rst", "acep_plot_st", "acep_postag", "acep_postag_hibrido", "acep_process_chunks", "acep_regex_cache_size", "acep_result", "acep_sst", "acep_svo", "acep_together", "acep_token", "acep_token_plot", "acep_token_table", "acep_upos", "pipe_clean", "pipe_count", "pipe_intensity", "pipe_timeseries"]
        },
        {
          "type": "list",
          "attributes": {
            "names": {
              "type": "character",
              "attributes": {},
              "value": ["acep_clean", "acep_count", "acep_svo"]
            }
          },
          "value": [
            {
              "type": "character",
              "attributes": {
                "names": {
                  "type": "character",
                  "attributes": {},
                  "value": ["x", "tolower", "rm_cesp", "rm_emoji", "rm_hashtag", "rm_users", "rm_punt", "rm_num", "rm_url", "rm_meses", "rm_dias", "rm_stopwords", "rm_shortwords", "rm_newline", "rm_whitespace", "other_sw", "u"]
                }
              },
              "value": ["", "TRUE", "TRUE", "TRUE", "TRUE", "TRUE", "TRUE", "TRUE", "TRUE", "TRUE", "TRUE", "TRUE", "TRUE", "TRUE", "TRUE", "NULL", "1"]
            },
            {
              "type": "character",
              "attributes": {
                "names": {
                  "type": "character",
                  "attributes": {},
                  "value": ["texto", "dic", "use_cache"]
                }
              },
              "value": ["", "", "TRUE"]
            },
            {
              "type": "character",
              "attributes": {
                "names": {
                  "type": "character",
                  "attributes": {},
                  "value": ["acep_tokenindex", "prof_s", "prof_o", "u"]
                }
              },
              "value": ["", "3", "3", "1"]
            }
          ]
        }
      ]
    }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_count.R
\name{acep_clear_regex_cache}
\alias{acep_clear_regex_cache}
\title{Limpiar caché de expresiones regulares}
\usage{
acep_clear_regex_cache()
}
\value{
Devuelve `NULL` invisiblemente.
}
\description{
Elimina todos los patrones regex almacenados en el caché interno de `acep_count()`.
Útil para liberar memoria cuando se han procesado muchos diccionarios diferentes
o cuando se quiere forzar la recompilación de patrones.
}
\examples{
# Limpiar el caché
acep_clear_regex_cache()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_gemini.R
\name{acep_gemini}
\alias{acep_gemini}
\title{Interaccion con modelos Gemini usando Structured Outputs}
\usage{
acep_gemini(
  texto,
  instrucciones,
  modelo = "gemini-2.5-flash",
  api_key = Sys.getenv("GEMINI_API_KEY"),
  schema = NULL,
  parse_json = TRUE,
  temperature = 0,
  max_tokens = 2000,
  top_p = 0.95,
  top_k = 40,
  timeout = 120
)
}
\arguments{
\item{texto}{Texto a analizar con Gemini. Puede ser una noticia, tweet, documento, etc.}

\item{instrucciones}{Instrucciones en lenguaje natural que indican al modelo que hacer
con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".}

\item{modelo}{Modelo de Gemini a utilizar. Opciones recomendadas:
- Gemini 2.5: `"gemini-2.5-flash"` (rapido, multimodal, por defecto),
  `"gemini-2.5-flash-lite"` (mas economico), `"gemini-2.5-pro"` (mas potente)
- Gemini 2.0: `"gemini-2.0-flash"`, `"gemini-2.0-flash-lite"`
Por defecto: `"gemini-2.5-flash"`. Ver: https://ai.google.dev/gemini-api/docs/models}

\item{api_key}{Clave de API de Google Gemini. Si no se proporciona, busca la variable de
entorno `GEMINI_API_KEY`. Para obtener una clave: https://aistudio.google.com/apikey}

\item{schema}{Esquema JSON que define la estructura de la respuesta. Puede usar
`acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
Si es `NULL`, usa un esquema simple con campo "respuesta".
NOTA: Gemini usa un subconjunto de OpenAPI 3.0 Schema para definir estructuras.}

\item{parse_json}{Logico. Si `TRUE` (por defecto), parsea automaticamente el JSON
a un objeto R (lista o data frame). Si `FALSE`, devuelve el JSON como string.}

\item{temperature}{Parametro de temperatura (0-2). Valores bajos (0-0.3) generan
respuestas mas deterministas. Valores altos (0.7-1) mas creativas. Por defecto: 0.
Valor recomendado por Google: 1.0.}

\item{max_tokens}{Numero maximo de tokens en la respuesta. Por defecto: 2000.}

\item{top_p}{Parametro top-p para nucleus sampling (0-1). Controla la diversidad
de la respuesta. Por defecto: 0.95 (valor tipico para Gemini).}

\item{top_k}{Parametro top-k. Limita la seleccion a los K tokens mas probables.
Por defecto: 40 (valor tipico para Gemini).}

\item{timeout}{Tiempo maximo de espera de la peticion HTTP en segundos.
Por defecto: 120. Evita que una conexion estancada bloquee la sesion de R.}
}
\value{
Si `parse_json=TRUE`, devuelve una lista o data frame con la respuesta
  estructurada segun el esquema. Si `parse_json=FALSE`, devuelve un string JSON.
}
\description{
Funcion para interactuar con la API de Google Gemini utilizando Structured Outputs
nativos. Gemini soporta generacion de JSON estructurado mediante el parametro
`responseSchema` que garantiza que las respuestas cumplan con el esquema definido.
Compatible con todos los modelos Gemini 2.5 y Gemini 2.0. Acceso gratuito para
uso limitado disponible en Google AI Studio.
}
\details{
La API de Gemini usa un enfoque diferente para structured outputs:
- Define `responseMimeType: "application/json"` en `generationConfig`
- Usa `responseSchema` con formato OpenAPI 3.0 Schema
- Soporta tipos: string, integer, number, boolean, array, object
- Campo opcional `propertyOrdering` controla orden de propiedades en respuesta

Diferencias importantes con OpenAI:
- No requiere campo `additionalProperties: false` (se maneja automaticamente)
- Los campos son opcionales por defecto (usar `required` para campos obligatorios)
- El esquema cuenta como tokens de entrada
}
\examples{
\dontrun{
# Configurar API key
Sys.setenv(GEMINI_API_KEY = "tu-api-key")

# Extraer entidades con Gemini 2.5 Flash
texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
resultado <- acep_gemini(texto, "Extrae las entidades nombradas",
                         schema = acep_gpt_schema("extraccion_entidades"))

# Analisis de sentimiento con modelo economico
resultado <- acep_gemini(texto, "Analiza el sentimiento",
                         modelo = "gemini-2.5-flash-lite",
                         schema = acep_gpt_schema("sentimiento"))

# Usar Gemini 2.0 Flash Lite (mas rapido)
resultado <- acep_gemini(texto, "Extrae entidades",
                         modelo = "gemini-2.0-flash-lite",
                         schema = acep_gpt_schema("extraccion_entidades"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_min.R
\name{acep_min}
\alias{acep_min}
\title{Convierte caracteres a minúsculas.}
\usage{
acep_min(x)
}
\arguments{
\item{x}{Un vector de texto (caracteres) que se desea convertir a minúsculas.}
}
\value{
Devuelve un nuevo vector con todas las letras en minúsculas.
}
\description{
Esta función toma un vector de texto y convierte todas las letras
mayusculas en minúsculas, manteniendo el resto de los caracteres sin cambios.
}
\examples{
vector_texto <- c("SUTEBA", "Sindicato", "PEN")
acep_min(vector_texto)
vector_numeros <- c(1, 2, 3, 4, 5)
acep_min(vector_numeros)
vector_mezclado <- c("Soip", 123, "CGT")
acep_min(vector_mezclado)
}
\keyword{caracteres}
\keyword{texto}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_gpt.R
\name{acep_gpt_schema}
\alias{acep_gpt_schema}
\title{Esquemas JSON predefinidos para analisis de texto con GPT}
\usage{
acep_gpt_schema(tipo = "extraccion_entidades")
}
\arguments{
\item{tipo}{Tipo de esquema a devolver. Opciones:
\itemize{
  \item \code{"extraccion_entidades"}: Extrae personas, organizaciones, lugares, fechas y eventos
  \item \code{"clasificacion"}: Clasifica el texto en categorias con nivel de confianza
  \item \code{"sentimiento"}: Analiza sentimiento general y por aspectos especificos
  \item \code{"resumen"}: Genera resumenes cortos y detallados con puntos clave
  \item \code{"qa"}: Responde preguntas con citas textuales y nivel de confianza
  \item \code{"tripletes"}: Extrae relaciones sujeto-predicado-objeto
  \item \code{"protesta_breve"}: Extrae informacion basica de acciones de protesta (fecha, sujeto, accion, objeto, lugar)
  \item \code{"protesta_detallada"}: Extrae informacion detallada de multiples acciones de protesta con 9 campos por accion
  \item \code{"verdadero_falso"}: Devuelve una respuesta booleana simple (TRUE o FALSE) con nivel de confianza (0 a 1) y justificacion opcional
}}
}
\value{
Lista con esquema JSON compatible con OpenAI Structured Outputs.
  Puede usarse directamente en el parametro `schema` de `acep_gpt()` o `acep_ollama()`.
}
\description{
Proporciona esquemas JSON predefinidos y validados para casos de uso comunes
en analisis de texto con GPT. Estos esquemas garantizan respuestas estructuradas
y consistentes para tareas como extraccion de entidades, clasificacion, analisis
de sentimiento, resumen, pregunta-respuesta, extraccion de tripletes y analisis
de acciones de protesta.
}
\examples{
# Obtener esquema para extraccion de entidades
schema_entidades <- acep_gpt_schema("extraccion_entidades")
names(schema_entidades$properties)  # personas, organizaciones, lugares, fechas, eventos

# Obtener esquema para clasificacion
schema_clasif <- acep_gpt_schema("clasificacion")
names(schema_clasif$properties)  # categoria, confianza, justificacion

# Obtener esquema para analisis de sentimiento
schema_sent <- acep_gpt_schema("sentimiento")
names(schema_sent$properties)  # sentimiento_general, puntuacion, aspectos

# Obtener esquema para analisis breve de protestas
schema_protesta <- acep_gpt_schema("protesta_breve")
names(schema_protesta$properties)  # fecha, sujeto, accion, objeto, lugar

# Obtener esquema para analisis detallado de protestas
schema_protesta_det <- acep_gpt_schema("protesta_detallada")
names(schema_protesta_det$properties)  # acciones (array con 9 campos cada una)

# Obtener esquema para respuesta verdadero/falso
schema_bool <- acep_gpt_schema("verdadero_falso")
names(schema_bool$properties)  # respuesta, nivel_confianza, justificacion
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_pipeline.R
\name{\%>\%}
\alias{\%>\%}
\title{Pipe operator para ACEP}
\usage{
lhs \%>\% rhs
}
\description{
Pipe operator para ACEP
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_plot_rst.R
\name{acep_plot_rst}
\alias{acep_plot_rst}
\title{Resumen visual de la serie temporal de los indices de conflictividad.}
\usage{
acep_plot_rst(datos, tagx = "horizontal")
}
\arguments{
\item{datos}{data frame con datos procesados.}

\item{tagx}{orientación de las etiquetas del
eje x ('horizontal' | 'vertical').}
}
\value{
Si todas las entradas son correctas,
la salida sera una imagen de cuatro paneles.
}
\description{
Función que devuelve un panel visual de cuatro gráficos
de barras con variables proxy de los indices de conflictividad agrupados
por segmento de tiempo.
}
\examples{
datos <- acep_bases$rp_procesada
datos_procesados_anio <- acep_sst(datos, st = 'anio')
acep_plot_rst(datos_procesados_anio, tagx = 'vertical')
}
\keyword{visualización}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_upos.R
\name{acep_upos}
\alias{acep_upos}
\title{Función para etiquetado POS, lematización, tokenización.}
\source{
\href{https://universaldependencies.org/}{Dependencias Universales para taggeo POS}

\href{https://ufal.mff.cuni.cz/~straka/papers/2017-conll_udpipe.pdf}{Sobre el modelo UDPipe}

\href{https://CRAN.R-project.org/package=rsyntax}{Sobre el paquete rsyntax}
}
\usage{
acep_upos(texto, modelo = "spanish")
}
\arguments{
\item{texto}{vector con los textos a procesar.}

\item{modelo}{idioma del modelo de etiquetado POS del paquete \code{udpipe}.}
}
\value{
Si todas las entradas son correctas, la salida sera un marco de datos con 17 variables.
}
\description{
Función que devuelve un marco de datos objetos con
etiquetado POS (modelo udpipe) para su posterior
procesamiento con la función acep_postag.
}
\examples{
\dontrun{
texto <- "El SOIP declara la huelga en demanda de aumento salarial."
acep_upos(texto)
}
}
\references{
Welbers, K., Atteveldt, W. van, & Kleinnijenhuis, J. 2021. Extracting semantic relations using syntax: An R package for querying and reshaping dependency trees. Computational Communication Research, 3-2, 1-16.
\doi{10.5117/CCR2021.2.003.WELB}
}
\keyword{sintaxis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_postag.R
\name{acep_postag}
\alias{acep_postag}
\title{Etiquetado POS adaptativo con optimizaciones avanzadas}
\usage{
acep_postag(
  texto,
  core = "es_core_news_lg",
  bajar_core = TRUE,
  inst_spacy = FALSE,
  inst_miniconda = FALSE,
  inst_reticulate = FALSE,
  chunk_size = 1000,
  geocode_cache_file = "geocode_cache.json",
  use_cache = TRUE,
  show_progress = TRUE
)
}
\arguments{
\item{texto}{Vector de caracteres con los textos a procesar.}

\item{core}{Idioma del modelo de etiquetado POS del paquete \code{spacyr}. Opciones disponibles:
'es_core_news_sm', 'es_core_news_md', 'es_core_news_lg' (espanol),
'pt_core_news_sm', 'pt_core_news_md', 'pt_core_news_lg' (portugues),
'en_core_web_sm', 'en_core_web_md', 'en_core_web_lg', 'en_core_web_trf' (ingles).
Default: "es_core_news_lg".}

\item{bajar_core}{Parametro booleano que define si descargar o no el modelo de etiquetado POS.
Default: TRUE.}

\item{inst_spacy}{Parametro booleano que define si instalar o no spacy (Python).
Default: FALSE.}

\item{inst_miniconda}{Parametro booleano que define si instalar o no miniconda.
Default: FALSE.}

\item{inst_reticulate}{Parametro booleano que define si instalar o no el paquete \code{reticulate}.
Default: FALSE.}

\item{chunk_size}{Tamano de los lotes para procesamiento chunking. Ajustar segun RAM disponible:
500 para sistemas con 2-4 GB RAM, 1000 para 8 GB RAM (default), 2000-5000 para 16+ GB RAM.
Default: 1000.}

\item{geocode_cache_file}{Ruta del archivo JSON para guardar cache de geocodificacion.
Permite evitar consultas repetidas a la API de Nominatim y compartir cache entre proyectos.
Default: "geocode_cache.json".}

\item{use_cache}{Parametro booleano que activa/desactiva el sistema de cache de geocodificacion.
Desactivar para forzar re-geocodificacion de todas las ubicaciones.
Default: TRUE.}

\item{show_progress}{Parametro booleano que controla la visualizacion de mensajes de progreso
durante el procesamiento. Util para operaciones largas.
Default: TRUE.}
}
\value{
Lista con seis elementos en formato tabular:
\itemize{
  \item \code{texto_tag}: Data frame con tokens etiquetados (POS, lemas, dependencias, etc.)
  \item \code{texto_tag_entity}: Data frame con entidades nombradas consolidadas
  \item \code{texto_only_entity}: Data frame con solo las entidades extraidas
  \item \code{texto_only_entity_loc}: Data frame con entidades de tipo LOC geocodificadas (lat/long)
  \item \code{texto_nounphrase}: Data frame con frases nominales consolidadas
  \item \code{texto_only_nounphrase}: Data frame con solo las frases nominales extraidas
}
}
\description{
Version optimizada de acep_postag que se adapta automaticamente al tamano del input.
Implementa procesamiento por lotes (chunking) para grandes volumenes, cache de geocodificacion
para evitar consultas repetidas, y estrategias de procesamiento adaptativas segun la cantidad
de textos. Puede procesar desde 10 hasta millones de textos de forma eficiente.
}
\details{
La funcion implementa dos estrategias de procesamiento automaticas:
\itemize{
  \item \strong{Batch Processing} (<= 100 textos): Procesa todos los textos en una sola llamada
  para maxima velocidad.
  \item \strong{Chunking} (> 100 textos): Divide los textos en lotes del tamano especificado
  en \code{chunk_size} para controlar el uso de memoria y permitir procesamiento de grandes volumenes.
}

El sistema de cache de geocodificacion guarda las coordenadas de ubicaciones ya consultadas
en formato JSON, evitando consultas repetidas a la API de Nominatim (que tiene limite de 1 req/seg).
Esto puede reducir el tiempo de procesamiento en 50-90% en ejecuciones posteriores con ubicaciones repetidas.

Para datasets muy grandes (>100,000 textos), se recomienda procesar en lotes usando la funcion
auxiliar proporcionada en los ejemplos y guardar resultados incrementalmente.
}
\examples{
\dontrun{
# Ejemplo basico con pocos textos
textos <- c(
  "En Mar del Plata el SOIP declara la huelga en demanda de aumento salarial.",
  "La manifestacion se realizo en Buenos Aires el 15 de marzo.",
  "El presidente visito Cordoba para inaugurar la nueva planta."
)
resultado <- acep_postag(texto = textos, bajar_core = FALSE)
head(resultado$texto_tag)

# Ejemplo con dataset mediano y configuracion personalizada
resultado <- acep_postag(
  texto = mis_1000_textos,
  bajar_core = FALSE,
  chunk_size = 500,
  geocode_cache_file = "cache/ubicaciones_argentina.json",
  use_cache = TRUE
)

# Ver ubicaciones geocodificadas
head(resultado$texto_only_entity_loc)

# Procesamiento incremental para datasets muy grandes
procesar_incremental <- function(textos, batch_size = 10000) {
  dir.create("resultados", showWarnings = FALSE)
  n_batches <- ceiling(length(textos) / batch_size)

  for (i in 1:n_batches) {
    start_idx <- (i - 1) * batch_size + 1
    end_idx <- min(i * batch_size, length(textos))
    batch <- textos[start_idx:end_idx]

    resultado <- acep_postag(
      texto = batch,
      bajar_core = FALSE,
      chunk_size = 2000,
      use_cache = TRUE,
      geocode_cache_file = "cache_global.json"
    )

    saveRDS(resultado, sprintf("resultados/batch_\%04d.rds", i))
    message(sprintf("Batch \%d/\%d completado", i, n_batches))
  }
}

# Usar funcion incremental
procesar_incremental(mis_millones_de_textos, batch_size = 10000)

# Ver contenido del cache
cache <- jsonlite::read_json("geocode_cache.json", simplifyVector = TRUE)
print(paste("Ubicaciones en cache:", nrow(cache)))
}
}
\keyword{cache}
\keyword{chunking}
\keyword{etiquetado}
\keyword{optimizacion}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_classes.R
\name{acep_corpus}
\alias{acep_corpus}
\title{Constructor de corpus para analisis de texto}
\usage{
acep_corpus(texto, metadata = NULL, id = NULL)
}
\arguments{
\item{texto}{Vector de caracteres con los textos a almacenar en el corpus.}

\item{metadata}{Lista opcional con informacion adicional sobre el corpus
(ej: fuente, fecha de recoleccion, categorias tematicas).}

\item{id}{Vector opcional de identificadores unicos para cada texto.
Si no se proporciona, se asignan IDs secuenciales (1, 2, 3, ...).}
}
\value{
Objeto de clase `acep_corpus` con la siguiente estructura:
\itemize{
  \item \code{texto_original}: Vector con los textos originales sin procesar
  \item \code{texto_procesado}: Vector con textos procesados (NULL inicialmente)
  \item \code{id}: Identificadores de cada texto
  \item \code{metadata}: Metadata adicional del corpus
  \item \code{procesamiento}: Registro de transformaciones aplicadas
}
}
\description{
Crea un objeto de clase `acep_corpus` que encapsula una coleccion de textos
junto con su metadata asociada. Este objeto esta disenado para trabajar
con las funciones pipeline de ACEP (`pipe_clean`, `pipe_count`, etc.),
permitiendo un flujo de trabajo encadenado y manteniendo trazabilidad
de las transformaciones aplicadas.
}
\examples{
# Crear corpus simple
textos <- c("El SUTEBA va al paro", "SOIP protesta por salarios")
corpus <- acep_corpus(textos)
print(corpus)

# Crear corpus con metadata e IDs personalizados
corpus <- acep_corpus(
  texto = c("Noticia 1", "Noticia 2"),
  metadata = list(fuente = "Diario La Nacion", year = 2024),
  id = c("LN001", "LN002")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_ollama.R
\name{acep_ollama_setup}
\alias{acep_ollama_setup}
\title{Guia de instalacion y uso de Ollama}
\usage{
acep_ollama_setup()
}
\description{
Imprime instrucciones para instalar y configurar Ollama en tu sistema.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_count.R
\name{acep_count}
\alias{acep_count}
\title{Conteo de menciones de palabras de un diccionario}
\usage{
acep_count(texto, dic, use_cache = TRUE)
}
\arguments{
\item{texto}{vector de textos al que se le aplica la función de conteo.}

\item{dic}{vector de palabras del diccionario utilizado.}

\item{use_cache}{logical, usar caché de regex (default TRUE).}
}
\value{
Vector con frecuencia de palabras del diccionario.
}
\description{
Cuenta el número de veces que aparecen palabras de un diccionario en cada texto.
Los términos se tratan como texto literal: sus metacaracteres de expresiones
regulares se escapan automáticamente. De forma predeterminada también cuenta
coincidencias parciales (por ejemplo, "paro" coincide dentro de "paros"); para
exigir límites de palabra (word boundaries) y evitar coincidencias parciales,
rodeá cada término con espacios (por ejemplo, " paro "). Incluye un sistema de
caché que almacena los patrones regex compilados para acelerar ejecuciones
repetidas con el mismo diccionario. El caché se acota automáticamente
(máximo 1000 patrones) para evitar crecimiento de memoria sin límite en
sesiones largas; también podés vaciarlo manualmente con
`acep_clear_regex_cache()`.
}
\examples{
df <- data.frame(texto = c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
"El SOIP lleva adelante un plan de lucha con paros y piquetes."))
diccionario <- c("paro", "lucha", "piquetes")
df$detect <- acep_count(df$texto, diccionario)
df
}
\keyword{frecuencia}
\keyword{indicadores}
\keyword{tokens}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_diccionarios.R
\docType{data}
\name{acep_diccionarios}
\alias{acep_diccionarios}
\title{Colección de diccionarios.}
\format{
Es un objeto de clase 'list' con 3 componentes. Cada componente es
una cadena de caracteres de longitud 1 con la URL de un diccionario `.rds`
descargable; usá \code{\link{acep_load_base}} para obtener el vector de
palabras correspondiente.
\describe{
\item{dicc_confl_gp}{URL al diccionario de términos que refieren a conflictos}
\item{dicc_viol_gp}{URL al diccionario de términos que refieren a violencia}
\item{dicc_confl_sismos}{URL al diccionario de términos que refieren a conflictos}
}
}
\source{
\href{https://revistapuerto.com.ar/}{Revista Puerto}

\href{https://www.lanueva.com/}{La Nueva}
}
\usage{
data(acep_diccionarios)
}
\description{
Colección de diccionarios que reúne diccionarios
de diferentes orígenes. El diccionario dicc_confl_sismos fue construido
en el marco del Observatorio de Conflictividad de la UNMdP.
Los diccionarios dicc_confl_gp y dicc_viol_gp fueron extraídos de
Albrieu y Palazzo (2020).
}
\examples{
\dontrun{
diccionario <- acep_load_base(acep_diccionarios$dicc_viol_gp)
diccionario
}
}
\references{
Albrieu, Ramiro y Gabriel Palazzo 2020 «Categorización de
conflictos sociales en el ámbito de los recursos naturales: un estudio
de las actividades extractivas mediante la minería de textos».
Revista CEPAL (131):29-59.
(\href{https://observatoriodeconflictividad.org/RVE131_AP.pdf}{Revista CEPAL})

Laitano, Guillermina y Agustín Nieto
«Análisis computacional de la conflictividad laboral en Mar del Plata
durante el gobierno de Cambiemos». Ponencia presentado en VI Workshop -
Los conflictos laborales en la Argentina del siglo XX y XXI:
un abordaje interdisciplinario de conceptos, problemas y escalas de análisis,
Tandil, 2021.
}
\keyword{diccionarios}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_db.R
\name{acep_db}
\alias{acep_db}
\title{Frecuencia, menciones e intensidad.}
\usage{
acep_db(db, t, d, n)
}
\arguments{
\item{db}{data frame con los textos a procesar.}

\item{t}{columna de data frame que contiene el vector
de textos a procesar.}

\item{d}{diccionario en formato vector.}

\item{n}{cantidad de decimales del indice de intensidad.}
}
\value{
Si todas las entradas son correctas, la salida sera una
base de datos en formato tabular con tres nuevas variables.
}
\description{
Función que usa las funciones acep_frec, acep_count y acep_int y
devuelve una tabla con tres columnas nuevas: numero de palabras,
número de menciones del diccionario, indice de intensidad.
}
\examples{
df <- data.frame(texto = c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
"El SOIP lleva adelante un plan de lucha con paros y piquetes."))
diccionario <- c("paro", "lucha", "piquetes")
acep_db(df, df$texto, diccionario, 4)
}
\keyword{frecuencia}
\keyword{indicadores}
\keyword{tokens}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_pipeline.R
\name{pipe_intensity}
\alias{pipe_intensity}
\title{Cálculo de intensidad en pipeline}
\usage{
pipe_intensity(result, decimales = 4)
}
\arguments{
\item{result}{Objeto `acep_result` que debe contener una columna `frecuencia`.
Típicamente proviene de `pipe_count()`.}

\item{decimales}{Número de decimales para redondear el índice de intensidad.
Por defecto: 4.}
}
\value{
Objeto `acep_result` con tipo `"intensidad"` que incluye columnas
  adicionales: `n_palabras` e `intensidad`.
}
\description{
Calcula el índice de intensidad normalizado dentro de un flujo pipeline.
La intensidad se define como la proporción de menciones del diccionario
respecto al total de palabras: intensidad = frecuencia / n_palabras.
}
\examples{
# Calcular intensidad desde resultado de conteo
textos <- c("El SUTEBA va al paro", "SOIP en lucha y paro")
corpus <- acep_corpus(textos)
diccionario <- c("paro", "lucha")
resultado <- pipe_count(corpus, diccionario)
resultado_intensidad <- pipe_intensity(resultado, decimales = 4)
print(resultado_intensidad)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_pipeline.R
\name{pipe_count}
\alias{pipe_count}
\title{Conteo de menciones en pipeline}
\usage{
pipe_count(corpus, dic, ...)
}
\arguments{
\item{corpus}{Objeto `acep_corpus`. Debe ser un corpus válido creado con
`acep_corpus()` o resultado de `pipe_clean()`.}

\item{dic}{Vector de caracteres con las palabras del diccionario a buscar.}

\item{...}{Argumentos adicionales (actualmente no utilizados, reservado para
futuras extensiones).}
}
\value{
Objeto `acep_result` con tipo `"frecuencia"` que contiene un data frame con:
  `id`, `texto` y `frecuencia` de menciones por texto.
}
\description{
Cuenta las menciones de palabras de un diccionario dentro de un flujo pipeline.
Esta función extrae los textos de un `acep_corpus` (procesados o originales)
y aplica `acep_count()` para detectar ocurrencias del diccionario.
}
\examples{
# Contar menciones en corpus
textos <- c("El SUTEBA va al paro", "SOIP en lucha y paro")
corpus <- acep_corpus(textos)
diccionario <- c("paro", "lucha", "protesta")
resultado <- pipe_count(corpus, diccionario)
print(resultado)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_extract.R
\name{acep_extract}
\alias{acep_extract}
\title{Función para buscar y extraer palabras en un texto.}
\usage{
acep_extract(texto, dic, sep = "; ", izq = "\\\\b\\\\w*", der = "\\\\w*\\\\b")
}
\arguments{
\item{texto}{El texto en el que se buscaran las palabras clave.}

\item{dic}{Un vector con las palabras clave a buscar.}

\item{sep}{El separador utilizado para concatenar las
palabras clave encontradas (por defecto: " | ").}

\item{izq}{expresión regular para incorporar otros caracteres a
la izquierda de los términos del vector 'dic'.}

\item{der}{expresión regular para incorporar otros caracteres a
la derecha de los términos del vector 'dic'.}
}
\value{
Si todas las entradas son correctas,
la salida sera un vector de tipo caracter.
procesado y el contexto de las palabras y/o frases entradas.
}
\description{
Esta función busca palabras clave en un texto y
extrae los resultados en un formato especifico.
}
\examples{
texto <- "Los obreros del pescado, en el marco de una huelga y
realizaron una manifestación con piquete en el puerto de la ciudad."
dicc <- c("huel", "manif", "piq")
acep_extract(texto, dicc)
}
\keyword{buscar}
\keyword{diccionario}
\keyword{palabras}
\keyword{texto}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_bases.R
\docType{data}
\name{acep_bases}
\alias{acep_bases}
\title{Coleccion de notas y recursos de prueba}
\format{
Lista con 11 objetos:
\describe{
\item{la_nueva}{URL para descargar una muestra del corpus de notas
del diario La Nueva Provincia de Bahia Blanca.}
\item{rev_puerto}{URL para descargar el corpus de notas
de Revista Puerto.}
\item{rp_procesada}{data frame con indicadores de conflictividad
construidos a partir de Revista Puerto.}
\item{lc_mdp}{URL para descargar el corpus de notas
del diario La Capital (Mar del Plata).}
\item{rp_mdp}{URL para descargar el corpus de notas
de Revista Puerto (edicion Mar del Plata).}
\item{ed_neco}{URL para descargar el corpus de notas
del diario Ecos Diarios (Necochea).}
\item{ln_bb}{URL para descargar el corpus de notas
de La Nueva (Bahia Blanca).}
\item{ln_arg}{URL para descargar un subconjunto de notas
de La Nacion.}
\item{lc_720}{data frame con 720 notas de La Capital publicadas entre
2016 y 2019, curado y documentado por Guillermina Laitano. Contiene
etiquetas binarias manuales que permiten evaluar el diccionario de
conflictividad, la extraccion de tripletes semanticos y la capacidad
de distintos modelos generativos para tareas de clasificacion binaria
y extraccion estructurada de eventos de protesta.}
\item{spacy_postag}{data frame con una oracion procesada con \pkg{spacyr}
que sirve como ejemplo para funciones de dependencias y SVO.}
\item{titulares}{vector con titulares sinteticos referidos a
conflictos sociales.}
}
}
\source{
\href{https://revistapuerto.com.ar/}{Revista Puerto}

\href{https://www.lanueva.com/}{La Nueva}
}
\usage{
data(acep_bases)
}
\description{
Lista con fuentes de datos y muestras preprocesadas
utilizadas en los ejemplos del paquete. Incluye enlaces de descarga
para distintos portales, resumenes estadisticos y conjuntos anotados
manualmente que permiten evaluar diccionarios, extraccion de tripletes
y desempeno de modelos generativos.
}
\examples{
acep_bases$rp_procesada[1:6, ]
}
\references{
Nieto, Agustin 2020 "Intersecciones entre historia digital e
historia social: un ejercicio de lectura distante sobre la
conflictividad maritima en la historia argentina reciente".
Drassana: revista del Museu Maritim (28):122-42.
(\href{https://observatoriodeconflictividad.org/nietohd.pdf}{Revista Drassana})
}
\keyword{datos}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_may.R
\name{acep_may}
\alias{acep_may}
\title{Convierte caracteres a mayusculas.}
\usage{
acep_may(x)
}
\arguments{
\item{x}{es un vector de texto (caracteres) que se desea convertir a mayúsculas.}
}
\value{
Devuelve un nuevo vector con todas las letras en mayúsculas.
}
\description{
Esta función toma un vector de texto y convierte todas las letras
minúsculas en mayúsculas, manteniendo el resto de los caracteres sin cambios.
}
\examples{
vector_texto <- c("soip", "cGt", "Sutna")
acep_may(vector_texto)
vector_numeros <- c(1, 2, 3, 4, 5)
acep_may(vector_numeros)
vector_mezclado <- c("sutna", 123, "Ate")
acep_may(vector_mezclado)
}
\keyword{caracteres}
\keyword{texto}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_count.R
\name{acep_regex_cache_size}
\alias{acep_regex_cache_size}
\title{Consultar tamaño del caché de regex}
\usage{
acep_regex_cache_size()
}
\value{
Número entero con la cantidad de patrones en caché.
}
\description{
Devuelve el número de patrones regex almacenados actualmente en el caché
interno de `acep_count()`. Cada diccionario único genera una entrada en el caché.
}
\examples{
# Ver cuántos patrones hay en caché
acep_regex_cache_size()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_count.R
\docType{data}
\name{.acep_regex_cache_max}
\alias{.acep_regex_cache_max}
\title{Tope de entradas del caché de regex}
\format{
An object of class \code{integer} of length 1.
}
\usage{
.acep_regex_cache_max
}
\description{
Cuando el caché alcanza este tamaño se vacía antes de agregar una nueva
entrada, evitando crecimiento de memoria sin límite en sesiones largas que
usan muchos diccionarios distintos. Un fallo de caché solo recompila el
patrón, por lo que la cota no afecta la correctitud de los conteos.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_int.R
\name{acep_int}
\alias{acep_int}
\title{Índice de intensidad.}
\usage{
acep_int(pc, pt, decimales = 4)
}
\arguments{
\item{pc}{vector numérico con la frecuencia de palabras conflictivas
presentes en cada texto.}

\item{pt}{vector de palabras totales en cada texto.}

\item{decimales}{cantidad de decimales, por defecto tiene 4
pero se puede modificar.}
}
\value{
Si todas las entradas son correctas,
la salida sera un vector numérico.
}
\description{
Función que elabora un indice de intensidad en
base a la relación entre palabras totales y palabras del diccionario
presentes en el texto.
}
\examples{
conflictos <- c(1, 5, 0, 3, 7)
palabras <- c(4, 11, 12, 9, 34)
acep_int(conflictos, palabras, 3)
}
\keyword{frecuencia}
\keyword{indicadores}
\keyword{tokens}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_process_chunks.R
\name{acep_process_chunks}
\alias{acep_process_chunks}
\title{Procesamiento de textos en lotes para optimizar memoria}
\usage{
acep_process_chunks(
  texto,
  funcion,
  chunk_size = 1000,
  show_progress = TRUE,
  ...
)
}
\arguments{
\item{texto}{Vector de caracteres con los textos a procesar.}

\item{funcion}{Funcion de ACEP a aplicar a cada lote. Ejemplos: `acep_clean`,
`acep_token`, `acep_count`, `acep_upos`, etc. Debe ser una funcion que acepte
un vector de textos como primer argumento.}

\item{chunk_size}{Numero de textos por lote. Valores mas bajos reducen el consumo
de memoria pero aumentan el tiempo total de procesamiento. Por defecto: 1000.}

\item{show_progress}{Logico. Si `TRUE`, muestra mensajes informativos sobre el
progreso del procesamiento (que lote se esta procesando). Por defecto: `TRUE`.}

\item{...}{Argumentos adicionales que se pasan directamente a la funcion especificada
en el parametro `funcion`. Ejemplo: si `funcion = acep_clean`, puede pasar
`rm_stopwords = TRUE`, `tolower = TRUE`, etc.}
}
\value{
El tipo de resultado depende de la funcion aplicada:
\itemize{
  \item Si la funcion retorna un vector, devuelve un vector combinado
  \item Si la funcion retorna un data frame, devuelve un data frame combinado (rbind)
  \item Si la funcion retorna una lista, devuelve una lista de listas
}
}
\description{
Divide un vector grande de textos en lotes (chunks) mas pequenos y los procesa
secuencialmente aplicando una funcion de ACEP. Esta estrategia permite analizar
corpus extensos (millones de documentos) sin superar la capacidad de memoria RAM
disponible. La funcion combina automaticamente los resultados de todos los lotes.
}
\examples{
\dontrun{
# Procesar 10,000 textos con limpieza en lotes de 1000
textos_limpios <- acep_process_chunks(
  texto = corpus_grande,
  funcion = acep_clean,
  chunk_size = 1000,
  rm_stopwords = TRUE
)

# Tokenizar corpus masivo
tokens <- acep_process_chunks(
  texto = corpus_masivo,
  funcion = acep_token,
  chunk_size = 500,
  tolower = TRUE
)

# Contar menciones en corpus grande
diccionario <- c("paro", "huelga", "protesta")
frecuencias <- acep_process_chunks(
  texto = corpus_grande,
  funcion = acep_count,
  chunk_size = 2000,
  dic = diccionario
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_claude.R, R/acep_gpt.R
\name{proteger_arrays_schema}
\alias{proteger_arrays_schema}
\title{Funcion auxiliar para proteger arrays en esquemas JSON}
\usage{
proteger_arrays_schema(schema)

proteger_arrays_schema(schema)
}
\arguments{
\item{schema}{Esquema JSON como lista de R}
}
\value{
Esquema con arrays protegidos

Esquema con arrays protegidos
}
\description{
Protege arrays en esquemas JSON para evitar que jsonlite::toJSON los convierta
incorrectamente. Aplica I() a campos 'required' y 'enum' recursivamente.

Protege arrays en esquemas JSON para evitar que jsonlite::toJSON los convierta
incorrectamente. Aplica I() a campos 'required' y 'enum' recursivamente.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_clean.R
\name{acep_clean}
\alias{acep_clean}
\title{Limpieza de texto.}
\usage{
acep_clean(
  x,
  tolower = TRUE,
  rm_cesp = TRUE,
  rm_emoji = TRUE,
  rm_hashtag = TRUE,
  rm_users = TRUE,
  rm_punt = TRUE,
  rm_num = TRUE,
  rm_url = TRUE,
  rm_meses = TRUE,
  rm_dias = TRUE,
  rm_stopwords = TRUE,
  rm_shortwords = TRUE,
  rm_newline = TRUE,
  rm_whitespace = TRUE,
  other_sw = NULL,
  u = 1
)
}
\arguments{
\item{x}{vector de textos al que se le aplica la
función de limpieza de texto.}

\item{tolower}{convierte los textos a minúsculas.}

\item{rm_cesp}{remueve caracteres especiales.}

\item{rm_emoji}{remueve los emojis.}

\item{rm_hashtag}{remueve los hashtags.}

\item{rm_users}{remueve las menciones de usuarixs de redes sociales.}

\item{rm_punt}{remueve la puntuación.}

\item{rm_num}{remueve números.}

\item{rm_url}{remueve las url.}

\item{rm_meses}{remueve los meses del año.}

\item{rm_dias}{remueve los dias de la semana.}

\item{rm_stopwords}{remueve palabras vacías.}

\item{rm_shortwords}{remueve las palabras cortas.}

\item{rm_newline}{remueve los saltos de linea.}

\item{rm_whitespace}{remueve los espacios en blanco.}

\item{other_sw}{su valor por defecto es NULL, sirve para ampliar el
listado de stopwords con un nuevo vector de palabras.}

\item{u}{umbral de caracteres para la función rm_shortwords.}
}
\value{
Si todas las entradas son correctas,
la salida sera un vector de textos normalizados.
}
\description{
Función que limpia y normaliza las notas/textos.
}
\examples{
acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_stopwords = FALSE)
acep_clean("El SUTEBA fue al paro. Reclaman mejoras salariales.", rm_stopwords = TRUE)
}
\keyword{limpieza}
\keyword{normalización}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_provider.R
\name{.acep_provider_post}
\alias{.acep_provider_post}
\title{Ejecutar una peticion POST a un proveedor de IA}
\usage{
.acep_provider_post(url, headers, body, timeout = 120, encode = "raw")
}
\description{
Centraliza la construccion de la peticion HTTP comun a los proveedores en la
nube: serializa el cuerpo a JSON con `auto_unbox = TRUE`, arma las cabeceras,
aplica un timeout y realiza el POST. El ensamblado del `body` y el parseo de
la respuesta siguen siendo especificos de cada proveedor.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_postag_hibrido.R
\name{acep_postag_hibrido}
\alias{acep_postag_hibrido}
\title{Etiquetado POS, lematizacion y extraccion de entidades con spacyr}
\usage{
acep_postag_hibrido(
  texto,
  core = "es_core_news_lg",
  bajar_core = TRUE,
  inst_spacy = FALSE,
  inst_miniconda = FALSE,
  inst_reticulate = FALSE,
  chunk_size = 1000,
  parallel_chunks = FALSE,
  n_cores = NULL,
  geocode_cache_file = "geocode_cache.json",
  use_cache = TRUE,
  show_progress = TRUE
)
}
\arguments{
\item{texto}{Vector de caracteres con los textos a analizar.}

\item{core}{Modelo de lenguaje de spaCy a utilizar. Opciones: `"es_core_news_sm"`,
`"es_core_news_md"`, `"es_core_news_lg"` (espanol), `"en_core_web_sm"`,
`"en_core_web_md"`, `"en_core_web_lg"` (ingles), `"pt_core_news_sm"`,
`"pt_core_news_md"`, `"pt_core_news_lg"` (portugues). Por defecto: `"es_core_news_lg"`.}

\item{bajar_core}{Logico. Si `TRUE`, descarga automaticamente el modelo si no esta instalado.}

\item{inst_spacy}{Logico. Si `TRUE`, instala la biblioteca spaCy en el entorno Python.}

\item{inst_miniconda}{Logico. Si `TRUE`, instala Miniconda (necesario para spaCy).}

\item{inst_reticulate}{Logico. Si `TRUE`, instala el paquete reticulate de R.}

\item{chunk_size}{Numero de textos a procesar por lote. Valores mas bajos consumen
menos memoria pero tardan mas. Por defecto: 1000.}

\item{parallel_chunks}{Logico. Si `TRUE`, procesa los lotes en paralelo usando
multiples nucleos del CPU. Requiere los paquetes `future` y `furrr`. Por defecto: `FALSE`.}

\item{n_cores}{Numero de nucleos de CPU a usar en modo paralelo. Si es `NULL`,
detecta automaticamente el numero de nucleos disponibles menos uno.}

\item{geocode_cache_file}{Ruta al archivo JSON donde se almacena el cache de
geocodificacion para evitar consultas repetidas. Por defecto: `"geocode_cache.json"`.}

\item{use_cache}{Logico. Si `TRUE`, usa y actualiza el cache de geocodificacion.}

\item{show_progress}{Logico. Si `TRUE`, muestra mensajes de progreso en la consola.}
}
\value{
Lista con 6 data frames que contienen diferentes niveles de analisis:
\itemize{
  \item \code{texto_tag}: Tokenizacion completa con etiquetas POS, lemas, dependencias
    sintacticas y atributos morfologicos para cada token
  \item \code{texto_tag_entity}: Tokens con entidades nombradas consolidadas
    (ej: "Mar del Plata" como una sola entidad en lugar de 3 tokens separados)
  \item \code{texto_only_entity}: Solo las entidades nombradas extraidas
    (personas, organizaciones, ubicaciones, fechas, etc.)
  \item \code{texto_only_entity_loc}: Entidades de tipo ubicacion (LOC)
    con coordenadas geograficas (latitud/longitud) obtenidas mediante geocodificacion
  \item \code{texto_nounphrase}: Tokens con frases nominales consolidadas
  \item \code{texto_only_nounphrase}: Solo las frases nominales extraidas
}
}
\description{
Realiza analisis linguistico completo de textos usando la biblioteca spaCy a traves
de spacyr. Incluye: etiquetado POS (Part-of-Speech), lematizacion, tokenizacion,
extraccion de entidades nombradas, frases nominales y geocodificacion de ubicaciones.
La funcion procesa automaticamente grandes volumenes de texto dividiendolos en lotes
(chunks) y soporta procesamiento paralelo para acelerar el analisis
}
\examples{
\dontrun{
# Analisis basico de un texto
texto <- "El SUTEBA convoco a un paro en Mar del Plata el 15 de marzo."
resultado <- acep_postag_hibrido(texto)

# Ver tokens con etiquetas POS
head(resultado$texto_tag)

# Ver entidades nombradas
print(resultado$texto_only_entity)

# Ver ubicaciones geocodificadas
print(resultado$texto_only_entity_loc)

# Procesar multiples textos con procesamiento paralelo
textos <- c("Primera noticia sobre conflictos.",
            "Segunda noticia sobre protestas.",
            "Tercera noticia sobre reclamos.")
resultado <- acep_postag_hibrido(textos,
                                 parallel_chunks = TRUE,
                                 chunk_size = 100)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_claude.R
\name{acep_claude}
\alias{acep_claude}
\title{Interaccion con modelos Claude usando Structured Outputs}
\usage{
acep_claude(
  texto,
  instrucciones,
  modelo = "claude-sonnet-4-20250514",
  api_key = Sys.getenv("ANTHROPIC_API_KEY"),
  schema = NULL,
  parse_json = TRUE,
  temperature = 0,
  max_tokens = 2000,
  top_p = 0.2,
  top_k = NULL,
  timeout = 120
)
}
\arguments{
\item{texto}{Texto a analizar con Claude. Puede ser una noticia, tweet, documento, etc.}

\item{instrucciones}{Instrucciones en lenguaje natural que indican al modelo que hacer
con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".}

\item{modelo}{Modelo de Claude a utilizar. Modelos disponibles (ordenados por potencia):
- Claude 4.5: `"claude-sonnet-4-5-20250929"` (mas reciente y potente),
  `"claude-haiku-4-5-20251001"` (rapido)
- Claude 4.1: `"claude-opus-4-1-20250805"` (razonamiento excepcional)
- Claude 4: `"claude-opus-4-20250514"`, `"claude-sonnet-4-20250514"`
- Claude 3.7: `"claude-3-7-sonnet-20250219"`
- Claude 3.5: `"claude-3-5-haiku-20241022"` (rapido y economico)
- Claude 3: `"claude-3-opus-20240229"`, `"claude-3-haiku-20240307"`
Por defecto: `"claude-sonnet-4-20250514"`. Ver: https://docs.anthropic.com/en/docs/about-claude/models}

\item{api_key}{Clave de API de Anthropic. Si no se proporciona, busca la variable de
entorno `ANTHROPIC_API_KEY`. Para obtener una clave: https://console.anthropic.com/}

\item{schema}{Esquema JSON que define la estructura de la respuesta. Puede usar
`acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
Si es `NULL`, usa un esquema simple con campo "respuesta".}

\item{parse_json}{Logico. Si `TRUE` (por defecto), parsea automaticamente el JSON
a un objeto R (lista o data frame). Si `FALSE`, devuelve el JSON como string.}

\item{temperature}{Parametro de temperatura (0-1). Valores bajos (0-0.3) generan
respuestas mas deterministas y consistentes. Valores altos (0.7-1) mas creativas.
Por defecto: 0 (maxima determinismo).
NOTA: Anthropic no permite usar `temperature` y `top_p` simultaneamente.}

\item{max_tokens}{Numero maximo de tokens en la respuesta. Por defecto: 2000.}

\item{top_p}{Parametro top-p para nucleus sampling (0-1). Controla la diversidad
de la respuesta. Por defecto: 0.2.
NOTA: Solo se usa si `temperature` es 0 (valor por defecto).}

\item{top_k}{Parametro top-k (solo disponible en Claude). Limita la seleccion a los
K tokens mas probables. Por defecto: NULL (deshabilitado).}

\item{timeout}{Tiempo maximo de espera de la peticion HTTP en segundos.
Por defecto: 120. Evita que una conexion estancada bloquee la sesion de R.}
}
\value{
Si `parse_json=TRUE`, devuelve una lista o data frame con la respuesta
  estructurada segun el esquema. Si `parse_json=FALSE`, devuelve un string JSON.
}
\description{
Funcion para interactuar con la API de Anthropic Claude utilizando Tool Calling
para garantizar respuestas en formato JSON que cumplen estrictamente con un esquema
predefinido. A diferencia de OpenAI, Anthropic utiliza "forced tool use" para
lograr structured outputs, definiendo el esquema deseado como input_schema de una
herramienta y forzando al modelo a usarla. Compatible con todos los modelos Claude.
}
\details{
**Diferencias importantes entre modelos Claude:**

- **Claude Sonnet 4.5** (`claude-sonnet-4-5`): NO permite `temperature` y `top_p`
  simultaneamente. La funcion solo envia uno de los dos si fue modificado del default.

- **Otros modelos Claude** (`claude-sonnet-4-20250514`, `claude-3-5-haiku-20241022`,
  `claude-3-opus-20240229`, etc.): SI permiten ambos parametros simultaneamente.

La funcion detecta automaticamente el modelo y aplica la logica correcta.
}
\examples{
\dontrun{
# Extraer entidades de un texto
texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
instrucciones <- "Extrae todas las entidades nombradas del texto."
schema <- acep_gpt_schema("extraccion_entidades")
resultado <- acep_claude(texto, instrucciones, schema = schema)
print(resultado)

# Analisis de sentimiento
texto <- "La protesta fue pacifica y bien organizada."
schema <- acep_gpt_schema("sentimiento")
resultado <- acep_claude(texto, "Analiza el sentimiento del texto", schema = schema)
print(resultado$sentimiento_general)

# Clasificar noticia
texto <- "Trabajadores reclamaron mejoras salariales."
schema <- acep_gpt_schema("clasificacion")
resultado <- acep_claude(texto, "Clasifica esta noticia", schema = schema)
print(resultado$categoria)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_gpt.R
\name{acep_gpt}
\alias{acep_gpt}
\title{Interaccion con modelos GPT usando Structured Outputs}
\usage{
acep_gpt(
  texto,
  instrucciones,
  modelo = "gpt-4o-mini",
  api_key = Sys.getenv("OPENAI_API_KEY"),
  schema = NULL,
  parse_json = TRUE,
  temperature = 0,
  max_tokens = 2000,
  top_p = 0.2,
  frequency_penalty = 0.2,
  seed = 123456,
  timeout = 120
)
}
\arguments{
\item{texto}{Texto a analizar con GPT. Puede ser una noticia, tweet, documento, etc.}

\item{instrucciones}{Instrucciones en lenguaje natural que indican al modelo que hacer
con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".}

\item{modelo}{Modelo de OpenAI a utilizar. Compatible con Structured Outputs:
`"gpt-4o-mini"` (mas rapido y economico), `"gpt-4o"`, `"gpt-4o-2024-08-06"` (mas potente),
`"gpt-4.1"`, `"gpt-5-nano"`, `"gpt-5-mini"`, `"o1-mini"`, `"o4-mini"`, entre otros.
Por defecto: `"gpt-4o-mini"`. Ver: https://platform.openai.com/docs/guides/structured-outputs}

\item{api_key}{Clave de API de OpenAI. Si no se proporciona, busca la variable de
entorno `OPENAI_API_KEY`. Para obtener una clave: https://platform.openai.com/api-keys}

\item{schema}{Esquema JSON que define la estructura de la respuesta. Puede usar
`acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
Si es `NULL`, usa un esquema simple con campo "respuesta".}

\item{parse_json}{Logico. Si `TRUE` (por defecto), parsea automaticamente el JSON
a un objeto R (lista o data frame). Si `FALSE`, devuelve el JSON como string.}

\item{temperature}{Parametro de temperatura (0-2). Valores bajos (0-0.3) generan
respuestas mas deterministas y consistentes. Valores altos (0.7-1) mas creativas.
Por defecto: 0 (maxima determinismo).
NOTA: Los modelos gpt-5, o1 y o4 solo aceptan temperature = 1 (default de OpenAI).}

\item{max_tokens}{Numero maximo de tokens en la respuesta. Por defecto: 2000.}

\item{top_p}{Parametro top-p para nucleus sampling (0-1). Controla la diversidad
de la respuesta. Por defecto: 0.2.
NOTA: Ignorado en modelos gpt-5, o1 y o4.}

\item{frequency_penalty}{Penalizacion por repeticion de tokens frecuentes (-2 a 2).
Por defecto: 0.2. NOTA: Ignorado en modelos gpt-5, o1 y o4.}

\item{seed}{Semilla numerica para reproducibilidad. Usar el mismo seed con los
mismos parametros genera respuestas identicas. Por defecto: 123456.
NOTA: Ignorado en modelos gpt-5, o1 y o4.}

\item{timeout}{Tiempo maximo de espera de la peticion HTTP en segundos.
Por defecto: 120. Evita que una conexion estancada bloquee la sesion de R.}
}
\value{
Si `parse_json=TRUE`, devuelve una lista o data frame con la respuesta
  estructurada segun el esquema. Si `parse_json=FALSE`, devuelve un string JSON.
}
\description{
Funcion para interactuar con la API de OpenAI utilizando Structured Outputs,
una funcionalidad que garantiza respuestas en formato JSON que cumplen estrictamente
con un esquema predefinido. Esto elimina la necesidad de parseo y validacion manual,
haciendo las respuestas mas confiables y estructuradas. Compatible con los modelos
de OpenAI que soportan Structured Outputs: series `gpt-4o`, `gpt-4.1`, `gpt-5`,
`o1` y `o4` (ver `@details`).
}
\details{
**Diferencias entre modelos:**

- **Modelos GPT-4o/GPT-4.1**: Soportan todos los parametros (temperature, top_p,
  frequency_penalty, seed). Usan `max_tokens`.

- **Modelos GPT-5/o1/o4**: Solo aceptan temperature = 1 (default). Los parametros
  temperature, top_p, frequency_penalty y seed son automaticamente omitidos.
  Usan `max_completion_tokens` en lugar de `max_tokens`.

La funcion maneja estas diferencias automaticamente segun el modelo especificado.
}
\examples{
\dontrun{
# Extraer entidades de un texto
texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
instrucciones <- "Extrae todas las entidades nombradas del texto."
schema <- acep_gpt_schema("extraccion_entidades")
resultado <- acep_gpt(texto, instrucciones, schema = schema)
print(resultado)

# Analisis de sentimiento
texto <- "La protesta fue pacifica y bien organizada."
schema <- acep_gpt_schema("sentimiento")
resultado <- acep_gpt(texto, "Analiza el sentimiento del texto", schema = schema)
print(resultado$sentimiento_general)

# Clasificar noticia
texto <- "Trabajadores reclamaron mejoras salariales."
schema <- acep_gpt_schema("clasificacion")
resultado <- acep_gpt(texto, "Clasifica esta noticia", schema = schema)
print(resultado$categoria)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_token_plot.R
\name{acep_token_plot}
\alias{acep_token_plot}
\title{Gráfico de barras de palabras más recurrentes en un corpus.}
\usage{
acep_token_plot(x, u = 10, frec = TRUE)
}
\arguments{
\item{x}{vector de palabras tokenizadas.}

\item{u}{numero de corte para el top de palabras mas frecuentes.}

\item{frec}{parámetro para determinar si los valores se visualizaran
como frecuencia absoluta o relativa.}
}
\value{
Si todas las entradas son correctas,
la salida sera un gráfico de barras.
}
\description{
Función que devuelve un gráfico de barras con las palabras
mas recurrentes en un corpus textual.
}
\examples{
tokens <- c(rep("paro",15), rep("piquete",25), rep("corte",20), rep("manifestación",10),
rep("bloqueo",5), rep("alerta",16), rep("ciudad",12), rep("sindicato",11), rep("paritaria",14),
rep("huelga",14), rep("escrache",15))
acep_token_plot(tokens)
}
\keyword{visualizacion}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_pipeline.R
\name{pipe_clean}
\alias{pipe_clean}
\title{Limpieza de texto en pipeline}
\usage{
pipe_clean(corpus, ...)
}
\arguments{
\item{corpus}{Objeto `acep_corpus` o vector de caracteres. Si se pasa un vector,
se crea automáticamente un objeto `acep_corpus`.}

\item{...}{Argumentos para `acep_clean()`. Ejemplos: `rm_stopwords = TRUE`,
`rm_num = TRUE`, `tolower = TRUE`, `rm_punt = TRUE`.}
}
\value{
Objeto `acep_corpus` con el campo `texto_procesado` actualizado
  y registro de la transformación en `procesamiento$limpieza`.
}
\description{
Aplica limpieza y normalización de texto dentro de un flujo pipeline.
Esta función actúa como adaptador de `acep_clean()` para trabajar con
objetos `acep_corpus`, registrando las transformaciones aplicadas.
}
\examples{
# Crear corpus y limpiar
textos <- c("El SUTEBA va al paro!!!", "SOIP protesta 123")
corpus <- acep_corpus(textos)
corpus_limpio <- pipe_clean(corpus, rm_punt = TRUE, rm_num = TRUE)
print(corpus_limpio)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_together.R
\name{acep_together}
\alias{acep_together}
\title{Interaccion con modelos de IA usando TogetherAI}
\usage{
acep_together(
  texto,
  instrucciones,
  modelo = "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
  api_key = Sys.getenv("TOGETHER_API_KEY"),
  schema = NULL,
  parse_json = TRUE,
  temperature = 0,
  max_tokens = 2000,
  top_p = 0.2,
  top_k = 50,
  repetition_penalty = 1,
  stop = NULL,
  prompt_system = "json",
  timeout = 120
)
}
\arguments{
\item{texto}{Texto a analizar. Puede ser una noticia, tweet, documento, etc.}

\item{instrucciones}{Instrucciones en lenguaje natural que indican al modelo que hacer
con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".}

\item{modelo}{Modelo a utilizar. Ejemplos populares:
- Moonshot: `"moonshotai/Kimi-K2-Instruct-0905"` (128K context)
- Meta Llama: `"meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo"`, `"meta-llama/Llama-3.3-70B-Instruct-Turbo"`
- Qwen: `"Qwen/Qwen2.5-72B-Instruct-Turbo"`, `"Qwen/QwQ-32B-Preview"`
- Mistral: `"mistralai/Mixtral-8x22B-Instruct-v0.1"`, `"mistralai/Mistral-7B-Instruct-v0.3"`
- DeepSeek: `"deepseek-ai/DeepSeek-V3"`, `"deepseek-ai/DeepSeek-R1"`
- Google: `"google/gemma-2-27b-it"`, `"google/gemma-2-9b-it"`
Por defecto: `"meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo"`.
Ver lista completa: https://docs.together.ai/docs/chat-models}

\item{api_key}{Clave de API de TogetherAI. Si no se proporciona, busca la variable de
entorno `TOGETHER_API_KEY`. Para obtener una clave: https://api.together.xyz/settings/api-keys}

\item{schema}{Esquema JSON que define la estructura de la respuesta. Puede usar
`acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
Si es `NULL`, usa un esquema simple con campo "respuesta".
NOTA: TogetherAI soporta JSON mode con \code{response_format: \{type: "json_object"\}} para
modelos compatibles. Consulta la lista de modelos soportados en:
https://docs.together.ai/docs/json-mode}

\item{parse_json}{Logico. Si `TRUE` (por defecto), parsea automaticamente el JSON
a un objeto R (lista o data frame). Si `FALSE`, devuelve el JSON como string.}

\item{temperature}{Parametro de temperatura (0-2). Valores bajos (0-0.3) generan
respuestas mas deterministas. Valores altos (0.7-1) mas creativas. Por defecto: 0.}

\item{max_tokens}{Numero maximo de tokens en la respuesta. Por defecto: 2000.}

\item{top_p}{Parametro top-p para nucleus sampling (0-1). Por defecto: 0.2.}

\item{top_k}{Parametro top-k para muestreo. Limita las opciones a los k tokens mas probables.
Por defecto: 50. Usar 0 o -1 para desactivar.}

\item{repetition_penalty}{Penalizacion por repeticion de tokens (0.1-2.0).
Valores > 1 penalizan repeticiones. Por defecto: 1.}

\item{stop}{Secuencias de parada opcionales. Vector de strings que detienen la generacion.
Por defecto: NULL.}

\item{prompt_system}{Prompt del sistema que define el comportamiento del modelo. Opciones:
- `"json"` (por defecto): Usa un prompt estructurado que instruye al modelo a responder
  SOLO en formato JSON siguiendo el esquema proporcionado. Agrega \code{response_format: \{type: "json_object"\}}
- `"texto"`: Usa un prompt simple para respuestas en texto plano sin estructura.
  Elimina automaticamente el contenido de pensamiento (<think>...</think>) de modelos como Qwen3-Thinking
- String personalizado: Cualquier texto que definas como prompt del sistema}

\item{timeout}{Tiempo maximo de espera de la peticion HTTP en segundos.
Por defecto: 120. Evita que una conexion estancada bloquee la sesion de R.}
}
\value{
Si `parse_json=TRUE`, devuelve una lista o data frame con la respuesta
  estructurada segun el esquema. Si `parse_json=FALSE`, devuelve un string JSON.
}
\description{
Funcion para interactuar con modelos de IA a traves de la API de TogetherAI.
TogetherAI proporciona acceso a modelos open-source de alta calidad como
Llama, Qwen, Mistral, DeepSeek y muchos otros. Soporta JSON mode para
respuestas estructuradas. La API es compatible con el formato de OpenAI,
lo que facilita la integracion.
}
\details{
**Sobre TogetherAI:**

TogetherAI es una plataforma especializada en modelos open-source que ofrece:
- Precios competitivos y modelos gratuitos
- Alta velocidad de inferencia optimizada
- Acceso a modelos de ultima generacion (Llama, Qwen, DeepSeek, etc.)
- API compatible con formato OpenAI

**JSON Mode:**

La funcion utiliza JSON mode de TogetherAI para obtener respuestas estructuradas.
Cuando `prompt_system = "json"`, la funcion:
1. Incluye el esquema JSON en el prompt del sistema (REQUERIDO por TogetherAI)
2. Agrega \code{response_format: \{type: "json_object"\}} al body de la peticion
3. Instruye explicitamente al modelo a responder SOLO en JSON

Esta combinacion de esquema textual + \code{response_format} asegura respuestas JSON validas
y consistentes en cada llamada.

**Modelos compatibles con JSON mode:**

Los modelos mas recientes que soportan JSON mode incluyen:
- Qwen3, Qwen2.5 (Instruct, Coder, VL, Thinking)
- DeepSeek-R1, DeepSeek-V3
- Meta Llama 3.1, 3.3, 4
- Mistral 7B Instruct
- Google Gemma

Ver lista completa: https://docs.together.ai/docs/json-mode

**Validaciones:**

La funcion incluye validacion de limite de tokens. Si la respuesta es truncada
por `max_tokens`, devuelve un mensaje claro indicando que se necesitan mas tokens.
}
\examples{
\dontrun{
# Configurar API key
Sys.setenv(TOGETHER_API_KEY = "tu-api-key")

# Usar Llama 3.1 70B (rapido y potente)
texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
resultado <- acep_together(texto, "Extrae las entidades nombradas",
                           modelo = "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                           schema = acep_gpt_schema("extraccion_entidades"))

# Usar Qwen para analisis de sentimiento
resultado_qwen <- acep_together(texto, "Clasifica el sentimiento",
                                modelo = "Qwen/Qwen2.5-72B-Instruct-Turbo",
                                schema = acep_gpt_schema("sentimiento"))

# Usar DeepSeek-V3
resultado_ds <- acep_together(texto, "Analiza el texto",
                              modelo = "deepseek-ai/DeepSeek-V3",
                              schema = acep_gpt_schema("clasificacion"))

# Usar Moonshot Kimi con 128K context
resultado_kimi <- acep_together(texto, "Resume el texto",
                                modelo = "moonshotai/Kimi-K2-Instruct-0905",
                                schema = acep_gpt_schema("resumen"))

# Usar modo texto plano (sin estructura JSON)
resultado_texto <- acep_together(texto, "Resume este texto en una frase",
                                 modelo = "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                                 prompt_system = "texto",
                                 parse_json = FALSE)
print(resultado_texto)  # Devuelve string de texto plano

# Usar prompt del sistema personalizado
resultado_custom <- acep_together(
  texto,
  "Analiza el sentimiento",
  modelo = "Qwen/Qwen2.5-72B-Instruct-Turbo",
  prompt_system = paste(
    "Eres un experto en analisis de sentimientos politicos.",
    "Se objetivo y neutral."
  ),
  parse_json = FALSE
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_token_table.R
\name{acep_token_table}
\alias{acep_token_table}
\title{Tabla de frecuencia de palabras tokenizadas.}
\usage{
acep_token_table(x, u = 10)
}
\arguments{
\item{x}{vector de palabras tokenizadas.}

\item{u}{número de corte para el top de palabras más frecuentes.}
}
\value{
Si todas las entradas son correctas,
la salida sera una tabla con la frecuencia relativa y
absoluta de palabras tokenizadas.
}
\description{
Función que cuenta la frecuencia de palabras tokenizadas.
}
\examples{
tokens <- c(rep("paro",15), rep("piquete",25), rep("corte",20), rep("manifestación",10),
rep("bloqueo",5), rep("alerta",16), rep("ciudad",12), rep("sindicato",11), rep("paritaria",14),
rep("huelga",14), rep("escrache",15))
acep_token_table(tokens)
}
\keyword{tablas}
\keyword{tokens}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_provider.R
\name{.acep_provider_strip_key}
\alias{.acep_provider_strip_key}
\title{Eliminar recursivamente una clave de un esquema con estructura anidada}
\usage{
.acep_provider_strip_key(schema, key)
}
\description{
Recorre el esquema (objetos anidados, items de arrays) y elimina todas las
apariciones de `key`, no solo la del nivel raiz. Util para proveedores cuyo
formato de esquema no admite ciertas claves (por ejemplo `additionalProperties`
en el responseSchema de Gemini).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_provider.R
\name{.acep_provider_http_error_message}
\alias{.acep_provider_http_error_message}
\title{Extraer un mensaje de error legible de una respuesta HTTP de error}
\usage{
.acep_provider_http_error_message(error_content)
}
\description{
Maneja de forma robusta los distintos cuerpos de error que puede devolver una
API: una lista con `error$message`, un campo `error` atomico (una cadena en
lugar de un objeto), un cuerpo de texto plano (p. ej. una pagina HTML de un
gateway 5xx) o un contenido nulo. Nunca falla con
"$ operator is invalid for atomic vectors".
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_count.R
\name{.acep_count_pattern}
\alias{.acep_count_pattern}
\title{Crear patrón regex para diccionarios de conteo}
\usage{
.acep_count_pattern(dic)
}
\description{
Escapa los metacaracteres de cada término (para tratarlos como texto literal,
evitando coincidencias accidentales o errores de regex) y convierte un espacio
inicial o final en un límite de palabra (\\b), de modo que " paro " coincida
solo con la palabra completa.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_load_base.R
\name{acep_load_base}
\alias{acep_load_base}
\title{Carga los corpus y las bases creadas por el Observatorio.}
\usage{
acep_load_base(tag)
}
\arguments{
\item{tag}{etiqueta identificatoria del data frame a cargar:
acep_bases$rp_mdp, acep_bases$ln_bb, acep_bases$lc_mdp,
acep_bases$ed_neco, acep_bases$ln_arg}
}
\value{
Si la descarga es exitosa, devuelve una base de datos en formato
tabular con un corpus de notas. Ante un fallo de red, una URL inexistente o
un error de lectura, informa el problema con un `message()` y devuelve `NULL`
de forma invisible.
}
\description{
Función para cargar bases de datos disponibles online.
Por ahora están disponibles las siguientes bases: Revista Puerto 'rp_mdp';
La Nueva 'ln_bb', La Capital 'lc_mdp', Ecos Diarios 'ed_neco',
La Nación 'ln_arg'
}
\examples{
\dontrun{
bd_sismos <- acep_bases$rev_puerto
head(acep_load_base(tag = bd_sismos))
}
}
\keyword{datos}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_svo.R
\name{acep_svo}
\alias{acep_svo}
\title{Función para extraer tripletes SVO (Sujeto-Verbo-Objeto).}
\source{
\href{https://universaldependencies.org/}{Dependencias Universales para taggeo POS}

\href{https://CRAN.R-project.org/package=rsyntax}{Sobre el paquete rsyntax}
}
\usage{
acep_svo(acep_tokenindex, prof_s = 3, prof_o = 3, u = 1)
}
\arguments{
\item{acep_tokenindex}{data.frame con el etiquetado POS y las relaciones
de dependencia generado con la función acep_postag.}

\item{prof_s}{es un numero entero positivo que determina la profundidad a la que se
buscan las relaciones dentro del sujeto. Este parámetro se hereda del la función children()
del paquete \{rsyntax\}. Se recomienda no superar el valor 2.}

\item{prof_o}{es un numero entero positivo que determina la profundidad a la que se
buscan las relaciones dentro del objeto. Este parámetro se hereda del la función children()
del paquete \{rsyntax\}. Se recomienda no superar el valor 2.}

\item{u}{numero entero que indica el umbral de palabras del objeto en la reconstrucción SVO.}
}
\value{
Si todas las entradas son correctas, la salida sera una lista con tres bases de datos en formato tabular.
}
\description{
Función que devuelve seis objetos data.frame con
etiquetado POS (modelo spacyr) y relaciones sintácticas
(modelo rsyntax) que permiten reconstruir estructuras
sintácticas como SVO y Sujeto-Predicado. Una vez seleccionadas
las notas periodísticas referidas a conflictos, esta función permite
extraer sujetos de la protesta, acción realizada y objeto(s) de la acción.
También devuelve entidades nombradas (NER).
}
\examples{
\dontrun{
acep_svo(acep_bases$spacy_postag)
}
}
\references{
Welbers, K., Atteveldt, W. van, & Kleinnijenhuis, J. 2021. Extracting semantic relations using syntax:
An R package for querying and reshaping dependency trees. Computational Communication Research, 3-2, 1-16.
\doi{10.5117/CCR2021.2.003.WELB}
}
\keyword{sintaxis}
\keyword{tripletes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_pipeline.R
\name{acep_pipeline}
\alias{acep_pipeline}
\title{Pipeline completo de análisis de conflictividad}
\usage{
acep_pipeline(texto, dic, clean = TRUE, ...)
}
\arguments{
\item{texto}{Vector de caracteres con los textos a analizar.}

\item{dic}{Vector de caracteres con las palabras del diccionario de conflictividad
(o cualquier otro diccionario temático) a buscar en los textos.}

\item{clean}{Lógico. Si `TRUE` (por defecto), aplica limpieza y normalización
al texto antes del análisis usando `acep_clean()`.}

\item{...}{Argumentos adicionales para pasar a `acep_clean()` cuando `clean = TRUE`.
Por ejemplo: `rm_stopwords = TRUE`, `rm_num = TRUE`, `tolower = TRUE`.}
}
\value{
Objeto de clase `acep_result` con tipo `"intensidad"` que contiene:
\itemize{
  \item \code{id}: Identificadores de cada texto
  \item \code{texto}: Textos analizados (limpios si `clean = TRUE`)
  \item \code{frecuencia}: Número de menciones del diccionario por texto
  \item \code{n_palabras}: Número total de palabras por texto
  \item \code{intensidad}: Índice normalizado de intensidad (frecuencia/n_palabras)
}
}
\description{
Ejecuta un flujo de trabajo completo de análisis de texto que incluye:
limpieza opcional, conteo de menciones de un diccionario, y cálculo de
intensidad. Esta función encadena automáticamente las funciones `pipe_clean()`,
`pipe_count()` y `pipe_intensity()` para facilitar análisis rápidos.
}
\examples{
\dontrun{
# Pipeline completo con limpieza
textos <- c("El SUTEBA va al paro por mejoras salariales",
            "SOIP en lucha contra despidos")
dic_conflictos <- c("paro", "lucha", "reclamo", "protesta")
resultado <- acep_pipeline(textos, dic_conflictos,
                           clean = TRUE, rm_stopwords = TRUE)
print(resultado)

# Pipeline sin limpieza
resultado <- acep_pipeline(textos, dic_conflictos, clean = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_count.R
\docType{data}
\name{.acep_regex_cache}
\alias{.acep_regex_cache}
\title{Cache para regex compilados}
\format{
An object of class \code{environment} of length 0.
}
\usage{
.acep_regex_cache
}
\description{
Cache para regex compilados
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_sst.R
\name{acep_sst}
\alias{acep_sst}
\title{Serie temporal de índices de conflictividad.}
\usage{
acep_sst(datos, st = "mes", u = 2, d = 4)
}
\arguments{
\item{datos}{data frame con las variables 'fecha' (en formato Date),
'n_palabras' (numérica), conflictos' (numérica), 'intensidad' (numérica).
Las ultimas tres se pueden construir en un solo paso con la función 'acep_db'
o en tres pasos con las funciones 'acep_frec', 'acep_men', 'acep_int'.}

\item{st}{parámetro para establecer el segmento temporal
a ser agrupado: 'anio', 'mes', 'dia'.}

\item{u}{umbral de menciones para contabilizar una nota
como nota que refiere a un conflicto, por defecto tiene 2
pero se puede modificar.}

\item{d}{cantidad de decimales, por defecto tiene 4 pero
se puede modificar.}
}
\value{
Si todas las entradas son correctas,
la salida sera una base de datos en formato tabular
con nuevas variables.
}
\description{
Función que devuelve los indices de conflictividad
agrupados por segmento de tiempo: 'dia', 'mes', 'anio'. Esta función
viene a reemplazar a acep_rst. Simplifica los parámetros.
}
\examples{
datos <- acep_bases$rp_procesada
head(datos)
datos_procesados_anio <- acep_sst(datos, st='anio', u=4)
datos_procesados_mes <- acep_sst(datos)
datos_procesados_dia <- acep_sst(datos, st ='dia', d=3)
head(datos_procesados_anio)
head(datos_procesados_mes)
head(datos_procesados_dia)
}
\keyword{resumen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_detect.R
\name{acep_detect}
\alias{acep_detect}
\title{Detección de menciones de palabras.}
\usage{
acep_detect(x, y, u = 1, tolower = TRUE)
}
\arguments{
\item{x}{vector de textos al que se le aplica la función de
detección de menciones de palabras del diccionario.}

\item{y}{vector de palabras del diccionario utilizado.}

\item{u}{umbral para atribuir valor positivo a la
detección de las menciones.}

\item{tolower}{convierte los textos a minúsculas.}
}
\value{
Si todas las entradas son correctas,
la salida sera un vector numérico.
}
\description{
Función que detecta de menciones de palabras que
refieren a conflictos en cada una de las notas/textos.
}
\examples{
df <- data.frame(texto = c("El SUTEBA fue al paro. Reclaman mejoras salariales.",
"El SOIP lleva adelante un plan de lucha con paros y piquetes."))
diccionario <- c("paro", "lucha", "piquetes")
df$detect <- acep_detect(df$texto, diccionario)
df
}
\keyword{frecuencia}
\keyword{indicadores}
\keyword{tokens}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_frec.R
\name{acep_frec}
\alias{acep_frec}
\title{Frecuencia de palabras totales.}
\usage{
acep_frec(x)
}
\arguments{
\item{x}{vector de textos al que se le aplica la función de conteo
de la frecuencia de palabras.}
}
\value{
Si todas las entradas son correctas, la salida sera un vector
con una frecuencia de palabras.
}
\description{
Función que cuenta la frecuencia de palabras totales
en cada una de las notas/textos.
}
\examples{
acep_frec("El SUTEBA fue al paro. Reclaman mejoras salariales.")
}
\keyword{frecuencia}
\keyword{indicadores}
\keyword{tokens}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_rs.R
\docType{data}
\name{acep_rs}
\alias{acep_rs}
\title{Cadenas de caracteres para limpiar y normalizar textos.}
\format{
Es un objeto de clase 'list' con 14 componentes (cadenas de
caracteres y expresiones regulares).
\describe{
\item{sw1}{es un string de palabras vacias.}
\item{sw2}{es un string de palabras vacias (complemento de sw1).}
\item{dias}{es un string de dias.}
\item{meses}{es un string de meses.}
\item{emojis}{es un string con expresiones regulares para emojis.}
\item{sintildes}{es un string de letras sin tildes.}
\item{tildes}{es un string de letras con tildes.}
\item{punt}{es un string de puntuación.}
\item{num}{es una expresión regular para números.}
\item{hashtag}{es una expresión regular para hashtag.}
\item{espacios}{es una expresión regular para espacios.}
\item{saltos}{es una expresión regular para saltos de línea.}
\item{url}{es una expresión regular para urls.}
\item{users}{es una expresión regular para usuarixs.}
}
}
\usage{
data(acep_rs)
}
\description{
Cadenas de caracteres y expresiones regulares
para limpiar y normalizar textos.
}
\examples{
print(acep_rs)
}
\keyword{expresiones}
\keyword{regulares}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_pipeline.R
\name{pipe_timeseries}
\alias{pipe_timeseries}
\title{Generación de series temporales en pipeline}
\usage{
pipe_timeseries(data, st = "mes", u = 2, d = 4)
}
\arguments{
\item{data}{Data frame o objeto `acep_result` que contenga columnas:
`fecha` (o variable temporal), `n_palabras`, `conflictos`, `intensidad`.}

\item{st}{Segmento temporal para agrupar. Valores: `"dia"`, `"mes"`, `"anio"`.
Por defecto: `"mes"`.}

\item{u}{Umbral para calcular métricas categóricas. Por defecto: 2.}

\item{d}{Número de decimales para redondear. Por defecto: 4.}
}
\value{
Objeto `acep_result` con tipo `"serie_temporal"` que contiene
  agregaciones por período temporal.
}
\description{
Crea agregaciones temporales de índices de conflictividad dentro de un flujo pipeline.
Agrupa los resultados por segmentos temporales (día, mes, año) y calcula
estadísticas resumidas usando `acep_sst()`.
}
\examples{
\dontrun{
# Crear serie temporal desde data frame con fechas
data <- data.frame(
  fecha = as.Date(c("2024-01-15", "2024-01-20", "2024-02-10")),
  n_palabras = c(100, 150, 120),
  conflictos = c(5, 8, 6),
  intensidad = c(0.05, 0.053, 0.05)
)
serie <- pipe_timeseries(data, st = "mes", u = 2)
print(serie)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_ollama.R
\name{acep_ollama}
\alias{acep_ollama}
\title{Interaccion con modelos Ollama locales y cloud usando Structured Outputs}
\usage{
acep_ollama(
  texto,
  instrucciones,
  modelo = "qwen3:1.7b",
  schema = NULL,
  parse_json = TRUE,
  temperature = 0,
  max_tokens = 4000,
  host = "http://localhost:11434",
  api_key = Sys.getenv("OLLAMA_API_KEY"),
  seed = 123456
)
}
\arguments{
\item{texto}{Texto a analizar. Puede ser una noticia, tweet, documento, etc.}

\item{instrucciones}{Instrucciones en lenguaje natural que indican al modelo que hacer
con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".}

\item{modelo}{Modelo de Ollama a utilizar.
- Para Ollama local: "qwen3:1.7b", "llama3.2:latest", "mistral", "phi3", "gemma2"
  (debe estar previamente descargado con `ollama pull nombre_modelo`)
- Para Ollama Cloud API: modelos cloud especificos disponibles sin GPU local:
  "deepseek-v3.1:671b-cloud", "gpt-oss:20b-cloud", "gpt-oss:120b-cloud",
  "kimi-k2:1t-cloud", "qwen3-coder:480b-cloud", "glm-4.6:cloud", "minimax-m2:cloud"
Por defecto: "qwen3:1.7b"}

\item{schema}{Esquema JSON que define la estructura de la respuesta. Puede usar
`acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
Si es NULL, usa un esquema simple con campo "respuesta".}

\item{parse_json}{Logico. Si TRUE (por defecto), parsea automaticamente el JSON
a un objeto R (lista o data frame). Si FALSE, devuelve el JSON como string.}

\item{temperature}{Parametro de temperatura (0-2). Valores bajos (0-0.3) generan
respuestas mas deterministas. Valores altos (0.7-1) mas creativas. Por defecto: 0.}

\item{max_tokens}{Numero maximo de tokens en la respuesta. Por defecto: 4000.}

\item{host}{URL del servidor Ollama. Por defecto: "http://localhost:11434" para uso local.
Para usar Ollama Cloud API, especificar "https://ollama.com" (sin /api, se agrega automaticamente).}

\item{api_key}{API key para Ollama API remota. Solo requerido si usas un servidor remoto.
Por defecto busca la variable de entorno OLLAMA_API_KEY. Para uso local (localhost) no es necesario.}

\item{seed}{Semilla numerica para reproducibilidad. Por defecto: 123456.}
}
\value{
Si parse_json=TRUE, devuelve una lista o data frame con la respuesta
  estructurada segun el esquema. Si parse_json=FALSE, devuelve un string JSON.
}
\description{
Funcion para interactuar con modelos de lenguaje usando Ollama.
Soporta tanto modelos locales (ejecutados en tu computadora sin costos)
como modelos cloud de Ollama (modelos grandes como DeepSeek 671B, Qwen3 Coder 480B,
Kimi 1T que se ejecutan en la nube sin necesidad de GPU local).
Utiliza structured outputs para garantizar respuestas en formato JSON que
cumplen con un esquema predefinido.
}
\examples{
\dontrun{
# Primero, instalar Ollama y descargar un modelo:
# Terminal: ollama pull llama3.1

# Extraer entidades de un texto
texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
instrucciones <- "Extrae todas las entidades nombradas del texto."
schema <- acep_gpt_schema("extraccion_entidades")
resultado <- acep_ollama(texto, instrucciones, schema = schema)
print(resultado)

# Analisis de sentimiento
texto <- "La protesta fue pacifica y bien organizada."
schema <- acep_gpt_schema("sentimiento")
resultado <- acep_ollama(texto, "Analiza el sentimiento del texto", schema = schema)
print(resultado$sentimiento_general)

# Usar Ollama Cloud API (requiere API key)
# Los modelos cloud se ejecutan sin necesidad de GPU local
Sys.setenv(OLLAMA_API_KEY = "tu-api-key")
resultado_remoto <- acep_ollama(
  texto = texto,
  instrucciones = "Extrae entidades",
  modelo = "deepseek-v3.1:671b-cloud",  # Modelo cloud de 671B parametros
  host = "https://ollama.com",
  schema = acep_gpt_schema("extraccion_entidades")
)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_token.R
\name{acep_token}
\alias{acep_token}
\title{Tokenizador.}
\usage{
acep_token(x, tolower = TRUE, cleaning = TRUE)
}
\arguments{
\item{x}{vector de textos al que se le aplica la función de tokenización.}

\item{tolower}{convierte los textos a minúsculas.}

\item{cleaning}{hace una limpieza de los textos.}
}
\value{
Si todas las entradas son correctas,
la salida será un data.frame con las palabras tokenizadas.
}
\description{
Función que tokeniza las notas/textos.
}
\examples{
acep_token("Huelga de obreros del pescado en el puerto")
}
\keyword{tokenizar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_plot_st.R
\name{acep_plot_st}
\alias{acep_plot_st}
\title{Gráfico de barras de la serie temporal de indices de conflictividad.}
\usage{
acep_plot_st(
  x,
  y,
  t = "",
  ejex = "",
  ejey = "",
  etiquetax = "horizontal",
  color = "mint"
)
}
\arguments{
\item{x}{vector de valores del eje x (por ejemplo, fechas).}

\item{y}{vector de valores numéricos del eje y (por ejemplo, menciones).}

\item{t}{titulo del gráfico.}

\item{ejex}{nombre del eje x.}

\item{ejey}{nombre del eje y.}

\item{etiquetax}{orientación de las etiquetas del
eje x ('horizontal' | 'vertical').}

\item{color}{color de las barras.}
}
\value{
Si todas las entradas son correctas,
la salida sera una imagen de un panel.
}
\description{
Función que devuelve un gráfico de barras con la serie
temporal de indices de conflictividad por dia, mes o anio.
}
\examples{
datos <- acep_bases$rp_procesada
dpa <- acep_sst(datos, st = 'anio')
acep_plot_st(
dpa$st, dpa$frecm,
t = 'Evoluci\u00f3n de la conflictividad en el sector pesquero argentino',
ejex = 'A\u00f1os analizados',
ejey = 'Menciones de t\u00e9rminos del diccionario de conflictos',
etiquetax = 'horizontal')
}
\keyword{visualizacion}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_openrouter.R
\name{acep_openrouter}
\alias{acep_openrouter}
\title{Interaccion con modelos de IA usando OpenRouter}
\usage{
acep_openrouter(
  texto,
  instrucciones,
  modelo = "openai/gpt-4o-mini",
  api_key = Sys.getenv("OPENROUTER_API_KEY"),
  schema = NULL,
  parse_json = TRUE,
  temperature = 0,
  max_tokens = 2000,
  top_p = 0.2,
  app_name = NULL,
  site_url = NULL,
  use_fallback = FALSE,
  fallback_provider_order = NULL,
  fallback_models = NULL,
  timeout = 120
)
}
\arguments{
\item{texto}{Texto a analizar. Puede ser una noticia, tweet, documento, etc.}

\item{instrucciones}{Instrucciones en lenguaje natural que indican al modelo que hacer
con el texto. Ejemplo: "Extrae todas las entidades nombradas", "Clasifica el sentimiento".}

\item{modelo}{Modelo a utilizar con formato "proveedor/modelo". Ejemplos populares:
- OpenAI: `"openai/gpt-4o-mini"` (rapido y economico), `"openai/gpt-4o"` (potente)
- Anthropic: `"anthropic/claude-sonnet-4.5"`, `"anthropic/claude-3.5-haiku"`
- Google: `"google/gemini-2.5-flash"`, `"google/gemini-2.0-flash-001"`
- Meta: `"meta-llama/llama-3.3-70b-instruct"`, `"meta-llama/llama-4-maverick:free"`
- Qwen: `"qwen/qwen3-next-80b-a3b-instruct-2509"`
- DeepSeek: `"deepseek/deepseek-chat-v3-0324:free"`, `"deepseek/deepseek-r1:free"`
Por defecto: `"openai/gpt-4o-mini"`. Ver lista completa: https://openrouter.ai/models}

\item{api_key}{Clave de API de OpenRouter. Si no se proporciona, busca la variable de
entorno `OPENROUTER_API_KEY`. Para obtener una clave: https://openrouter.ai/settings/keys}

\item{schema}{Esquema JSON que define la estructura de la respuesta. Puede usar
`acep_gpt_schema()` para obtener esquemas predefinidos o crear uno personalizado.
Si es `NULL`, usa un esquema simple con campo "respuesta".
NOTA: Structured Outputs solo funciona con modelos compatibles (OpenAI GPT-4o+, Fireworks).
Para otros modelos, se usara JSON mode basico.}

\item{parse_json}{Logico. Si `TRUE` (por defecto), parsea automaticamente el JSON
a un objeto R (lista o data frame). Si `FALSE`, devuelve el JSON como string.}

\item{temperature}{Parametro de temperatura (0-2). Valores bajos (0-0.3) generan
respuestas mas deterministas. Valores altos (0.7-1) mas creativas. Por defecto: 0.}

\item{max_tokens}{Numero maximo de tokens en la respuesta. Por defecto: 2000.}

\item{top_p}{Parametro top-p para nucleus sampling (0-1). Por defecto: 0.2.}

\item{app_name}{Nombre de tu aplicacion (opcional). Se muestra en openrouter.ai/activity.}

\item{site_url}{URL de tu aplicacion (opcional). Para estadisticas en OpenRouter.}

\item{use_fallback}{Logico. Si `TRUE`, OpenRouter usara modelos alternativos si el
proveedor primario falla. Por defecto: FALSE (sin fallbacks).}

\item{fallback_provider_order}{Vector opcional de slugs de proveedores para forzar
un orden especifico de enrutamiento (ej.: `c("openai", "anthropic")`). Requiere
`use_fallback = TRUE` para habilitar intentos sucesivos.}

\item{fallback_models}{Vector opcional de modelos alternativos (en formato
`"proveedor/modelo"`) que se probaran en orden si el modelo principal devuelve un
error recuperable (429, 5xx, timeouts). Ideal para definir variantes pagas cuando
la version `:free` alcance su limite.}

\item{timeout}{Tiempo maximo de espera de la peticion HTTP en segundos.
Por defecto: 120. Evita que una conexion estancada bloquee la sesion de R.}
}
\value{
Si `parse_json=TRUE`, devuelve una lista o data frame con la respuesta
  estructurada segun el esquema. Si `parse_json=FALSE`, devuelve un string JSON.
}
\description{
Funcion para interactuar con multiples proveedores de IA (OpenAI, Anthropic, Google,
Meta, etc.) a traves de la API unificada de OpenRouter. Soporta Structured Outputs
para modelos compatibles (OpenAI GPT-4o+, Fireworks, y otros). OpenRouter normaliza
las diferencias entre proveedores, permitiendo acceder a 400+ modelos con una sola API.
Ideal para comparar modelos o usar fallbacks automaticos.
}
\details{
OpenRouter abstrae las diferencias entre proveedores, mapeando automaticamente los
parametros a la interfaz nativa de cada modelo. Los parametros no soportados por un
modelo son ignorados silenciosamente. Esto permite usar la misma funcion para
cualquier modelo sin preocuparse por las especificidades de cada API.

Cuando `use_fallback = TRUE`, la funcion configura el objeto `provider` de OpenRouter
para conservar la resiliencia ante errores transitorios y, si se define
`fallback_models`, intenta llamar secuencialmente a cada modelo alternativo ante
codigos recuperables (429, 5xx, timeouts). Esto evita depender del campo `route`,
ya deprecado en la API.

Para Structured Outputs estrictos, recomendamos usar modelos OpenAI (gpt-4o+) o
Fireworks. Otros modelos intentaran seguir el esquema pero sin garantias estrictas.
}
\section{MODELOS COMPATIBLES Y PROBADOS (ACTUALIZADO)}{

\subsection{OpenAI - Familia GPT-5 (Ultima Generacion)}{
\itemize{
  \item \code{openai/gpt-5} - Modelo principal GPT-5
  \item \code{openai/gpt-5-pro} - Version Pro, maxima precision
  \item \code{openai/gpt-5-mini} - Version mini, economica
  \item \code{openai/gpt-5-nano} - Version nano, ultrarrapida
  \item \code{openai/gpt-5-chat} - Optimizado para chat
}
}
\subsection{OpenAI - Familia GPT-4.1}{
\itemize{
  \item \code{openai/gpt-4.1} - Modelo principal GPT-4.1
  \item \code{openai/gpt-4.1-mini} - Version mini
  \item \code{openai/gpt-4.1-nano} - Version nano
}
}
\subsection{OpenAI - Familia GPT-4}{
\itemize{
  \item \code{openai/gpt-4o} - GPT-4 optimizado
  \item \code{openai/gpt-4o-mini} - Economico, rapido, ideal para produccion
  \item \code{openai/gpt-4-turbo} - Version turbo
  \item \code{openai/gpt-4} - Modelo base GPT-4
}
}
\subsection{OpenAI - Familia GPT-3.5}{
\itemize{
  \item \code{openai/gpt-3.5-turbo} - Version turbo, economica
}
}
\subsection{OpenAI - Modelos OSS (Open Source Style)}{
\itemize{
  \item \code{openai/gpt-oss-120b} - Modelo grande (120B parametros)
  \item \code{openai/gpt-oss-120b:exacto} - Version exacta
  \item \code{openai/gpt-oss-20b} - Modelo pequeno (20B parametros)
  \item \code{openai/gpt-oss-20b:free} - Version gratuita
}
}
\subsection{xAI - Familia Grok 4 (Ultima Generacion)}{
\itemize{
  \item \code{x-ai/grok-4} - Modelo principal Grok 4
  \item \code{x-ai/grok-4-fast} - Version rapida, optimizada
}
}
\subsection{xAI - Familia Grok 3}{
\itemize{
  \item \code{x-ai/grok-3} - Modelo principal Grok 3
  \item \code{x-ai/grok-3-mini} - Version mini, economica
  \item \code{x-ai/grok-3-beta} - Version beta
  \item \code{x-ai/grok-3-mini-beta} - Version mini beta
}
}
\subsection{DeepSeek - Familia V3 (Ultima Generacion)}{
\itemize{
  \item \code{deepseek/deepseek-v3.2-exp} - Version experimental 3.2
  \item \code{deepseek/deepseek-v3.1-terminus} - Version terminus 3.1
  \item \code{deepseek/deepseek-v3.1-terminus:exacto} - Version terminus exacta
}
}
\subsection{DeepSeek - Familia R1 (Razonamiento)}{
\itemize{
  \item \code{deepseek/deepseek-r1-0528} - Version R1 de mayo 2028
  \item \code{deepseek/deepseek-r1} - Modelo principal R1
  \item \code{deepseek/deepseek-r1:free} - Version R1 gratuita
}
}
\subsection{DeepSeek - R1 Distilled (Basado en Llama)}{
\itemize{
  \item \code{deepseek/deepseek-r1-distill-llama-70b} - R1 destilado en Llama 70B
  \item \code{deepseek/deepseek-r1-distill-llama-70b:free} - Version gratuita
}
}
\subsection{DeepSeek - Chat y Otros}{
\itemize{
  \item \code{deepseek/deepseek-chat} - Optimizado para chat
}
}
\subsection{DeepCogito - Modelos Especializados}{
\itemize{
  \item \code{deepcogito/cogito-v2-preview-deepseek-671b} - Modelo cogito 671B basado en DeepSeek
}
}
\subsection{Meta Llama - Familia 4}{
\itemize{
  \item \code{meta-llama/llama-4-maverick} - Llama 4 Maverick
}
}
\subsection{Meta Llama - Familia 3.3}{
\itemize{
  \item \code{meta-llama/llama-3.3-70b-instruct} - Version de pago
  \item \code{meta-llama/llama-3.3-70b-instruct:free} - Version gratuita
}
}
\subsection{Meta Llama - Familia 3.2}{
\itemize{
  \item \code{meta-llama/llama-3.2-90b-vision-instruct} - Con capacidades de vision
}
}
\subsection{Meta Llama - Familia 3.1}{
\itemize{
  \item \code{meta-llama/llama-3.1-405b-instruct} - Modelo grande 405B
  \item \code{meta-llama/llama-3.1-70b-instruct} - Modelo mediano 70B
}
}
\subsection{NousResearch - Modelos Hermes}{
\itemize{
  \item \code{nousresearch/hermes-3-llama-3.1-405b} - Hermes 3 basado en Llama 405B
}
}
\subsection{Mistral AI - Familia Magistral}{
\itemize{
  \item \code{mistralai/magistral-medium-2506} - Magistral medium
  \item \code{mistralai/magistral-medium-2506:thinking} - Con razonamiento extendido
}
}
\subsection{Mistral AI - Otros Modelos}{
\itemize{
  \item \code{mistralai/mistral-large-2407} - Mistral large
  \item \code{mistralai/mixtral-8x22b-instruct} - Mixtral 8x22B (MoE)
}
}
\subsection{MoonshotAI - Familia Kimi K2}{
\itemize{
  \item \code{moonshotai/kimi-k2} - Modelo principal Kimi K2
  \item \code{moonshotai/kimi-k2-0905} - Version 09/05
  \item \code{moonshotai/kimi-k2-0905:exacto} - Version exacta
  \item \code{moonshotai/kimi-k2-thinking} - Con razonamiento extendido
}
}
\subsection{Qwen - Familia Qwen3 (235B - Modelos Grandes)}{
\itemize{
  \item \code{qwen/qwen3-235b-a22b-2507} - Modelo grande 235B (version 2507)
  \item \code{qwen/qwen3-235b-a22b} - Modelo grande 235B
  \item \code{qwen/qwen3-235b-a22b-thinking-2507} - Con razonamiento extendido
  \item \code{qwen/qwen3-max} - Version maxima
}
}
\subsection{Qwen - Familia Qwen3 (30B-80B - Modelos Medianos)}{
\itemize{
  \item \code{qwen/qwen3-next-80b-a3b-instruct} - Modelo next 80B
  \item \code{qwen/qwen3-32b} - Modelo 32B
  \item \code{qwen/qwen3-30b-a3b} - Modelo 30B
  \item \code{qwen/qwen3-30b-a3b-instruct-2507} - Version instruct 2507
  \item \code{qwen/qwen3-30b-a3b:free} - Version 30B gratuita
}
}
\subsection{Qwen - Familia Qwen3 (Modelos Pequenos)}{
\itemize{
  \item \code{qwen/qwen3-14b:free} - Modelo 14B gratuito
  \item \code{qwen/qwen3-4b:free} - Modelo 4B gratuito, ultrarrapido
}
}
\subsection{Qwen - Familia Qwen 2.5}{
\itemize{
  \item \code{qwen/qwen-2.5-72b-instruct} - Modelo 2.5 generacion anterior
  \item \code{qwen/qwen-plus} - Version plus
}
}
\subsection{Google Gemini - Familia 2.5 (Ultima Generacion)}{
\itemize{
  \item \code{google/gemini-2.5-flash} - Rapido, ultima generacion
  \item \code{google/gemini-2.5-pro} - Mayor precision, ultima generacion
  \item \code{google/gemini-2.5-flash-lite} - Ultrarrapido, ligero
  \item \code{google/gemini-2.5-flash-preview-09-2025} - Preview version septiembre
  \item \code{google/gemini-2.5-flash-lite-preview-09-2025} - Preview lite septiembre
}
}
\subsection{Google Gemini - Familia 2.0}{
\itemize{
  \item \code{google/gemini-2.0-flash-001} - Version estable 2.0
  \item \code{google/gemini-2.0-flash-lite-001} - Version ligera 2.0
}
}
\subsection{Google Gemini - Familia 1.5}{
\itemize{
  \item \code{google/gemini-pro-1.5} - Version anterior, estable
}
}
\subsection{Anthropic Claude - Familia Haiku (Rapidos y Economicos)}{
\itemize{
  \item \code{anthropic/claude-3.5-haiku} - Version 3.5, muy rapido
  \item \code{anthropic/claude-3-haiku} - Version 3, economico
  \item \code{anthropic/claude-haiku-4.5} - Ultima version, mas preciso
}
}
\subsection{Anthropic Claude - Familia Sonnet (Equilibrados)}{
\itemize{
  \item \code{anthropic/claude-3.5-sonnet} - Popular, buen balance
  \item \code{anthropic/claude-3.7-sonnet} - Version mejorada
  \item \code{anthropic/claude-3.7-sonnet:thinking} - Con razonamiento extendido
  \item \code{anthropic/claude-sonnet-4} - Generacion 4
  \item \code{anthropic/claude-sonnet-4.5} - Ultima version, mas preciso
}
}
\subsection{Anthropic Claude - Familia Opus (Maxima Precision)}{
\itemize{
  \item \code{anthropic/claude-3-opus} - Version 3, muy preciso
  \item \code{anthropic/claude-opus-4} - Generacion 4
  \item \code{anthropic/claude-opus-4.1} - Ultima version disponible
}
}
**Importante:** los modelos etiquetados como `:free` operan con cuotas comunitarias
y suelen estar sometidos a limites de tasa estrictos por parte de OpenRouter. Es
frecuente recibir respuestas HTTP 429 (Too Many Requests) cuando la demanda supera
la cuota disponible; este codigo indica que el proveedor rechazo la peticion para
proteger la infraestructura compartida. Si ocurre, espera unos segundos y reintenta,
o selecciona la variante de pago equivalente (sin sufijo `:free`) o activa `use_fallback`
para que OpenRouter cambie automaticamente a un modelo disponible.
}

\examples{
\dontrun{
# Configurar API key
Sys.setenv(OPENROUTER_API_KEY = "tu-api-key")

# Usar GPT-4o mini (rapido y economico)
texto <- "El SUTEBA convoco a un paro en Buenos Aires el 15 de marzo."
resultado <- acep_openrouter(texto, "Extrae las entidades nombradas",
                              modelo = "openai/gpt-4o-mini",
                              schema = acep_gpt_schema("extraccion_entidades"))

# Comparar con Claude
resultado_claude <- acep_openrouter(texto, "Extrae las entidades nombradas",
                                     modelo = "anthropic/claude-sonnet-4.5",
                                     schema = acep_gpt_schema("extraccion_entidades"))

# Usar modelo gratuito
resultado_free <- acep_openrouter(texto, "Clasifica el sentimiento",
                                   modelo = "meta-llama/llama-4-maverick:free",
                                   schema = acep_gpt_schema("sentimiento"))

# Definir fallback hacia variantes pagas o proveedores alternativos
resultado_resiliente <- acep_openrouter(
  texto,
  "Extrae las entidades nombradas",
  modelo = "meta-llama/llama-4-maverick:free",
  schema = acep_gpt_schema("extraccion_entidades"),
  use_fallback = TRUE,
  fallback_models = c("meta-llama/llama-4-maverick", "openai/gpt-4o-mini")
)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_classes.R
\name{acep_result}
\alias{acep_result}
\title{Constructor de resultados de analisis}
\usage{
acep_result(data, tipo = "general", metadata = NULL)
}
\arguments{
\item{data}{Data frame con los resultados del analisis.}

\item{tipo}{Tipo de resultado que contiene el objeto. Valores comunes:
`"frecuencia"`, `"intensidad"`, `"svo"`, `"serie_temporal"`, `"general"`.
Este parametro determina el comportamiento de los metodos de impresion y visualizacion.}

\item{metadata}{Lista opcional con informacion sobre el analisis realizado
(ej: diccionario utilizado, parametros aplicados, corpus de origen).}
}
\value{
Objeto de clase `acep_result` con la siguiente estructura:
\itemize{
  \item \code{data}: Data frame con los resultados del analisis
  \item \code{tipo}: Etiqueta del tipo de resultado
  \item \code{metadata}: Informacion adicional del analisis
  \item \code{fecha_creacion}: Timestamp de creacion del objeto
}
}
\description{
Crea un objeto de clase `acep_result` que encapsula los resultados de
un analisis de texto realizado con funciones de ACEP. Este objeto
proporciona metodos especializados para visualizacion (`plot()`),
resumen (`summary()`) y conversion a data frame (`as.data.frame()`).
}
\examples{
# Crear resultado de analisis de frecuencias
datos <- data.frame(
  texto = c("El SUTEBA va al paro", "SOIP protesta"),
  frecuencia = c(5, 3)
)
resultado <- acep_result(datos, tipo = "frecuencia")
print(resultado)
summary(resultado)

# Convertir a data frame
df <- as.data.frame(resultado)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_count.R
\name{.acep_escape_regex}
\alias{.acep_escape_regex}
\title{Escapar metacaracteres de expresiones regulares en términos literales}
\usage{
.acep_escape_regex(x)
}
\description{
Escapar metacaracteres de expresiones regulares en términos literales
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_context.R
\name{acep_context}
\alias{acep_context}
\title{Función para extraer contexto de palabras o frases.}
\usage{
acep_context(texto, clave, izq = 1, der = 1, ci = "\\\\b", cd = "\\\\S*")
}
\arguments{
\item{texto}{vector con los textos a procesar.}

\item{clave}{vector de palabras clave a contextualizar.}

\item{izq}{número de palabras de la ventana hacia la izquierda.}

\item{der}{número de palabras de la ventana hacia la derecha.}

\item{ci}{expresión regular a la izquierda de la palabra clave.}

\item{cd}{expresión regular a la derecha de la palabra clave.}
}
\value{
Data frame con id de textos y contexto de palabras/frases.
}
\description{
Versión optimizada que usa vectorización en lugar de bucles anidados.
Mejora de rendimiento de 70-80% respecto a la versión anterior.
}
\examples{
texto <- "El SOIP para por aumento de salarios"
texto_context <- acep_context(texto = texto, clave = "para")
texto_context
}
\keyword{contexto}
\keyword{tokens}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_prompt_gpt.R
\docType{data}
\name{acep_prompt_gpt}
\alias{acep_prompt_gpt}
\title{Colección de instrucciones para GPT.}
\format{
Es un objeto de clase 'list' con 6 componentes.
\describe{
\item{instruccion_breve_sao_es}{es un texto en castellano con instrucciones breves para
 extraer eventos de protesta y codificarlos con las siguientes claves: 'fecha',
 'sujeto', 'accion', 'objeto', 'lugar'.}
 \item{instruccion_larga_sao_es}{es un texto en castellano con instrucciones largas para
 extraer eventos de protesta y codificarlos con las siguientes claves: 'id',
 'cronica', 'fecha', 'sujeto', 'organizacion', 'participacion', 'accion',
 'objeto', 'lugar'.}
\item{instruccion_breve_sao_en}{es un texto en inglés con instrucciones breves para
 extraer eventos de protesta y codificarlos con las siguientes claves: 'date',
 'subject', 'action', 'object', 'place'.}
 \item{instruccion_larga_sao_en}{es un texto en inglés con instrucciones largas para
 extraer eventos de protesta y codificarlos con las siguientes claves: 'id',
 'chronicle', 'date', 'subject', 'organization', 'participation', 'action',
 'object', 'place'.}
\item{system_prompt_01_es}{es el prompt de sistema en castellano que utiliza
 \code{\link{acep_ollama}} para guiar el análisis de texto.}
\item{system_prompt_01_en}{es el prompt de sistema en inglés equivalente a
 system_prompt_01_es.}
}
}
\usage{
data(acep_prompt_gpt)
}
\description{
Colección de instrucciones para interactuar con los modelos
de OpenAI. Las instrucciones fueron testeadas en el marco de las tareas
que realizamos en el Observatorio de Conflictividad Social de la
Universidad Nacional de Mar del Plata.
}
\examples{
prompt01 <- acep_prompt_gpt$instruccion_larga_sao_es
prompt01
}
\keyword{diccionarios}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acep_count.R
\name{.acep_count_cache_key}
\alias{.acep_count_cache_key}
\title{Crear una clave de caché estable para diccionarios de conteo}
\usage{
.acep_count_cache_key(dic)
}
\description{
Crear una clave de caché estable para diccionarios de conteo
}
\keyword{internal}
